@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package javax.annotation;
 
 import java.lang.annotation.ElementType;
@@ -23,9 +21,11 @@ import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.lang.annotation.Target;
 
+/**
+ * @since Common Annotations 1.0
+ */
 @Target({ElementType.TYPE, ElementType.METHOD, ElementType.FIELD})
 @Retention(RetentionPolicy.RUNTIME)
-
 public @interface Resource {
     public enum AuthenticationType {
         CONTAINER,
@@ -38,4 +38,8 @@ public @interface Resource {
     public boolean shareable() default true;
     public String description() default "";
     public String mappedName() default "";
+    /**
+     * @since Common Annotations 1.1
+     */
+    public String lookup() default "";
 }
