@@ -175,10 +175,6 @@ public class BeanELResolver extends ELResolver {
 
     @Override
     public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext context, Object base) {
-        if (context == null) {
-            throw new NullPointerException();
-        }
-
         if (base == null) {
             return null;
         }
@@ -200,10 +196,6 @@ public class BeanELResolver extends ELResolver {
 
     @Override
     public Class<?> getCommonPropertyType(ELContext context, Object base) {
-        if (context == null) {
-            throw new NullPointerException();
-        }
-
         if (base != null) {
             return Object.class;
         }
@@ -292,7 +284,7 @@ public class BeanELResolver extends ELResolver {
                 if (this.write == null) {
                     throw new PropertyNotFoundException(message(ctx,
                             "propertyNotWritable", new Object[] {
-                                    type.getName(), descriptor.getName() }));
+                                    owner.getName(), descriptor.getName() }));
                 }
             }
             return this.write;
@@ -304,7 +296,7 @@ public class BeanELResolver extends ELResolver {
                 if (this.read == null) {
                     throw new PropertyNotFoundException(message(ctx,
                             "propertyNotReadable", new Object[] {
-                                    type.getName(), descriptor.getName() }));
+                                    owner.getName(), descriptor.getName() }));
                 }
             }
             return this.read;
