@@ -71,7 +71,6 @@ import javax.servlet.ServletResponse;
  * information on handling multiple threads in a Java program.
  *
  * @author  Various
- * @version $Version$
  */
 public abstract class HttpServlet extends GenericServlet {
 
@@ -750,8 +749,12 @@ class NoBodyResponse extends HttpServletResponseWrapper {
 
     // file private
     void setContentLength() {
-        if (!didSetContentLength)
-          super.setContentLength(noBody.getContentLength());
+        if (!didSetContentLength) {
+            if (writer != null) {
+                writer.flush();
+            }
+            super.setContentLength(noBody.getContentLength());
+        }
     }
 
 
