@@ -27,8 +27,9 @@ import org.apache.catalina.core.StandardPipeline;
  * alternative defaults.
  *
  * @deprecated There was no demand for this capability and it will be removed in
- *             Tomact 8.0.x
+ *             Tomcat 8.0.x
  */
+@Deprecated
 public class CatalinaFactory {
 
     private static CatalinaFactory factory = new CatalinaFactory();
