@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -36,9 +36,7 @@ package org.apache.catalina;
  *
  * @author Craig R. McClanahan
  * @author Peter Donald
- * @version $Id: Pipeline.java 939350 2010-04-29 15:36:29Z kkolinko $
  */
-
 public interface Pipeline {
 
 
@@ -54,7 +52,7 @@ public interface Pipeline {
 
     /**
      * <p>Set the Valve instance that has been distinguished as the basic
-     * Valve for this Pipeline (if any).  Prioer to setting the basic Valve,
+     * Valve for this Pipeline (if any).  Prior to setting the basic Valve,
      * the Valve's <code>setContainer()</code> will be called, if it
      * implements <code>Contained</code>, with the owning Container as an
      * argument.  The method may throw an <code>IllegalArgumentException</code>
@@ -80,11 +78,15 @@ public interface Pipeline {
      * be associated with this Container, or <code>IllegalStateException</code>
      * if it is already associated with a different Container.</p>
      *
+     * <p>Implementation note: Implementations are expected to trigger the
+     * {@link Container#ADD_VALVE_EVENT} for the associated container if this
+     * call is successful.</p>
+     *
      * @param valve Valve to be added
      *
      * @exception IllegalArgumentException if this Container refused to
      *  accept the specified Valve
-     * @exception IllegalArgumentException if the specifie Valve refuses to be
+     * @exception IllegalArgumentException if the specified Valve refuses to be
      *  associated with this Container
      * @exception IllegalStateException if the specified Valve is already
      *  associated with a different Container
@@ -106,6 +108,10 @@ public interface Pipeline {
      * found and removed, the Valve's <code>setContainer(null)</code> method
      * will be called if it implements <code>Contained</code>.
      *
+     * <p>Implementation note: Implementations are expected to trigger the
+     * {@link Container#REMOVE_VALVE_EVENT} for the associated container if this
+     * call is successful.</p>
+     *
      * @param valve Valve to be removed
      */
     public void removeValve(Valve valve);
@@ -117,5 +123,24 @@ public interface Pipeline {
      */
     public Valve getFirst();
 
+    /**
+     * Returns true if all the valves in this pipeline support async, false otherwise
+     * @return true if all the valves in this pipeline support async, false otherwise
+     */
+    public boolean isAsyncSupported();
+
+
+    /**
+     * Return the Container with which this Pipeline is associated.
+     */
+    public Container getContainer();
+
+
+    /**
+     * Set the Container with which this Pipeline is associated.
+     *
+     * @param container The new associated container
+     */
+    public void setContainer(Container container);
 
 }
