@@ -44,7 +44,7 @@ import org.apache.tools.ant.BuildException;
  *           host="127.0.0.1"
  *           port="9014"
  *           ref=""
- *           name="Catalina:type=Manager,path="/ClusterTest",host=localhost" 
+ *           name="Catalina:type=Manager,context="/ClusterTest",host=localhost" 
  *           attribute="maxActiveSessions"
  *           value="100"
  *           type="int"
@@ -58,10 +58,8 @@ import org.apache.tools.ant.BuildException;
  * These tasks require Ant 1.6 or later interface.
  *
  * @author Peter Rossbach
- * @version $Id: JMXAccessorSetTask.java 939336 2010-04-29 15:00:41Z kkolinko $
  * @since 5.5.10
  */
-
 public class JMXAccessorSetTask extends JMXAccessorTask {
 
     // ----------------------------------------------------- Instance Variables
@@ -83,6 +81,7 @@ public class JMXAccessorSetTask extends JMXAccessorTask {
      * corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -156,6 +155,7 @@ public class JMXAccessorSetTask extends JMXAccessorTask {
      * @exception Exception
      *                if an error occurs
      */
+    @Override
     public String jmxExecute(MBeanServerConnection jmxServerConnection)
         throws Exception {
 
@@ -191,11 +191,10 @@ public class JMXAccessorSetTask extends JMXAccessorTask {
                 attribute, realValue));
         return null;
     }
-    
 
 
     /**
-     * Get MBean Attriute from Mbean Server
+     * Get MBean Attribute from Mbean Server
      * @param jmxServerConnection
      * @param name
      * @param attribute
