@@ -85,18 +85,16 @@ import org.apache.tools.ant.Project;
  * </b>: These tasks require Ant 1.6 or later interface.
  * 
  * @author Peter Rossbach
- * @version $Revision: 467222 $ $Date: 2006-10-24 05:17:11 +0200 (Di, 24. Okt 2006) $
  * @since 5.5.10
  */
-
 public class JMXAccessorTask extends BaseRedirectorHelperTask {
 
+    public static final String JMX_SERVICE_PREFIX = "service:jmx:rmi:///jndi/rmi://";
+
+    public static final String JMX_SERVICE_SUFFIX = "/jmxrmi";
+
     // ----------------------------------------------------- Instance Variables
 
-    public static String JMX_SERVICE_PREFIX = "service:jmx:rmi:///jndi/rmi://";
-
-    public static String JMX_SERVICE_SUFFIX = "/jmxrmi";
-
     private String name = null;
 
     private String resultproperty;
@@ -338,6 +336,7 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
      * @exception BuildException
      *                if a validation error occurs
      */
+    @Override
     public void execute() throws BuildException {
         if (testIfCondition() && testUnlessCondition()) {
             try {
@@ -350,11 +349,11 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
                     // or error line will be logged twice
                     throw new BuildException(error);
                 }
-            } catch (Throwable t) {
+            } catch (Exception e) {
                 if (isFailOnError()) {
-                    throw new BuildException(t);
+                    throw new BuildException(e);
                 } else {
-                    handleErrorOutput(t.getMessage());
+                    handleErrorOutput(e.getMessage());
                 }
             } finally {
                 closeRedirector();
@@ -374,12 +373,12 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
         else
             urlForJMX = JMX_SERVICE_PREFIX + host + ":" + port
                     + JMX_SERVICE_SUFFIX;
-        Map environment = null;
+        Map<String, String[]> environment = null;
         if (username != null && password != null) {
             String[] credentials = new String[2];
             credentials[0] = username;
             credentials[1] = password;
-            environment = new HashMap();
+            environment = new HashMap<String, String[]>();
             environment.put(JMXConnector.CREDENTIALS, credentials);
         }
         return JMXConnectorFactory.connect(new JMXServiceURL(urlForJMX),
@@ -419,6 +418,7 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
      * @throws MalformedURLException
      * @throws IOException
      */
+    @SuppressWarnings("null")
     public static MBeanServerConnection accessJMXConnection(Project project,
             String url, String host, String port, String username,
             String password, String refId) throws MalformedURLException,
@@ -430,10 +430,8 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
             try {
                 jmxServerConnection = (MBeanServerConnection) pref;
             } catch (ClassCastException cce) {
-                if (project != null) {
-                    project.log("wrong object reference " + refId + " - "
+                project.log("wrong object reference " + refId + " - "
                             + pref.getClass());
-                }
                 return null;
             }
         }
@@ -536,7 +534,7 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
             }
         } else if ("java.lang.Boolean".equals(valueType)
                 || "boolean".equals(valueType)) {
-            convertValue = new Boolean(value);
+            convertValue = Boolean.valueOf(value);
         } else if ("java.lang.Float".equals(valueType)
                 || "float".equals(valueType)) {
             try {
@@ -605,7 +603,7 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
      * an array and isSeparateArrayResults is true, resultproperty used as
      * prefix (<code>resultproperty.0-array.length</code> and store the
      * result array length at <code>resultproperty.length</code>. Other
-     * option is that you delemit your result with a delimiter
+     * option is that you delimit your result with a delimiter
      * (java.util.StringTokenizer is used).
      * 
      * @param propertyPrefix
@@ -617,12 +615,12 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
         if (result instanceof CompositeDataSupport) {
             CompositeDataSupport data = (CompositeDataSupport) result;
             CompositeType compositeType = data.getCompositeType();
-            Set keys = compositeType.keySet();
-            for (Iterator iter = keys.iterator(); iter.hasNext();) {
-                String key = (String) iter.next();
+            Set<String> keys = compositeType.keySet();
+            for (Iterator<String> iter = keys.iterator(); iter.hasNext();) {
+                String key = iter.next();
                 Object value = data.get(key);
-                OpenType type = compositeType.getType(key);
-                if (type instanceof SimpleType) {
+                OpenType<?> type = compositeType.getType(key);
+                if (type instanceof SimpleType<?>) {
                     setProperty(propertyPrefix + "." + key, value);
                 } else {
                     createProperty(propertyPrefix + "." + key, value);
@@ -630,15 +628,15 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
             }
         } else if (result instanceof TabularDataSupport) {
             TabularDataSupport data = (TabularDataSupport) result;
-            for (Iterator iter = data.keySet().iterator(); iter.hasNext();) {
+            for (Iterator<Object> iter = data.keySet().iterator(); iter.hasNext();) {
                 Object key = iter.next();
-                for (Iterator iter1 = ((List) key).iterator(); iter1.hasNext();) {
+                for (Iterator<?> iter1 = ((List<?>) key).iterator(); iter1.hasNext();) {
                     Object key1 = iter1.next();
                     CompositeData valuedata = data.get(new Object[] { key1 });
                     Object value = valuedata.get("value");
-                    OpenType type = valuedata.getCompositeType().getType(
+                    OpenType<?> type = valuedata.getCompositeType().getType(
                             "value");
-                    if (type instanceof SimpleType) {
+                    if (type instanceof SimpleType<?>) {
                         setProperty(propertyPrefix + "." + key1, value);
                     } else {
                         createProperty(propertyPrefix + "." + key1, value);
@@ -681,22 +679,9 @@ public class JMXAccessorTask extends BaseRedirectorHelperTask {
     }
 
     /**
-     * get all properties, when project is there got all project Properties
-     * @return properties
-     */
-    public Map getProperties() {
-        Project currentProject = getProject();
-        if (currentProject != null) {
-            return currentProject.getProperties();
-        } else {
-            return properties;
-        }        
-    }
-    
-    /**
-     * get all Properties
-     * @param property
-     * @return The property
+     * Get Property
+     * @param property name
+     * @return The property value
      */
     public String getProperty(String property) {
         Project currentProject = getProject();
