@@ -50,7 +50,6 @@ import org.apache.tomcat.util.http.parser.HttpParser;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: DigestAuthenticator.java 1405416 2012-11-03 20:55:42Z markt $
  */
 
 public class DigestAuthenticator extends AuthenticatorBase {
@@ -113,6 +112,14 @@ public class DigestAuthenticator extends AuthenticatorBase {
     protected Map<String,NonceInfo> nonces;
 
 
+    /**
+     * The last timestamp used to generate a nonce. Each nonce should get a
+     * unique timestamp.
+     */
+    protected long lastTimestamp = 0;
+    protected final Object lastTimestampLock = new Object();
+
+
     /**
      * Maximum number of server nonces to keep in the cache. If not specified,
      * the default value of 1000 is used.
@@ -399,6 +406,13 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
         long currentTime = System.currentTimeMillis();
 
+        synchronized (lastTimestampLock) {
+            if (currentTime > lastTimestamp) {
+                lastTimestamp = currentTime;
+            } else {
+                currentTime = ++lastTimestamp;
+            }
+        }
 
         String ipTimeKey =
             request.getRemoteAddr() + ":" + currentTime + ":" + getKey();
