@@ -5,17 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.connector;
 
 import java.io.IOException;
@@ -25,15 +23,15 @@ import java.io.IOException;
  * of a request by a remote client.
  *
  * @author Glenn L. Nielsen
- * @version $Id: ClientAbortException.java 939336 2010-04-29 15:00:41Z kkolinko $
  */
 
 public final class ClientAbortException extends IOException {
 
+    private static final long serialVersionUID = 1L;
+
 
     //------------------------------------------------------------ Constructors
 
-
     /**
      * Construct a new ClientAbortException with no other information.
      */
@@ -105,6 +103,7 @@ public final class ClientAbortException extends IOException {
     /**
      * Returns the message associated with this exception, if any.
      */
+    @Override
     public String getMessage() {
 
         return (message);
@@ -115,19 +114,21 @@ public final class ClientAbortException extends IOException {
     /**
      * Returns the cause that caused this exception, if any.
      */
+    @Override
     public Throwable getCause() {
-        
+
         return (throwable);
-        
+
     }
 
-    
+
     /**
      * Return a formatted string that describes this exception.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("ClientAbortException:  ");
+        StringBuilder sb = new StringBuilder("ClientAbortException:  ");
         if (message != null) {
             sb.append(message);
             if (throwable != null) {
