@@ -22,6 +22,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
+import java.lang.reflect.InvocationTargetException;
 import java.nio.charset.Charset;
 import java.security.Principal;
 import java.text.SimpleDateFormat;
@@ -40,6 +41,7 @@ import java.util.TimeZone;
 import java.util.TreeMap;
 import java.util.concurrent.atomic.AtomicBoolean;
 
+import javax.naming.NamingException;
 import javax.security.auth.Subject;
 import javax.servlet.AsyncContext;
 import javax.servlet.DispatcherType;
@@ -74,7 +76,7 @@ import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.util.ParameterMap;
 import org.apache.catalina.util.StringParser;
 import org.apache.coyote.ActionCode;
-import org.apache.coyote.http11.upgrade.UpgradeInbound;
+import org.apache.coyote.http11.upgrade.servlet31.HttpUpgradeHandler;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
@@ -91,8 +93,11 @@ import org.apache.tomcat.util.http.fileupload.FileUploadBase.InvalidContentTypeE
 import org.apache.tomcat.util.http.fileupload.FileUploadException;
 import org.apache.tomcat.util.http.fileupload.disk.DiskFileItemFactory;
 import org.apache.tomcat.util.http.fileupload.servlet.ServletFileUpload;
+import org.apache.tomcat.util.http.fileupload.servlet.ServletRequestContext;
 import org.apache.tomcat.util.http.mapper.MappingData;
 import org.apache.tomcat.util.res.StringManager;
+import org.ietf.jgss.GSSCredential;
+import org.ietf.jgss.GSSException;
 
 
 /**
@@ -100,9 +105,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Id: Request.java 1335733 2012-05-08 20:07:23Z markt $
  */
-
 public class Request
     implements HttpServletRequest {
 
@@ -198,12 +201,6 @@ public class Request
      */
     protected boolean sslAttributesParsed = false;
 
-    /**
-     * List of read only attributes for this Request.
-     */
-    private final HashMap<String,Object> readOnlyAttributes =
-        new HashMap<String,Object>();
-
 
     /**
      * The preferred Locales associated with this Request.
@@ -470,7 +467,16 @@ public class Request
         subject = null;
         sessionParsed = false;
         parametersParsed = false;
-        parts = null;
+        if (parts != null) {
+            for (Part part: parts) {
+                try {
+                    part.delete();
+                } catch (IOException ignored) {
+                    // ApplicationPart.delete() never throws an IOEx
+                }
+            }
+            parts = null;
+        }
         partsParseException = null;
         cookiesParsed = false;
         locales.clear();
@@ -728,7 +734,7 @@ public class Request
     }
 
     /**
-     * URI byte to char converter (not recycled).
+     * URI byte to char converter.
      */
     protected B2CConverter URIConverter = null;
 
@@ -1454,12 +1460,6 @@ public class Request
     @Override
     public void removeAttribute(String name) {
         // Remove the specified attribute
-        // Check for read only attribute
-        // requests are per thread so synchronization unnecessary
-        if (readOnlyAttributes.containsKey(name)) {
-            return;
-        }
-
         // Pass special attributes to the native layer
         if (name.startsWith("org.apache.tomcat.")) {
             coyoteRequest.getAttributes().remove(name);
@@ -1507,12 +1507,6 @@ public class Request
         }
 
         // Add or replace the specified attribute
-        // Check for read only attribute
-        // requests are per thread so synchronization unnecessary
-        if (readOnlyAttributes.containsKey(name)) {
-            return;
-        }
-
         // Do the security check before any updates are made
         if (Globals.IS_SECURITY_ENABLED &&
                 name.equals(Globals.SENDFILE_FILENAME_ATTR)) {
@@ -1930,7 +1924,6 @@ public class Request
      *
      * @return the URL decoded request URI
      */
-    @Deprecated
     public MessageBytes getDecodedRequestURIMB() {
         return coyoteRequest.decodedURI();
     }
@@ -2451,6 +2444,27 @@ public class Request
     @Override
     public Principal getUserPrincipal() {
         if (userPrincipal instanceof GenericPrincipal) {
+            GSSCredential gssCredential =
+                    ((GenericPrincipal) userPrincipal).getGssCredential();
+            if (gssCredential != null) {
+                int left = -1;
+                try {
+                    left = gssCredential.getRemainingLifetime();
+                } catch (GSSException e) {
+                    log.warn(sm.getString("coyoteRequest.gssLifetimeFail",
+                            userPrincipal.getName()), e);
+                }
+                if (left == 0) {
+                    // GSS credential has expired. Need to re-authenticate.
+                    try {
+                        logout();
+                    } catch (ServletException e) {
+                        // Should never happen (no code called by logout()
+                        // throws a ServletException
+                    }
+                    return null;
+                }
+            }
             return ((GenericPrincipal) userPrincipal).getUserPrincipal();
         }
 
@@ -2710,7 +2724,8 @@ public class Request
 
             parts = new ArrayList<Part>();
             try {
-                List<FileItem> items = upload.parseRequest(this);
+                List<FileItem> items =
+                        upload.parseRequest(new ServletRequestContext(this));
                 int maxPostSize = getConnector().getMaxPostSize();
                 int postSize = 0;
                 String enc = getCharacterEncoding();
@@ -2723,15 +2738,19 @@ public class Request
                     }
                 }
                 for (FileItem item : items) {
-                    ApplicationPart part = new ApplicationPart(item, mce);
+                    ApplicationPart part = new ApplicationPart(item, location);
                     parts.add(part);
-                    if (part.getFilename() == null) {
+                    if (part.getSubmittedFileName() == null) {
                         String name = part.getName();
                         String value = null;
                         try {
                             String encoding = parameters.getEncoding();
                             if (encoding == null) {
-                                encoding = Parameters.DEFAULT_ENCODING;
+                                if (enc == null) {
+                                    encoding = Parameters.DEFAULT_ENCODING;
+                                } else {
+                                    encoding = enc;
+                                }
                             }
                             value = part.getString(encoding);
                         } catch (UnsupportedEncodingException uee) {
@@ -2805,12 +2824,16 @@ public class Request
     }
 
 
-    // --------------------------------------------------------- Upgrade Methods
+    // --------------------------------- Tomcat proprietary HTTP upgrade methods
 
-    public void doUpgrade(UpgradeInbound inbound)
+    /**
+     * @deprecated  Will be removed in Tomcat 8.0.x.
+     */
+    @Deprecated
+    public void doUpgrade(org.apache.coyote.http11.upgrade.UpgradeInbound inbound)
             throws IOException {
 
-        coyoteRequest.action(ActionCode.UPGRADE, inbound);
+        coyoteRequest.action(ActionCode.UPGRADE_TOMCAT, inbound);
 
         // Output required by RFC2616. Protocol specific headers should have
         // already been set.
@@ -2819,6 +2842,35 @@ public class Request
     }
 
 
+    // ---------------------------------- Servlet 3.1 based HTTP upgrade methods
+
+    @SuppressWarnings("unchecked")
+    public <T extends HttpUpgradeHandler> T upgrade(
+            Class<T> httpUpgradeHandlerClass) throws ServletException {
+            
+        T handler;
+        try {
+            handler = (T) context.getInstanceManager().newInstance(httpUpgradeHandlerClass);
+        } catch (InstantiationException e) {
+            throw new ServletException(e);
+        } catch (IllegalAccessException e) {
+            throw new ServletException(e);
+        } catch (InvocationTargetException e) {
+            throw new ServletException(e);
+        } catch (NamingException e) {
+            throw new ServletException(e);
+        }
+
+        coyoteRequest.action(ActionCode.UPGRADE, handler);
+
+        // Output required by RFC2616. Protocol specific headers should have
+        // already been set.
+        response.setStatus(HttpServletResponse.SC_SWITCHING_PROTOCOLS);
+
+        return handler;
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
