@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
 import java.io.IOException;
@@ -24,6 +22,7 @@ import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.Servlet;
 import javax.servlet.ServletException;
@@ -36,6 +35,7 @@ import javax.servlet.UnavailableException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import org.apache.catalina.AsyncDispatcher;
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
 import org.apache.catalina.InstanceEvent;
@@ -46,7 +46,8 @@ import org.apache.catalina.connector.RequestFacade;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.connector.ResponseFacade;
 import org.apache.catalina.util.InstanceSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Standard implementation of <code>RequestDispatcher</code> that allows a
@@ -59,52 +60,86 @@ import org.apache.catalina.util.StringManager;
  * <code>javax.servlet.ServletResponseWrapper</code>.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 836371 $ $Date: 2009-11-15 16:17:08 +0100 (So, 15. Nov 2009) $
  */
+final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher {
 
-final class ApplicationDispatcher
-    implements RequestDispatcher {
+    protected static final boolean STRICT_SERVLET_COMPLIANCE;
+
+    protected static final boolean WRAP_SAME_OBJECT;
 
 
-    protected class PrivilegedForward implements PrivilegedExceptionAction {
+    static {
+        STRICT_SERVLET_COMPLIANCE = Globals.STRICT_SERVLET_COMPLIANCE;
+        
+        String wrapSameObject = System.getProperty(
+                "org.apache.catalina.core.ApplicationDispatcher.WRAP_SAME_OBJECT");
+        if (wrapSameObject == null) {
+            WRAP_SAME_OBJECT = STRICT_SERVLET_COMPLIANCE;
+        } else {
+            WRAP_SAME_OBJECT =
+                Boolean.valueOf(wrapSameObject).booleanValue();
+        }
+    }
+
+
+    protected class PrivilegedForward
+            implements PrivilegedExceptionAction<Void> {
         private ServletRequest request;
         private ServletResponse response;
 
-        PrivilegedForward(ServletRequest request, ServletResponse response)
-        {
+        PrivilegedForward(ServletRequest request, ServletResponse response) {
             this.request = request;
             this.response = response;
         }
 
-        public Object run() throws java.lang.Exception {
+        @Override
+        public Void run() throws java.lang.Exception {
             doForward(request,response);
             return null;
         }
     }
 
-    protected class PrivilegedInclude implements PrivilegedExceptionAction {
+    protected class PrivilegedInclude implements
+            PrivilegedExceptionAction<Void> {
         private ServletRequest request;
         private ServletResponse response;
 
-        PrivilegedInclude(ServletRequest request, ServletResponse response)
-        {
+        PrivilegedInclude(ServletRequest request, ServletResponse response) {
             this.request = request;
             this.response = response;
         }
 
-        public Object run() throws ServletException, IOException {
-            doInclude(request,response);
+        @Override
+        public Void run() throws ServletException, IOException {
+            doInclude(request, response);
             return null;
         }
     }
 
-    
+    protected class PrivilegedDispatch implements
+            PrivilegedExceptionAction<Void> {
+        private final ServletRequest request;
+        private final ServletResponse response;
+
+        PrivilegedDispatch(ServletRequest request, ServletResponse response) {
+            this.request = request;
+            this.response = response;
+        }
+
+        @Override
+        public Void run() throws ServletException, IOException {
+            doDispatch(request, response);
+            return null;
+        }
+    }
+
+
     /**
      * Used to pass state when the request dispatcher is used. Using instance
      * variables causes threading issues and state is too complex to pass and
      * return single ServletRequest or ServletResponse objects.
      */
-    private class State {
+    private static class State {
         State(ServletRequest request, ServletResponse response,
                 boolean including) {
             this.outerRequest = request;
@@ -277,7 +312,7 @@ final class ApplicationDispatcher
     /**
      * Forward this request and response to another resource for processing.
      * Any runtime exception, IOException, or ServletException thrown by the
-     * called servlet will be propogated to the caller.
+     * called servlet will be propagated to the caller.
      *
      * @param request The servlet request to be forwarded
      * @param response The servlet response to be forwarded
@@ -285,6 +320,7 @@ final class ApplicationDispatcher
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet exception occurs
      */
+    @Override
     public void forward(ServletRequest request, ServletResponse response)
         throws ServletException, IOException
     {
@@ -321,7 +357,7 @@ final class ApplicationDispatcher
         // Set up to handle the specified request and response
         State state = new State(request, response, false);
 
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
+        if (WRAP_SAME_OBJECT) {
             // Check SRV.8.2 / SRV.14.2.5.1 compliance
             checkSameObjects(request, response);
         }
@@ -349,16 +385,17 @@ final class ApplicationDispatcher
                 (ApplicationHttpRequest) wrapRequest(state);
             String contextPath = context.getPath();
             HttpServletRequest hrequest = state.hrequest;
-            if (hrequest.getAttribute(Globals.FORWARD_REQUEST_URI_ATTR) == null) {
-                wrequest.setAttribute(Globals.FORWARD_REQUEST_URI_ATTR,
+            if (hrequest.getAttribute(
+                    RequestDispatcher.FORWARD_REQUEST_URI) == null) {
+                wrequest.setAttribute(RequestDispatcher.FORWARD_REQUEST_URI,
                                       hrequest.getRequestURI());
-                wrequest.setAttribute(Globals.FORWARD_CONTEXT_PATH_ATTR,
+                wrequest.setAttribute(RequestDispatcher.FORWARD_CONTEXT_PATH,
                                       hrequest.getContextPath());
-                wrequest.setAttribute(Globals.FORWARD_SERVLET_PATH_ATTR,
+                wrequest.setAttribute(RequestDispatcher.FORWARD_SERVLET_PATH,
                                       hrequest.getServletPath());
-                wrequest.setAttribute(Globals.FORWARD_PATH_INFO_ATTR,
+                wrequest.setAttribute(RequestDispatcher.FORWARD_PATH_INFO,
                                       hrequest.getPathInfo());
-                wrequest.setAttribute(Globals.FORWARD_QUERY_STRING_ATTR,
+                wrequest.setAttribute(RequestDispatcher.FORWARD_QUERY_STRING,
                                       hrequest.getQueryString());
             }
  
@@ -374,6 +411,12 @@ final class ApplicationDispatcher
             processRequest(request,response,state);
         }
 
+        if (request.getAsyncContext() != null) {
+            // An async request was started during the forward, don't close the
+            // response as it may be written to during the async handling
+            return;
+        }
+
         // This is not a real close in order to support error processing
         if (wrapper.getLogger().isDebugEnabled() )
             wrapper.getLogger().debug(" Disabling the response for futher output");
@@ -381,7 +424,7 @@ final class ApplicationDispatcher
         if  (response instanceof ResponseFacade) {
             ((ResponseFacade) response).finish();
         } else {
-            // Servlet SRV.6.2.2. The Resquest/Response may have been wrapped
+            // Servlet SRV.6.2.2. The Request/Response may have been wrapped
             // and may no longer be instance of RequestFacade 
             if (wrapper.getLogger().isDebugEnabled()){
                 wrapper.getLogger().debug( " The Response is vehiculed using a wrapper: " 
@@ -397,12 +440,12 @@ final class ApplicationDispatcher
                     ServletOutputStream stream = response.getOutputStream();
                     stream.close();
                 } catch (IllegalStateException f) {
-                    ;
+                    // Ignore
                 } catch (IOException f) {
-                    ;
+                    // Ignore
                 }
             } catch (IOException e) {
-                ;
+                // Ignore
             }
         }
 
@@ -423,22 +466,33 @@ final class ApplicationDispatcher
                                 State state)
         throws IOException, ServletException {
                 
-        Integer disInt = (Integer) request.getAttribute
-            (ApplicationFilterFactory.DISPATCHER_TYPE_ATTR);
+        DispatcherType disInt = (DispatcherType) request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
         if (disInt != null) {
-            if (disInt.intValue() != ApplicationFilterFactory.ERROR) {
-                state.outerRequest.setAttribute
-                    (ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
-                     getCombinedPath());
-                state.outerRequest.setAttribute
-                    (ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
-                     Integer.valueOf(ApplicationFilterFactory.FORWARD));
-                invoke(state.outerRequest, response, state);
-            } else {
-                invoke(state.outerRequest, response, state);
+            boolean doInvoke = true;
+            
+            if (context.getFireRequestListenersOnForwards() &&
+                    !context.fireRequestInitEvent(request)) {
+                doInvoke = false;
+            }
+
+            if (doInvoke) {
+                if (disInt != DispatcherType.ERROR) {
+                    state.outerRequest.setAttribute(
+                            Globals.DISPATCHER_REQUEST_PATH_ATTR,
+                            getCombinedPath());
+                    state.outerRequest.setAttribute(
+                            Globals.DISPATCHER_TYPE_ATTR,
+                            DispatcherType.FORWARD);
+                    invoke(state.outerRequest, response, state);
+                } else {
+                    invoke(state.outerRequest, response, state);
+                }
+                
+                if (context.getFireRequestListenersOnForwards()) {
+                    context.fireRequestDestroyEvent(request);
+                }
             }
         }
-
     }
     
     
@@ -462,7 +516,7 @@ final class ApplicationDispatcher
     /**
      * Include the response from another resource in the current response.
      * Any runtime exception, IOException, or ServletException thrown by the
-     * called servlet will be propogated to the caller.
+     * called servlet will be propagated to the caller.
      *
      * @param request The servlet request that is including this one
      * @param response The servlet response to be appended to
@@ -470,6 +524,7 @@ final class ApplicationDispatcher
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet exception occurs
      */
+    @Override
     public void include(ServletRequest request, ServletResponse response)
         throws ServletException, IOException
     {
@@ -485,17 +540,17 @@ final class ApplicationDispatcher
                 throw (IOException) e;
             }
         } else {
-            doInclude(request,response);
+            doInclude(request, response);
         }
     }
 
     private void doInclude(ServletRequest request, ServletResponse response)
-        throws ServletException, IOException
-    {
+            throws ServletException, IOException {
+
         // Set up to handle the specified request and response
         State state = new State(request, response, true);
 
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
+        if (WRAP_SAME_OBJECT) {
             // Check SRV.8.2 / SRV.14.2.5.1 compliance
             checkSameObjects(request, response);
         }
@@ -511,10 +566,9 @@ final class ApplicationDispatcher
             wrequest.setAttribute(Globals.NAMED_DISPATCHER_ATTR, name);
             if (servletPath != null)
                 wrequest.setServletPath(servletPath);
-            wrequest.setAttribute(ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
-                    Integer.valueOf(ApplicationFilterFactory.INCLUDE));
-            wrequest.setAttribute(
-                    ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
+            wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
+                    DispatcherType.INCLUDE);
+            wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
                     getCombinedPath());
             invoke(state.outerRequest, state.outerResponse, state);
         }
@@ -526,27 +580,26 @@ final class ApplicationDispatcher
                 (ApplicationHttpRequest) wrapRequest(state);
             String contextPath = context.getPath();
             if (requestURI != null)
-                wrequest.setAttribute(Globals.INCLUDE_REQUEST_URI_ATTR,
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_REQUEST_URI,
                                       requestURI);
             if (contextPath != null)
-                wrequest.setAttribute(Globals.INCLUDE_CONTEXT_PATH_ATTR,
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_CONTEXT_PATH,
                                       contextPath);
             if (servletPath != null)
-                wrequest.setAttribute(Globals.INCLUDE_SERVLET_PATH_ATTR,
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH,
                                       servletPath);
             if (pathInfo != null)
-                wrequest.setAttribute(Globals.INCLUDE_PATH_INFO_ATTR,
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_PATH_INFO,
                                       pathInfo);
             if (queryString != null) {
-                wrequest.setAttribute(Globals.INCLUDE_QUERY_STRING_ATTR,
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_QUERY_STRING,
                                       queryString);
                 wrequest.setQueryParams(queryString);
             }
             
-            wrequest.setAttribute(ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
-                    Integer.valueOf(ApplicationFilterFactory.INCLUDE));
-            wrequest.setAttribute(
-                    ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
+            wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
+                    DispatcherType.INCLUDE);
+            wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
                     getCombinedPath());
             invoke(state.outerRequest, state.outerResponse, state);
         }
@@ -554,6 +607,59 @@ final class ApplicationDispatcher
     }
 
 
+    @Override
+    public void dispatch(ServletRequest request, ServletResponse response)
+            throws ServletException, IOException {
+        if (Globals.IS_SECURITY_ENABLED) {
+            try {
+                PrivilegedDispatch dp = new PrivilegedDispatch(request,response);
+                AccessController.doPrivileged(dp);
+            } catch (PrivilegedActionException pe) {
+                Exception e = pe.getException();
+
+                if (e instanceof ServletException)
+                    throw (ServletException) e;
+                throw (IOException) e;
+            }
+        } else {
+            doDispatch(request, response);
+        }
+    }
+
+    private void doDispatch(ServletRequest request, ServletResponse response)
+            throws ServletException, IOException {
+
+        // Set up to handle the specified request and response
+        State state = new State(request, response, false);
+
+        // Create a wrapped response to use for this request
+        wrapResponse(state);
+
+        ApplicationHttpRequest wrequest =
+            (ApplicationHttpRequest) wrapRequest(state);
+
+        if (queryString != null) {
+            wrequest.setQueryParams(queryString);
+        }
+
+        wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
+                DispatcherType.ASYNC);
+        wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
+                getCombinedPath());
+
+        wrequest.setContextPath(context.getPath());
+        wrequest.setRequestURI(requestURI);
+        wrequest.setServletPath(servletPath);
+        wrequest.setPathInfo(pathInfo);
+        if (queryString != null) {
+            wrequest.setQueryString(queryString);
+            wrequest.setQueryParams(queryString);
+        }
+
+        invoke(state.outerRequest, state.outerResponse, state);
+    }
+
+
     // -------------------------------------------------------- Private Methods
 
 
@@ -618,8 +724,8 @@ final class ApplicationDispatcher
             wrapper.getLogger().error(sm.getString("applicationDispatcher.allocateException",
                              wrapper.getName()), StandardWrapper.getRootCause(e));
             servletException = e;
-            servlet = null;
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.allocateException",
                              wrapper.getName()), e);
             servletException = new ServletException
@@ -632,13 +738,9 @@ final class ApplicationDispatcher
         ApplicationFilterFactory factory = ApplicationFilterFactory.getInstance();
         ApplicationFilterChain filterChain = factory.createFilterChain(request,
                                                                 wrapper,servlet);
+        
         // Call the service() method for the allocated servlet instance
         try {
-            String jspFile = wrapper.getJspFile();
-            if (jspFile != null)
-                request.setAttribute(Globals.JSP_FILE_ATTR, jspFile);
-            else
-                request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.BEFORE_DISPATCH_EVENT,
                                       servlet, request, response);
             // for includes/forwards
@@ -646,23 +748,19 @@ final class ApplicationDispatcher
                filterChain.doFilter(request, response);
              }
             // Servlet Service Method is called by the FilterChain
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
         } catch (ClientAbortException e) {
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
             ioException = e;
         } catch (IOException e) {
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
                              wrapper.getName()), e);
             ioException = e;
         } catch (UnavailableException e) {
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
@@ -670,7 +768,6 @@ final class ApplicationDispatcher
             servletException = e;
             wrapper.unavailable(e);
         } catch (ServletException e) {
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
             Throwable rootCause = StandardWrapper.getRootCause(e);
@@ -680,7 +777,6 @@ final class ApplicationDispatcher
             }
             servletException = e;
         } catch (RuntimeException e) {
-            request.removeAttribute(Globals.JSP_FILE_ATTR);
             support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
                                       servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
@@ -693,9 +789,10 @@ final class ApplicationDispatcher
             if (filterChain != null)
                 filterChain.release();
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             wrapper.getLogger().error(sm.getString("standardWrapper.releaseFilters",
                              wrapper.getName()), e);
-            // FIXME: Exception handling needs to be simpiler to what is in the StandardWrapperValue
+            // FIXME: Exception handling needs to be similar to what is in the StandardWrapperValue
         }
 
         // Deallocate the allocated servlet instance
@@ -708,6 +805,7 @@ final class ApplicationDispatcher
                              wrapper.getName()), e);
             servletException = e;
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.deallocateException",
                              wrapper.getName()), e);
             servletException = new ServletException
@@ -745,6 +843,12 @@ final class ApplicationDispatcher
         if (state.wrapRequest == null)
             return;
 
+        if (state.outerRequest.isAsyncStarted()) {
+            if (!state.outerRequest.getAsyncContext().hasOriginalRequestAndResponse()) {
+                return;
+            }
+        }
+
         ServletRequest previous = null;
         ServletRequest current = state.outerRequest;
         while (current != null) {
@@ -772,8 +876,7 @@ final class ApplicationDispatcher
         }
 
     }
-
-
+    
     /**
      * Unwrap the response if we have wrapped it.
      */
@@ -782,6 +885,12 @@ final class ApplicationDispatcher
         if (state.wrapResponse == null)
             return;
 
+        if (state.outerRequest.isAsyncStarted()) {
+            if (!state.outerRequest.getAsyncContext().hasOriginalRequestAndResponse()) {
+                return;
+            }
+        }
+
         ServletResponse previous = null;
         ServletResponse current = state.outerResponse;
         while (current != null) {
@@ -823,17 +932,12 @@ final class ApplicationDispatcher
         while (current != null) {
             if(state.hrequest == null && (current instanceof HttpServletRequest))
                 state.hrequest = (HttpServletRequest)current;
-            if ("org.apache.catalina.servlets.InvokerHttpRequest".
-                equals(current.getClass().getName()))
-                break; // KLUDGE - Make nested RD.forward() using invoker work
             if (!(current instanceof ServletRequestWrapper))
                 break;
             if (current instanceof ApplicationHttpRequest)
                 break;
             if (current instanceof ApplicationRequest)
                 break;
-            if (current instanceof Request)
-                break;
             previous = current;
             current = ((ServletRequestWrapper) current).getRequest();
         }
@@ -851,8 +955,8 @@ final class ApplicationDispatcher
                 (state.outerRequest instanceof HttpServletRequest)) {
                 HttpServletRequest houterRequest = 
                     (HttpServletRequest) state.outerRequest;
-                Object contextPath = houterRequest.getAttribute
-                    (Globals.INCLUDE_CONTEXT_PATH_ATTR);
+                Object contextPath = houterRequest.getAttribute(
+                        RequestDispatcher.INCLUDE_CONTEXT_PATH);
                 if (contextPath == null) {
                     // Forward
                     contextPath = houterRequest.getContextPath();
@@ -895,8 +999,6 @@ final class ApplicationDispatcher
                 break;
             if (current instanceof ApplicationResponse)
                 break;
-            if (current instanceof Response)
-                break;
             previous = current;
             current = ((ServletResponseWrapper) current).getResponse();
         }
