@@ -21,12 +21,14 @@ package org.apache.catalina.core;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.NoSuchElementException;
 
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletRequestWrapper;
@@ -34,11 +36,9 @@ import javax.servlet.http.HttpSession;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
-import org.apache.catalina.Session;
 import org.apache.catalina.Manager;
-import org.apache.catalina.util.Enumerator;
+import org.apache.catalina.Session;
 import org.apache.catalina.util.RequestUtil;
-import org.apache.catalina.util.StringManager;
 
 
 /**
@@ -55,9 +55,7 @@ import org.apache.catalina.util.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 739532 $ $Date: 2009-01-31 10:52:13 +0100 (Sa, 31. Jan 2009) $
  */
-
 class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
@@ -68,18 +66,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * The set of attribute names that are special for request dispatchers.
      */
     protected static final String specials[] =
-    { Globals.INCLUDE_REQUEST_URI_ATTR, Globals.INCLUDE_CONTEXT_PATH_ATTR,
-      Globals.INCLUDE_SERVLET_PATH_ATTR, Globals.INCLUDE_PATH_INFO_ATTR,
-      Globals.INCLUDE_QUERY_STRING_ATTR, Globals.FORWARD_REQUEST_URI_ATTR, 
-      Globals.FORWARD_CONTEXT_PATH_ATTR, Globals.FORWARD_SERVLET_PATH_ATTR, 
-      Globals.FORWARD_PATH_INFO_ATTR, Globals.FORWARD_QUERY_STRING_ATTR };
-
-
-    /**
-     * The string manager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
+    { RequestDispatcher.INCLUDE_REQUEST_URI,
+      RequestDispatcher.INCLUDE_CONTEXT_PATH,
+      RequestDispatcher.INCLUDE_SERVLET_PATH,
+      RequestDispatcher.INCLUDE_PATH_INFO,
+      RequestDispatcher.INCLUDE_QUERY_STRING,
+      RequestDispatcher.FORWARD_REQUEST_URI, 
+      RequestDispatcher.FORWARD_CONTEXT_PATH,
+      RequestDispatcher.FORWARD_SERVLET_PATH, 
+      RequestDispatcher.FORWARD_PATH_INFO,
+      RequestDispatcher.FORWARD_QUERY_STRING };
 
 
     // ----------------------------------------------------------- Constructors
@@ -117,7 +113,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
     /**
-     * If this request is cross context, since this changes session accesss
+     * If this request is cross context, since this changes session access
      * behavior.
      */
     protected boolean crossContext = false;
@@ -126,7 +122,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * The current dispatcher type.
      */
-    protected Object dispatcherType = null;
+    protected DispatcherType dispatcherType = null;
 
 
     /**
@@ -140,7 +136,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * The request parameters for this request.  This is initialized from the
      * wrapped request, but updates are allowed.
      */
-    protected Map parameters = null;
+    protected Map<String, String[]> parameters = null;
 
 
     /**
@@ -195,7 +191,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Special attributes.
      */
     protected Object[] specialAttributes = new Object[specials.length];
-
+    
 
     // ------------------------------------------------- ServletRequest Methods
 
@@ -205,6 +201,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the attribute to retrieve
      */
+    @Override
     public Object getAttribute(String name) {
 
         if (name.equals(Globals.DISPATCHER_TYPE_ATTR)) {
@@ -239,7 +236,8 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getAttributeNames()</code> method of the wrapped
      * request.
      */
-    public Enumeration getAttributeNames() {
+    @Override
+    public Enumeration<String> getAttributeNames() {
         return (new AttributeNamesEnumerator());
     }
 
@@ -250,6 +248,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the attribute to remove
      */
+    @Override
     public void removeAttribute(String name) {
 
         if (!removeSpecial(name))
@@ -265,10 +264,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * @param name Name of the attribute to set
      * @param value Value of the attribute to set
      */
+    @Override
     public void setAttribute(String name, Object value) {
 
         if (name.equals(Globals.DISPATCHER_TYPE_ATTR)) {
-            dispatcherType = value;
+            dispatcherType = (DispatcherType)value;
             return;
         } else if (name.equals(Globals.DISPATCHER_REQUEST_PATH_ATTR)) {
             requestDispatcherPath = value;
@@ -288,6 +288,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param path Path of the resource to be wrapped
      */
+    @Override
     public RequestDispatcher getRequestDispatcher(String path) {
 
         if (context == null)
@@ -301,7 +302,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
         // Convert a request-relative path to a context-relative one
         String servletPath = 
-            (String) getAttribute(Globals.INCLUDE_SERVLET_PATH_ATTR);
+            (String) getAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH);
         if (servletPath == null)
             servletPath = getServletPath();
 
@@ -327,6 +328,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
     }
 
+    
+    /**
+     * Override the getDispatcherType() method of the wrapped request.
+     * 
+     */
+    @Override
+    public DispatcherType getDispatcherType() {
+        return dispatcherType;
+    }
+
 
     // --------------------------------------------- HttpServletRequest Methods
 
@@ -335,6 +346,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getContextPath()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getContextPath() {
 
         return (this.contextPath);
@@ -347,9 +359,10 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the requested parameter
      */
+    @Override
     public String getParameter(String name) {
 
-	parseParameters();
+        parseParameters();
 
         Object value = parameters.get(name);
         if (value == null)
@@ -368,9 +381,10 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getParameterMap()</code> method of the
      * wrapped request.
      */
-    public Map getParameterMap() {
+    @Override
+    public Map<String, String[]> getParameterMap() {
 
-	parseParameters();
+        parseParameters();
         return (parameters);
 
     }
@@ -380,11 +394,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getParameterNames()</code> method of the
      * wrapped request.
      */
-    public Enumeration getParameterNames() {
-
-	parseParameters();
-        return (new Enumerator(parameters.keySet()));
+    @Override
+    public Enumeration<String> getParameterNames() {
 
+        parseParameters();
+        return Collections.enumeration(parameters.keySet());
     }
 
 
@@ -394,12 +408,13 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param name Name of the requested parameter
      */
+    @Override
     public String[] getParameterValues(String name) {
 
-	parseParameters();
+        parseParameters();
         Object value = parameters.get(name);
         if (value == null)
-            return ((String[]) null);
+            return null;
         else if (value instanceof String[])
             return ((String[]) value);
         else if (value instanceof String) {
@@ -418,6 +433,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * Override the <code>getPathInfo()</code> method of the wrapped request.
      */
+    @Override
     public String getPathInfo() {
 
         return (this.pathInfo);
@@ -429,6 +445,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getQueryString()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getQueryString() {
 
         return (this.queryString);
@@ -440,6 +457,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getRequestURI()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getRequestURI() {
 
         return (this.requestURI);
@@ -451,6 +469,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getRequestURL()</code> method of the wrapped
      * request.
      */
+    @Override
     public StringBuffer getRequestURL() {
 
         StringBuffer url = new StringBuffer();
@@ -478,6 +497,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Override the <code>getServletPath()</code> method of the wrapped
      * request.
      */
+    @Override
     public String getServletPath() {
 
         return (this.servletPath);
@@ -489,6 +509,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Return the session associated with this Request, creating one
      * if necessary.
      */
+    @Override
     public HttpSession getSession() {
         return (getSession(true));
     }
@@ -500,6 +521,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param create Create a new session if one does not exist
      */
+    @Override
     public HttpSession getSession(boolean create) {
 
         if (crossContext) {
@@ -525,6 +547,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
                 try {
                     localSession =
                         context.getManager().findSession(other.getId());
+                    if (localSession != null && !localSession.isValid()) {
+                        localSession = null;
+                    }
                 } catch (IOException e) {
                     // Ignore
                 }
@@ -554,6 +579,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * @return true if the request specifies a JSESSIONID that is valid within
      * the context of this ApplicationHttpRequest, false otherwise.
      */
+    @Override
     public boolean isRequestedSessionIdValid() {
 
         if (crossContext) {
@@ -570,7 +596,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
             try {
                 session = manager.findSession(requestedSessionId);
             } catch (IOException e) {
-                session = null;
+                // Ignore
             }
             if ((session != null) && session.isValid()) {
                 return (true);
@@ -612,16 +638,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      *
      * @param orig Origin Map to be copied
      */
-    Map copyMap(Map orig) {
+    Map<String, String[]> copyMap(Map<String, String[]> orig) {
 
         if (orig == null)
-            return (new HashMap());
-        HashMap dest = new HashMap();
-        Iterator keys = orig.keySet().iterator();
-        while (keys.hasNext()) {
-            String key = (String) keys.next();
-            dest.put(key, orig.get(key));
+            return (new HashMap<String, String[]>());
+        HashMap<String, String[]> dest = new HashMap<String, String[]>();
+        
+        for (Map.Entry<String, String[]> entry : orig.entrySet()) {
+            dest.put(entry.getKey(), entry.getValue());
         }
+
         return (dest);
 
     }
@@ -673,7 +699,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         super.setRequest(request);
 
         // Initialize the attributes for this request
-        dispatcherType = request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
+        dispatcherType = (DispatcherType)request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
         requestDispatcherPath = 
             request.getAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR);
 
@@ -719,11 +745,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     void parseParameters() {
 
-	if (parsedParams) {
-	    return;
-	}
+        if (parsedParams) {
+            return;
+        }
 
-        parameters = new HashMap();
+        parameters = new HashMap<String, String[]>();
         parameters = copyMap(getRequest().getParameterMap());
         mergeParameters();
         parsedParams = true;
@@ -740,10 +766,8 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         this.queryParamString = queryString;
     }
 
-
     // ------------------------------------------------------ Protected Methods
 
-
     /**
      * Is this attribute name one of the special ones that is added only for
      * included servlets?
@@ -817,11 +841,11 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     protected String[] mergeValues(Object values1, Object values2) {
 
-        ArrayList results = new ArrayList();
+        ArrayList<Object> results = new ArrayList<Object>();
 
-        if (values1 == null)
-            ;
-        else if (values1 instanceof String)
+        if (values1 == null) {
+            // Skip - nothing to merge
+        } else if (values1 instanceof String)
             results.add(values1);
         else if (values1 instanceof String[]) {
             String values[] = (String[]) values1;
@@ -830,9 +854,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         } else
             results.add(values1.toString());
 
-        if (values2 == null)
-            ;
-        else if (values2 instanceof String)
+        if (values2 == null) {
+            // Skip - nothing to merge
+        } else if (values2 instanceof String)
             results.add(values2);
         else if (values2 instanceof String[]) {
             String values[] = (String[]) values2;
@@ -842,7 +866,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
             results.add(values2.toString());
 
         String values[] = new String[results.size()];
-        return ((String[]) results.toArray(values));
+        return results.toArray(values);
 
     }
 
@@ -861,19 +885,15 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         if ((queryParamString == null) || (queryParamString.length() < 1))
             return;
 
-        HashMap queryParameters = new HashMap();
+        HashMap<String, String[]> queryParameters = new HashMap<String, String[]>();
         String encoding = getCharacterEncoding();
         if (encoding == null)
             encoding = "ISO-8859-1";
-        try {
-            RequestUtil.parseParameters
-                (queryParameters, queryParamString, encoding);
-        } catch (Exception e) {
-            ;
-        }
-        Iterator keys = parameters.keySet().iterator();
+        RequestUtil.parseParameters(queryParameters, queryParamString,
+                encoding);
+        Iterator<String> keys = parameters.keySet().iterator();
         while (keys.hasNext()) {
-            String key = (String) keys.next();
+            String key = keys.next();
             Object value = queryParameters.get(key);
             if (value == null) {
                 queryParameters.put(key, parameters.get(key));
@@ -894,28 +914,31 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      * Utility class used to expose the special attributes as being available
      * as request attributes.
      */
-    protected class AttributeNamesEnumerator implements Enumeration {
+    protected class AttributeNamesEnumerator implements Enumeration<String> {
 
         protected int pos = -1;
         protected int last = -1;
-        protected Enumeration parentEnumeration = null;
+        protected Enumeration<String> parentEnumeration = null;
         protected String next = null;
 
         public AttributeNamesEnumerator() {
             parentEnumeration = getRequest().getAttributeNames();
-            for (int i = 0; i < specialAttributes.length; i++) {
+            for (int i = specialAttributes.length - 1; i >= 0; i--) {
                 if (getAttribute(specials[i]) != null) {
                     last = i;
+                    break;
                 }
             }
         }
 
+        @Override
         public boolean hasMoreElements() {
             return ((pos != last) || (next != null) 
                     || ((next = findNext()) != null));
         }
 
-        public Object nextElement() {
+        @Override
+        public String nextElement() {
             if (pos != last) {
                 for (int i = pos + 1; i <= last; i++) {
                     if (getAttribute(specials[i]) != null) {
@@ -936,7 +959,7 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         protected String findNext() {
             String result = null;
             while ((result == null) && (parentEnumeration.hasMoreElements())) {
-                String current = (String) parentEnumeration.nextElement();
+                String current = parentEnumeration.nextElement();
                 if (!isSpecial(current)) {
                     result = current;
                 }
