@@ -30,6 +30,7 @@ import javax.imageio.ImageIO;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
 
+import org.apache.catalina.Globals;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleListener;
@@ -59,12 +60,30 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     private static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
+    private static final boolean IS_JAVA_7_OR_LATER;
+
+    static {
+        boolean isJava7OrLater;
+        try {
+            Class.forName("java.util.Objects");
+            isJava7OrLater = true;
+        } catch (ClassNotFoundException e) {
+            isJava7OrLater = false;
+        }
+        IS_JAVA_7_OR_LATER = isJava7OrLater;
+    }
+
     /**
      * Protect against the memory leak caused when the first call to
      * <code>sun.awt.AppContext.getAppContext()</code> is triggered by a web
-     * application. Defaults to <code>true</code>.
+     * application. Defaults to <code>true</code> for Java 6 and earlier (since
+     * it is used by {@link java.beans.Introspector#flushCaches()}) but defaults
+     * to <code>false</code> for Java 7 and later since
+     * {@link java.beans.Introspector#flushCaches()} no longer uses AppContext
+     * from 1.7.0_02 onwards. Also, from 1.7.0_25 onwards, calling this method
+     * requires a graphical environment and starts an AWT thread.
      */
-    private boolean appContextProtection = true;
+    private boolean appContextProtection = !IS_JAVA_7_OR_LATER;
     public boolean isAppContextProtection() { return appContextProtection; }
     public void setAppContextProtection(boolean appContextProtection) {
         this.appContextProtection = appContextProtection;
@@ -218,7 +237,6 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
         this.classesToInitialize = classesToInitialize;
     }
     
-    
     @Override
     public void lifecycleEvent(LifecycleEvent event) {
         // Initialise these classes when Tomcat starts
@@ -249,8 +267,15 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                  * due to eventual calls to getAppContext() are:
                  * - Google Web Toolkit via its use of javax.imageio
                  * - Tomcat via its use of java.beans.Introspector.flushCaches()
-                 *   in 1.6.0_15 onwards
+                 *   in 1.6.0_15 to 1.7.0_01. From 1.7.0_02 onwards use of
+                 *   AppContext by Introspector.flushCaches() was replaced with
+                 *   ThreadGroupContext
                  * - others TBD
+                 *
+                 * From 1.7.0_25 onwards, a call to
+                 * sun.awt.AppContext.getAppContext() results in a thread being
+                 * started named AWT-AppKit that requires a graphic environment
+                 * to be available.
                  */
                 
                 // Trigger a call to sun.awt.AppContext.getAppContext(). This
@@ -279,12 +304,17 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 }
 
                 /*
-                 * Several components end up calling:
-                 * sun.misc.GC.requestLatency(long)
+                 * Several components end up calling
+                 * sun.misc.GC.requestLatency(long) which creates a daemon
+                 * thread without setting the TCCL.
                  * 
                  * Those libraries / components known to trigger memory leaks
                  * due to eventual calls to requestLatency(long) are:
                  * - javax.management.remote.rmi.RMIConnectorServer.start()
+                 *
+                 * Note: Long.MAX_VALUE is a special case that causes the thread
+                 *       to terminate
+                 *
                  */
                 if (gcDaemonProtection) {
                     try {
@@ -292,10 +322,9 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                         Method method = clazz.getDeclaredMethod(
                                 "requestLatency",
                                 new Class[] {long.class});
-                        method.invoke(null, Long.valueOf(3600000));
+                        method.invoke(null, Long.valueOf(Long.MAX_VALUE - 1));
                     } catch (ClassNotFoundException e) {
-                        if (System.getProperty("java.vendor").startsWith(
-                                "Sun")) {
+                        if (Globals.IS_ORACLE_JVM) {
                             log.error(sm.getString(
                                     "jreLeakListener.gcDaemonFail"), e);
                         } else {
@@ -427,8 +456,7 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                     try {
                         Class.forName("com.sun.jndi.ldap.LdapPoolManager");
                     } catch (ClassNotFoundException e) {
-                        if (System.getProperty("java.vendor").startsWith(
-                                "Sun")) {
+                        if (Globals.IS_ORACLE_JVM) {
                             log.error(sm.getString(
                                     "jreLeakListener.ldapPoolManagerFail"), e);
                         } else {
