@@ -21,8 +21,10 @@ package org.apache.catalina.core;
 
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
+import java.lang.reflect.Constructor;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.Iterator;
@@ -39,13 +41,11 @@ import org.apache.catalina.Container;
 import org.apache.catalina.ContainerEvent;
 import org.apache.catalina.ContainerListener;
 import org.apache.catalina.Context;
-import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Server;
-import org.apache.catalina.Service;
 import org.apache.catalina.deploy.ContextEjb;
 import org.apache.catalina.deploy.ContextEnvironment;
 import org.apache.catalina.deploy.ContextHandler;
@@ -56,7 +56,6 @@ import org.apache.catalina.deploy.ContextResourceLink;
 import org.apache.catalina.deploy.ContextService;
 import org.apache.catalina.deploy.ContextTransaction;
 import org.apache.catalina.deploy.NamingResources;
-import org.apache.catalina.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.naming.ContextAccessController;
@@ -70,6 +69,7 @@ import org.apache.naming.ResourceRef;
 import org.apache.naming.ServiceRef;
 import org.apache.naming.TransactionRef;
 import org.apache.tomcat.util.modeler.Registry;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -77,13 +77,11 @@ import org.apache.tomcat.util.modeler.Registry;
  * with each context and server.
  *
  * @author Remy Maucherat
- * @version $Id: NamingContextListener.java 939336 2010-04-29 15:00:41Z kkolinko $
  */
-
 public class NamingContextListener
     implements LifecycleListener, ContainerListener, PropertyChangeListener {
 
-    private static Log log = LogFactory.getLog(NamingContextListener.class);
+    private static final Log log = LogFactory.getLog(NamingContextListener.class);
 
 
     // ----------------------------------------------------- Instance Variables
@@ -137,18 +135,45 @@ public class NamingContextListener
     /**
      * Objectnames hashtable.
      */
-    protected HashMap objectNames = new HashMap();
+    protected HashMap<String, ObjectName> objectNames =
+        new HashMap<String, ObjectName>();
     
 
+    /**
+     * Determines if an attempt to write to a read-only context results in an
+     * exception or if the request is ignored.
+     */
+    private boolean exceptionOnFailedWrite = true;
+
+
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
     // ------------------------------------------------------------- Properties
 
+    /**
+     * Returns whether or not an attempt to modify the JNDI context will trigger
+     * an exception or if the request will be ignored.
+     */
+    public boolean getExceptionOnFailedWrite() {
+        return exceptionOnFailedWrite;
+    }
+
+
+    /**
+     * Controls whether or not an attempt to modify the JNDI context will
+     * trigger an exception or if the request will be ignored.
+     *
+     * @param exceptionOnFailedWrite    The new value
+     */
+    public void setExceptionOnFailedWrite(boolean exceptionOnFailedWrite) {
+        this.exceptionOnFailedWrite = exceptionOnFailedWrite;
+    }
+
 
     /**
      * Return the "name" property.
@@ -171,6 +196,7 @@ public class NamingContextListener
     /**
      * Return the comp context.
      */
+    @Deprecated
     public javax.naming.Context getCompContext() {
         return this.compCtx;
     }
@@ -187,6 +213,7 @@ public class NamingContextListener
     /**
      * Return the associated naming context.
      */
+    @Deprecated
     public NamingContext getNamingContext() {
         return (this.namingContext);
     }
@@ -200,6 +227,7 @@ public class NamingContextListener
      *
      * @param event LifecycleEvent that has occurred
      */
+    @Override
     public void lifecycleEvent(LifecycleEvent event) {
 
         container = event.getLifecycle();
@@ -213,12 +241,12 @@ public class NamingContextListener
             return;
         }
 
-        if (event.getType() == Lifecycle.START_EVENT) {
+        if (Lifecycle.CONFIGURE_START_EVENT.equals(event.getType())) {
 
             if (initialized)
                 return;
 
-            Hashtable contextEnv = new Hashtable();
+            Hashtable<String, Object> contextEnv = new Hashtable<String, Object>();
             try {
                 namingContext = new NamingContext(contextEnv, getName());
             } catch (NamingException e) {
@@ -230,6 +258,10 @@ public class NamingContextListener
                 log.debug("Bound " + container );
             }
 
+            // Configure write when read-only behaviour
+            namingContext.setExceptionOnFailedWrite(
+                    getExceptionOnFailedWrite());
+
             // Setting the context in read/write mode
             ContextAccessController.setWritable(getName(), container);
 
@@ -273,7 +305,7 @@ public class NamingContextListener
 
             initialized = true;
 
-        } else if (event.getType() == Lifecycle.STOP_EVENT) {
+        } else if (Lifecycle.CONFIGURE_STOP_EVENT.equals(event.getType())) {
 
             if (!initialized)
                 return;
@@ -296,6 +328,13 @@ public class NamingContextListener
             }
 
             ContextAccessController.unsetSecurityToken(getName(), container);
+            
+            // unregister mbeans.
+            Collection<ObjectName> names = objectNames.values();
+            for (ObjectName objectName : names) {
+                Registry.getRegistry(null, null).unregisterComponent(objectName);
+            }
+            objectNames.clear();
 
             namingContext = null;
             envCtx = null;
@@ -317,6 +356,7 @@ public class NamingContextListener
      *
      * @param event ContainerEvent that has occurred
      */
+    @Override
     public void containerEvent(ContainerEvent event) {
 
         if (!initialized)
@@ -454,6 +494,7 @@ public class NamingContextListener
      *
      * @param event The property change event that has occurred
      */
+    @Override
     public void propertyChange(PropertyChangeEvent event) {
 
         if (!initialized)
@@ -661,9 +702,9 @@ public class NamingContextListener
                 compCtx.bind("UserTransaction", ref);
                 ContextTransaction transaction = namingResources.getTransaction();
                 if (transaction != null) {
-                    Iterator params = transaction.listProperties();
+                    Iterator<String> params = transaction.listProperties();
                     while (params.hasNext()) {
-                        String paramName = (String) params.next();
+                        String paramName = params.next();
                         String paramValue = (String) transaction.getProperty(paramName);
                         StringRefAddr refAddr = new StringRefAddr(paramName, paramValue);
                         ref.add(refAddr);
@@ -718,14 +759,12 @@ public class NamingContextListener
                         ",class=" + resource.getType() + 
                         ",name=" + quotedResourceName);
         } else if (container instanceof Context) {                    
-            String path = ((Context)container).getPath();
-            if (path.length() < 1)
-                path = "/";
+            String contextName = ((Context)container).getName();
+            if (!contextName.startsWith("/"))
+                contextName = "/" + contextName;
             Host host = (Host) ((Context)container).getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
             name = new ObjectName(domain + ":type=DataSource" +
-                        ",path=" + path + 
+                        ",context=" + contextName + 
                         ",host=" + host.getName() +
                         ",class=" + resource.getType() +
                         ",name=" + quotedResourceName);
@@ -745,9 +784,9 @@ public class NamingContextListener
         Reference ref = new EjbRef
             (ejb.getType(), ejb.getHome(), ejb.getRemote(), ejb.getLink());
         // Adding the additional parameters, if any
-        Iterator params = ejb.listProperties();
+        Iterator<String> params = ejb.listProperties();
         while (params.hasNext()) {
-            String paramName = (String) params.next();
+            String paramName = params.next();
             String paramValue = (String) ejb.getProperty(paramName);
             StringRefAddr refAddr = new StringRefAddr(paramName, paramValue);
             ref.add(refAddr);
@@ -776,25 +815,25 @@ public class NamingContextListener
                 value = env.getValue();
             } else if (type.equals("java.lang.Byte")) {
                 if (env.getValue() == null) {
-                    value = new Byte((byte) 0);
+                    value = Byte.valueOf((byte) 0);
                 } else {
                     value = Byte.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Short")) {
                 if (env.getValue() == null) {
-                    value = new Short((short) 0);
+                    value = Short.valueOf((short) 0);
                 } else {
                     value = Short.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Integer")) {
                 if (env.getValue() == null) {
-                    value = new Integer(0);
+                    value = Integer.valueOf(0);
                 } else {
                     value = Integer.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Long")) {
                 if (env.getValue() == null) {
-                    value = new Long(0);
+                    value = Long.valueOf(0);
                 } else {
                     value = Long.decode(env.getValue());
                 }
@@ -802,28 +841,32 @@ public class NamingContextListener
                 value = Boolean.valueOf(env.getValue());
             } else if (type.equals("java.lang.Double")) {
                 if (env.getValue() == null) {
-                    value = new Double(0);
+                    value = Double.valueOf(0);
                 } else {
                     value = Double.valueOf(env.getValue());
                 }
             } else if (type.equals("java.lang.Float")) {
                 if (env.getValue() == null) {
-                    value = new Float(0);
+                    value = Float.valueOf(0);
                 } else {
                     value = Float.valueOf(env.getValue());
                 }
             } else if (type.equals("java.lang.Character")) {
                 if (env.getValue() == null) {
-                    value = new Character((char) 0);
+                    value = Character.valueOf((char) 0);
                 } else {
                     if (env.getValue().length() == 1) {
-                        value = new Character(env.getValue().charAt(0));
+                        value = Character.valueOf(env.getValue().charAt(0));
                     } else {
                         throw new IllegalArgumentException();
                     }
                 }
             } else {
-                logger.error(sm.getString("naming.invalidEnvEntryType", env.getName()));
+                value = constructEnvEntry(env.getType(), env.getValue());
+                if (value == null) {
+                    logger.error(sm.getString(
+                            "naming.invalidEnvEntryType", env.getName()));
+                }
             }
         } catch (NumberFormatException e) {
             logger.error(sm.getString("naming.invalidEnvEntryValue", env.getName()));
@@ -846,13 +889,39 @@ public class NamingContextListener
     }
 
 
+    private Object constructEnvEntry(String type, String value) {
+        try {
+            Class<?> clazz = Class.forName(type);
+            Constructor<?> c = null;
+            try {
+                 c = clazz.getConstructor(String.class);
+                 return c.newInstance(value);
+            } catch (NoSuchMethodException e) {
+                // Ignore
+            }
+
+            if (value.length() != 1) {
+                return null;
+            }
+
+            try {
+                c = clazz.getConstructor(char.class);
+                return c.newInstance(Character.valueOf(value.charAt(0)));
+            } catch (NoSuchMethodException e) {
+                // Ignore
+            }
+        } catch (Exception e) {
+            // Ignore
+        }
+        return null;
+    }
+
     /**
      * Set the specified local EJBs in the naming context.
      */
-    public void addLocalEjb(ContextLocalEjb localEjb) {
-
-
-
+    public void addLocalEjb(
+            @SuppressWarnings("unused") ContextLocalEjb localEjb) {
+        // NO-OP
     }
 
 
@@ -867,7 +936,7 @@ public class NamingContextListener
             try {
                 wsdlURL = new URL(service.getWsdlfile());
             } catch (MalformedURLException e) {
-                wsdlURL = null;
+                // Ignore and carry on
             }
             if (wsdlURL == null) {
                 try {
@@ -875,7 +944,7 @@ public class NamingContextListener
                                                     getServletContext().
                                                     getResource(service.getWsdlfile());
                 } catch (MalformedURLException e) {
-                    wsdlURL = null;
+                    // Ignore and carry on
                 }
             }
             if (wsdlURL == null) {
@@ -901,7 +970,7 @@ public class NamingContextListener
             try {
                 jaxrpcURL = new URL(service.getJaxrpcmappingfile());
             } catch (MalformedURLException e) {
-                jaxrpcURL = null;
+                // Ignore and carry on
             }
             if (jaxrpcURL == null) {
                 try {
@@ -909,7 +978,7 @@ public class NamingContextListener
                                                     getServletContext().
                                                     getResource(service.getJaxrpcmappingfile());
                 } catch (MalformedURLException e) {
-                    jaxrpcURL = null;
+                    // Ignore and carry on
                 }
             }
             if (jaxrpcURL == null) {
@@ -934,31 +1003,31 @@ public class NamingContextListener
             (service.getName(), service.getType(), service.getServiceqname(),
              service.getWsdlfile(), service.getJaxrpcmappingfile());
         // Adding the additional port-component-ref, if any
-        Iterator portcomponent = service.getServiceendpoints();
+        Iterator<String> portcomponent = service.getServiceendpoints();
         while (portcomponent.hasNext()) {
-            String serviceendpoint = (String) portcomponent.next();
+            String serviceendpoint = portcomponent.next();
             StringRefAddr refAddr = new StringRefAddr(ServiceRef.SERVICEENDPOINTINTERFACE, serviceendpoint);
             ref.add(refAddr);
-            String portlink = (String) service.getPortlink(serviceendpoint);
+            String portlink = service.getPortlink(serviceendpoint);
             refAddr = new StringRefAddr(ServiceRef.PORTCOMPONENTLINK, portlink);
             ref.add(refAddr);
         }
         // Adding the additional parameters, if any
-        Iterator handlers = service.getHandlers();
+        Iterator<String> handlers = service.getHandlers();
         while (handlers.hasNext()) {
-            String handlername = (String) handlers.next();
-            ContextHandler handler = (ContextHandler) service.getHandler(handlername);
+            String handlername = handlers.next();
+            ContextHandler handler = service.getHandler(handlername);
             HandlerRef handlerRef = new HandlerRef(handlername, handler.getHandlerclass());
-            Iterator localParts = handler.getLocalparts();
+            Iterator<String> localParts = handler.getLocalparts();
             while (localParts.hasNext()) {
-                String localPart = (String) localParts.next();
-                String namespaceURI = (String) handler.getNamespaceuri(localPart);
+                String localPart = localParts.next();
+                String namespaceURI = handler.getNamespaceuri(localPart);
                 handlerRef.add(new StringRefAddr(HandlerRef.HANDLER_LOCALPART, localPart));
                 handlerRef.add(new StringRefAddr(HandlerRef.HANDLER_NAMESPACE, namespaceURI));
             }
-            Iterator params = handler.listProperties();
+            Iterator<String> params = handler.listProperties();
             while (params.hasNext()) {
-                String paramName = (String) params.next();
+                String paramName = params.next();
                 String paramValue = (String) handler.getProperty(paramName);
                 handlerRef.add(new StringRefAddr(HandlerRef.HANDLER_PARAMNAME, paramName));
                 handlerRef.add(new StringRefAddr(HandlerRef.HANDLER_PARAMVALUE, paramValue));
@@ -994,11 +1063,12 @@ public class NamingContextListener
         // Create a reference to the resource.
         Reference ref = new ResourceRef
             (resource.getType(), resource.getDescription(),
-             resource.getScope(), resource.getAuth());
+             resource.getScope(), resource.getAuth(),
+             resource.getSingleton());
         // Adding the additional parameters, if any
-        Iterator params = resource.listProperties();
+        Iterator<String> params = resource.listProperties();
         while (params.hasNext()) {
-            String paramName = (String) params.next();
+            String paramName = params.next();
             String paramValue = (String) resource.getProperty(paramName);
             StringRefAddr refAddr = new StringRefAddr(paramName, paramValue);
             ref.add(refAddr);
@@ -1014,7 +1084,8 @@ public class NamingContextListener
             logger.error(sm.getString("naming.bindFailed", e));
         }
 
-        if ("javax.sql.DataSource".equals(ref.getClassName())) {
+        if ("javax.sql.DataSource".equals(ref.getClassName()) &&
+                resource.getSingleton()) {
             try {
                 ObjectName on = createObjectName(resource);
                 Object actualResource = envCtx.lookup(resource.getName());
@@ -1036,9 +1107,9 @@ public class NamingContextListener
         // Create a reference to the resource env.
         Reference ref = new ResourceEnvRef(resourceEnvRef.getType());
         // Adding the additional parameters, if any
-        Iterator params = resourceEnvRef.listProperties();
+        Iterator<String> params = resourceEnvRef.listProperties();
         while (params.hasNext()) {
-            String paramName = (String) params.next();
+            String paramName = params.next();
             String paramValue = (String) resourceEnvRef.getProperty(paramName);
             StringRefAddr refAddr = new StringRefAddr(paramName, paramValue);
             ref.add(refAddr);
@@ -1062,7 +1133,16 @@ public class NamingContextListener
 
         // Create a reference to the resource.
         Reference ref = new ResourceLinkRef
-            (resourceLink.getType(), resourceLink.getGlobal());
+            (resourceLink.getType(), resourceLink.getGlobal(), resourceLink.getFactory(), null);
+        Iterator<String> i = resourceLink.listProperties();
+        while (i.hasNext()) {
+            String key = i.next().toString();
+            Object val = resourceLink.getProperty(key);
+            if (val!=null) {
+                StringRefAddr refAddr = new StringRefAddr(key, val.toString());
+                ref.add(refAddr);
+            }
+        }
         javax.naming.Context ctx = 
             "UserTransaction".equals(resourceLink.getName()) 
             ? compCtx : envCtx;
@@ -1145,7 +1225,7 @@ public class NamingContextListener
             logger.error(sm.getString("naming.unbindFailed", e));
         }
 
-        ObjectName on = (ObjectName) objectNames.get(name);
+        ObjectName on = objectNames.get(name);
         if (on != null) {
             Registry.getRegistry(null, null).unregisterComponent(on);
         }
