@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -27,6 +27,7 @@ import java.util.regex.Pattern;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
@@ -46,14 +47,12 @@ import org.apache.tomcat.util.ExceptionUtils;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: StandardHost.java 1343337 2012-05-28 17:50:14Z kkolinko $
  */
-
 public class StandardHost extends ContainerBase implements Host {
 
     private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( StandardHost.class );
-    
+
     // ----------------------------------------------------------- Constructors
 
 
@@ -75,7 +74,7 @@ public class StandardHost extends ContainerBase implements Host {
      * The set of aliases for this Host.
      */
     private String[] aliases = new String[0];
-    
+
     private final Object aliasesLock = new Object();
 
 
@@ -120,7 +119,7 @@ public class StandardHost extends ContainerBase implements Host {
     /**
      * deploy Context XML config files property.
      */
-    private boolean deployXML = true;
+    private boolean deployXML = !Globals.IS_SECURITY_ENABLED;
 
 
     /**
@@ -132,7 +131,7 @@ public class StandardHost extends ContainerBase implements Host {
 
 
     /**
-     * The Java class name of the default error reporter implementation class 
+     * The Java class name of the default error reporter implementation class
      * for deployed web applications.
      */
     private String errorReportValveClass =
@@ -162,15 +161,15 @@ public class StandardHost extends ContainerBase implements Host {
      */
      private boolean createDirs = true;
 
-     
+
      /**
       * Track the class loaders for the child web applications so memory leaks
       * can be detected.
       */
      private Map<ClassLoader, String> childClassLoaders =
          new WeakHashMap<ClassLoader, String>();
-     
-     
+
+
      /**
       * Any file or directory in {@link #appBase} that this pattern matches will
       * be ignored by the automatic deployment process (both
@@ -179,12 +178,27 @@ public class StandardHost extends ContainerBase implements Host {
      private Pattern deployIgnore = null;
 
 
+    private boolean undeployOldVersions = false;
+
+
     // ------------------------------------------------------------- Properties
 
-     @Override
-     public ExecutorService getStartStopExecutor() {
-         return startStopExecutor;
-     }
+    @Override
+    public boolean getUndeployOldVersions() {
+        return undeployOldVersions;
+    }
+
+
+    @Override
+    public void setUndeployOldVersions(boolean undeployOldVersions) {
+        this.undeployOldVersions = undeployOldVersions;
+    }
+
+
+    @Override
+    public ExecutorService getStartStopExecutor() {
+        return startStopExecutor;
+    }
 
 
     /**
@@ -225,7 +239,7 @@ public class StandardHost extends ContainerBase implements Host {
         return (this.xmlBase);
 
     }
-    
+
 
     /**
      * Set the Xml root for this Host.  This can be an absolute
@@ -264,7 +278,7 @@ public class StandardHost extends ContainerBase implements Host {
     }
 
     /**
-     * Return the value of the auto deploy flag.  If true, it indicates that 
+     * Return the value of the auto deploy flag.  If true, it indicates that
      * this host's child webapps will be dynamically deployed.
      */
     @Override
@@ -277,7 +291,7 @@ public class StandardHost extends ContainerBase implements Host {
 
     /**
      * Set the auto deploy flag value for this host.
-     * 
+     *
      * @param autoDeploy The new auto deploy flag
      */
     @Override
@@ -285,7 +299,7 @@ public class StandardHost extends ContainerBase implements Host {
 
         boolean oldAutoDeploy = this.autoDeploy;
         this.autoDeploy = autoDeploy;
-        support.firePropertyChange("autoDeploy", oldAutoDeploy, 
+        support.firePropertyChange("autoDeploy", oldAutoDeploy,
                                    this.autoDeploy);
 
     }
@@ -348,8 +362,8 @@ public class StandardHost extends ContainerBase implements Host {
 
 
     /**
-     * Return the value of the deploy on startup flag.  If true, it indicates 
-     * that this host's child webapps should be discovered and automatically 
+     * Return the value of the deploy on startup flag.  If true, it indicates
+     * that this host's child webapps should be discovered and automatically
      * deployed at startup time.
      */
     @Override
@@ -362,7 +376,7 @@ public class StandardHost extends ContainerBase implements Host {
 
     /**
      * Set the deploy on startup flag value for this host.
-     * 
+     *
      * @param deployOnStartup The new deploy on startup flag
      */
     @Override
@@ -370,7 +384,7 @@ public class StandardHost extends ContainerBase implements Host {
 
         boolean oldDeployOnStartup = this.deployOnStartup;
         this.deployOnStartup = deployOnStartup;
-        support.firePropertyChange("deployOnStartup", oldDeployOnStartup, 
+        support.firePropertyChange("deployOnStartup", oldDeployOnStartup,
                                    this.deployOnStartup);
 
     }
@@ -416,8 +430,8 @@ public class StandardHost extends ContainerBase implements Host {
         this.copyXML= copyXML;
 
     }
-    
-    
+
+
     /**
      * Return the Java class name of the error report valve class
      * for new web applications.
@@ -440,12 +454,12 @@ public class StandardHost extends ContainerBase implements Host {
         String oldErrorReportValveClassClass = this.errorReportValveClass;
         this.errorReportValveClass = errorReportValveClass;
         support.firePropertyChange("errorReportValveClass",
-                                   oldErrorReportValveClassClass, 
+                                   oldErrorReportValveClassClass,
                                    this.errorReportValveClass);
 
     }
-    
-    
+
+
     /**
      * Return the canonical, fully qualified, name of the virtual host
      * this Container represents.
@@ -529,7 +543,7 @@ public class StandardHost extends ContainerBase implements Host {
     public String getDeployIgnore() {
         if (deployIgnore == null) {
             return null;
-        } 
+        }
         return this.deployIgnore.toString();
     }
 
@@ -564,7 +578,7 @@ public class StandardHost extends ContainerBase implements Host {
             this.deployIgnore = Pattern.compile(deployIgnore);
         }
         support.firePropertyChange("deployIgnore",
-                                   oldDeployIgnore, 
+                                   oldDeployIgnore,
                                    deployIgnore);
     }
 
@@ -636,8 +650,8 @@ public class StandardHost extends ContainerBase implements Host {
             }
         }
     }
-    
-    
+
+
     /**
      * Attempt to identify the contexts that have a class loader memory leak.
      * This is usually triggered on context reload. Note: This method attempts
@@ -645,11 +659,11 @@ public class StandardHost extends ContainerBase implements Host {
      * caution on a production system.
      */
     public String[] findReloadedContextMemoryLeaks() {
-        
+
         System.gc();
-        
+
         List<String> result = new ArrayList<String>();
-        
+
         for (Map.Entry<ClassLoader, String> entry :
                 childClassLoaders.entrySet()) {
             ClassLoader cl = entry.getKey();
@@ -659,7 +673,7 @@ public class StandardHost extends ContainerBase implements Host {
                 }
             }
         }
-        
+
         return result.toArray(new String[result.size()]);
     }
 
@@ -747,7 +761,7 @@ public class StandardHost extends ContainerBase implements Host {
         return (sb.toString());
 
     }
-    
+
     /**
      * Start this component and implement the requirements
      * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
@@ -757,7 +771,7 @@ public class StandardHost extends ContainerBase implements Host {
      */
     @Override
     protected synchronized void startInternal() throws LifecycleException {
-        
+
         // Set error report valve
         String errorValve = getErrorReportValveClass();
         if ((errorValve != null) && (!errorValve.equals(""))) {
@@ -821,5 +835,5 @@ public class StandardHost extends ContainerBase implements Host {
 
         return keyProperties.toString();
     }
-    
+
 }
