@@ -14,18 +14,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.deploy;
 
-
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.Serializable;
+import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.util.HashMap;
-import java.util.Hashtable;
+import java.util.HashSet;
+import java.util.Set;
 
 import javax.naming.NamingException;
 
@@ -36,6 +35,7 @@ import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Server;
 import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.util.Introspection;
 import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
@@ -49,9 +49,7 @@ import org.apache.tomcat.util.res.StringManager;
  * Naming Context and their associated JNDI context.
  *
  * @author Remy Maucherat
- * @version $Id: NamingResources.java 1201569 2011-11-14 01:36:07Z kkolinko $
  */
-
 public class NamingResources extends LifecycleMBeanBase implements Serializable {
 
     private static final long serialVersionUID = 1L;
@@ -84,11 +82,9 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
 
 
     /**
-     * List of naming entries, keyed by name. The value is the entry type, as
-     * declared by the user.
+     * Set of naming entries, keyed by name.
      */
-    private Hashtable<String, String> entries =
-        new Hashtable<String, String>();
+    private Set<String> entries = new HashSet<String>();
 
 
     /**
@@ -204,10 +200,10 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addEjb(ContextEjb ejb) {
 
-        if (entries.containsKey(ejb.getName())) {
+        if (entries.contains(ejb.getName())) {
             return;
         } else {
-            entries.put(ejb.getName(), ejb.getType());
+            entries.add(ejb.getName());
         }
 
         synchronized (ejbs) {
@@ -226,7 +222,7 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addEnvironment(ContextEnvironment environment) {
 
-        if (entries.containsKey(environment.getName())) {
+        if (entries.contains(environment.getName())) {
             ContextEnvironment ce = findEnvironment(environment.getName());
             ContextResourceLink rl = findResourceLink(environment.getName());
             if (ce != null) {
@@ -251,7 +247,13 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
             }
         }
         
-        entries.put(environment.getName(), environment.getType());
+        if (!checkResourceType(environment)) {
+            throw new IllegalArgumentException(sm.getString(
+                    "namingResources.resourceTypeFail", environment.getName(),
+                    environment.getType()));
+        }
+
+        entries.add(environment.getName());
 
         synchronized (envs) {
             environment.setNamingResources(this);
@@ -292,10 +294,10 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addLocalEjb(ContextLocalEjb ejb) {
 
-        if (entries.containsKey(ejb.getName())) {
+        if (entries.contains(ejb.getName())) {
             return;
         } else {
-            entries.put(ejb.getName(), ejb.getType());
+            entries.add(ejb.getName());
         }
 
         synchronized (localEjbs) {
@@ -314,10 +316,15 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addMessageDestinationRef(MessageDestinationRef mdr) {
 
-        if (entries.containsKey(mdr.getName())) {
+        if (entries.contains(mdr.getName())) {
             return;
         } else {
-            entries.put(mdr.getName(), mdr.getType());
+            if (!checkResourceType(mdr)) {
+                throw new IllegalArgumentException(sm.getString(
+                        "namingResources.resourceTypeFail", mdr.getName(),
+                        mdr.getType()));
+            }
+            entries.add(mdr.getName());
         }
 
         synchronized (mdrs) {
@@ -348,10 +355,15 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addResource(ContextResource resource) {
 
-        if (entries.containsKey(resource.getName())) {
+        if (entries.contains(resource.getName())) {
             return;
         } else {
-            entries.put(resource.getName(), resource.getType());
+            if (!checkResourceType(resource)) {
+                throw new IllegalArgumentException(sm.getString(
+                        "namingResources.resourceTypeFail", resource.getName(),
+                        resource.getType()));
+            }
+            entries.add(resource.getName());
         }
 
         synchronized (resources) {
@@ -379,10 +391,15 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addResourceEnvRef(ContextResourceEnvRef resource) {
 
-        if (entries.containsKey(resource.getName())) {
+        if (entries.contains(resource.getName())) {
             return;
         } else {
-            entries.put(resource.getName(), resource.getType());
+            if (!checkResourceType(resource)) {
+                throw new IllegalArgumentException(sm.getString(
+                        "namingResources.resourceTypeFail", resource.getName(),
+                        resource.getType()));
+            }
+            entries.add(resource.getName());
         }
 
         synchronized (resourceEnvRefs) {
@@ -401,14 +418,10 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addResourceLink(ContextResourceLink resourceLink) {
 
-        if (entries.containsKey(resourceLink.getName())) {
+        if (entries.contains(resourceLink.getName())) {
             return;
         } else {
-            String value = resourceLink.getType();
-            if (value == null) {
-                value = "";
-            }
-            entries.put(resourceLink.getName(), value);
+            entries.add(resourceLink.getName());
         }
 
         synchronized (resourceLinks) {
@@ -436,14 +449,10 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
      */
     public void addService(ContextService service) {
 
-        if (entries.containsKey(service.getName())) {
+        if (entries.contains(service.getName())) {
             return;
         } else {
-            String value = service.getType();
-            if (value == null) {
-                value = "";
-            }
-            entries.put(service.getName(), value);
+            entries.add(service.getName());
         }
         
         synchronized (services) {
@@ -697,7 +706,7 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
     @Deprecated
     public boolean exists(String name) {
 
-        return (entries.containsKey(name));
+        return (entries.contains(name));
 
     }
 
@@ -1104,4 +1113,132 @@ public class NamingResources extends LifecycleMBeanBase implements Serializable
         // Server or just unknown
         return "type=NamingResources";
     }
+
+    /**
+     * Checks that the configuration of the type for the specified resource is
+     * consistent with any injection targets and if the type is not specified,
+     * tries to configure the type based on the injection targets
+     *
+     * @param resource  The resource to check
+     *
+     * @return  <code>true</code> if the type for the resource is now valid (if
+     *          previously <code>null</code> this means it is now set) or
+     *          <code>false</code> if the current resource type is inconsistent
+     *          with the injection targets and/or cannot be determined
+     */
+    private boolean checkResourceType(ResourceBase resource) {
+        if (!(container instanceof Context)) {
+            // Only Context's will have injection targets
+            return true;
+        }
+
+        if (resource.getInjectionTargets() == null ||
+                resource.getInjectionTargets().size() == 0) {
+            // No injection targets so use the defined type for the resource
+            return true;
+        }
+
+        Context context = (Context) container;
+
+        String typeName = resource.getType();
+        Class<?> typeClass = null;
+        if (typeName != null) {
+            typeClass = Introspection.loadClass(context, typeName);
+            if (typeClass == null) {
+                // Can't load the type - will trigger a failure later so don't
+                // fail here
+                return true;
+            }
+        }
+
+        Class<?> compatibleClass =
+                getCompatibleType(context, resource, typeClass);
+        if (compatibleClass == null) {
+            // Indicates that a compatible type could not be identified that
+            // worked for all injection targets
+            return false;
+        }
+
+        resource.setType(compatibleClass.getCanonicalName());
+        return true;
+    }
+
+    private Class<?> getCompatibleType(Context context,
+            ResourceBase resource, Class<?> typeClass) {
+
+        Class<?> result = null;
+
+        for (InjectionTarget injectionTarget : resource.getInjectionTargets()) {
+            Class<?> clazz = Introspection.loadClass(
+                    context, injectionTarget.getTargetClass());
+            if (clazz == null) {
+                // Can't load class - therefore ignore this target
+                continue;
+            }
+
+            // Look for a match
+            String targetName = injectionTarget.getTargetName();
+            // Look for a setter match first
+            Class<?> targetType = getSetterType(clazz, targetName);
+            if (targetType == null) {
+                // Try a field match if no setter match
+                targetType = getFieldType(clazz,targetName);
+            }
+            if (targetType == null) {
+                // No match - ignore this injection target
+                continue;
+            }
+            targetType = Introspection.convertPrimitiveType(targetType);
+
+            if (typeClass == null) {
+                // Need to find a common type amongst the injection targets
+                if (result == null) {
+                    result = targetType;
+                } else if (targetType.isAssignableFrom(result)) {
+                    // NO-OP - This will work
+                } else if (result.isAssignableFrom(targetType)) {
+                    // Need to use more specific type
+                    result = targetType;
+                } else {
+                    // Incompatible types
+                    return null;
+                }
+            } else {
+                // Each injection target needs to be consistent with the defined
+                // type
+                if (targetType.isAssignableFrom(typeClass)) {
+                    result = typeClass;
+                } else {
+                    // Incompatible types
+                    return null;
+                }
+            }
+        }
+        return result;
+    }
+
+    private Class<?> getSetterType(Class<?> clazz, String name) {
+        Method[] methods = Introspection.getDeclaredMethods(clazz);
+        if (methods != null && methods.length > 0) {
+            for (Method method : methods) {
+                if (Introspection.isValidSetter(method) &&
+                        Introspection.getPropertyName(method).equals(name)) {
+                    return method.getParameterTypes()[0];
+                }
+            }
+        }
+        return null;
+    }
+
+    private Class<?> getFieldType(Class<?> clazz, String name) {
+        Field[] fields = Introspection.getDeclaredFields(clazz);
+        if (fields != null && fields.length > 0) {
+            for (Field field : fields) {
+                if (field.getName().equals(name)) {
+                    return field.getType();
+                }
+            }
+        }
+        return null;
+    }
 }
