@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.deploy;
 
 import java.io.Serializable;
@@ -29,9 +27,7 @@ import java.util.List;
  * Representation of an Context element
  *
  * @author Peter Rossbach (pero@apache.org)
- * @version $Id: ResourceBase.java 1027762 2010-10-26 22:19:49Z markt $
  */
-
 public class ResourceBase implements Serializable, Injectable {
 
     private static final long serialVersionUID = 1L;
@@ -86,7 +82,8 @@ public class ResourceBase implements Serializable, Injectable {
     /**
      * Holder for our configured properties.
      */
-    private HashMap<String, Object> properties = new HashMap<String, Object>();
+    private final HashMap<String, Object> properties =
+            new HashMap<String, Object>();
 
     /**
      * Return a configured property.
@@ -116,7 +113,7 @@ public class ResourceBase implements Serializable, Injectable {
         return properties.keySet().iterator();
     }
 
-    private List<InjectionTarget> injectionTargets = new ArrayList<InjectionTarget>();
+    private final List<InjectionTarget> injectionTargets = new ArrayList<InjectionTarget>();
 
     @Override
     public void addInjectionTarget(String injectionTargetName, String jndiName) {
@@ -129,8 +126,75 @@ public class ResourceBase implements Serializable, Injectable {
         return injectionTargets;
     }
 
-    // -------------------------------------------------------- Package Methods
 
+    @Override
+    public int hashCode() {
+        final int prime = 31;
+        int result = 1;
+        result = prime * result +
+                ((description == null) ? 0 : description.hashCode());
+        result = prime * result +
+                ((injectionTargets == null) ? 0 : injectionTargets.hashCode());
+        result = prime * result + ((name == null) ? 0 : name.hashCode());
+        result = prime * result +
+                ((properties == null) ? 0 : properties.hashCode());
+        result = prime * result + ((type == null) ? 0 : type.hashCode());
+        return result;
+    }
+
+
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (obj == null) {
+            return false;
+        }
+        if (getClass() != obj.getClass()) {
+            return false;
+        }
+        ResourceBase other = (ResourceBase) obj;
+        if (description == null) {
+            if (other.description != null) {
+                return false;
+            }
+        } else if (!description.equals(other.description)) {
+            return false;
+        }
+        if (injectionTargets == null) {
+            if (other.injectionTargets != null) {
+                return false;
+            }
+        } else if (!injectionTargets.equals(other.injectionTargets)) {
+            return false;
+        }
+        if (name == null) {
+            if (other.name != null) {
+                return false;
+            }
+        } else if (!name.equals(other.name)) {
+            return false;
+        }
+        if (properties == null) {
+            if (other.properties != null) {
+                return false;
+            }
+        } else if (!properties.equals(other.properties)) {
+            return false;
+        }
+        if (type == null) {
+            if (other.type != null) {
+                return false;
+            }
+        } else if (!type.equals(other.type)) {
+            return false;
+        }
+        return true;
+    }
+
+
+    // -------------------------------------------------------- Package Methods
 
     /**
      * The NamingResources with which we are associated (if any).
@@ -144,6 +208,4 @@ public class ResourceBase implements Serializable, Injectable {
     void setNamingResources(NamingResources resources) {
         this.resources = resources;
     }
-
-
 }
