@@ -193,7 +193,7 @@ public class ClusterSingleSignOn
     @Override
     protected void associate(String ssoId, Session session) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
@@ -233,7 +233,7 @@ public class ClusterSingleSignOn
     @Override
     protected void deregister(String ssoId, Session session) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
@@ -271,7 +271,7 @@ public class ClusterSingleSignOn
     @Override
     protected void deregister(String ssoId) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
@@ -310,7 +310,7 @@ public class ClusterSingleSignOn
     protected void register(String ssoId, Principal principal, String authType,
                   String username, String password) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
@@ -374,7 +374,7 @@ public class ClusterSingleSignOn
     protected void update(String ssoId, Principal principal, String authType,
                           String username, String password) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
@@ -418,7 +418,7 @@ public class ClusterSingleSignOn
     @Override
     protected void removeSession(String ssoId, Session session) {
 
-        if (cluster != null) {
+        if (cluster != null && cluster.getMembers().length > 0) {
             messageNumber++;
             SingleSignOnMessage msg =
                 new SingleSignOnMessage(cluster.getLocalMember(),
