@@ -48,8 +48,8 @@ import org.apache.tomcat.util.res.StringManager;
  * </p>
  * Any host can act as the admin, and will have three directories
  * <ul>
- * <li>deployDir - the directory where we watch for changes</li>
- * <li>applicationDir - the directory where we install applications</li>
+ * <li>watchDir - the directory where we watch for changes</li>
+ * <li>deployDir - the directory where we install applications</li>
  * <li>tempDir - a temporaryDirectory to store binary data when downloading a
  * war from the cluster</li>
  * </ul>
@@ -58,7 +58,6 @@ import org.apache.tomcat.util.res.StringManager;
  * 
  * @author Filip Hanik
  * @author Peter Rossbach
- * @version $Revision: 1064881 $
  */
 public class FarmWarDeployer extends ClusterListener
         implements ClusterDeployer, FileChangeListener {
@@ -73,16 +72,28 @@ public class FarmWarDeployer extends ClusterListener
     private static final String info = "FarmWarDeployer/1.2";
 
     /*--Instance Variables--------------------------------------*/
-    protected boolean started = false; //default 5 seconds
+    protected boolean started = false;
 
     protected HashMap<String, FileMessageFactory> fileFactories =
         new HashMap<String, FileMessageFactory>();
 
+    /**
+     * Deployment directory.
+     */
     protected String deployDir;
+    private File deployDirFile = null;
 
+    /**
+     * Temporary directory.
+     */
     protected String tempDir;
+    private File tempDirFile = null;
 
+    /**
+     * Watch directory.
+     */
     protected String watchDir;
+    private File watchDirFile = null;
 
     protected boolean watchEnabled = false;
 
@@ -125,6 +136,11 @@ public class FarmWarDeployer extends ClusterListener
      */
     protected ObjectName oname = null;
 
+    /**
+     * The maximum valid time(in seconds) for FileMessageFactory.
+     */
+    protected int maxValidTime = 5 * 60;
+
     /*--Constructor---------------------------------------------*/
     public FarmWarDeployer() {
     }
@@ -171,17 +187,23 @@ public class FarmWarDeployer extends ClusterListener
             return;
         }
         if (watchEnabled) {
-            watcher = new WarWatcher(this, new File(getWatchDir()));
+            watcher = new WarWatcher(this, getWatchDirFile());
             if (log.isInfoEnabled()) {
                 log.info(sm.getString(
                         "farmWarDeployer.watchDir", getWatchDir()));
             }
         }
-         
-        configBase = new File(
-                System.getProperty(Globals.CATALINA_BASE_PROP), "conf");
-        configBase = new File(configBase, engine.getName());
-        configBase = new File(configBase, hostname);
+
+        if (host.getXmlBase()!=null) {
+            configBase = getAbsolutePath(host.getXmlBase());
+        } else {
+            StringBuilder xmlDir = new StringBuilder("conf");
+            xmlDir.append('/');
+            xmlDir.append(engine.getName());
+            xmlDir.append('/');
+            xmlDir.append(host.getName());
+            configBase = getAbsolutePath(xmlDir.toString());
+        }
 
         // Retrieve the MBean server
         mBeanServer = Registry.getRegistry(null, null).getMBeanServer();
@@ -241,7 +263,7 @@ public class FarmWarDeployer extends ClusterListener
                     String name = factory.getFile().getName();
                     if (!name.endsWith(".war"))
                         name = name + ".war";
-                    File deployable = new File(getDeployDir(), name);
+                    File deployable = new File(getDeployDirFile(), name);
                     try {
                         String contextName = fmsg.getContextName();
                         if (!isServiced(contextName)) {
@@ -312,10 +334,11 @@ public class FarmWarDeployer extends ClusterListener
      */
     public synchronized FileMessageFactory getFactory(FileMessage msg)
             throws java.io.FileNotFoundException, java.io.IOException {
-        File writeToFile = new File(getTempDir(), msg.getFileName());
+        File writeToFile = new File(getTempDirFile(), msg.getFileName());
         FileMessageFactory factory = fileFactories.get(msg.getFileName());
         if (factory == null) {
             factory = FileMessageFactory.getInstance(writeToFile, true);
+            factory.setMaxValidTime(maxValidTime);
             fileFactories.put(msg.getFileName(), factory);
         }
         return factory;
@@ -373,6 +396,8 @@ public class FarmWarDeployer extends ClusterListener
     @Override
     public void install(String contextName, File webapp) throws IOException {
         Member[] members = getCluster().getMembers();
+        if (members.length == 0) return;
+
         Member localMember = getCluster().getLocalMember();
         FileMessageFactory factory =
             FileMessageFactory.getInstance(webapp, false);
@@ -420,15 +445,17 @@ public class FarmWarDeployer extends ClusterListener
     @Override
     public void remove(String contextName, boolean undeploy)
             throws IOException {
-        if (log.isInfoEnabled())
-            log.info(sm.getString("farmWarDeployer.removeStart", contextName));
-        Member localMember = getCluster().getLocalMember();
-        UndeployMessage msg = new UndeployMessage(localMember, System
-                .currentTimeMillis(), "Undeploy:" + contextName + ":"
-                + System.currentTimeMillis(), contextName, undeploy);
-        if (log.isDebugEnabled())
-            log.debug(sm.getString("farmWarDeployer.removeTxMsg", contextName));
-        cluster.send(msg);
+        if (getCluster().getMembers().length > 0) {
+            if (log.isInfoEnabled())
+                log.info(sm.getString("farmWarDeployer.removeStart", contextName));
+            Member localMember = getCluster().getLocalMember();
+            UndeployMessage msg = new UndeployMessage(localMember, System
+                    .currentTimeMillis(), "Undeploy:" + contextName + ":"
+                    + System.currentTimeMillis(), contextName, undeploy);
+            if (log.isDebugEnabled())
+                log.debug(sm.getString("farmWarDeployer.removeTxMsg", contextName));
+            cluster.send(msg);
+        }
         // remove locally
         if (undeploy) {
             try {
@@ -459,16 +486,28 @@ public class FarmWarDeployer extends ClusterListener
     @Override
     public void fileModified(File newWar) {
         try {
-            File deployWar = new File(getDeployDir(), newWar.getName());
-            copy(newWar, deployWar);
-            ContextName cn = new ContextName(deployWar.getName());
+            File deployWar = new File(getDeployDirFile(), newWar.getName());
+            ContextName cn = new ContextName(deployWar.getName(), true);
+            if (deployWar.exists() && deployWar.lastModified() > newWar.lastModified()) {
+                if (log.isInfoEnabled())
+                    log.info(sm.getString("farmWarDeployer.alreadyDeployed", cn.getName()));
+                return;
+            }
             if (log.isInfoEnabled())
                 log.info(sm.getString("farmWarDeployer.modInstall",
                         cn.getName(), deployWar.getAbsolutePath()));
-            try {
-                remove(cn.getName(), false);
-            } catch (Exception x) {
-                log.error(sm.getString("farmWarDeployer.modRemoveFail"), x);
+            // install local
+            if (!isServiced(cn.getName())) {
+                addServiced(cn.getName());
+                try {
+                    copy(newWar, deployWar);
+                    check(cn.getName());
+                } finally {
+                    removeServiced(cn.getName());
+                }
+            } else {
+                log.error(sm.getString("farmWarDeployer.servicingDeploy",
+                        cn.getName(), deployWar.getName()));
             }
             install(cn.getName(), deployWar);
         } catch (Exception x) {
@@ -484,7 +523,7 @@ public class FarmWarDeployer extends ClusterListener
     @Override
     public void fileRemoved(File removeWar) {
         try {
-            ContextName cn = new ContextName(removeWar.getName());
+            ContextName cn = new ContextName(removeWar.getName(), true);
             if (log.isInfoEnabled())
                 log.info(sm.getString("farmWarDeployer.removeLocal",
                         cn.getName()));
@@ -586,10 +625,13 @@ public class FarmWarDeployer extends ClusterListener
     @Override
     public void backgroundProcess() {
         if (started) {
-            count = (count + 1) % processDeployFrequency;
-            if (count == 0 && watchEnabled) {
-                watcher.check();
+            if (watchEnabled) {
+                count = (count + 1) % processDeployFrequency;
+                if (count == 0) {
+                    watcher.check();
+                }
             }
+            removeInvalidFileFactories();
         }
 
     }
@@ -649,6 +691,14 @@ public class FarmWarDeployer extends ClusterListener
         return deployDir;
     }
 
+    public File getDeployDirFile() {
+        if (deployDirFile != null) return deployDirFile;
+
+        File dir = getAbsolutePath(getDeployDir());
+        this.deployDirFile = dir;
+        return dir;
+    }
+
     public void setDeployDir(String deployDir) {
         this.deployDir = deployDir;
     }
@@ -657,6 +707,14 @@ public class FarmWarDeployer extends ClusterListener
         return tempDir;
     }
 
+    public File getTempDirFile() {
+        if (tempDirFile != null) return tempDirFile;
+
+        File dir = getAbsolutePath(getTempDir());
+        this.tempDirFile = dir;
+        return dir;
+    }
+
     public void setTempDir(String tempDir) {
         this.tempDir = tempDir;
     }
@@ -665,6 +723,14 @@ public class FarmWarDeployer extends ClusterListener
         return watchDir;
     }
 
+    public File getWatchDirFile() {
+        if (watchDirFile != null) return watchDirFile;
+
+        File dir = getAbsolutePath(getWatchDir());
+        this.watchDirFile = dir;
+        return dir;
+    }
+
     public void setWatchDir(String watchDir) {
         this.watchDir = watchDir;
     }
@@ -704,6 +770,14 @@ public class FarmWarDeployer extends ClusterListener
         this.processDeployFrequency = processExpiresFrequency;
     }
 
+    public int getMaxValidTime() {
+        return maxValidTime;
+    }
+
+    public void setMaxValidTime(int maxValidTime) {
+        this.maxValidTime = maxValidTime;
+    }
+
     /**
      * Copy a file to the specified temp directory.
      * @param from copy from temp
@@ -738,4 +812,26 @@ public class FarmWarDeployer extends ClusterListener
         return true;
     }
 
+    protected void removeInvalidFileFactories() {
+        String[] fileNames = fileFactories.keySet().toArray(new String[0]);
+        for (String fileName : fileNames) {
+            FileMessageFactory factory = fileFactories.get(fileName);
+            if (!factory.isValid()) {
+                fileFactories.remove(fileName);
+            }
+        }
+    }
+
+    private File getAbsolutePath(String path) {
+        File dir = new File(path);
+        File base = new File(System.getProperty(Globals.CATALINA_BASE_PROP));
+        if (!dir.isAbsolute()) {
+            dir = new File(base, dir.getPath());
+        }
+        try {
+            dir = dir.getCanonicalFile();
+        } catch (IOException e) {// ignore
+        }
+        return dir;
+    }
 }
