@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -20,10 +20,12 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
 
+import org.apache.catalina.Cluster;
 import org.apache.catalina.DistributedManager;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Session;
+import org.apache.catalina.ha.CatalinaCluster;
 import org.apache.catalina.ha.ClusterManager;
 import org.apache.catalina.ha.ClusterMessage;
 import org.apache.catalina.tribes.Channel;
@@ -31,6 +33,7 @@ import org.apache.catalina.tribes.tipis.AbstractReplicatedMap.MapOwner;
 import org.apache.catalina.tribes.tipis.LazyReplicatedMap;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  *@author Filip Hanik
@@ -39,20 +42,29 @@ import org.apache.juli.logging.LogFactory;
 public class BackupManager extends ClusterManagerBase
         implements MapOwner, DistributedManager {
 
-    private static final Log log = LogFactory.getLog(BackupManager.class);
+    private final Log log = LogFactory.getLog(BackupManager.class);
+
+    /**
+     * The string manager for this package.
+     */
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     protected static long DEFAULT_REPL_TIMEOUT = 15000;//15 seconds
 
-    /** Set to true if we don't want the sessions to expire on shutdown */
+    /**
+     * Set to true if we don't want the sessions to expire on shutdown
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
+     */
+    @Deprecated
     protected boolean mExpireSessionsOnShutdown = true;
-    
+
     /**
      * The name of this manager
      */
     protected String name;
 
     /**
-     * 
+     *
      */
     private int mapSendOptions = Channel.SEND_OPTIONS_SYNCHRONIZED_ACK|Channel.SEND_OPTIONS_USE_ACK;
 
@@ -61,6 +73,11 @@ public class BackupManager extends ClusterManagerBase
      */
     private long rpcTimeout = DEFAULT_REPL_TIMEOUT;
 
+    /**
+     * Flag for whether to terminate this map that failed to start.
+     */
+    private boolean terminateOnStartFailure = false;
+
     /**
      * Constructor, just calls super()
      *
@@ -71,18 +88,26 @@ public class BackupManager extends ClusterManagerBase
 
 
 //******************************************************************************/
-//      ClusterManager Interface     
+//      ClusterManager Interface
 //******************************************************************************/
 
     @Override
     public void messageDataReceived(ClusterMessage msg) {
     }
 
+    /**
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
+     */
+    @Deprecated
     public void setExpireSessionsOnShutdown(boolean expireSessionsOnShutdown)
     {
         mExpireSessionsOnShutdown = expireSessionsOnShutdown;
     }
 
+    /**
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
+     */
+    @Deprecated
     public boolean getExpireSessionsOnShutdown()
     {
         return mExpireSessionsOnShutdown;
@@ -117,7 +142,7 @@ public class BackupManager extends ClusterManagerBase
     public Session createEmptySession() {
         return new DeltaSession(this);
     }
-    
+
 
     @Override
     public String getName() {
@@ -132,31 +157,42 @@ public class BackupManager extends ClusterManagerBase
      * Starts the cluster communication channel, this will connect with the
      * other nodes in the cluster, and request the current session state to be
      * transferred to this node.
-     * 
+     *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
+    @SuppressWarnings("unchecked")
     @Override
     protected synchronized void startInternal() throws LifecycleException {
-        
+
         super.startInternal();
 
         try {
+            if (getCluster() == null) {
+                Cluster cluster = getContainer().getCluster();
+                if (cluster instanceof CatalinaCluster) {
+                    setCluster((CatalinaCluster)cluster);
+                } else {
+                    throw new LifecycleException(
+                            sm.getString("backupManager.noCluster", getName()));
+                }
+            }
             cluster.registerManager(this);
             LazyReplicatedMap map = new LazyReplicatedMap(this,
                                                           cluster.getChannel(),
                                                           rpcTimeout,
                                                           getMapName(),
-                                                          getClassLoaders());
+                                                          getClassLoaders(),
+                                                          terminateOnStartFailure);
             map.setChannelSendOptions(mapSendOptions);
             this.sessions = map;
         }  catch ( Exception x ) {
-            log.error("Unable to start BackupManager",x);
-            throw new LifecycleException("Failed to start BackupManager",x);
+            log.error(sm.getString("backupManager.startUnable", getName()),x);
+            throw new LifecycleException(sm.getString("backupManager.startFailed", getName()),x);
         }
         setState(LifecycleState.STARTING);
     }
-    
+
     public String getMapName() {
         String name = cluster.getManagerName(getName(),this)+"-"+"map";
         if ( log.isDebugEnabled() ) log.debug("Backup manager, Setting map name to:"+name);
@@ -167,7 +203,7 @@ public class BackupManager extends ClusterManagerBase
     /**
      * Stop this component and implement the requirements
      * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
-     * 
+     *
      * This will disconnect the cluster communication channel and stop the
      * listener thread.
      *
@@ -178,7 +214,7 @@ public class BackupManager extends ClusterManagerBase
     protected synchronized void stopInternal() throws LifecycleException {
 
         if (log.isDebugEnabled())
-            log.debug("Stopping");
+            log.debug(sm.getString("backupManager.stopped", getName()));
 
         setState(LifecycleState.STOPPING);
 
@@ -217,11 +253,19 @@ public class BackupManager extends ClusterManagerBase
         return rpcTimeout;
     }
 
+    public void setTerminateOnStartFailure(boolean terminateOnStartFailure) {
+        this.terminateOnStartFailure = terminateOnStartFailure;
+    }
+
+    public boolean isTerminateOnStartFailure() {
+        return terminateOnStartFailure;
+    }
+
     @Override
     public String[] getInvalidatedSessions() {
         return new String[0];
     }
-    
+
     @Override
     public ClusterManager cloneFromTemplate() {
         BackupManager result = new BackupManager();
@@ -229,6 +273,7 @@ public class BackupManager extends ClusterManagerBase
         result.mExpireSessionsOnShutdown = mExpireSessionsOnShutdown;
         result.mapSendOptions = mapSendOptions;
         result.rpcTimeout = rpcTimeout;
+        result.terminateOnStartFailure = terminateOnStartFailure;
         return result;
     }
 
