@@ -20,33 +20,33 @@ package org.apache.catalina.ha.session;
 
 /**
  * This class is used to track the series of actions that happens when
- * a request is executed. These actions will then translate into invokations of methods 
+ * a request is executed. These actions will then translate into invocations of methods 
  * on the actual session.
  * This class is NOT thread safe. One DeltaRequest per session
  * @author <a href="mailto:fhanik@apache.org">Filip Hanik</a>
  * @version 1.0
  */
 
+import java.io.ByteArrayOutputStream;
 import java.io.Externalizable;
+import java.io.IOException;
+import java.io.ObjectOutputStream;
 import java.security.Principal;
 import java.util.LinkedList;
 
 import org.apache.catalina.realm.GenericPrincipal;
-import org.apache.catalina.util.StringManager;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.io.ObjectOutputStream;
+import org.apache.tomcat.util.res.StringManager;
 
 
 public class DeltaRequest implements Externalizable {
 
-    public static org.apache.juli.logging.Log log =
+    public static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog( DeltaRequest.class );
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm = StringManager
+    protected static final StringManager sm = StringManager
             .getManager(Constants.Package);
 
     public static final int TYPE_ATTRIBUTE = 0;
@@ -64,8 +64,9 @@ public class DeltaRequest implements Externalizable {
     public static final String NAME_AUTHTYPE = "__SET__AUTHTYPE__";
 
     private String sessionId;
-    private LinkedList actions = new LinkedList();
-    private LinkedList actionPool = new LinkedList();
+    private LinkedList<AttributeInfo> actions = new LinkedList<AttributeInfo>();
+    private LinkedList<AttributeInfo> actionPool =
+        new LinkedList<AttributeInfo>();
     
     private boolean recordAllActions = false;
 
@@ -92,7 +93,7 @@ public class DeltaRequest implements Externalizable {
 
     public void setMaxInactiveInterval(int interval) {
         int action = ACTION_SET;
-        addAction(TYPE_MAXINTERVAL,action,NAME_MAXINTERVAL,new Integer(interval));
+        addAction(TYPE_MAXINTERVAL,action,NAME_MAXINTERVAL,Integer.valueOf(interval));
     }
     
     /**
@@ -117,7 +118,7 @@ public class DeltaRequest implements Externalizable {
 
     public void setNew(boolean n) {
         int action = ACTION_SET;
-        addAction(TYPE_ISNEW,action,NAME_ISNEW,new Boolean(n));
+        addAction(TYPE_ISNEW,action,NAME_ISNEW,Boolean.valueOf(n));
     }
 
     public void setAuthType(String authType) {
@@ -132,7 +133,7 @@ public class DeltaRequest implements Externalizable {
         AttributeInfo info = null;
         if ( this.actionPool.size() > 0 ) {
             try {
-                info = (AttributeInfo) actionPool.removeFirst();
+                info = actionPool.removeFirst();
             }catch ( Exception x ) {
                 log.error("Unable to remove element:",x);
                 info = new AttributeInfo(type, action, name, value);
@@ -154,16 +155,12 @@ public class DeltaRequest implements Externalizable {
         actions.addLast(info);
     }
     
-    public void execute(DeltaSession session) {
-        execute(session,true);
-    }
-
     public void execute(DeltaSession session, boolean notifyListeners) {
         if ( !this.sessionId.equals( session.getId() ) )
             throw new java.lang.IllegalArgumentException("Session id mismatch, not executing the delta request");
         session.access();
         for ( int i=0; i<actions.size(); i++ ) {
-            AttributeInfo info = (AttributeInfo)actions.get(i);
+            AttributeInfo info = actions.get(i);
             switch ( info.getType() ) {
                 case TYPE_ATTRIBUTE: {
                     if ( info.getAction() == ACTION_SET ) {
@@ -190,7 +187,7 @@ public class DeltaRequest implements Externalizable {
                     Principal p = null;
                     if ( info.getAction() == ACTION_SET ) {
                         SerializablePrincipal sp = (SerializablePrincipal)info.getValue();
-                        p = (Principal)sp.getPrincipal(session.getManager().getContainer().getRealm());
+                        p = sp.getPrincipal();
                     }
                     session.setPrincipal(p,false);
                     break;
@@ -213,7 +210,7 @@ public class DeltaRequest implements Externalizable {
     public void reset() {
         while ( actions.size() > 0 ) {
             try {
-                AttributeInfo info = (AttributeInfo) actions.removeFirst();
+                AttributeInfo info = actions.removeFirst();
                 info.recycle();
                 actionPool.addLast(info);
             }catch  ( Exception x ) {
@@ -241,6 +238,7 @@ public class DeltaRequest implements Externalizable {
         actionPool.clear();
     }
     
+    @Override
     public void readExternal(java.io.ObjectInput in) throws IOException,ClassNotFoundException {
         //sessionId - String
         //recordAll - boolean
@@ -251,29 +249,29 @@ public class DeltaRequest implements Externalizable {
         recordAllActions = in.readBoolean();
         int cnt = in.readInt();
         if (actions == null)
-            actions = new LinkedList();
+            actions = new LinkedList<AttributeInfo>();
         else
             actions.clear();
         for (int i = 0; i < cnt; i++) {
             AttributeInfo info = null;
             if (this.actionPool.size() > 0) {
                 try {
-                    info = (AttributeInfo) actionPool.removeFirst();
+                    info = actionPool.removeFirst();
                 } catch ( Exception x )  {
                     log.error("Unable to remove element",x);
-                    info = new AttributeInfo(-1,-1,null,null);
+                    info = new AttributeInfo();
                 }
             }
             else {
-                info = new AttributeInfo(-1,-1,null,null);
+                info = new AttributeInfo();
             }
             info.readExternal(in);
             actions.addLast(info);
         }//for
     }
-        
 
 
+    @Override
     public void writeExternal(java.io.ObjectOutput out ) throws java.io.IOException {
         //sessionId - String
         //recordAll - boolean
@@ -283,7 +281,7 @@ public class DeltaRequest implements Externalizable {
         out.writeBoolean(recordAllActions);
         out.writeInt(getSize());
         for ( int i=0; i<getSize(); i++ ) {
-            AttributeInfo info = (AttributeInfo)actions.get(i);
+            AttributeInfo info = actions.get(i);
             info.writeExternal(out);
         }
     }
@@ -292,7 +290,6 @@ public class DeltaRequest implements Externalizable {
      * serialize DeltaRequest
      * @see DeltaRequest#writeExternal(java.io.ObjectOutput)
      * 
-     * @param deltaRequest
      * @return serialized delta request
      * @throws IOException
      */
@@ -311,7 +308,9 @@ public class DeltaRequest implements Externalizable {
         private int action;
         private int type;
 
-        public AttributeInfo() {}
+        public AttributeInfo() {
+            this(-1, -1, null, null);
+        }
 
         public AttributeInfo(int type,
                              int action,
@@ -342,6 +341,7 @@ public class DeltaRequest implements Externalizable {
         public Object getValue() {
             return value;
         }
+        @Override
         public int hashCode() {
             return name.hashCode();
         }
@@ -357,12 +357,14 @@ public class DeltaRequest implements Externalizable {
             action=-1;
         }
 
+        @Override
         public boolean equals(Object o) {
             if ( ! (o instanceof AttributeInfo ) ) return false;
             AttributeInfo other =  (AttributeInfo)o;
             return other.getName().equals(this.getName());
         }
         
+        @Override
         public void readExternal(java.io.ObjectInput in ) throws IOException,ClassNotFoundException {
             //type - int
             //action - int
@@ -376,6 +378,7 @@ public class DeltaRequest implements Externalizable {
             if ( hasValue ) value = in.readObject();
         }
 
+        @Override
         public void writeExternal(java.io.ObjectOutput out) throws IOException {
             //type - int
             //action - int
@@ -389,8 +392,9 @@ public class DeltaRequest implements Externalizable {
             if (getValue()!=null) out.writeObject(getValue());
         }
         
+        @Override
         public String toString() {
-            StringBuffer buf = new StringBuffer("AttributeInfo[type=");
+            StringBuilder buf = new StringBuilder("AttributeInfo[type=");
             buf.append(getType()).append(", action=").append(getAction());
             buf.append(", name=").append(getName()).append(", value=").append(getValue());
             buf.append(", addr=").append(super.toString()).append("]");
