@@ -53,9 +53,7 @@ import org.apache.tomcat.util.res.StringManager;
  * track of deltas during a request.
  *
  * @author Filip Hanik
- * @version $Id: DeltaSession.java 1226162 2011-12-31 18:32:53Z markt $
  */
-
 public class DeltaSession extends StandardSession implements Externalizable,ClusterSession,ReplicatedMapEntry {
 
     public static final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(DeltaSession.class);
@@ -112,116 +110,140 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     // ----------------------------------------------------- ReplicatedMapEntry
 
     /**
-         * Has the object changed since last replication
-         * and is not in a locked state
-         * @return boolean
-         */
-        @Override
-        public boolean isDirty() {
-            return getDeltaRequest().getSize()>0;
-        }
+     * Has the object changed since last replication
+     * and is not in a locked state
+     * @return boolean
+     */
+    @Override
+    public boolean isDirty() {
+        return getDeltaRequest().getSize()>0;
+    }
 
-        /**
-         * If this returns true, the map will extract the diff using getDiff()
-         * Otherwise it will serialize the entire object.
-         * @return boolean
-         */
-        @Override
-        public boolean isDiffable() {
+    /**
+     * If this returns true, the map will extract the diff using getDiff()
+     * Otherwise it will serialize the entire object.
+     * @return boolean
+     */
+    @Override
+    public boolean isDiffable() {
+        return true;
+    }
+
+    /**
+     * Returns a diff and sets the dirty map to false
+     * @return byte[]
+     * @throws IOException
+     */
+    @Override
+    public byte[] getDiff() throws IOException {
+        try{
+            lock();
+            return getDeltaRequest().serialize();
+        }finally{
+            unlock();
+        }
+    }
+
+    public ClassLoader[] getClassLoaders() {
+        if ( manager instanceof BackupManager ) return ((BackupManager)manager).getClassLoaders();
+        else if ( manager instanceof ClusterManagerBase ) return ((ClusterManagerBase)manager).getClassLoaders();
+        else if ( manager instanceof StandardManager ) {
+            StandardManager sm = (StandardManager)manager;
+            return ClusterManagerBase.getClassLoaders(sm.getContainer());
+        } else if ( manager instanceof ManagerBase ) {
+            ManagerBase mb = (ManagerBase)manager;
+            return ClusterManagerBase.getClassLoaders(mb.getContainer());
+        }//end if
+        return null;
+    }
+
+    /**
+     * Applies a diff to an existing object.
+     * @param diff byte[]
+     * @param offset int
+     * @param length int
+     * @throws IOException
+     */
+    @Override
+    public void applyDiff(byte[] diff, int offset, int length) throws IOException, ClassNotFoundException {
+        try {
+            lock();
+            ReplicationStream stream = ( (ClusterManager) getManager()).getReplicationStream(diff, offset, length);
+            ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
+            try {
+                ClassLoader[] loaders = getClassLoaders();
+                if (loaders != null && loaders.length > 0)
+                    Thread.currentThread().setContextClassLoader(loaders[0]);
+                getDeltaRequest().readExternal(stream);
+                getDeltaRequest().execute(this, ((ClusterManager)getManager()).isNotifyListenersOnReplication());
+            } finally {
+                Thread.currentThread().setContextClassLoader(contextLoader);
+            }
+        }finally {
+            unlock();
+        }
+    }
+
+    /**
+     * Resets the current diff state and resets the dirty flag
+     */
+    @Override
+    public void resetDiff() {
+        resetDeltaRequest();
+    }
+
+    /**
+     * Lock during serialization
+     */
+    @Override
+    public void lock() {
+        diffLock.lock();
+    }
+
+    /**
+     * Unlock after serialization
+     */
+    @Override
+    public void unlock() {
+        diffLock.unlock();
+    }
+
+    @Override
+    public void setOwner(Object owner) {
+        if ( owner instanceof ClusterManager && getManager()==null) {
+            ClusterManager cm = (ClusterManager)owner;
+            this.setManager(cm);
+            this.setValid(true);
+            this.setPrimarySession(false);
+            this.access();
+            this.resetDeltaRequest();
+            this.endAccess();
+        }
+    }
+
+    /**
+     * If this returns true, to replicate that an object has been accessed
+     * @return boolean
+     */
+    @Override
+    public boolean isAccessReplicate() {
+        long replDelta = System.currentTimeMillis() - getLastTimeReplicated();
+        if (maxInactiveInterval >=0 && replDelta > (maxInactiveInterval * 1000)) {
             return true;
         }
+        return false;
+    }
 
-        /**
-         * Returns a diff and sets the dirty map to false
-         * @return byte[]
-         * @throws IOException
-         */
-        @Override
-        public byte[] getDiff() throws IOException {
-            try{
-                lock();
-                return getDeltaRequest().serialize();
-            }finally{
-                unlock();
-            }
-        }
+    /**
+     * Access to an existing object.
+     */
+    @Override
+    public void accessEntry() {
+        this.access();
+        this.setPrimarySession(false);
+        this.endAccess();
+    }
 
-        public ClassLoader[] getClassLoaders() {
-            if ( manager instanceof BackupManager ) return ((BackupManager)manager).getClassLoaders();
-            else if ( manager instanceof ClusterManagerBase ) return ((ClusterManagerBase)manager).getClassLoaders();
-            else if ( manager instanceof StandardManager ) {
-                StandardManager sm = (StandardManager)manager;
-                return ClusterManagerBase.getClassLoaders(sm.getContainer());
-            } else if ( manager instanceof ManagerBase ) {
-                ManagerBase mb = (ManagerBase)manager;
-                return ClusterManagerBase.getClassLoaders(mb.getContainer());
-            }//end if
-            return null;
-        }
-
-        /**
-         * Applies a diff to an existing object.
-         * @param diff byte[]
-         * @param offset int
-         * @param length int
-         * @throws IOException
-         */
-        @Override
-        public void applyDiff(byte[] diff, int offset, int length) throws IOException, ClassNotFoundException {
-            try {
-                lock();
-                ReplicationStream stream = ( (ClusterManager) getManager()).getReplicationStream(diff, offset, length);
-                ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
-                try {
-                    ClassLoader[] loaders = getClassLoaders();
-                    if (loaders != null && loaders.length > 0)
-                        Thread.currentThread().setContextClassLoader(loaders[0]);
-                    getDeltaRequest().readExternal(stream);
-                    getDeltaRequest().execute(this, ((ClusterManager)getManager()).isNotifyListenersOnReplication());
-                } finally {
-                    Thread.currentThread().setContextClassLoader(contextLoader);
-                }
-            }finally {
-                unlock();
-            }
-        }
-
-        /**
-         * Resets the current diff state and resets the dirty flag
-         */
-        @Override
-        public void resetDiff() {
-            resetDeltaRequest();
-        }
-
-        /**
-         * Lock during serialization
-         */
-        @Override
-        public void lock() {
-            diffLock.lock();
-        }
-
-        /**
-         * Unlock after serialization
-         */
-        @Override
-        public void unlock() {
-            diffLock.unlock();
-        }
-
-        @Override
-        public void setOwner(Object owner) {
-            if ( owner instanceof ClusterManager && getManager()==null) {
-                ClusterManager cm = (ClusterManager)owner;
-                this.setManager(cm);
-                this.setValid(true);
-                this.setPrimarySession(false);
-                this.access();
-                this.resetDeltaRequest();
-                this.endAccess();
-            }
-        }
     // ----------------------------------------------------- Session Properties
 
     /**
@@ -361,12 +383,12 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
      */
     @Override
     public boolean isValid() {
-        if (this.expiring) {
-            return true;
-        }
         if (!this.isValid) {
             return false;
         }
+        if (this.expiring) {
+            return true;
+        }
         if (ACTIVITY_CHECK && accessCount.get() > 0) {
             return true;
         }
@@ -421,30 +443,49 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
     }
 
     public void expire(boolean notify, boolean notifyCluster) {
-        if (expiring)
+
+        // Check to see if session has already been invalidated.
+        // Do not check expiring at this point as expire should not return until
+        // isValid is false
+        if (!isValid)
             return;
-        String expiredId = getIdInternal();
 
-        if(notifyCluster && expiredId != null && manager != null &&
-           manager instanceof DeltaManager) {
-            DeltaManager dmanager = (DeltaManager)manager;
-            CatalinaCluster cluster = dmanager.getCluster();
-            ClusterMessage msg = dmanager.requestCompleted(expiredId, true);
-            if (msg != null) {
-                cluster.send(msg);
+        synchronized (this) {
+            // Check again, now we are inside the sync so this code only runs once
+            // Double check locking - isValid needs to be volatile
+            if (!isValid)
+                return;
+
+            if (manager == null)
+                return;
+
+            // Mark this session as "being expired". The flag will be unset in
+            // the call to super.expire(notify)
+            expiring = true;
+
+            String expiredId = getIdInternal();
+
+            if(notifyCluster && expiredId != null &&
+                    manager instanceof DeltaManager) {
+                DeltaManager dmanager = (DeltaManager)manager;
+                CatalinaCluster cluster = dmanager.getCluster();
+                ClusterMessage msg = dmanager.requestCompleted(expiredId, true);
+                if (msg != null) {
+                    cluster.send(msg);
+                }
             }
-        }
 
-        super.expire(notify);
+            super.expire(notify);
 
-        if (notifyCluster) {
-            if (log.isDebugEnabled())
-                log.debug(sm.getString("deltaSession.notifying",
-                                       ((ClusterManager)manager).getName(), 
-                                       Boolean.valueOf(isPrimarySession()), 
-                                       expiredId));
-            if ( manager instanceof DeltaManager ) {
-                ( (DeltaManager) manager).sessionExpired(expiredId);
+            if (notifyCluster) {
+                if (log.isDebugEnabled())
+                    log.debug(sm.getString("deltaSession.notifying",
+                                           ((ClusterManager)manager).getName(),
+                                           Boolean.valueOf(isPrimarySession()),
+                                           expiredId));
+                if ( manager instanceof DeltaManager ) {
+                    ( (DeltaManager) manager).sessionExpired(expiredId);
+                }
             }
         }
     }
@@ -843,7 +884,8 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         }
     }
 
-    protected long getLastTimeReplicated() {
+    @Override
+    public long getLastTimeReplicated() {
         return lastTimeReplicated;
     }
 
@@ -852,7 +894,8 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         return version;
     }
 
-    protected void setLastTimeReplicated(long lastTimeReplicated) {
+    @Override
+    public void setLastTimeReplicated(long lastTimeReplicated) {
         this.lastTimeReplicated = lastTimeReplicated;
     }
 
