@@ -24,6 +24,9 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
+
+import javax.management.ObjectName;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
@@ -33,6 +36,7 @@ import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Valve;
 import org.apache.catalina.ha.CatalinaCluster;
@@ -40,43 +44,42 @@ import org.apache.catalina.ha.ClusterListener;
 import org.apache.catalina.ha.ClusterManager;
 import org.apache.catalina.ha.ClusterMessage;
 import org.apache.catalina.ha.ClusterValve;
+import org.apache.catalina.ha.session.ClusterSessionListener;
 import org.apache.catalina.ha.session.DeltaManager;
+import org.apache.catalina.ha.session.JvmRouteBinderValve;
+import org.apache.catalina.ha.session.JvmRouteSessionIDBinderListener;
+import org.apache.catalina.ha.session.SessionMessage;
 import org.apache.catalina.ha.util.IDynamicProperty;
+import org.apache.catalina.mbeans.MBeanUtils;
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.ChannelListener;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.MembershipListener;
 import org.apache.catalina.tribes.group.GroupChannel;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.tribes.group.interceptors.MessageDispatch15Interceptor;
+import org.apache.catalina.tribes.group.interceptors.TcpFailureDetector;
+import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.IntrospectionUtils;
-import org.apache.catalina.ha.session.ClusterSessionListener;
-import org.apache.catalina.tribes.group.interceptors.MessageDispatch15Interceptor;
-import org.apache.catalina.tribes.group.interceptors.TcpFailureDetector;
-import org.apache.catalina.ha.session.JvmRouteBinderValve;
-import org.apache.catalina.ha.session.JvmRouteSessionIDBinderListener;
-import org.apache.catalina.ha.session.SessionMessage;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * A <b>Cluster </b> implementation using simple multicast. Responsible for
  * setting up a cluster and provides callers with a valid multicast
  * receiver/sender.
  * 
- * FIXME remove install/remove/start/stop context dummys
  * FIXME wrote testcases 
  * 
  * @author Filip Hanik
  * @author Remy Maucherat
  * @author Peter Rossbach
- * @version $Id: SimpleTcpCluster.java 1377884 2012-08-27 21:46:29Z markt $
  */
-public class SimpleTcpCluster 
-    implements CatalinaCluster, Lifecycle, LifecycleListener, IDynamicProperty,
+public class SimpleTcpCluster extends LifecycleMBeanBase
+        implements CatalinaCluster, LifecycleListener, IDynamicProperty,
                MembershipListener, ChannelListener{
 
-    public static Log log = LogFactory.getLog(SimpleTcpCluster.class);
+    public static final Log log = LogFactory.getLog(SimpleTcpCluster.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -119,7 +122,7 @@ public class SimpleTcpCluster
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     /**
      * The cluster name to join
@@ -137,16 +140,6 @@ public class SimpleTcpCluster
      */
     protected Container container = null;
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-    /**
-     * Has this component been started?
-     */
-    protected boolean started = false;
-
     /**
      * The property change support for this component.
      */
@@ -155,18 +148,20 @@ public class SimpleTcpCluster
     /**
      * The context name <->manager association for distributed contexts.
      */
-    protected Map managers = new HashMap();
+    protected Map<String, ClusterManager> managers =
+        new HashMap<String, ClusterManager>();
 
     protected ClusterManager managerTemplate = new DeltaManager();
 
-    private List valves = new ArrayList();
+    private List<Valve> valves = new ArrayList<Valve>();
 
     private org.apache.catalina.ha.ClusterDeployer clusterDeployer;
+    private ObjectName onameClusterDeployer;
 
     /**
      * Listeners of messages
      */
-    protected List clusterListeners = new ArrayList();
+    protected List<ClusterListener> clusterListeners = new ArrayList<ClusterListener>();
 
     /**
      * Comment for <code>notifyLifecycleListenerOnFailure</code>
@@ -175,16 +170,23 @@ public class SimpleTcpCluster
 
     /**
      * dynamic sender <code>properties</code>
+     *
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
-    private Map properties = new HashMap();
+    @Deprecated
+    private Map<String, Object> properties = new HashMap<String, Object>();
     
     private int channelSendOptions = Channel.SEND_OPTIONS_ASYNCHRONOUS;
     
     private int channelStartOptions = Channel.DEFAULT;
 
+    private Map<Member,ObjectName> memberOnameMap =
+            new ConcurrentHashMap<Member,ObjectName>();
+
     // ------------------------------------------------------------- Properties
 
     public SimpleTcpCluster() {
+        // NO-OP
     }
 
     /**
@@ -192,6 +194,7 @@ public class SimpleTcpCluster
      * corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
         return (info);
     }
@@ -219,6 +222,7 @@ public class SimpleTcpCluster
      * @param clusterName
      *            The clustername to join
      */
+    @Override
     public void setClusterName(String clusterName) {
         this.clusterName = clusterName;
     }
@@ -229,6 +233,7 @@ public class SimpleTcpCluster
      * 
      * @return The name of the cluster associated with this server
      */
+    @Override
     public String getClusterName() {
         if(clusterName == null && container != null)
             return container.getName() ;
@@ -241,6 +246,7 @@ public class SimpleTcpCluster
      * @param container
      *            The Container to use
      */
+    @Override
     public void setContainer(Container container) {
         Container oldContainer = this.container;
         this.container = container;
@@ -252,6 +258,7 @@ public class SimpleTcpCluster
      * 
      * @return The Container associated with our Cluster
      */
+    @Override
     public Container getContainer() {
         return (this.container);
     }
@@ -276,27 +283,12 @@ public class SimpleTcpCluster
                 this.notifyLifecycleListenerOnFailure);
     }
 
-    /**
-     * @deprecated use getManagerTemplate().getClass().getName() instead.
-     * @return String
-     */
-    public String getManagerClassName() {
-        return managerTemplate.getClass().getName();
-    }
-
-    /**
-     * @deprecated use nested &lt;Manager&gt; element inside the cluster config instead.
-     * @param managerClassName String
-     */
-    public void setManagerClassName(String managerClassName) {
-        log.warn("setManagerClassName is deprecated, use nested <Manager> element inside the <Cluster> element instead, this request will be ignored.");
-    }
-
     /**
      * Add cluster valve 
      * Cluster Valves are only add to container when cluster is started!
      * @param valve The new cluster Valve.
      */
+    @Override
     public void addValve(Valve valve) {
         if (valve instanceof ClusterValve && (!valves.contains(valve)))
             valves.add(valve);
@@ -306,8 +298,9 @@ public class SimpleTcpCluster
      * get all cluster valves
      * @return current cluster valves
      */
+    @Override
     public Valve[] getValves() {
-        return (Valve[]) valves.toArray(new Valve[valves.size()]);
+        return valves.toArray(new Valve[valves.size()]);
     }
 
     /**
@@ -325,10 +318,11 @@ public class SimpleTcpCluster
     }
 
     /**
-     * add cluster message listener and register cluster to this listener
+     * Add cluster message listener and register cluster to this listener.
      * 
-     * @see org.apache.catalina.ha.CatalinaCluster#addClusterListener(org.apache.catalina.ha.MessageListener)
+     * @see org.apache.catalina.ha.CatalinaCluster#addClusterListener(org.apache.catalina.ha.ClusterListener)
      */
+    @Override
     public void addClusterListener(ClusterListener listener) {
         if (listener != null && !clusterListeners.contains(listener)) {
             clusterListeners.add(listener);
@@ -337,10 +331,11 @@ public class SimpleTcpCluster
     }
 
     /**
-     * remove message listener and deregister Cluster from listener
+     * Remove message listener and deregister Cluster from listener.
      * 
-     * @see org.apache.catalina.ha.CatalinaCluster#removeClusterListener(org.apache.catalina.ha.MessageListener)
+     * @see org.apache.catalina.ha.CatalinaCluster#removeClusterListener(org.apache.catalina.ha.ClusterListener)
      */
+    @Override
     public void removeClusterListener(ClusterListener listener) {
         if (listener != null) {
             clusterListeners.remove(listener);
@@ -351,6 +346,7 @@ public class SimpleTcpCluster
     /**
      * get current Deployer
      */
+    @Override
     public org.apache.catalina.ha.ClusterDeployer getClusterDeployer() {
         return clusterDeployer;
     }
@@ -358,11 +354,13 @@ public class SimpleTcpCluster
     /**
      * set a new Deployer, must be set before cluster started!
      */
+    @Override
     public void setClusterDeployer(
             org.apache.catalina.ha.ClusterDeployer clusterDeployer) {
         this.clusterDeployer = clusterDeployer;
     }
 
+    @Override
     public void setChannel(Channel channel) {
         this.channel = channel;
     }
@@ -379,6 +377,7 @@ public class SimpleTcpCluster
      * has members
      */
     protected boolean hasMembers = false;
+    @Override
     public boolean hasMembers() {
         return hasMembers;
     }
@@ -387,6 +386,7 @@ public class SimpleTcpCluster
      * Get all current cluster members
      * @return all members or empty array 
      */
+    @Override
     public Member[] getMembers() {
         return channel.getMembers();
     }
@@ -396,6 +396,7 @@ public class SimpleTcpCluster
      * 
      * @return Member
      */
+    @Override
     public Member getLocalMember() {
         return channel.getLocalMember(true);
     }
@@ -408,7 +409,9 @@ public class SimpleTcpCluster
      * 
      * @param name
      * @param value
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     public boolean setProperty(String name, String value) {
         return setProperty(name, (Object) value);
     }
@@ -418,9 +421,18 @@ public class SimpleTcpCluster
      * 
      * @param name
      * @param value
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Override
+    @Deprecated
     public boolean setProperty(String name, Object value) {
+        if (log.isTraceEnabled())
+            log.trace(sm.getString("SimpleTcpCluster.setProperty", name, value,properties.get(name)));
         properties.put(name, value);
+        //using a dynamic way of setting properties is nice, but a security risk
+        //if exposed through JMX. This way you can sit and try to guess property names,
+        //we will only allow explicit property names
+        log.warn("Dynamic setProperty("+name+",value) has been disabled, please use explicit properties for the element you are trying to identify");
         return false;
     }
 
@@ -429,7 +441,11 @@ public class SimpleTcpCluster
      * 
      * @param key
      * @return The property
+     *
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Override
+    @Deprecated
     public Object getProperty(String key) {
         if (log.isTraceEnabled())
             log.trace(sm.getString("SimpleTcpCluster.getProperty", key));
@@ -440,8 +456,11 @@ public class SimpleTcpCluster
      * Get all properties keys
      * 
      * @return An iterator over the property names.
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
-    public Iterator getPropertyNames() {
+    @Override
+    @Deprecated
+    public Iterator<String> getPropertyNames() {
         return properties.keySet().iterator();
     }
 
@@ -449,7 +468,10 @@ public class SimpleTcpCluster
      * remove a configured property.
      * 
      * @param key
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Override
+    @Deprecated
     public void removeProperty(String key) {
         properties.remove(key);
     }
@@ -458,11 +480,13 @@ public class SimpleTcpCluster
      * transfer properties from cluster configuration to subelement bean.
      * @param prefix
      * @param bean
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     protected void transferProperty(String prefix, Object bean) {
         if (prefix != null) {
-            for (Iterator iter = getPropertyNames(); iter.hasNext();) {
-                String pkey = (String) iter.next();
+            for (Iterator<String> iter = getPropertyNames(); iter.hasNext();) {
+                String pkey = iter.next();
                 if (pkey.startsWith(prefix)) {
                     String key = pkey.substring(prefix.length() + 1);
                     Object value = getProperty(pkey);
@@ -477,10 +501,12 @@ public class SimpleTcpCluster
     /**
      * @return Returns the managers.
      */
-    public Map getManagers() {
+    @Override
+    public Map<String, ClusterManager> getManagers() {
         return managers;
     }
 
+    @Override
     public Channel getChannel() {
         return channel;
     }
@@ -499,24 +525,28 @@ public class SimpleTcpCluster
      * @param name
      *            Context Name of this manager
      * @see org.apache.catalina.Cluster#createManager(java.lang.String)
-     * @see #addManager(String, Manager)
      * @see DeltaManager#start()
      */
+    @Override
     public synchronized Manager createManager(String name) {
-        if (log.isDebugEnabled()) log.debug("Creating ClusterManager for context " + name + " using class " + getManagerClassName());
-        Manager manager = null;
+        if (log.isDebugEnabled()) {
+            log.debug("Creating ClusterManager for context " + name +
+                    " using class " + getManagerTemplate().getClass().getName());
+        }
+        ClusterManager manager = null;
         try {
             manager = managerTemplate.cloneFromTemplate();
-            ((ClusterManager)manager).setName(name);
+            manager.setName(name);
         } catch (Exception x) {
             log.error("Unable to clone cluster manager, defaulting to org.apache.catalina.ha.session.DeltaManager", x);
             manager = new org.apache.catalina.ha.session.DeltaManager();
         } finally {
-            if ( manager != null && (manager instanceof ClusterManager)) ((ClusterManager)manager).setCluster(this);
+            if ( manager != null) manager.setCluster(this);
         }
         return manager;
     }
     
+    @Override
     public void registerManager(Manager manager) {
     
         if (! (manager instanceof ClusterManager)) {
@@ -526,39 +556,40 @@ public class SimpleTcpCluster
         ClusterManager cmanager = (ClusterManager) manager ;
         cmanager.setDistributable(true);
         // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(BEFORE_MANAGERREGISTER_EVENT, manager);
+        fireLifecycleEvent(BEFORE_MANAGERREGISTER_EVENT, manager);
         String clusterName = getManagerName(cmanager.getName(), manager);
         cmanager.setName(clusterName);
         cmanager.setCluster(this);
-        cmanager.setDefaultMode(false);
     
-        managers.put(clusterName, manager);
+        managers.put(clusterName, cmanager);
         // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(AFTER_MANAGERREGISTER_EVENT, manager);    
+        fireLifecycleEvent(AFTER_MANAGERREGISTER_EVENT, manager);    
     }
 
     /**
-     * remove an application form cluster replication bus
+     * Remove an application from cluster replication bus.
      * 
-     * @see org.apache.catalina.ha.CatalinaCluster#removeManager(java.lang.String,Manager)
+     * @see org.apache.catalina.Cluster#removeManager(Manager)
      */
+    @Override
     public void removeManager(Manager manager) {
         if (manager != null && manager instanceof ClusterManager ) {
             ClusterManager cmgr = (ClusterManager) manager;
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(BEFORE_MANAGERUNREGISTER_EVENT,manager);
+            fireLifecycleEvent(BEFORE_MANAGERUNREGISTER_EVENT,manager);
             managers.remove(getManagerName(cmgr.getName(),manager));
             cmgr.setCluster(null);
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(AFTER_MANAGERUNREGISTER_EVENT, manager);
+            fireLifecycleEvent(AFTER_MANAGERUNREGISTER_EVENT, manager);
         }
     }
 
     /**
      * @param name
      * @param manager
-     * @return
+     * @return TODO
      */
+    @Override
     public String getManagerName(String name, Manager manager) {
         String clusterName = name ;
         if ( clusterName == null ) clusterName = manager.getContainer().getName();
@@ -566,8 +597,10 @@ public class SimpleTcpCluster
             Container context = manager.getContainer() ;
             if(context != null && context instanceof Context) {
                 Container host = ((Context)context).getParent();
-                if(host != null && host instanceof Host && clusterName!=null && !(clusterName.indexOf("#")>=0))
+                if(host != null && host instanceof Host && clusterName!=null && 
+                        !(clusterName.startsWith(host.getName() +"#"))) {
                     clusterName = host.getName() +"#" + clusterName ;
+                }
             }
         }
         return clusterName;
@@ -578,8 +611,9 @@ public class SimpleTcpCluster
      * 
      * @see org.apache.catalina.ha.CatalinaCluster#getManager(java.lang.String)
      */
+    @Override
     public Manager getManager(String name) {
-        return (Manager) managers.get(name);
+        return managers.get(name);
     }
     
     // ------------------------------------------------------ Lifecycle Methods
@@ -593,42 +627,17 @@ public class SimpleTcpCluster
      * @see org.apache.catalina.tribes.group.GroupChannel.HeartbeatThread#run()
      * 
      */
+    @Override
     public void backgroundProcess() {
         if (clusterDeployer != null) clusterDeployer.backgroundProcess();
        
         //send a heartbeat through the channel        
         if ( isHeartbeatBackgroundEnabled() && channel !=null ) channel.heartbeat();
+
+        // periodic event
+        fireLifecycleEvent(Lifecycle.PERIODIC_EVENT, null);
     }
 
-    /**
-     * Add a lifecycle event listener to this component.
-     * 
-     * @param listener
-     *            The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-        lifecycle.addLifecycleListener(listener);
-    }
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     * 
-     * @param listener
-     *            The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-        lifecycle.removeLifecycleListener(listener);
-    }
 
     /**
      * Use as base to handle start/stop/periodic Events from host. Currently
@@ -636,34 +645,41 @@ public class SimpleTcpCluster
      * 
      * @see org.apache.catalina.LifecycleListener#lifecycleEvent(org.apache.catalina.LifecycleEvent)
      */
+    @Override
     public void lifecycleEvent(LifecycleEvent lifecycleEvent) {
         if (log.isTraceEnabled())
             log.trace(sm.getString("SimpleTcpCluster.event.log", lifecycleEvent.getType(), lifecycleEvent.getData()));
     }
 
+
     // ------------------------------------------------------ public
 
+    @SuppressWarnings("deprecation")
+    @Override
+    protected void initInternal() throws LifecycleException {
+        super.initInternal();
+        if (clusterDeployer != null) {
+            StringBuilder name = new StringBuilder("type=Cluster");
+            Container container = getContainer();
+            name.append(MBeanUtils.getContainerKeyProperties(container));
+            name.append(",component=Deployer");
+            onameClusterDeployer = register(clusterDeployer, name.toString());
+        }
+    }
+    
+    
     /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component. This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized. <BR>
-     * Starts the cluster communication channel, this will connect with the
-     * other nodes in the cluster, and request the current session state to be
-     * transferred to this node.
-     * 
-     * @exception IllegalStateException
-     *                if this component has already been started
-     * @exception LifecycleException
-     *                if this component detects a fatal error that prevents this
-     *                component from being used
+     * Start Cluster and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-        if (started)
-            throw new LifecycleException(sm.getString("cluster.alreadyStarted"));
+    @Override
+    protected void startInternal() throws LifecycleException {
+
         if (log.isInfoEnabled()) log.info("Cluster is about to start");
 
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(BEFORE_START_EVENT, this);
         try {
             checkDefaults();
             registerClusterValve();
@@ -671,13 +687,13 @@ public class SimpleTcpCluster
             channel.addChannelListener(this);
             channel.start(channelStartOptions);
             if (clusterDeployer != null) clusterDeployer.start();
-            this.started = true;
-            // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(AFTER_START_EVENT, this);
+            registerMember(channel.getLocalMember(false));
         } catch (Exception x) {
             log.error("Unable to start cluster.", x);
             throw new LifecycleException(x);
         }
+        
+        setState(LifecycleState.STARTING);
     }
 
     protected void checkDefaults() {
@@ -704,7 +720,7 @@ public class SimpleTcpCluster
      */
     protected void registerClusterValve() throws Exception {
         if(container != null ) {
-            for (Iterator iter = valves.iterator(); iter.hasNext();) {
+            for (Iterator<Valve> iter = valves.iterator(); iter.hasNext();) {
                 ClusterValve valve = (ClusterValve) iter.next();
                 if (log.isDebugEnabled())
                     log.debug("Invoking addValve on " + getContainer()
@@ -714,8 +730,8 @@ public class SimpleTcpCluster
                             new Object[] { valve },
                             new Class[] { org.apache.catalina.Valve.class });
 
+                    valve.setCluster(this);
                 }
-                valve.setCluster(this);
             }
         }
     }
@@ -726,39 +742,34 @@ public class SimpleTcpCluster
      * @throws ClassNotFoundException
      */
     protected void unregisterClusterValve() throws Exception {
-        for (Iterator iter = valves.iterator(); iter.hasNext();) {
+        for (Iterator<Valve> iter = valves.iterator(); iter.hasNext();) {
             ClusterValve valve = (ClusterValve) iter.next();
             if (log.isDebugEnabled())
                 log.debug("Invoking removeValve on " + getContainer()
                         + " with class=" + valve.getClass().getName());
             if (valve != null) {
-                    IntrospectionUtils.callMethodN(getContainer(), "removeValve",
-                        new Object[] { valve }, new Class[] { org.apache.catalina.Valve.class });
+                IntrospectionUtils.callMethodN(getContainer(), "removeValve",
+                    new Object[] { valve },
+                    new Class[] { org.apache.catalina.Valve.class });
+                valve.setCluster(this);
             }
-            valve.setCluster(this);
         }
     }
 
+    
     /**
-     * Gracefully terminate the active cluster component.<br/>
-     * This will disconnect the cluster communication channel, stop the
-     * listener and deregister the valves from host or engine.<br/><br/>
-     * <b>Note:</b><br/>The sub elements receiver, sender, membership,
-     * listener or valves are not removed. You can easily start the cluster again.
-     * 
-     * @exception IllegalStateException
-     *                if this component has not been started
-     * @exception LifecycleException
-     *                if this component detects a fatal error that needs to be
-     *                reported
+     * Stop Cluster and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
-        if (!started)
-            throw new IllegalStateException(sm.getString("cluster.notStarted"));
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(BEFORE_STOP_EVENT, this);
+        setState(LifecycleState.STOPPING);
 
+        unregisterMember(channel.getLocalMember(false));
         if (clusterDeployer != null) clusterDeployer.stop();
         this.managers.clear();
         try {
@@ -768,15 +779,37 @@ public class SimpleTcpCluster
             channel.removeMembershipListener(this);
             this.unregisterClusterValve();
         } catch (Exception x) {
-            log.error("Unable to stop cluster valve.", x);
+            log.error("Unable to stop cluster.", x);
         }
-        started = false;
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(AFTER_STOP_EVENT, this);
-   }
+    }
 
     
+    @Override
+    protected void destroyInternal() throws LifecycleException {
+        if (onameClusterDeployer != null) {
+            unregister(onameClusterDeployer);
+            onameClusterDeployer = null;
+        }
+        super.destroyInternal();
+    }
 
+    
+    /**
+     * Return a String rendering of this object.
+     */
+    @Override
+    public String toString() {
+        StringBuilder sb = new StringBuilder(this.getClass().getName());
+        sb.append('[');
+        if (container == null) {
+            sb.append("Container is null");
+        } else {
+            sb.append(container.getName());
+        }
+        sb.append(']');
+        return sb.toString();
+    }
+    
 
     /**
      * send message to all cluster members
@@ -784,19 +817,10 @@ public class SimpleTcpCluster
      * 
      * @see org.apache.catalina.ha.CatalinaCluster#send(org.apache.catalina.ha.ClusterMessage)
      */
+    @Override
     public void send(ClusterMessage msg) {
         send(msg, null);
     }
-
-    /**
-     * send message to all cluster members same cluster domain
-     * 
-     * @see org.apache.catalina.ha.CatalinaCluster#send(org.apache.catalina.ha.ClusterMessage)
-     */
-    public void sendClusterDomain(ClusterMessage msg) {
-        send(msg,null);
-    } 
-
     
     /**
      * send a cluster message to one member
@@ -804,8 +828,9 @@ public class SimpleTcpCluster
      * @param msg message to transfer
      * @param dest Receiver member
      * @see org.apache.catalina.ha.CatalinaCluster#send(org.apache.catalina.ha.ClusterMessage,
-     *      org.apache.catalina.ha.Member)
+     *      org.apache.catalina.tribes.Member)
      */
+    @Override
     public void send(ClusterMessage msg, Member dest) {
         try {
             msg.setAddress(getLocalMember());
@@ -820,8 +845,9 @@ public class SimpleTcpCluster
                 } else
                     log.error("Unable to send message to local member " + msg);
             } else {
-                if (channel.getMembers().length>0)
-                    channel.send(channel.getMembers(),msg, sendOptions);
+                Member[] destmembers = channel.getMembers();
+                if (destmembers.length>0)
+                    channel.send(destmembers,msg, sendOptions);
                 else if (log.isDebugEnabled()) 
                     log.debug("No members in cluster, ignoring message:"+msg);
             }
@@ -833,16 +859,20 @@ public class SimpleTcpCluster
     /**
      * New cluster member is registered
      * 
-     * @see org.apache.catalina.ha.MembershipListener#memberAdded(org.apache.catalina.ha.Member)
+     * @see org.apache.catalina.tribes.MembershipListener#memberAdded(org.apache.catalina.tribes.Member)
      */
+    @Override
     public void memberAdded(Member member) {
         try {
             hasMembers = channel.hasMembers();
             if (log.isInfoEnabled()) log.info("Replication member added:" + member);
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(BEFORE_MEMBERREGISTER_EVENT, member);
+            fireLifecycleEvent(BEFORE_MEMBERREGISTER_EVENT, member);
+
+            registerMember(member);
+
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(AFTER_MEMBERREGISTER_EVENT, member);
+            fireLifecycleEvent(AFTER_MEMBERREGISTER_EVENT, member);
         } catch (Exception x) {
             log.error("Unable to connect to replication system.", x);
         }
@@ -852,16 +882,20 @@ public class SimpleTcpCluster
     /**
      * Cluster member is gone
      * 
-     * @see org.apache.catalina.ha.MembershipListener#memberDisappeared(org.apache.catalina.ha.Member)
+     * @see org.apache.catalina.tribes.MembershipListener#memberDisappeared(org.apache.catalina.tribes.Member)
      */
+    @Override
     public void memberDisappeared(Member member) {
         try {
             hasMembers = channel.hasMembers();            
             if (log.isInfoEnabled()) log.info("Received member disappeared:" + member);
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(BEFORE_MEMBERUNREGISTER_EVENT, member);
+            fireLifecycleEvent(BEFORE_MEMBERUNREGISTER_EVENT, member);
+
+            unregisterMember(member);
+
             // Notify our interested LifecycleListeners
-            lifecycle.fireLifecycleEvent(AFTER_MEMBERUNREGISTER_EVENT, member);
+            fireLifecycleEvent(AFTER_MEMBERUNREGISTER_EVENT, member);
         } catch (Exception x) {
             log.error("Unable remove cluster node from replication system.", x);
         }
@@ -872,16 +906,18 @@ public class SimpleTcpCluster
 
     /**
      * notify all listeners from receiving a new message is not ClusterMessage
-     * emitt Failure Event to LifecylceListener
+     * emit Failure Event to LifecylceListener
      * 
-     * @param message
-     *            receveived Message
+     * @param msg
+     *            received Message
      */
+    @Override
     public boolean accept(Serializable msg, Member sender) {
         return (msg instanceof ClusterMessage);
     }
     
     
+    @Override
     public void messageReceived(Serializable message, Member sender) {
         ClusterMessage fwd = (ClusterMessage)message;
         fwd.setAddress(sender);
@@ -890,7 +926,6 @@ public class SimpleTcpCluster
 
     public void messageReceived(ClusterMessage message) {
 
-        long start = 0;
         if (log.isDebugEnabled() && message != null)
             log.debug("Assuming clocks are synched: Replication for "
                     + message.getUniqueId() + " took="
@@ -900,49 +935,52 @@ public class SimpleTcpCluster
         //invoke all the listeners
         boolean accepted = false;
         if (message != null) {
-            for (Iterator iter = clusterListeners.iterator(); iter.hasNext();) {
-                ClusterListener listener = (ClusterListener) iter.next();
+            for (Iterator<ClusterListener> iter = clusterListeners.iterator();
+                    iter.hasNext();) {
+                ClusterListener listener = iter.next();
                 if (listener.accept(message)) {
                     accepted = true;
                     listener.messageReceived(message);
                 }
             }
-        }
-        if (!accepted && log.isDebugEnabled()) {
-            if (notifyLifecycleListenerOnFailure) {
+            if (!accepted && notifyLifecycleListenerOnFailure) {
                 Member dest = message.getAddress();
                 // Notify our interested LifecycleListeners
-                lifecycle.fireLifecycleEvent(RECEIVE_MESSAGE_FAILURE_EVENT,
+                fireLifecycleEvent(RECEIVE_MESSAGE_FAILURE_EVENT,
                         new SendMessageData(message, dest, null));
+                if (log.isDebugEnabled()) {
+                    log.debug("Message " + message.toString() + " from type "
+                            + message.getClass().getName()
+                            + " transfered but no listener registered");
+                }
             }
-            log.debug("Message " + message.toString() + " from type "
-                    + message.getClass().getName()
-                    + " transfered but no listener registered");
         }
         return;
     }
 
     // --------------------------------------------------------- Logger
 
+    @Override
     public Log getLogger() {
         return log;
     }
 
 
-
-    
     // ------------------------------------------------------------- deprecated
 
     /**
      * 
      * @see org.apache.catalina.Cluster#setProtocol(java.lang.String)
      */
+    @Override
     public void setProtocol(String protocol) {
+        // NO-OP
     }
 
     /**
      * @see org.apache.catalina.Cluster#getProtocol()
      */
+    @Override
     public String getProtocol() {
         return null;
     }
@@ -954,4 +992,53 @@ public class SimpleTcpCluster
     public void setChannelStartOptions(int channelStartOptions) {
         this.channelStartOptions = channelStartOptions;
     }
+
+
+    // --------------------------------------------------------------------- JMX
+
+    @SuppressWarnings("deprecation")
+    @Override
+    protected String getDomainInternal() {
+        Container container = getContainer();
+        if (container == null) {
+            return null;
+        }
+        return MBeanUtils.getDomain(container);
+    }
+
+    @SuppressWarnings("deprecation")
+    @Override
+    protected String getObjectNameKeyProperties() {
+        StringBuilder name = new StringBuilder("type=Cluster");
+
+        Container container = getContainer();
+        if (container != null) {
+            name.append(MBeanUtils.getContainerKeyProperties(container));
+        }
+
+        return name.toString();
+    }
+
+    @SuppressWarnings("deprecation")
+    private void registerMember(Member member) {
+        // JMX registration
+        StringBuilder name = new StringBuilder("type=Cluster");
+        Container container = getContainer();
+        if (container != null) {
+            name.append(MBeanUtils.getContainerKeyProperties(container));
+        }
+        name.append(",component=Member,name=");
+        name.append(ObjectName.quote(member.getName()));
+
+        ObjectName oname = register(member, name.toString());
+        memberOnameMap.put(member, oname);
+    }
+
+    private void unregisterMember(Member member) {
+        if (member == null) return;
+        ObjectName oname = memberOnameMap.remove(member);
+        if (oname != null) {
+            unregister(oname);
+        }
+    }
 }
