@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -79,7 +79,6 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: WebappLoader.java 1430536 2013-01-08 21:07:06Z kkolinko $
  */
 
 public class WebappLoader extends LifecycleMBeanBase
@@ -148,7 +147,7 @@ public class WebappLoader extends LifecycleMBeanBase
 
     /**
      * The Java class name of the ClassLoader implementation to be used.
-     * This class should extend WebappClassLoader, otherwise, a different 
+     * This class should extend WebappClassLoader, otherwise, a different
      * loader implementation must be used.
      */
     private String loaderClass =
@@ -440,7 +439,7 @@ public class WebappLoader extends LifecycleMBeanBase
     /**
      * Return the set of repositories defined for this class loader.
      * If none are defined, a zero-length array is returned.
-     * For security reason, returns a clone of the Array (since 
+     * For security reason, returns a clone of the Array (since
      * String are immutable).
      */
     @Override
@@ -481,7 +480,7 @@ public class WebappLoader extends LifecycleMBeanBase
     }
 
 
-    /** 
+    /**
      * Classpath, as set in org.apache.catalina.jsp_classpath context
      * property
      *
@@ -558,7 +557,7 @@ public class WebappLoader extends LifecycleMBeanBase
             setState(LifecycleState.STARTING);
             return;
         }
-        
+
         // Register a stream handler factory for the JNDI protocol
         URLStreamHandlerFactory streamHandlerFactory =
                 DirContextURLStreamHandlerFactory.getInstance();
@@ -572,7 +571,7 @@ public class WebappLoader extends LifecycleMBeanBase
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
                 // This is likely a dual registration
-                log.info("Dual registration of jndi stream handler: " 
+                log.info("Dual registration of jndi stream handler: "
                          + t.getMessage());
             }
         }
@@ -617,7 +616,7 @@ public class WebappLoader extends LifecycleMBeanBase
             String contextName = ctx.getName();
             if (!contextName.startsWith("/")) {
                 contextName = "/" + contextName;
-            }   
+            }
             ObjectName cloname = new ObjectName
                 (MBeanUtils.getDomain(ctx) + ":type=WebappClassLoader,context="
                  + contextName + ",host=" + ctx.getParent().getName());
@@ -668,7 +667,7 @@ public class WebappLoader extends LifecycleMBeanBase
             String contextName = ctx.getName();
             if (!contextName.startsWith("/")) {
                 contextName = "/" + contextName;
-            }   
+            }
             ObjectName cloname = new ObjectName
                 (MBeanUtils.getDomain(ctx) + ":type=WebappClassLoader,context="
                  + contextName + ",host=" + ctx.getParent().getName());
@@ -758,7 +757,7 @@ public class WebappLoader extends LifecycleMBeanBase
                 classLoader.addPermission
                     (new FilePermission(workDirPath, "read,write"));
                 classLoader.addPermission
-                    (new FilePermission(workDirPath + File.separator + "-", 
+                    (new FilePermission(workDirPath + File.separator + "-",
                                         "read,write,delete"));
             } catch (IOException e) {
                 // Ignore
@@ -833,7 +832,7 @@ public class WebappLoader extends LifecycleMBeanBase
     /**
      * Configure the repositories for our class loader, based on the
      * associated Context.
-     * @throws IOException 
+     * @throws IOException
      */
     private void setRepositories() throws IOException {
 
@@ -852,7 +851,7 @@ public class WebappLoader extends LifecycleMBeanBase
             log.info("No work dir for " + servletContext);
         }
 
-        if( log.isDebugEnabled() && workDir != null) 
+        if( log.isDebugEnabled() && workDir != null)
             log.debug(sm.getString("webappLoader.deploy", workDir.getAbsolutePath()));
 
         classLoader.setWorkDir(workDir);
@@ -984,12 +983,12 @@ public class WebappLoader extends LifecycleMBeanBase
                     ioe.initCause(e);
                     throw ioe;
                 }
-                
+
                 if (!(obj instanceof Resource))
                     continue;
-                    
+
                 Resource jarResource = (Resource) obj;
-                    
+
                 if (copyJars) {
                     if (!copy(jarResource.streamContent(),
                               new FileOutputStream(destFile))) {
@@ -1003,10 +1002,10 @@ public class WebappLoader extends LifecycleMBeanBase
                     classLoader.addJar(filename, jarFile, destFile);
                 } catch (Exception ex) {
                     // Catch the exception if there is an empty jar file
-                    // Should ignore and continue loading other jar files 
+                    // Should ignore and continue loading other jar files
                     // in the dir
                 }
-                    
+
                 loaderRepositories.add( filename );
             }
         }
@@ -1028,7 +1027,7 @@ public class WebappLoader extends LifecycleMBeanBase
             return;
 
         if (container instanceof StandardContext) {
-            String baseClasspath = 
+            String baseClasspath =
                 ((StandardContext) container).getCompilerClasspath();
             if (baseClasspath != null) {
                 servletContext.setAttribute(Globals.CLASS_PATH_ATTR,
@@ -1041,44 +1040,27 @@ public class WebappLoader extends LifecycleMBeanBase
 
         // Assemble the class path information from our class loader chain
         ClassLoader loader = getClassLoader();
-        int n = 0;
+
+        if (delegate && loader != null) {
+            // Skip the webapp loader for now as delegation is enabled
+            loader = loader.getParent();
+        }
+
         while (loader != null) {
-            if (!(loader instanceof URLClassLoader)) {
-                String cp=getClasspath( loader );
-                if( cp==null ) {
-                    log.info( "Unknown loader " + loader + " " + loader.getClass());
-                } else {
-                    if (n > 0) 
-                        classpath.append(File.pathSeparator);
-                    classpath.append(cp);
-                    n++;
-                }
+            if (!buildClassPath(servletContext, classpath, loader)) {
                 break;
-                //continue;
-            }
-            URL repositories[] =
-                ((URLClassLoader) loader).getURLs();
-            for (int i = 0; i < repositories.length; i++) {
-                String repository = repositories[i].toString();
-                if (repository.startsWith("file://"))
-                    repository = utf8Decode(repository.substring(7));
-                else if (repository.startsWith("file:"))
-                    repository = utf8Decode(repository.substring(5));
-                else if (repository.startsWith("jndi:"))
-                    repository =
-                        servletContext.getRealPath(repository.substring(5));
-                else
-                    continue;
-                if (repository == null)
-                    continue;
-                if (n > 0)
-                    classpath.append(File.pathSeparator);
-                classpath.append(repository);
-                n++;
             }
             loader = loader.getParent();
         }
 
+        if (delegate) {
+            // Delegation was enabled, go back and add the webapp paths
+            loader = getClassLoader();
+            if (loader != null) {
+                buildClassPath(servletContext, classpath, loader);
+            }
+        }
+
         this.classpath=classpath.toString();
 
         // Store the assembled class path as a servlet context attribute
@@ -1087,6 +1069,43 @@ public class WebappLoader extends LifecycleMBeanBase
 
     }
 
+
+    private boolean buildClassPath(ServletContext servletContext,
+            StringBuilder classpath, ClassLoader loader) {
+        if (loader instanceof URLClassLoader) {
+            URL repositories[] =
+                    ((URLClassLoader) loader).getURLs();
+                for (int i = 0; i < repositories.length; i++) {
+                    String repository = repositories[i].toString();
+                    if (repository.startsWith("file://"))
+                        repository = utf8Decode(repository.substring(7));
+                    else if (repository.startsWith("file:"))
+                        repository = utf8Decode(repository.substring(5));
+                    else if (repository.startsWith("jndi:"))
+                        repository =
+                            servletContext.getRealPath(repository.substring(5));
+                    else
+                        continue;
+                    if (repository == null)
+                        continue;
+                    if (classpath.length() > 0)
+                        classpath.append(File.pathSeparator);
+                    classpath.append(repository);
+                }
+        } else {
+            String cp = getClasspath(loader);
+            if (cp == null) {
+                log.info( "Unknown loader " + loader + " " + loader.getClass());
+            } else {
+                if (classpath.length() > 0)
+                    classpath.append(File.pathSeparator);
+                classpath.append(cp);
+            }
+            return false;
+        }
+        return true;
+    }
+
     private String utf8Decode(String input) {
         String result = null;
         try {
@@ -1197,19 +1216,19 @@ public class WebappLoader extends LifecycleMBeanBase
 
     @Override
     protected String getObjectNameKeyProperties() {
-        
+
         StringBuilder name = new StringBuilder("type=Loader");
-        
+
         if (container instanceof Context) {
             name.append(",context=");
             Context context = (Context) container;
-            
+
             String contextName = context.getName();
             if (!contextName.startsWith("/")) {
                 name.append("/");
-            }   
+            }
             name.append(contextName);
-            
+
             name.append(",host=");
             name.append(context.getParent().getName());
         } else {
@@ -1217,7 +1236,7 @@ public class WebappLoader extends LifecycleMBeanBase
             name.append(",container=");
             name.append(container.getName());
         }
-        
+
         return name.toString();
     }
 
