@@ -148,9 +148,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: ManagerServlet.java 1162172 2011-08-26 17:12:33Z markt $
  */
-
 public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
     private static final long serialVersionUID = 1L;
@@ -193,7 +191,9 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
     /**
      * Path used to store context descriptors.
+     * @deprecated  Unused
      */
+    @Deprecated
     protected File contextDescriptors = null;
 
 
@@ -205,10 +205,12 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
     
     /**
      * The host appBase.
+     * @deprecated  Unused
      */
+    @Deprecated
     protected File appBase = null;
-    
-    
+
+
     /**
      * MBean server.
      */
@@ -315,7 +317,8 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                       HttpServletResponse response)
         throws IOException, ServletException {
 
-        StringManager smClient = getStringManager(request);
+        StringManager smClient = StringManager.getManager(
+                Constants.Package, request.getLocales());
 
         // Identify the request parameters that we need
         String command = request.getPathInfo();
@@ -402,7 +405,8 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                       HttpServletResponse response)
         throws IOException, ServletException {
 
-        StringManager smClient = getStringManager(request);
+        StringManager smClient = StringManager.getManager(
+                Constants.Package, request.getLocales());
 
         // Identify the request parameters that we need
         String command = request.getPathInfo();
@@ -859,10 +863,11 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                 }
             }
             context = (Context) host.findChild(name);
-            if (context != null && context.getConfigured() && context.getAvailable()) {
+            if (context != null && context.getConfigured() &&
+                    context.getState().isAvailable()) {
                 writer.println(smClient.getString(
                         "managerServlet.deployed", displayPath));
-            } else if (context!=null && !context.getAvailable()) {
+            } else if (context!=null && !context.getState().isAvailable()) {
                 writer.println(smClient.getString(
                         "managerServlet.deployedButNotStarted", displayPath));
             } else {
@@ -900,7 +905,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                 String displayPath = context.getPath();
                 if( displayPath.equals("") )
                     displayPath = "/";
-                if (context.getAvailable()) {
+                if (context.getState().isAvailable()) {
                     writer.println(smClient.getString("managerServlet.listitem",
                             displayPath,
                             "running",
@@ -1191,7 +1196,11 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
      *
      * @param writer Writer to render to
      * @param cn Name of the application to list session information for
+     *
+     * @deprecated  Use {@link #sessions(PrintWriter, ContextName, int,
+     *              StringManager)}
      */
+    @Deprecated
     protected void sessions(PrintWriter writer, ContextName cn,
             StringManager smClient) {
         sessions(writer, cn, -1, smClient);
@@ -1245,7 +1254,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                 return;
             }
             context.start();
-            if (context.getAvailable())
+            if (context.getState().isAvailable())
                 writer.println(smClient.getString("managerServlet.started",
                         displayPath));
             else
@@ -1566,6 +1575,11 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
     }
 
 
+    /**
+     * @deprecated Use {@link StringManager#getManager(String, Enumeration)}.
+     *             This method will be removed in Tomcat 8.
+     */
+    @Deprecated
     protected StringManager getStringManager(HttpServletRequest req) {
         Enumeration<Locale> requestedLocales = req.getLocales();
         while (requestedLocales.hasMoreElements()) {
