@@ -35,6 +35,7 @@ import org.apache.catalina.Container;
 import org.apache.catalina.Host;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
 * Servlet that enables remote management of the virtual hosts deployed
@@ -55,12 +56,12 @@ import org.apache.catalina.util.ServerInfo;
 * @author Malcolm Edgar
 * @author Glenn L. Nielsen
 * @author Peter Rossbach
-* @version $Id: HTMLHostManagerServlet.java 1030547 2010-11-03 17:17:59Z markt $
-* @see ManagerServlet
+* @see org.apache.catalina.manager.ManagerServlet
 */
-
 public final class HTMLHostManagerServlet extends HostManagerServlet {
 
+    private static final long serialVersionUID = 1L;
+
     // --------------------------------------------------------- Public Methods
 
     /**
@@ -72,10 +73,55 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     public void doGet(HttpServletRequest request,
                       HttpServletResponse response)
         throws IOException, ServletException {
 
+        StringManager smClient = StringManager.getManager(
+                Constants.Package, request.getLocales());
+
+        // Identify the request parameters that we need
+        String command = request.getPathInfo();
+
+        // Prepare our output writer to generate the response message
+        response.setContentType("text/html; charset=" + Constants.CHARSET);
+
+        String message = "";
+        // Process the requested command
+        if (command == null) {
+            // No command == list
+        } else if (command.equals("/list")) {
+            // Nothing to do - always generate list
+        } else if (command.equals("/add") || command.equals("/remove") ||
+                command.equals("/start") || command.equals("/stop")) {
+            message = smClient.getString(
+                    "hostManagerServlet.postCommand", command);
+        } else {
+            message = smClient.getString(
+                    "hostManagerServlet.unknownCommand", command);
+        }
+
+        list(request, response, message, smClient);
+    }
+
+    
+    /**
+     * Process a POST request for the specified resource.
+     *
+     * @param request The servlet request we are processing
+     * @param response The servlet response we are creating
+     *
+     * @exception IOException if an input/output error occurs
+     * @exception ServletException if a servlet-specified error occurs
+     */
+    @Override
+    public void doPost(HttpServletRequest request, HttpServletResponse response)
+            throws ServletException, IOException {
+
+        StringManager smClient = StringManager.getManager(
+                Constants.Package, request.getLocales());
+
         // Identify the request parameters that we need
         String command = request.getPathInfo();
 
@@ -85,44 +131,39 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
         response.setContentType("text/html; charset=" + Constants.CHARSET);
 
         String message = "";
+        
         // Process the requested command
         if (command == null) {
+            // No command == list
         } else if (command.equals("/add")) {
-            message = add(request, name);
+            message = add(request, name, smClient);
         } else if (command.equals("/remove")) {
-            message = remove(name);
-        } else if (command.equals("/list")) {
+            message = remove(name, smClient);
         } else if (command.equals("/start")) {
-            message = start(name);
+            message = start(name, smClient);
         } else if (command.equals("/stop")) {
-            message = stop(name);
+            message = stop(name, smClient);
         } else {
-            message =
-                sm.getString("hostManagerServlet.unknownCommand", command);
+            //Try GET
+            doGet(request, response);
         }
 
-        list(request, response, message);
+        list(request, response, message, smClient);
     }
 
-    
-    @Override
-    public void doPost(HttpServletRequest request, HttpServletResponse response)
-            throws IOException, ServletException {
-        doGet(request, response);
-    }
-    
-    
+
     /**
      * Add a host using the specified parameters.
      *
      * @param name host name
      */
-    protected String add(HttpServletRequest request,String name) {
+    protected String add(HttpServletRequest request,String name,
+            StringManager smClient) {
 
         StringWriter stringWriter = new StringWriter();
         PrintWriter printWriter = new PrintWriter(stringWriter);
 
-        super.add(request,printWriter,name,true);
+        super.add(request,printWriter,name,true, smClient);
 
         return stringWriter.toString();
     }
@@ -131,15 +172,14 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
     /**
      * Remove the specified host.
      *
-     * @param writer Writer to render results to
      * @param name host name
      */
-    protected String remove(String name) {
+    protected String remove(String name, StringManager smClient) {
 
         StringWriter stringWriter = new StringWriter();
         PrintWriter printWriter = new PrintWriter(stringWriter);
 
-        super.remove(printWriter, name);
+        super.remove(printWriter, name, smClient);
 
         return stringWriter.toString();
     }
@@ -150,12 +190,12 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
      *
      * @param name Host name
      */
-    protected String start(String name) {
+    protected String start(String name, StringManager smClient) {
 
         StringWriter stringWriter = new StringWriter();
         PrintWriter printWriter = new PrintWriter(stringWriter);
 
-        super.start(printWriter, name);
+        super.start(printWriter, name, smClient);
 
         return stringWriter.toString();
     }
@@ -166,12 +206,12 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
      *
      * @param name Host name
      */
-    protected String stop(String name) {
+    protected String stop(String name, StringManager smClient) {
 
         StringWriter stringWriter = new StringWriter();
         PrintWriter printWriter = new PrintWriter(stringWriter);
 
-        super.stop(printWriter, name);
+        super.stop(printWriter, name, smClient);
 
         return stringWriter.toString();
     }
@@ -187,23 +227,28 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
      */
     public void list(HttpServletRequest request,
                      HttpServletResponse response,
-                     String message) throws IOException {
+                     String message,
+                     StringManager smClient) throws IOException {
+
+        if (debug >= 1) {
+            log(sm.getString("hostManagerServlet.list", engine.getName()));
+        }
 
         PrintWriter writer = response.getWriter();
 
         // HTML Header Section
-        writer.print(Constants.HTML_HEADER_SECTION);
+        writer.print(org.apache.catalina.manager.Constants.HTML_HEADER_SECTION);
 
         // Body Header Section
         Object[] args = new Object[2];
         args[0] = request.getContextPath();
-        args[1] = sm.getString("htmlHostManagerServlet.title");
+        args[1] = smClient.getString("htmlHostManagerServlet.title");
         writer.print(MessageFormat.format
                      (Constants.BODY_HEADER_SECTION, args));
 
         // Message Section
         args = new Object[3];
-        args[0] = sm.getString("htmlHostManagerServlet.messageLabel");
+        args[0] = smClient.getString("htmlHostManagerServlet.messageLabel");
         if (message == null || message.length() == 0) {
             args[1] = "OK";
         } else {
@@ -213,26 +258,26 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
 
         // Manager Section
         args = new Object[9];
-        args[0] = sm.getString("htmlHostManagerServlet.manager");
+        args[0] = smClient.getString("htmlHostManagerServlet.manager");
         args[1] = response.encodeURL(request.getContextPath() + "/html/list");
-        args[2] = sm.getString("htmlHostManagerServlet.list");
+        args[2] = smClient.getString("htmlHostManagerServlet.list");
         args[3] = response.encodeURL
             (request.getContextPath() + "/" +
-             sm.getString("htmlHostManagerServlet.helpHtmlManagerFile"));
-        args[4] = sm.getString("htmlHostManagerServlet.helpHtmlManager");
+             smClient.getString("htmlHostManagerServlet.helpHtmlManagerFile"));
+        args[4] = smClient.getString("htmlHostManagerServlet.helpHtmlManager");
         args[5] = response.encodeURL
             (request.getContextPath() + "/" +
-             sm.getString("htmlHostManagerServlet.helpManagerFile"));
-        args[6] = sm.getString("htmlHostManagerServlet.helpManager");
+             smClient.getString("htmlHostManagerServlet.helpManagerFile"));
+        args[6] = smClient.getString("htmlHostManagerServlet.helpManager");
         args[7] = response.encodeURL("/manager/status");
-        args[8] = sm.getString("statusServlet.title");
+        args[8] = smClient.getString("statusServlet.title");
         writer.print(MessageFormat.format(Constants.MANAGER_SECTION, args));
 
          // Hosts Header Section
         args = new Object[3];
-        args[0] = sm.getString("htmlHostManagerServlet.hostName");
-        args[1] = sm.getString("htmlHostManagerServlet.hostAliases");
-        args[2] = sm.getString("htmlHostManagerServlet.hostTasks");
+        args[0] = smClient.getString("htmlHostManagerServlet.hostName");
+        args[1] = smClient.getString("htmlHostManagerServlet.hostAliases");
+        args[2] = smClient.getString("htmlHostManagerServlet.hostTasks");
         writer.print(MessageFormat.format(HOSTS_HEADER_SECTION, args));
 
         // Hosts Row Section
@@ -250,22 +295,25 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
             sortedHostNamesMap.put(displayPath, hostNames[i]);
         }
 
-        String hostsStart = sm.getString("htmlHostManagerServlet.hostsStart");
-        String hostsStop = sm.getString("htmlHostManagerServlet.hostsStop");
-        String hostsRemove = sm.getString("htmlHostManagerServlet.hostsRemove");
+        String hostsStart =
+            smClient.getString("htmlHostManagerServlet.hostsStart");
+        String hostsStop =
+            smClient.getString("htmlHostManagerServlet.hostsStop");
+        String hostsRemove =
+            smClient.getString("htmlHostManagerServlet.hostsRemove");
 
         Iterator<Map.Entry<String,String>> iterator =
             sortedHostNamesMap.entrySet().iterator();
         while (iterator.hasNext()) {
             Map.Entry<String,String> entry = iterator.next();
-            String hostName = (String) entry.getKey();
+            String hostName = entry.getKey();
             Host host = (Host) engine.findChild(hostName);
 
             if (host != null ) {
                 args = new Object[2];
                 args[0] = RequestUtil.filter(hostName);
                 String[] aliases = host.findAliases();
-                StringBuffer buf = new StringBuffer();
+                StringBuilder buf = new StringBuilder();
                 if (aliases.length > 0) {
                     buf.append(aliases[0]);
                     for (int j = 1; j < aliases.length; j++) {
@@ -283,88 +331,82 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
                 writer.print
                     (MessageFormat.format(HOSTS_ROW_DETAILS_SECTION, args));
 
-                args = new Object[7];
-                args[0] = response.encodeURL
-                    (request.getContextPath() +
-                     "/html/start?name=" +
-                     URLEncoder.encode(hostName, "UTF-8"));
-                args[1] = hostsStart;
-                args[2] = response.encodeURL
+                args = new Object[4];
+                if (host.getState().isAvailable()) {
+                    args[0] = response.encodeURL
                     (request.getContextPath() +
                      "/html/stop?name=" +
                      URLEncoder.encode(hostName, "UTF-8"));
-                args[3] = hostsStop;
-                args[4] = response.encodeURL
+                    args[1] = hostsStop;
+                } else {
+                    args[0] = response.encodeURL
+                        (request.getContextPath() +
+                         "/html/start?name=" +
+                         URLEncoder.encode(hostName, "UTF-8"));
+                    args[1] = hostsStart;
+                }
+                args[2] = response.encodeURL
                     (request.getContextPath() +
                      "/html/remove?name=" +
                      URLEncoder.encode(hostName, "UTF-8"));
-                args[5] = hostsRemove;
-                args[6] = RequestUtil.filter(hostName);
-                if (host == this.host) {
+                args[3] = hostsRemove;
+                if (host == this.installedHost) {
                     writer.print(MessageFormat.format(
                         MANAGER_HOST_ROW_BUTTON_SECTION, args));
                 } else {
                     writer.print(MessageFormat.format(
                         HOSTS_ROW_BUTTON_SECTION, args));
                 }
-
             }
         }
 
         // Add Section
         args = new Object[6];
-        args[0] = sm.getString("htmlHostManagerServlet.addTitle");
-        args[1] = sm.getString("htmlHostManagerServlet.addHost");
+        args[0] = smClient.getString("htmlHostManagerServlet.addTitle");
+        args[1] = smClient.getString("htmlHostManagerServlet.addHost");
         args[2] = response.encodeURL(request.getContextPath() + "/html/add");
-        args[3] = sm.getString("htmlHostManagerServlet.addName");
-        args[4] = sm.getString("htmlHostManagerServlet.addAliases");
-        args[5] = sm.getString("htmlHostManagerServlet.addAppBase");
+        args[3] = smClient.getString("htmlHostManagerServlet.addName");
+        args[4] = smClient.getString("htmlHostManagerServlet.addAliases");
+        args[5] = smClient.getString("htmlHostManagerServlet.addAppBase");
         writer.print(MessageFormat.format(ADD_SECTION_START, args));
  
         args = new Object[3];
-        args[0] = sm.getString("htmlHostManagerServlet.addAutoDeploy");
+        args[0] = smClient.getString("htmlHostManagerServlet.addAutoDeploy");
         args[1] = "autoDeploy";
         args[2] = "checked";
         writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
-        args[0] = sm.getString("htmlHostManagerServlet.addDeployOnStartup");
+        args[0] = smClient.getString(
+                "htmlHostManagerServlet.addDeployOnStartup");
         args[1] = "deployOnStartup";
         args[2] = "checked";
         writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
-        args[0] = sm.getString("htmlHostManagerServlet.addDeployXML");
+        args[0] = smClient.getString("htmlHostManagerServlet.addDeployXML");
         args[1] = "deployXML";
         args[2] = "checked";
         writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
-        args[0] = sm.getString("htmlHostManagerServlet.addUnpackWARs");
+        args[0] = smClient.getString("htmlHostManagerServlet.addUnpackWARs");
         args[1] = "unpackWARs";
         args[2] = "checked";
         writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
-        args[0] = sm.getString("htmlHostManagerServlet.addXmlNamespaceAware");
-        args[1] = "xmlNamespaceAware";
-        args[2] = "";
-        writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
-        args[0] = sm.getString("htmlHostManagerServlet.addXmlValidation");
-        args[1] = "xmlValidation";
-        args[2] = "";
-        writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
 
-        args[0] = sm.getString("htmlHostManagerServlet.addManager");
+        args[0] = smClient.getString("htmlHostManagerServlet.addManager");
         args[1] = "manager";
         args[2] = "checked";
         writer.print(MessageFormat.format(ADD_SECTION_BOOLEAN, args));
         
         args = new Object[1];
-        args[0] = sm.getString("htmlHostManagerServlet.addButton");
+        args[0] = smClient.getString("htmlHostManagerServlet.addButton");
         writer.print(MessageFormat.format(ADD_SECTION_END, args));
 
         // Server Header Section
         args = new Object[7];
-        args[0] = sm.getString("htmlHostManagerServlet.serverTitle");
-        args[1] = sm.getString("htmlHostManagerServlet.serverVersion");
-        args[2] = sm.getString("htmlHostManagerServlet.serverJVMVersion");
-        args[3] = sm.getString("htmlHostManagerServlet.serverJVMVendor");
-        args[4] = sm.getString("htmlHostManagerServlet.serverOSName");
-        args[5] = sm.getString("htmlHostManagerServlet.serverOSVersion");
-        args[6] = sm.getString("htmlHostManagerServlet.serverOSArch");
+        args[0] = smClient.getString("htmlHostManagerServlet.serverTitle");
+        args[1] = smClient.getString("htmlHostManagerServlet.serverVersion");
+        args[2] = smClient.getString("htmlHostManagerServlet.serverJVMVersion");
+        args[3] = smClient.getString("htmlHostManagerServlet.serverJVMVendor");
+        args[4] = smClient.getString("htmlHostManagerServlet.serverOSName");
+        args[5] = smClient.getString("htmlHostManagerServlet.serverOSVersion");
+        args[6] = smClient.getString("htmlHostManagerServlet.serverOSArch");
         writer.print(MessageFormat.format
                      (Constants.SERVER_HEADER_SECTION, args));
 
@@ -390,8 +432,8 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
     // ------------------------------------------------------ Private Constants
 
     // These HTML sections are broken in relatively small sections, because of
-    // limited number of subsitutions MessageFormat can process
-    // (maximium of 10).
+    // limited number of substitutions MessageFormat can process
+    // (maximum of 10).
 
     private static final String HOSTS_HEADER_SECTION =
         "<table border=\"1\" cellspacing=\"0\" cellpadding=\"3\">\n" +
@@ -413,20 +455,19 @@ public final class HTMLHostManagerServlet extends HostManagerServlet {
     private static final String MANAGER_HOST_ROW_BUTTON_SECTION =
         " <td class=\"row-left\">\n" +
         "  <small>\n" +
-        "  &nbsp;{1}&nbsp;\n" +
-        "  &nbsp;{3}&nbsp;\n" +
-        "  &nbsp;{5}&nbsp;\n" +
+        sm.getString("htmlHostManagerServlet.hostThis") +
         "  </small>\n" +
         " </td>\n" +
         "</tr>\n";
 
     private static final String HOSTS_ROW_BUTTON_SECTION =
         " <td class=\"row-left\" NOWRAP>\n" +
-        "  <small>\n" +
-        "  &nbsp;<a href=\"{0}\" onclick=\"return(confirm(''{1} {6}\\n\\nAre you sure?''))\">{1}</a>&nbsp;\n" +
-        "  &nbsp;<a href=\"{2}\" onclick=\"return(confirm(''{3} {6}\\n\\nAre you sure?''))\">{3}</a>&nbsp;\n" +
-        "  &nbsp;<a href=\"{4}\" onclick=\"return(confirm(''{5} {6}\\n\\nAre you sure?''))\">{5}</a>&nbsp;\n" +
-        "  </small>\n" +
+        "  <form class=\"inline\" method=\"POST\" action=\"{0}\">" +
+        "   <small><input type=\"submit\" value=\"{1}\"></small>" +
+        "  </form>\n" +
+        "  <form class=\"inline\" method=\"POST\" action=\"{2}\">" +
+        "   <small><input type=\"submit\" value=\"{3}\"></small>" +
+        "  </form>\n" +
         " </td>\n" +
         "</tr>\n";
 
