@@ -25,6 +25,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PrintWriter;
+import java.util.Enumeration;
+import java.util.Locale;
 import java.util.StringTokenizer;
 
 import javax.management.MBeanServer;
@@ -40,13 +42,13 @@ import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.Wrapper;
+import org.apache.catalina.core.ContainerBase;
 import org.apache.catalina.core.StandardHost;
 import org.apache.catalina.startup.HostConfig;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.modeler.Registry;
-import org.apache.catalina.core.ContainerBase;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -81,11 +83,6 @@ import org.apache.catalina.core.ContainerBase;
  * this servlet itself will not succeed.  Therefore, this servlet should
  * generally be deployed in a separate virtual host.
  * <p>
- * <b>NOTE</b> - For security reasons, this application will not operate
- * when accessed via the invoker servlet.  You must explicitly map this servlet
- * with a servlet mapping, and you will always want to protect it with
- * appropriate security constraints as well.
- * <p>
  * The following servlet initialization parameters are recognized:
  * <ul>
  * <li><b>debug</b> - The debugging detail level that controls the amount
@@ -94,26 +91,19 @@ import org.apache.catalina.core.ContainerBase;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: HostManagerServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
  */
-
 public class HostManagerServlet
     extends HttpServlet implements ContainerServlet {
 
+    private static final long serialVersionUID = 1L;
 
     // ----------------------------------------------------- Instance Variables
 
 
-    /**
-     * Path where context descriptors should be deployed.
-     */
-    protected File configBase = null;
-
-
     /**
      * The Context container associated with our web application.
      */
-    protected Context context = null;
+    protected transient Context context = null;
 
 
     /**
@@ -125,32 +115,32 @@ public class HostManagerServlet
     /**
      * The associated host.
      */
-    protected Host host = null;
+    protected transient Host installedHost = null;
 
     
     /**
      * The associated engine.
      */
-    protected Engine engine = null;
+    protected transient Engine engine = null;
 
     
     /**
      * MBean server.
      */
-    protected MBeanServer mBeanServer = null;
+    protected transient MBeanServer mBeanServer = null;
 
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
     /**
      * The Wrapper container associated with this servlet.
      */
-    protected Wrapper wrapper = null;
+    protected transient Wrapper wrapper = null;
 
 
     // ----------------------------------------------- ContainerServlet Methods
@@ -159,6 +149,7 @@ public class HostManagerServlet
     /**
      * Return the Wrapper with which we are associated.
      */
+    @Override
     public Wrapper getWrapper() {
 
         return (this.wrapper);
@@ -171,17 +162,18 @@ public class HostManagerServlet
      *
      * @param wrapper The new wrapper
      */
+    @Override
     public void setWrapper(Wrapper wrapper) {
 
         this.wrapper = wrapper;
         if (wrapper == null) {
             context = null;
-            host = null;
+            installedHost = null;
             engine = null;
         } else {
             context = (Context) wrapper.getParent();
-            host = (Host) context.getParent();
-            engine = (Engine) host.getParent();
+            installedHost = (Host) context.getParent();
+            engine = (Engine) installedHost.getParent();
         }
 
         // Retrieve the MBean server
@@ -196,9 +188,10 @@ public class HostManagerServlet
     /**
      * Finalize this servlet.
      */
+    @Override
     public void destroy() {
 
-        ;       // No actions necessary
+        // No actions necessary
 
     }
 
@@ -212,14 +205,13 @@ public class HostManagerServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     public void doGet(HttpServletRequest request,
                       HttpServletResponse response)
         throws IOException, ServletException {
 
-        // Verify that we were not accessed using the invoker servlet
-        if (request.getAttribute(Globals.INVOKED_ATTR) != null)
-            throw new UnavailableException
-                (sm.getString("hostManagerServlet.cannotInvoke"));
+        StringManager smClient = StringManager.getManager(
+                Constants.Package, request.getLocales());
 
         // Identify the request parameters that we need
         String command = request.getPathInfo();
@@ -235,15 +227,15 @@ public class HostManagerServlet
         if (command == null) {
             writer.println(sm.getString("hostManagerServlet.noCommand"));
         } else if (command.equals("/add")) {
-            add(request, writer, name, false);
+            add(request, writer, name, false, smClient);
         } else if (command.equals("/remove")) {
-            remove(writer, name);
+            remove(writer, name, smClient);
         } else if (command.equals("/list")) {
-            list(writer);
+            list(writer, smClient);
         } else if (command.equals("/start")) {
-            start(writer, name);
+            start(writer, name, smClient);
         } else if (command.equals("/stop")) {
-            stop(writer, name);
+            stop(writer, name, smClient);
         } else {
             writer.println(sm.getString("hostManagerServlet.unknownCommand",
                                         command));
@@ -264,7 +256,8 @@ public class HostManagerServlet
      * @param name The host name
      * @param htmlMode Flag value
      */
-    protected void add(HttpServletRequest request, PrintWriter writer, String name, boolean htmlMode ) {
+    protected void add(HttpServletRequest request, PrintWriter writer,
+            String name, boolean htmlMode, StringManager smClient) {
         String aliases = request.getParameter("aliases");
         String appBase = request.getParameter("appBase");
         boolean manager = booleanParameter(request, "manager", false, htmlMode);
@@ -272,25 +265,21 @@ public class HostManagerServlet
         boolean deployOnStartup = booleanParameter(request, "deployOnStartup", true, htmlMode);
         boolean deployXML = booleanParameter(request, "deployXML", true, htmlMode);
         boolean unpackWARs = booleanParameter(request, "unpackWARs", true, htmlMode);
-        boolean xmlNamespaceAware = booleanParameter(request, "xmlNamespaceAware", false, htmlMode);
-        boolean xmlValidation = booleanParameter(request, "xmlValidation", false, htmlMode);
         add(writer, name, aliases, appBase, manager,
             autoDeploy,
             deployOnStartup,
             deployXML,                                       
             unpackWARs,
-            xmlNamespaceAware,
-            xmlValidation);
+            smClient);
     }
 
 
     /**
-     * extract boolean value from checkbox with default
+     * Extract boolean value from checkbox with default.
      * @param request
      * @param parameter
      * @param theDefault
      * @param htmlMode
-     * @return
      */
     protected boolean booleanParameter(HttpServletRequest request,
             String parameter, boolean theDefault, boolean htmlMode) {
@@ -317,6 +306,7 @@ public class HostManagerServlet
     /**
      * Initialize this servlet.
      */
+    @Override
     public void init() throws ServletException {
 
         // Ensure that our ContainerServlet properties have been set
@@ -324,21 +314,13 @@ public class HostManagerServlet
             throw new UnavailableException
                 (sm.getString("hostManagerServlet.noWrapper"));
 
-        // Verify that we were not accessed using the invoker servlet
-        String servletName = getServletConfig().getServletName();
-        if (servletName == null)
-            servletName = "";
-        if (servletName.startsWith("org.apache.catalina.INVOKER."))
-            throw new UnavailableException
-                (sm.getString("hostManagerServlet.cannotInvoke"));
-
         // Set our properties from the initialization parameters
         String value = null;
         try {
             value = getServletConfig().getInitParameter("debug");
             debug = Integer.parseInt(value);
         } catch (Throwable t) {
-            ;
+            ExceptionUtils.handleThrowable(t);
         }
 
     }
@@ -364,40 +346,45 @@ public class HostManagerServlet
          boolean deployOnStartup,
          boolean deployXML,                                       
          boolean unpackWARs,
-         boolean xmlNamespaceAware,
-         boolean xmlValidation) {
+         StringManager smClient) {
         if (debug >= 1) {
-            log("add: Adding host '" + name + "'");
+            log(sm.getString("hostManagerServlet.add", name));
         }
 
         // Validate the requested host name
         if ((name == null) || name.length() == 0) {
-            writer.println(sm.getString("hostManagerServlet.invalidHostName", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.invalidHostName", name));
             return;
         }
 
         // Check if host already exists
         if (engine.findChild(name) != null) {
-            writer.println
-                (sm.getString("hostManagerServlet.alreadyHost", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.alreadyHost", name));
             return;
         }
 
         // Validate and create appBase
         File appBaseFile = null;
+        File file = null;
         if (appBase == null || appBase.length() == 0) {
-            appBase = name;
+            file = new File(name);
+        } else {
+            file = new File(appBase);
         }
-        File file = new File(appBase);
         if (!file.isAbsolute())
-            file = new File(System.getProperty("catalina.base"), appBase);
+            file = new File(System.getProperty(Globals.CATALINA_BASE_PROP), file.getPath());
         try {
             appBaseFile = file.getCanonicalFile();
         } catch (IOException e) {
             appBaseFile = file;
         }
-        if (!appBaseFile.exists()) {
-            appBaseFile.mkdirs();
+        if (!appBaseFile.mkdirs() && !appBaseFile.isDirectory()) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.appBaseCreateFail",
+                    appBaseFile.toString(), name));
+            return;
         }
         
         // Create base for config files
@@ -405,6 +392,11 @@ public class HostManagerServlet
         
         // Copy manager.xml if requested
         if (manager) {
+            if (configBaseFile == null) {
+                writer.println(smClient.getString(
+                        "hostManagerServlet.configBaseCreateFail", name));
+                return;
+            }
             InputStream is = null;
             OutputStream os = null;
             try {
@@ -419,20 +411,22 @@ public class HostManagerServlet
                     os.write(buffer, 0, len);
                 }
             } catch (IOException e) {
-                writer.println
-                    (sm.getString("hostManagerServlet.managerXml"));
+                writer.println(smClient.getString(
+                        "hostManagerServlet.managerXml"));
                 return;
             } finally {
                 if (is != null) {
                     try {
                         is.close();
                     } catch (IOException e) {
+                        // Ignore
                     }
                 }
                 if (os != null) {
                     try {
                         os.close();
                     } catch (IOException e) {
+                        // Ignore
                     }
                 }
             }
@@ -455,24 +449,23 @@ public class HostManagerServlet
         host.setDeployOnStartup(deployOnStartup);
         host.setDeployXML(deployXML);
         host.setUnpackWARs(unpackWARs);
-        host.setXmlNamespaceAware(xmlNamespaceAware);
-        host.setXmlValidation(xmlValidation);
         
         // Add new host
         try {
             engine.addChild(host);
         } catch (Exception e) {
-            writer.println(sm.getString("hostManagerServlet.exception",
-                                        e.toString()));
+            writer.println(smClient.getString("hostManagerServlet.exception",
+                    e.toString()));
             return;
         }
         
         host = (StandardHost) engine.findChild(name);
         if (host != null) {
-            writer.println(sm.getString("hostManagerServlet.add", name));
+            writer.println(smClient.getString("hostManagerServlet.add", name));
         } else {
             // Something failed
-            writer.println(sm.getString("hostManagerServlet.addFailed", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.addFailed", name));
         }
         
     }
@@ -484,29 +477,31 @@ public class HostManagerServlet
      * @param writer Writer to render results to
      * @param name host name
      */
-    protected synchronized void remove(PrintWriter writer, String name) {
+    protected synchronized void remove(PrintWriter writer, String name,
+            StringManager smClient) {
 
         if (debug >= 1) {
-            log("remove: Removing host '" + name + "'");
+            log(sm.getString("hostManagerServlet.remove", name));
         }
 
         // Validate the requested host name
         if ((name == null) || name.length() == 0) {
-            writer.println(sm.getString("hostManagerServlet.invalidHostName", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.invalidHostName", name));
             return;
         }
 
         // Check if host exists
         if (engine.findChild(name) == null) {
-            writer.println
-                (sm.getString("hostManagerServlet.noHost", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.noHost", name));
             return;
         }
 
         // Prevent removing our own host
-        if (engine.findChild(name) == host) {
-            writer.println
-                (sm.getString("hostManagerServlet.cannotRemoveOwnHost", name));
+        if (engine.findChild(name) == installedHost) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.cannotRemoveOwnHost", name));
             return;
         }
 
@@ -517,17 +512,19 @@ public class HostManagerServlet
             engine.removeChild(child);
             if ( child instanceof ContainerBase ) ((ContainerBase)child).destroy();
         } catch (Exception e) {
-            writer.println(sm.getString("hostManagerServlet.exception",
-                                        e.toString()));
+            writer.println(smClient.getString("hostManagerServlet.exception",
+                    e.toString()));
             return;
         }
         
         Host host = (StandardHost) engine.findChild(name);
         if (host == null) {
-            writer.println(sm.getString("hostManagerServlet.remove", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.remove", name));
         } else {
             // Something failed
-            writer.println(sm.getString("hostManagerServlet.removeFailed", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.removeFailed", name));
         }
         
     }
@@ -538,27 +535,27 @@ public class HostManagerServlet
      *
      * @param writer Writer to render to
      */
-    protected void list(PrintWriter writer) {
+    protected void list(PrintWriter writer, StringManager smClient) {
 
-        if (debug >= 1)
-            log("list: Listing hosts for engine '" 
-                + engine.getName() + "'");
+        if (debug >= 1) {
+            log(sm.getString("hostManagerServlet.list", engine.getName()));
+        }
 
-        writer.println(sm.getString("hostManagerServlet.listed",
-                                    engine.getName()));
+        writer.println(smClient.getString("hostManagerServlet.listed",
+                engine.getName()));
         Container[] hosts = engine.findChildren();
         for (int i = 0; i < hosts.length; i++) {
             Host host = (Host) hosts[i];
             String name = host.getName();
             String[] aliases = host.findAliases();
-            StringBuffer buf = new StringBuffer();
+            StringBuilder buf = new StringBuilder();
             if (aliases.length > 0) {
                 buf.append(aliases[0]);
                 for (int j = 1; j < aliases.length; j++) {
                     buf.append(',').append(aliases[j]);
                 }
             }
-            writer.println(sm.getString("hostManagerServlet.listitem",
+            writer.println(smClient.getString("hostManagerServlet.listitem",
                                         name, buf.toString()));
         }
     }
@@ -570,43 +567,55 @@ public class HostManagerServlet
      * @param writer Writer to render to
      * @param name Host name
      */
-    protected void start(PrintWriter writer, String name) {
+    protected void start(PrintWriter writer, String name,
+            StringManager smClient) {
 
-        if (debug >= 1)
-            log("start: Starting host with name '" + name + "'");
+        if (debug >= 1) {
+            log(sm.getString("hostManagerServlet.start", name));
+        }
 
         // Validate the requested host name
         if ((name == null) || name.length() == 0) {
-            writer.println(sm.getString("hostManagerServlet.invalidHostName", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.invalidHostName", name));
             return;
         }
 
+        Container host = engine.findChild(name);
+        
         // Check if host exists
-        if (engine.findChild(name) == null) {
-            writer.println
-                (sm.getString("hostManagerServlet.noHost", name));
+        if (host == null) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.noHost", name));
             return;
         }
 
         // Prevent starting our own host
-        if (engine.findChild(name) == host) {
-            writer.println
-                (sm.getString("hostManagerServlet.cannotStartOwnHost", name));
+        if (host == installedHost) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.cannotStartOwnHost", name));
+            return;
+        }
+
+        // Don't start host if already started
+        if (host.getState().isAvailable()) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.alreadyStarted", name));
             return;
         }
 
         // Start host
         try {
-            ((Lifecycle) engine.findChild(name)).start();
-            writer.println
-                (sm.getString("hostManagerServlet.started", name));
-        } catch (Throwable t) {
+            host.start();
+            writer.println(smClient.getString(
+                    "hostManagerServlet.started", name));
+        } catch (Exception e) {
             getServletContext().log
-                (sm.getString("hostManagerServlet.startFailed", name), t);
-            writer.println
-                (sm.getString("hostManagerServlet.startFailed", name));
-            writer.println(sm.getString("hostManagerServlet.exception",
-                                        t.toString()));
+                (sm.getString("hostManagerServlet.startFailed", name), e);
+            writer.println(smClient.getString(
+                    "hostManagerServlet.startFailed", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.exception", e.toString()));
             return;
         }
         
@@ -614,48 +623,60 @@ public class HostManagerServlet
 
 
     /**
-     * Start the host with the specified name.
+     * Stop the host with the specified name.
      *
      * @param writer Writer to render to
      * @param name Host name
      */
-    protected void stop(PrintWriter writer, String name) {
+    protected void stop(PrintWriter writer, String name,
+            StringManager smClient) {
 
-        if (debug >= 1)
-            log("stop: Stopping host with name '" + name + "'");
+        if (debug >= 1) {
+            log(sm.getString("hostManagerServlet.stop", name));
+        }
 
         // Validate the requested host name
         if ((name == null) || name.length() == 0) {
-            writer.println(sm.getString("hostManagerServlet.invalidHostName", name));
+            writer.println(smClient.getString(
+                    "hostManagerServlet.invalidHostName", name));
             return;
         }
 
+        Container host = engine.findChild(name);
+
         // Check if host exists
-        if (engine.findChild(name) == null) {
-            writer.println
-                (sm.getString("hostManagerServlet.noHost", name));
+        if (host == null) {
+            writer.println(smClient.getString("hostManagerServlet.noHost",
+                    name));
             return;
         }
 
-        // Prevent starting our own host
-        if (engine.findChild(name) == host) {
-            writer.println
-                (sm.getString("hostManagerServlet.cannotStopOwnHost", name));
+        // Prevent stopping our own host
+        if (host == installedHost) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.cannotStopOwnHost", name));
             return;
         }
 
-        // Start host
+        // Don't stop host if already stopped
+        if (!host.getState().isAvailable()) {
+            writer.println(smClient.getString(
+                    "hostManagerServlet.alreadyStopped", name));
+            return;
+        }
+
+        // Stop host
         try {
-            ((Lifecycle) engine.findChild(name)).stop();
-            writer.println
-                (sm.getString("hostManagerServlet.stopped", name));
-        } catch (Throwable t) {
-            getServletContext().log
-                (sm.getString("hostManagerServlet.stopFailed", name), t);
-            writer.println
-                (sm.getString("hostManagerServlet.stopFailed", name));
-            writer.println(sm.getString("hostManagerServlet.exception",
-                                        t.toString()));
+            host.stop();
+            writer.println(smClient.getString("hostManagerServlet.stopped",
+                    name));
+        } catch (Exception e) {
+            getServletContext().log(sm.getString(
+                    "hostManagerServlet.stopFailed", name), e);
+            writer.println(smClient.getString("hostManagerServlet.stopFailed",
+                    name));
+            writer.println(smClient.getString("hostManagerServlet.exception",
+                    e.toString()));
             return;
         }
         
@@ -670,19 +691,39 @@ public class HostManagerServlet
      */
     protected File getConfigBase(String hostName) {
         File configBase = 
-            new File(System.getProperty("catalina.base"), "conf");
+            new File(System.getProperty(Globals.CATALINA_BASE_PROP), "conf");
         if (!configBase.exists()) {
             return null;
         }
         if (engine != null) {
             configBase = new File(configBase, engine.getName());
         }
-        if (host != null) {
+        if (installedHost != null) {
             configBase = new File(configBase, hostName);
         }
-        configBase.mkdirs();
+        if (!configBase.mkdirs() && !configBase.isDirectory()) {
+            return null;
+        }
         return configBase;
     }
 
 
+    /**
+     * @deprecated Use {@link StringManager#getManager(String, Enumeration)}.
+     *             This method will be removed in Tomcat 8.
+     */
+    @Deprecated
+    protected StringManager getStringManager(HttpServletRequest req) {
+        Enumeration<Locale> requestedLocales = req.getLocales();
+        while (requestedLocales.hasMoreElements()) {
+            Locale locale = requestedLocales.nextElement();
+            StringManager result = StringManager.getManager(Constants.Package,
+                    locale);
+            if (result.getLocale().equals(locale)) {
+                return result;
+            }
+        }
+        // Return the default
+        return sm;
+    }
 }
