@@ -36,9 +36,7 @@ import org.apache.tomcat.util.modeler.BaseModelMBean;
  * <code>org.apache.catalina.deploy.ContextEnvironment</code> component.</p>
  *
  * @author Amy Roh
- * @version $Id: ContextEnvironmentMBean.java 939518 2010-04-30 00:08:58Z kkolinko $
  */
-
 public class ContextEnvironmentMBean extends BaseModelMBean {
 
 
@@ -81,7 +79,8 @@ public class ContextEnvironmentMBean extends BaseModelMBean {
      * @exception ReflectionException if a Java reflection exception
      *  occurs when invoking the getter
      */
-     public void setAttribute(Attribute attribute)
+     @Override
+    public void setAttribute(Attribute attribute)
         throws AttributeNotFoundException, MBeanException,
         ReflectionException {
 
@@ -96,7 +95,7 @@ public class ContextEnvironmentMBean extends BaseModelMBean {
              throw new MBeanException(e);
         }
         
-        // cannot use side-efects.  It's removed and added back each time 
+        // cannot use side-effects.  It's removed and added back each time 
         // there is a modification in a resource.
         NamingResources nr = ce.getNamingResources();
         nr.removeEnvironment(ce.getName());
