@@ -57,9 +57,7 @@ import org.apache.catalina.valves.ValveBase;
  * <code>org.apache.catalina.core.StandardServer</code> component.</p>
  *
  * @author Amy Roh
- * @version $Id: MBeanFactory.java 1240334 2012-02-03 20:17:37Z markt $
  */
-
 public class MBeanFactory {
 
     private static final org.apache.juli.logging.Log log = 
@@ -78,8 +76,6 @@ public class MBeanFactory {
      * Construct a <code>ModelMBean</code> with default
      * <code>ModelMBeanInfo</code> information.
      *
-     * @exception javax.management.MBeanException if the initializer of an
-     *  object throws an exception
      * @exception javax.management.RuntimeOperationsException if an
      *  IllegalArgumentException occurs
      */
@@ -111,7 +107,10 @@ public class MBeanFactory {
      * Return the managed bean definition for the specified bean type
      *
      * @param type MBean type
+     *
+     * @deprecated  Unused
      */
+    @Deprecated
     public String findObjectName(String type) {
 
         if (type.equals("org.apache.catalina.core.StandardContext")) {
