@@ -67,9 +67,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Amy Roh
- * @version $Id: MBeanUtils.java 1240168 2012-02-03 13:49:58Z markt $
  */
-
 public class MBeanUtils {
 
     // ------------------------------------------------------- Static Variables
@@ -509,7 +507,7 @@ public class MBeanUtils {
 
         ObjectName name = null;
         Host host = (Host)context.getParent();
-        ContextName cn = new ContextName(context.getName());
+        ContextName cn = new ContextName(context.getName(), false);
         name = new ObjectName(domain + ":j2eeType=WebModule,name=//" +
                               host.getName()+ cn.getDisplayName() +
                               ",J2EEApplication=none,J2EEServer=none");
@@ -540,7 +538,7 @@ public class MBeanUtils {
                         ",resourcetype=Global,name=" + environment.getName());
         } else if (container instanceof Context) {        
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=Environment" + 
                         ",resourcetype=Context,context=" + cn.getDisplayName() + 
@@ -575,7 +573,7 @@ public class MBeanUtils {
                         ",name=" + quotedResourceName);
         } else if (container instanceof Context) {                    
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=Resource" +
                         ",resourcetype=Context,context=" + cn.getDisplayName() + 
@@ -613,7 +611,7 @@ public class MBeanUtils {
                         ",name=" + quotedResourceLinkName);
         } else if (container instanceof Context) {                    
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=ResourceLink" +
                         ",resourcetype=Context,context=" + cn.getDisplayName() + 
@@ -715,7 +713,7 @@ public class MBeanUtils {
                               container.getName());
         } else if (container instanceof Context) {
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=Loader,context=" +
                     cn.getDisplayName() + ",host=" + host.getName());
@@ -751,7 +749,7 @@ public class MBeanUtils {
                               container.getName());
         } else if (container instanceof Context) {
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=Manager,context=" +
                     cn.getDisplayName() + ",host=" + host.getName());
@@ -784,7 +782,7 @@ public class MBeanUtils {
                         ",resourcetype=Global");
         } else if (container instanceof Context) {        
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=NamingResources" + 
                         ",resourcetype=Context,context=" + cn.getDisplayName() + 
@@ -842,7 +840,7 @@ public class MBeanUtils {
                               container.getName());
         } else if (container instanceof Context) {
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             name = new ObjectName(domain + ":type=Realm,context=" +
                     cn.getDisplayName() + ",host=" + host.getName());
@@ -1015,7 +1013,7 @@ public class MBeanUtils {
                                     ext + local );
         } else if (container instanceof Context) {
             Context context = ((Context)container);
-            ContextName cn = new ContextName(context.getName());
+            ContextName cn = new ContextName(context.getName(), false);
             Container host = context.getParent();
             String local=",context=" + cn.getDisplayName() + ",host=" +
                     host.getName();
@@ -1618,7 +1616,9 @@ public class MBeanUtils {
         // The database itself
         ObjectName db = new ObjectName(
                 "Users:type=UserDatabase,database=" + userDatabase);
-        mserver.unregisterMBean(db);
+        if(mserver.isRegistered(db)) {
+            mserver.unregisterMBean(db);
+        }
     }
 
 
@@ -1748,7 +1748,7 @@ public class MBeanUtils {
                 keyProperties.append(c.getName());
             } else if (c instanceof Context) {
                 keyProperties.append(",context=");
-                ContextName cn = new ContextName(c.getName());
+                ContextName cn = new ContextName(c.getName(), false);
                 keyProperties.append(cn.getDisplayName());
             } else if (c instanceof Host) {
                 keyProperties.append(",host=");
