@@ -14,17 +14,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.realm;
 
-import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
-import java.nio.charset.Charset;
 import java.security.Principal;
 import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Hashtable;
@@ -54,9 +50,7 @@ import javax.naming.directory.SearchControls;
 import javax.naming.directory.SearchResult;
 
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.util.Base64;
-import org.apache.tomcat.util.buf.ByteChunk;
-import org.apache.tomcat.util.buf.CharChunk;
+
 import org.ietf.jgss.GSSCredential;
 
 /**
@@ -126,8 +120,10 @@ import org.ietf.jgss.GSSCredential;
  *         property.</li>
  *     <li>The <code>roleSearch</code> pattern optionally includes pattern
  *         replacements "{0}" for the distinguished name, and/or "{1}" for
- *         the username, of the authenticated user for which roles will be
- *         retrieved.</li>
+ *         the username, and/or "{2}" the value of an attribute from the
+ *         user's directory entry (the attribute is specified by the
+ *         <code>userRoleAttribute</code> property), of the authenticated user
+ *         for which roles will be retrieved.</li>
  *     <li>The <code>roleBase</code> property can be set to the element that
  *         is the base of the search for matching roles.  If not specified,
  *         the entire context will be searched.</li>
@@ -172,9 +168,7 @@ import org.ietf.jgss.GSSCredential;
  *
  * @author John Holman
  * @author Craig R. McClanahan
- * @version $Id: JNDIRealm.java 1138019 2011-06-21 14:29:49Z markt $
  */
-
 public class JNDIRealm extends RealmBase {
 
 
@@ -298,6 +292,14 @@ public class JNDIRealm extends RealmBase {
      */
     protected String userPassword = null;
 
+    /**
+     * The name of the attribute inside the users
+     * directory entry where the value will be
+     * taken to search for roles
+     * This attribute is not used during a nested search
+     */
+    protected String userRoleAttribute = null;
+
 
     /**
      * A string of LDAP user patterns or paths, ":"-separated
@@ -359,7 +361,8 @@ public class JNDIRealm extends RealmBase {
 
     /**
      * The message format used to select roles for a user, with "{0}" marking
-     * the spot where the distinguished name of the user goes.
+     * the spot where the distinguished name of the user goes. The "{1}"
+     * and "{2}" are described in the Configuration Reference.
      */
     protected String roleSearch = null;
 
@@ -425,6 +428,14 @@ public class JNDIRealm extends RealmBase {
     protected boolean useDelegatedCredential = true;
 
 
+    /**
+     * The QOP that should be used for the connection to the LDAP server after
+     * authentication. This value is used to set the
+     * <code>javax.security.sasl.qop</code> environment property for the LDAP
+     * connection.
+     */
+    protected String spnegoDelegationQop = "auth-conf";
+
     // ------------------------------------------------------------- Properties
 
     /**
@@ -770,6 +781,14 @@ public class JNDIRealm extends RealmBase {
     }
 
 
+    public boolean isRoleSearchAsUser() {
+        return roleSearchAsUser;
+    }
+
+    public void setRoleSearchAsUser(boolean roleSearchAsUser) {
+        this.roleSearchAsUser = roleSearchAsUser;
+    }
+
     /**
      * Return the "search subtree for roles" flag.
      */
@@ -835,6 +854,14 @@ public class JNDIRealm extends RealmBase {
     }
 
 
+    public String getUserRoleAttribute() {
+        return userRoleAttribute;
+    }
+
+    public void setUserRoleAttribute(String userRoleAttribute) {
+        this.userRoleAttribute = userRoleAttribute;
+    }
+
     /**
      * Return the message format pattern for selecting users in this Realm.
      */
@@ -845,6 +872,8 @@ public class JNDIRealm extends RealmBase {
     }
 
 
+
+
     /**
      * Set the message format pattern for selecting users in this Realm.
      * This may be one simple pattern, or multiple patterns to be tried,
@@ -960,7 +989,6 @@ public class JNDIRealm extends RealmBase {
     }
 
 
-    
     public boolean isUseDelegatedCredential() {
         return useDelegatedCredential;
     }
@@ -969,6 +997,16 @@ public class JNDIRealm extends RealmBase {
         this.useDelegatedCredential = useDelegatedCredential;
     }
 
+    
+    public String getSpnegoDelegationQop() {
+        return spnegoDelegationQop;
+    }
+
+    public void setSpnegoDelegationQop(String spnegoDelegationQop) {
+        this.spnegoDelegationQop = spnegoDelegationQop;
+    }
+
+    
     /**
      * Return descriptive information about this Realm implementation and
      * the corresponding version number, in the format
@@ -976,15 +1014,12 @@ public class JNDIRealm extends RealmBase {
      */
     @Override
     public String getInfo() {
-
         return info;
-
     }
 
 
     // ---------------------------------------------------------- Realm Methods
 
-
     /**
      * Return the Principal associated with the specified username and
      * credentials, if there is one; otherwise return <code>null</code>.
@@ -1021,7 +1056,7 @@ public class JNDIRealm extends RealmBase {
                    with broken SSL
                 */
                 // log the exception so we know it's there.
-                containerLog.warn(sm.getString("jndiRealm.exception"), e);
+                containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -1036,7 +1071,7 @@ public class JNDIRealm extends RealmBase {
             } catch (CommunicationException e) {
 
                 // log the exception so we know it's there.
-                containerLog.warn(sm.getString("jndiRealm.exception"), e);
+                containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -1051,7 +1086,7 @@ public class JNDIRealm extends RealmBase {
             } catch (ServiceUnavailableException e) {
 
                 // log the exception so we know it's there.
-                containerLog.warn(sm.getString("jndiRealm.exception"), e);
+                containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -1249,6 +1284,9 @@ public class JNDIRealm extends RealmBase {
             list.add(userPassword);
         if (userRoleName != null)
             list.add(userRoleName);
+        if (userRoleAttribute != null) {
+            list.add(userRoleAttribute);
+        }
         String[] attrIds = new String[list.size()];
         list.toArray(attrIds);
 
@@ -1284,7 +1322,7 @@ public class JNDIRealm extends RealmBase {
 
         // If no attributes are requested, no need to look for them
         if (attrIds == null || attrIds.length == 0) {
-            return new User(username, dn, null, null);
+            return new User(username, dn, null, null,null);
         }
 
         // Get required attributes from user entry
@@ -1302,12 +1340,17 @@ public class JNDIRealm extends RealmBase {
         if (userPassword != null)
             password = getAttributeValue(userPassword, attrs);
 
+        String userRoleAttrValue = null;
+        if (userRoleAttribute != null) {
+            userRoleAttrValue = getAttributeValue(userRoleAttribute, attrs);
+        }
+
         // Retrieve values of userRoleName attribute
         ArrayList<String> roles = null;
         if (userRoleName != null)
             roles = addAttributeValues(userRoleName, attrs, roles);
 
-        return new User(username, dn, password, roles);
+        return new User(username, dn, password, roles, userRoleAttrValue);
     }
 
 
@@ -1446,12 +1489,17 @@ public class JNDIRealm extends RealmBase {
         if (userPassword != null)
             password = getAttributeValue(userPassword, attrs);
 
+        String userRoleAttrValue = null;
+        if (userRoleAttribute != null) {
+            userRoleAttrValue = getAttributeValue(userRoleAttribute, attrs);
+        }
+
         // Retrieve values of userRoleName attribute
         ArrayList<String> roles = null;
         if (userRoleName != null)
             roles = addAttributeValues(userRoleName, attrs, roles);
 
-        return new User(username, dn, password, roles);
+        return new User(username, dn, password, roles, userRoleAttrValue);
     }
 
 
@@ -1511,78 +1559,16 @@ public class JNDIRealm extends RealmBase {
                                          String credentials)
         throws NamingException {
 
-        if (info == null || credentials == null)
-            return (false);
-
-        String password = info.getPassword();
-        if (password == null)
-            return (false);
-
         // Validate the credentials specified by the user
         if (containerLog.isTraceEnabled())
             containerLog.trace("  validating credentials");
 
-        boolean validated = false;
-        if (hasMessageDigest()) {
-            // Some directories prefix the password with the hash type
-            // The string is in a format compatible with Base64.encode not
-            // the Hex encoding of the parent class.
-            if (password.startsWith("{MD5}") || password.startsWith("{SHA}")) {
-                /* sync since super.digest() does this same thing */
-                synchronized (this) {
-                    password = password.substring(5);
-                    md.reset();
-                    md.update(credentials.getBytes(Charset.defaultCharset()));
-                    String digestedPassword = Base64.encode(md.digest());
-                    validated = password.equals(digestedPassword);
-                }
-            } else if (password.startsWith("{SSHA}")) {
-                // Bugzilla 32938
-                /* sync since super.digest() does this same thing */
-                synchronized (this) {
-                    password = password.substring(6);
+        if (info == null || credentials == null)
+            return (false);
 
-                    md.reset();
-                    md.update(credentials.getBytes(Charset.defaultCharset()));
-
-                    // Decode stored password.
-                    ByteChunk pwbc = new ByteChunk(password.length());
-                    try {
-                        pwbc.append(password.getBytes(Charset.defaultCharset()),
-                                0, password.length());
-                    } catch (IOException e) {
-                        // Should never happen
-                        containerLog.error("Could not append password bytes to chunk: ", e);
-                    }
-
-                    CharChunk decoded = new CharChunk();
-                    Base64.decode(pwbc, decoded);
-                    char[] pwarray = decoded.getBuffer();
-
-                    // Split decoded password into hash and salt.
-                    final int saltpos = 20;
-                    byte[] hash = new byte[saltpos];
-                    for (int i=0; i< hash.length; i++) {
-                        hash[i] = (byte) pwarray[i];
-                    }
-
-                    byte[] salt = new byte[pwarray.length - saltpos];
-                    for (int i=0; i< salt.length; i++)
-                        salt[i] = (byte)pwarray[i+saltpos];
-
-                    md.update(salt);
-                    byte[] dp = md.digest();
-
-                    validated = Arrays.equals(dp, hash);
-                } // End synchronized(this) block
-            } else {
-                // Hex hashes should be compared case-insensitive
-                validated = (digest(credentials).equalsIgnoreCase(password));
-            }
-        } else
-            validated = (digest(credentials).equals(password));
-        return (validated);
+        String password = info.getPassword();
 
+        return compareCredentials(credentials, password);
     }
 
 
@@ -1694,6 +1680,7 @@ public class JNDIRealm extends RealmBase {
 
         String dn = user.getDN();
         String username = user.getUserName();
+        String userRoleId = user.getUserRoleId();
 
         if (dn == null || username == null)
             return (null);
@@ -1721,7 +1708,7 @@ public class JNDIRealm extends RealmBase {
             return (list);
         
         // Set up parameters for an appropriate search
-        String filter = roleFormat.format(new String[] { doRFC2254Encoding(dn), username });
+        String filter = roleFormat.format(new String[] { doRFC2254Encoding(dn), username, userRoleId });
         SearchControls controls = new SearchControls();
         if (roleSubtree)
             controls.setSearchScope(SearchControls.SUBTREE_SCOPE);
@@ -1794,7 +1781,7 @@ public class JNDIRealm extends RealmBase {
                 Map<String, String> newThisRound = new HashMap<String, String>(); // Stores the groups we find in this iteration
 
                 for (Entry<String, String> group : newGroups.entrySet()) {
-                    filter = roleFormat.format(new String[] { group.getKey(), group.getValue() });
+                    filter = roleFormat.format(new String[] { group.getKey(), group.getValue(), group.getValue() });
 
                     if (containerLog.isTraceEnabled()) {
                         containerLog.trace("Perform a nested group search with base "+ roleBase + " and filter " + filter);
@@ -1980,7 +1967,7 @@ public class JNDIRealm extends RealmBase {
             } catch (CommunicationException e) {
 
                 // log the exception so we know it's there.
-                containerLog.warn(sm.getString("jndiRealm.exception"), e);
+                containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -1995,7 +1982,7 @@ public class JNDIRealm extends RealmBase {
             } catch (ServiceUnavailableException e) {
 
                 // log the exception so we know it's there.
-                containerLog.warn(sm.getString("jndiRealm.exception"), e);
+                containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
                 // close the connection so we know it will be reopened.
                 if (context != null)
@@ -2043,16 +2030,19 @@ public class JNDIRealm extends RealmBase {
 
         User user = null;
         List<String> roles = null;
+        Hashtable<?, ?> preservedEnvironment = null;
 
         try {
             if (gssCredential != null && isUseDelegatedCredential()) {
+                // Preserve the current context environment parameters
+                preservedEnvironment = context.getEnvironment();
                 // Set up context
                 context.addToEnvironment(
                         Context.SECURITY_AUTHENTICATION, "GSSAPI");
                 context.addToEnvironment(
                         "javax.security.sasl.server.authentication", "true");
                 context.addToEnvironment(
-                        "javax.security.sasl.qop", "auth-conf");
+                        "javax.security.sasl.qop", spnegoDelegationQop);
                 // Note: Subject already set in SPNEGO authenticator so no need
                 //       for Subject.doAs() here
             }
@@ -2061,24 +2051,12 @@ public class JNDIRealm extends RealmBase {
                 roles = getRoles(context, user);
             }
         } finally {
-            try {
-                context.removeFromEnvironment(
-                        Context.SECURITY_AUTHENTICATION);
-            } catch (NamingException e) {
-                // Ignore
-            }
-            try {
-                context.removeFromEnvironment(
-                        "javax.security.sasl.server.authentication");
-            } catch (NamingException e) {
-                // Ignore
-            }
-            try {
-                context.removeFromEnvironment(
-                        "javax.security.sasl.qop");
-            } catch (NamingException e) {
-                // Ignore
-            }
+            restoreEnvironmentParameter(context,
+                    Context.SECURITY_AUTHENTICATION, preservedEnvironment);
+            restoreEnvironmentParameter(context,
+                    "javax.security.sasl.server.authentication", preservedEnvironment);
+            restoreEnvironmentParameter(context, "javax.security.sasl.qop",
+                    preservedEnvironment);
         }
 
         if (user != null) {
@@ -2089,6 +2067,19 @@ public class JNDIRealm extends RealmBase {
         return null;
     }
 
+    private void restoreEnvironmentParameter(DirContext context,
+            String parameterName, Hashtable<?, ?> preservedEnvironment) {
+        try {
+            context.removeFromEnvironment(parameterName);
+            if (preservedEnvironment != null && preservedEnvironment.containsKey(parameterName)) {
+                context.addToEnvironment(parameterName,
+                        preservedEnvironment.get(parameterName));
+            }
+        } catch (NamingException e) {
+            // Ignore
+        }
+    }
+
     /**
      * Open (if necessary) and return a connection to the configured
      * directory server for this Realm.
@@ -2111,7 +2102,7 @@ public class JNDIRealm extends RealmBase {
             connectionAttempt = 1;
 
             // log the first exception.
-            containerLog.warn(sm.getString("jndiRealm.exception"), e);
+            containerLog.info(sm.getString("jndiRealm.exception.retry"), e);
 
             // Try connecting to the alternate url.
             context = new InitialDirContext(getDirectoryContextEnvironment());
@@ -2378,9 +2369,11 @@ public class JNDIRealm extends RealmBase {
         private final String dn;
         private final String password;
         private final List<String> roles;
+        private final String userRoleId;
+
 
         public User(String username, String dn, String password,
-                List<String> roles) {
+                List<String> roles, String userRoleId) {
             this.username = username;
             this.dn = dn;
             this.password = password;
@@ -2389,6 +2382,7 @@ public class JNDIRealm extends RealmBase {
             } else {
                 this.roles = Collections.unmodifiableList(roles);
             }
+            this.userRoleId = userRoleId;
         }
     
         public String getUserName() {
@@ -2406,6 +2400,12 @@ public class JNDIRealm extends RealmBase {
         public List<String> getRoles() {
             return roles;
         }
+
+        public String getUserRoleId() {
+            return userRoleId;
     }
+
+
+}
 }
 
