@@ -43,9 +43,7 @@ import org.apache.tomcat.util.digester.Digester;
  * synchronization is performed around accesses to the principals collection.
  *
  * @author Craig R. McClanahan
- * @version $Id: MemoryRealm.java 1026786 2010-10-24 13:08:26Z markt $
  */
-
 public class MemoryRealm  extends RealmBase {
 
     private static final Log log = LogFactory.getLog(MemoryRealm.class);
@@ -142,17 +140,7 @@ public class MemoryRealm  extends RealmBase {
 
         GenericPrincipal principal = principals.get(username);
 
-        boolean validated = false;
-        if (principal != null && credentials != null) {
-            if (hasMessageDigest()) {
-                // Hex hashes should be compared case-insensitive
-                validated = (digest(credentials)
-                             .equalsIgnoreCase(principal.getPassword()));
-            } else {
-                validated =
-                    (digest(credentials).equals(principal.getPassword()));
-            }
-        }
+        boolean validated = compareCredentials(credentials, principal.getPassword());
 
         if (validated) {
             if (log.isDebugEnabled())
@@ -266,7 +254,10 @@ public class MemoryRealm  extends RealmBase {
      * Returns the principals for this realm.
      *
      * @return The principals, keyed by user name (a String)
+     *
+     * @deprecated  Unused
      */
+    @Deprecated
     protected Map<String,GenericPrincipal> getPrincipals() {
         return principals;
     }
