@@ -233,12 +233,8 @@ import org.apache.catalina.util.IOTools;
  *
  * @author Martin T Dengler [root@martindengler.com]
  * @author Amy Roh
- * @version $Id: CGIServlet.java 1200163 2011-11-10 05:40:04Z kkolinko $
  * @since Tomcat 4.0
- *
  */
-
-
 public final class CGIServlet extends HttpServlet {
 
     /* some vars below copied from Craig R. McClanahan's InvokerServlet */
@@ -276,7 +272,7 @@ public final class CGIServlet extends HttpServlet {
     static Object expandFileLock = new Object();
 
     /** the shell environment variables to be passed to the CGI script */
-    static Hashtable<String,String> shellEnv = new Hashtable<String,String>();
+    Hashtable<String,String> shellEnv = new Hashtable<String,String>();
 
     /**
      * Sets instance variables.
@@ -646,9 +642,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      * </p>
      *
-     * @version  $Id: CGIServlet.java 1200163 2011-11-10 05:40:04Z kkolinko $
      * @since    Tomcat 4.0
-     *
      */
     protected class CGIEnvironment {
 
@@ -1377,10 +1371,7 @@ public final class CGIServlet extends HttpServlet {
      * The input and output streams can be set by the <code>setInput</code>
      * and <code>setResponse</code> methods, respectively.
      * </p>
-     *
-     * @version $Id: CGIServlet.java 1200163 2011-11-10 05:40:04Z kkolinko $
      */
-
     protected class CGIRunner {
 
         /** script/command to be executed */
