@@ -17,12 +17,6 @@
 
 package org.apache.catalina.session;
 
-import org.apache.catalina.Container;
-import org.apache.catalina.LifecycleException;
-import org.apache.catalina.Loader;
-import org.apache.catalina.Session;
-import org.apache.catalina.Store;
-import org.apache.catalina.util.CustomObjectInputStream;
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.ByteArrayInputStream;
@@ -39,22 +33,31 @@ import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Properties;
 
+import javax.naming.Context;
+import javax.naming.InitialContext;
+import javax.naming.NamingException;
+import javax.sql.DataSource;
+
+import org.apache.catalina.Container;
+import org.apache.catalina.LifecycleException;
+import org.apache.catalina.Loader;
+import org.apache.catalina.Session;
+import org.apache.catalina.util.CustomObjectInputStream;
+import org.apache.tomcat.util.ExceptionUtils;
+
 /**
  * Implementation of the <code>Store</code> interface that stores
  * serialized session objects in a database.  Sessions that are
  * saved are still subject to being expired based on inactivity.
  *
  * @author Bip Thelin
- * @version $Revision: 554109 $, $Date: 2007-07-07 03:40:19 +0200 (Sa, 07. Jul 2007) $
  */
-
-public class JDBCStore
-        extends StoreBase implements Store {
+public class JDBCStore extends StoreBase {
 
     /**
      * The descriptive information about this implementation.
      */
-    protected static String info = "JDBCStore/1.0";
+    protected static final String info = "JDBCStore/1.0";
 
     /**
      * Context name associated with this Store
@@ -102,6 +105,16 @@ public class JDBCStore
      */
     protected String driverName = null;
 
+    /**
+     * name of the JNDI resource
+     */
+    protected String dataSourceName = null;
+
+    /**
+     * DataSource to use
+     */
+    protected DataSource dataSource = null;
+    
     // ------------------------------------------------------------- Table & cols
 
     /**
@@ -176,6 +189,7 @@ public class JDBCStore
     /**
      * Return the info for this Store.
      */
+    @Override
     public String getInfo() {
         return (info);
     }
@@ -187,6 +201,9 @@ public class JDBCStore
         if (name == null) {
             Container container = manager.getContainer();
             String contextName = container.getName();
+            if (!contextName.startsWith("/")) {
+                contextName = "/" + contextName;
+            }
             String hostName = "";
             String engineName = "";
 
@@ -212,6 +229,7 @@ public class JDBCStore
     /**
      * Return the name for this Store, used for logging.
      */
+    @Override
     public String getStoreName() {
         return (storeName);
     }
@@ -431,6 +449,27 @@ public class JDBCStore
         return (this.sessionLastAccessedCol);
     }
 
+    /**
+     * Set the JNDI name of a DataSource-factory to use for db access
+     *
+     * @param dataSourceName The JNDI name of the DataSource-factory
+     */
+    public void setDataSourceName(String dataSourceName) {
+        if (dataSourceName == null || "".equals(dataSourceName.trim())) {
+            manager.getContainer().getLogger().warn(
+                    sm.getString(getStoreName() + ".missingDataSourceName"));
+            return;
+        }
+        this.dataSourceName = dataSourceName;
+    }
+
+    /**
+     * Return the name of the JNDI DataSource-factory
+     */
+    public String getDataSourceName() {
+        return this.dataSourceName;
+    }
+
     // --------------------------------------------------------- Public Methods
 
     /**
@@ -440,6 +479,7 @@ public class JDBCStore
      *
      * @exception IOException if an input/output error occurred
      */
+    @Override
     public String[] keys() throws IOException {
         ResultSet rst = null;
         String keys[] = null;
@@ -457,17 +497,17 @@ public class JDBCStore
                                 + sessionTable + " WHERE " + sessionAppCol
                                 + " = ?";
                         preparedKeysSql = _conn.prepareStatement(keysSql);
-					}
+                    }
 
                     preparedKeysSql.setString(1, getName());
                     rst = preparedKeysSql.executeQuery();
-                    ArrayList tmpkeys = new ArrayList();
+                    ArrayList<String> tmpkeys = new ArrayList<String>();
                     if (rst != null) {
                         while (rst.next()) {
                             tmpkeys.add(rst.getString(1));
                         }
                     }
-                    keys = (String[]) tmpkeys.toArray(new String[tmpkeys.size()]);
+                    keys = tmpkeys.toArray(new String[tmpkeys.size()]);
                     // Break out after the finally block
                     numberOfTries = 0;
                 } catch (SQLException e) {
@@ -482,7 +522,7 @@ public class JDBCStore
                             rst.close();
                         }
                     } catch (SQLException e) {
-                        ;
+                        // Ignore
                     }
 
                     release(_conn);
@@ -501,6 +541,7 @@ public class JDBCStore
      *
      * @exception IOException if an input/output error occurred
      */
+    @Override
     public int getSize() throws IOException {
         int size = 0;
         ResultSet rst = null;
@@ -520,7 +561,7 @@ public class JDBCStore
                                 + ") FROM " + sessionTable + " WHERE "
                                 + sessionAppCol + " = ?";
                         preparedSizeSql = _conn.prepareStatement(sizeSql);
-					}
+                    }
 
                     preparedSizeSql.setString(1, getName());
                     rst = preparedSizeSql.executeQuery();
@@ -538,7 +579,7 @@ public class JDBCStore
                         if (rst != null)
                             rst.close();
                     } catch (SQLException e) {
-                        ;
+                        // Ignore
                     }
 
                     release(_conn);
@@ -558,6 +599,7 @@ public class JDBCStore
      * @exception ClassNotFoundException if an error occurs
      * @exception IOException if an input/output error occurred
      */
+    @Override
     public Session load(String id)
             throws ClassNotFoundException, IOException {
         ResultSet rst = null;
@@ -576,6 +618,7 @@ public class JDBCStore
                     return (null);
                 }
 
+                ClassLoader oldThreadContextCL = Thread.currentThread().getContextClassLoader();
                 try {
                     if (preparedLoadSql == null) {
                         String loadSql = "SELECT " + sessionIdCol + ", "
@@ -598,6 +641,7 @@ public class JDBCStore
                             classLoader = loader.getClassLoader();
                         }
                         if (classLoader != null) {
+                            Thread.currentThread().setContextClassLoader(classLoader);
                             ois = new CustomObjectInputStream(bis,
                                     classLoader);
                         } else {
@@ -627,15 +671,16 @@ public class JDBCStore
                             rst.close();
                         }
                     } catch (SQLException e) {
-                        ;
+                        // Ignore
                     }
                     if (ois != null) {
                         try {
                             ois.close();
                         } catch (IOException e) {
-                            ;
+                            // Ignore
                         }
                     }
+                    Thread.currentThread().setContextClassLoader(oldThreadContextCL);
                     release(_conn);
                 }
                 numberOfTries--;
@@ -654,6 +699,7 @@ public class JDBCStore
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void remove(String id) throws IOException {
 
         synchronized (this) {
@@ -666,16 +712,7 @@ public class JDBCStore
                 }
 
                 try {
-                    if (preparedRemoveSql == null) {
-                        String removeSql = "DELETE FROM " + sessionTable
-                                + " WHERE " + sessionIdCol + " = ?  AND "
-                                + sessionAppCol + " = ?";
-                        preparedRemoveSql = _conn.prepareStatement(removeSql);
-                    }
-
-                    preparedRemoveSql.setString(1, id);
-                    preparedRemoveSql.setString(2, getName());
-                    preparedRemoveSql.execute();
+                    remove(id, _conn);
                     // Break out after the finally block
                     numberOfTries = 0;
                 } catch (SQLException e) {
@@ -694,11 +731,34 @@ public class JDBCStore
         }
     }
 
+    /**
+     * Remove the Session with the specified session identifier from
+     * this Store, if present.  If no such Session is present, this method
+     * takes no action.
+     * 
+     * @param id Session identifier of the Session to be removed
+     * @param _conn open connection to be used
+     * @throws SQLException if an error occurs while talking to the database
+     */
+    private void remove(String id, Connection _conn) throws SQLException {
+        if (preparedRemoveSql == null) {
+            String removeSql = "DELETE FROM " + sessionTable
+                    + " WHERE " + sessionIdCol + " = ?  AND "
+                    + sessionAppCol + " = ?";
+            preparedRemoveSql = _conn.prepareStatement(removeSql);
+        }
+
+        preparedRemoveSql.setString(1, id);
+        preparedRemoveSql.setString(2, getName());
+        preparedRemoveSql.execute();
+    }
+
     /**
      * Remove all of the Sessions in this Store.
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void clear() throws IOException {
 
         synchronized (this) {
@@ -738,6 +798,7 @@ public class JDBCStore
      * @param session the session to be stored
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void save(Session session) throws IOException {
         ObjectOutputStream oos = null;
         ByteArrayOutputStream bos = null;
@@ -752,12 +813,12 @@ public class JDBCStore
                     return;
                 }
 
-                // If sessions already exist in DB, remove and insert again.
-                // TODO:
-                // * Check if ID exists in database and if so use UPDATE.
-                remove(session.getIdInternal());
-
                 try {
+                    // If sessions already exist in DB, remove and insert again.
+                    // TODO:
+                    // * Check if ID exists in database and if so use UPDATE.
+                    remove(session.getIdInternal(), _conn);
+                    
                     bos = new ByteArrayOutputStream();
                     oos = new ObjectOutputStream(new BufferedOutputStream(bos));
 
@@ -777,7 +838,7 @@ public class JDBCStore
                            + sessionLastAccessedCol
                            + ") VALUES (?, ?, ?, ?, ?, ?)";
                        preparedSaveSql = _conn.prepareStatement(saveSql);
-					}
+                    }
 
                     preparedSaveSql.setString(1, session.getIdInternal());
                     preparedSaveSql.setString(2, getName());
@@ -793,7 +854,7 @@ public class JDBCStore
                     if (dbConnection != null)
                         close(dbConnection);
                 } catch (IOException e) {
-                    ;
+                    // Ignore
                 } finally {
                     if (oos != null) {
                         oos.close();
@@ -824,14 +885,16 @@ public class JDBCStore
      * <code>null</code> or closed try to reopen it.
      * Returns <code>null</code> if the connection could not be established.
      *
-     * @return <code>Connection</code> if the connection suceeded
+     * @return <code>Connection</code> if the connection succeeded
      */
     protected Connection getConnection() {
+        Connection conn = null;
         try {
-            if (dbConnection == null || dbConnection.isClosed()) {
+            conn = open();
+            if (conn == null || conn.isClosed()) {
                 manager.getContainer().getLogger().info(sm.getString(getStoreName() + ".checkConnectionDBClosed"));
-                open();
-                if (dbConnection == null || dbConnection.isClosed()) {
+                conn = open();
+                if (conn == null || conn.isClosed()) {
                     manager.getContainer().getLogger().info(sm.getString(getStoreName() + ".checkConnectionDBReOpenFail"));
                 }
             }
@@ -840,7 +903,7 @@ public class JDBCStore
                     ex.toString()));
         }
 
-        return dbConnection;
+        return conn;
     }
 
     /**
@@ -855,10 +918,27 @@ public class JDBCStore
         if (dbConnection != null)
             return (dbConnection);
 
+        if (dataSourceName != null && dataSource == null) {
+            Context initCtx;
+            try {
+                initCtx = new InitialContext();
+                Context envCtx = (Context) initCtx.lookup("java:comp/env");
+                this.dataSource = (DataSource) envCtx.lookup(this.dataSourceName);
+            } catch (NamingException e) {
+                manager.getContainer().getLogger().error(
+                        sm.getString(getStoreName() + ".wrongDataSource",
+                                this.dataSourceName), e);
+           }
+        }
+        
+        if (dataSource != null) {
+            return dataSource.getConnection();
+        }
+
         // Instantiate our database driver if necessary
         if (driver == null) {
             try {
-                Class clazz = Class.forName(driverName);
+                Class<?> clazz = Class.forName(driverName);
                 driver = (Driver) clazz.newInstance();
             } catch (ClassNotFoundException ex) {
                 manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".checkConnectionClassNotFoundException",
@@ -899,43 +979,52 @@ public class JDBCStore
         try {
             preparedSizeSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedSizeSql = null;
 
         try {
             preparedKeysSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedKeysSql = null;
 
         try {
             preparedSaveSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedSaveSql = null;
 
         try {
             preparedClearSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
          
-		try {
+        try {
             preparedRemoveSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedRemoveSql = null;
 
         try {
             preparedLoadSql.close();
         } catch (Throwable f) {
-            ;
+            ExceptionUtils.handleThrowable(f);
         }
         this.preparedLoadSql = null;
+        
+        // Commit if autoCommit is false
+        try {
+            if (!dbConnection.getAutoCommit()) {
+                dbConnection.commit();
+            }            
+        } catch (SQLException e) {
+            manager.getContainer().getLogger().error(sm.getString(getStoreName() + ".commitSQLException"), e);
+        }
 
         // Close this database connection, and log any errors
         try {
@@ -949,39 +1038,53 @@ public class JDBCStore
     }
 
     /**
-     * Release the connection, not needed here since the
-     * connection is not associated with a connection pool.
+     * Release the connection, if it
+     * is associated with a connection pool.
      *
      * @param conn The connection to be released
      */
     protected void release(Connection conn) {
-        ;
+        if (dataSource != null) {
+            close(conn); 
+        }
     }
 
     /**
-     * Called once when this Store is first started.
-     */
-    public void start() throws LifecycleException {
-        super.start();
-
-        // Open connection to the database
-        this.dbConnection = getConnection();
-    }
-
-    /**
-     * Gracefully terminate everything associated with our db.
-     * Called once when this Store is stopping.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
-        super.stop();
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
+
+        if (dataSourceName == null) {
+            // If not using a connection pool, open a connection to the database
+            this.dbConnection = getConnection();
+        }
+        
+        super.startInternal();
+    }
+
+    /**
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
+     */
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
+        
+        super.stopInternal();
 
         // Close and release everything associated with our db.
         if (dbConnection != null) {
             try {
                 dbConnection.commit();
             } catch (SQLException e) {
-                ;
+                // Ignore
             }
             close(dbConnection);
         }
