@@ -76,9 +76,7 @@ import org.apache.tomcat.util.res.StringManager;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Id: StandardSession.java 1239138 2012-02-01 14:09:59Z markt $
  */
-
 public class StandardSession implements HttpSession, Session, Serializable {
 
     private static final long serialVersionUID = 1L;
@@ -634,14 +632,14 @@ public class StandardSession implements HttpSession, Session, Serializable {
     @Override
     public boolean isValid() {
 
-        if (this.expiring) {
-            return true;
-        }
-
         if (!this.isValid) {
             return false;
         }
 
+        if (this.expiring) {
+            return true;
+        }
+
         if (ACTIVITY_CHECK && accessCount.get() > 0) {
             return true;
         }
@@ -659,7 +657,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
             }
         }
 
-        return (this.isValid);
+        return this.isValid;
     }
 
 
@@ -753,14 +751,16 @@ public class StandardSession implements HttpSession, Session, Serializable {
      */
     public void expire(boolean notify) {
 
-        // Check to see if expire is in progress or has previously been called
-        if (expiring || !isValid)
+        // Check to see if session has already been invalidated.
+        // Do not check expiring at this point as expire should not return until
+        // isValid is false
+        if (!isValid)
             return;
 
         synchronized (this) {
             // Check again, now we are inside the sync so this code only runs once
-            // Double check locking - expiring and isValid need to be volatile
-            if (expiring || !isValid)
+            // Double check locking - isValid needs to be volatile
+            if (!isValid)
                 return;
 
             if (manager == null)
@@ -834,7 +834,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
             if (ACTIVITY_CHECK) {
                 accessCount.set(0);
             }
-            setValid(false);
 
             // Remove this session from our manager's active sessions
             manager.remove(this, true);
@@ -857,6 +856,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
             }
 
             // We have completed expire of this session
+            setValid(false);
             expiring = false;
 
             // Unbind any objects associated with this session
@@ -1537,7 +1537,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
      * check.
      */
     protected boolean isValidInternal() {
-        return (this.isValid || this.expiring);
+        return this.isValid;
     }
 
     /**
