@@ -23,6 +23,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.lang.reflect.Constructor;
+import java.net.ConnectException;
 import java.net.Socket;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -34,7 +35,6 @@ import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Server;
-import org.apache.catalina.core.StandardServer;
 import org.apache.catalina.security.SecurityConfig;
 import org.apache.juli.ClassLoaderLogManager;
 import org.apache.tomcat.util.ExceptionUtils;
@@ -67,7 +67,6 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Catalina.java 1302650 2012-03-19 20:48:49Z markt $
  */
 public class Catalina {
 
@@ -107,13 +106,19 @@ public class Catalina {
 
     /**
      * Are we starting a new server?
+     *
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     protected boolean starting = false;
 
 
     /**
      * Are we stopping an existing server?
+     *
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     protected boolean stopping = false;
 
 
@@ -144,7 +149,10 @@ public class Catalina {
 
     // ------------------------------------------------------------- Properties
 
-
+    /**
+     * @deprecated  Use {@link #setConfigFile(String)}
+     */
+    @Deprecated
     public void setConfig(String file) {
         configFile = file;
     }
@@ -300,7 +308,7 @@ public class Catalina {
         attrs.add("className");
         fakeAttributes.put(Object.class, attrs);
         digester.setFakeAttributes(fakeAttributes);
-        digester.setClassLoader(StandardServer.class.getClassLoader());
+        digester.setUseContextClassLoader(true);
 
         // Configure the actions we will be using
         digester.addObjectCreate("Server",
@@ -420,6 +428,7 @@ public class Catalina {
 
         // Initialize the digester
         Digester digester = new Digester();
+        digester.setUseContextClassLoader(true);
 
         // Configure the rules we need for shutting down
         digester.addObjectCreate("Server",
@@ -449,12 +458,11 @@ public class Catalina {
         if( s == null ) {
             // Create and execute our Digester
             Digester digester = createStopDigester();
-            digester.setClassLoader(Thread.currentThread().getContextClassLoader());
             File file = configFile();
             FileInputStream fis = null;
             try {
                 InputSource is =
-                    new InputSource("file://" + file.getAbsolutePath());
+                    new InputSource(file.toURI().toURL().toString());
                 fis = new FileInputStream(file);
                 is.setByteStream(fis);
                 digester.push(this);
@@ -494,6 +502,12 @@ public class Catalina {
                     stream.write(shutdown.charAt(i));
                 }
                 stream.flush();
+            } catch (ConnectException ce) {
+                log.error(sm.getString("catalina.stopServer.connectException",
+                                       s.getAddress(),
+                                       String.valueOf(s.getPort())));
+                log.error("Catalina.stop: ", ce);
+                System.exit(1);
             } catch (IOException e) {
                 log.error("Catalina.stop: ", e);
                 System.exit(1);
@@ -542,7 +556,7 @@ public class Catalina {
         try {
             file = configFile();
             inputStream = new FileInputStream(file);
-            inputSource = new InputSource("file://" + file.getAbsolutePath());
+            inputSource = new InputSource(file.toURI().toURL().toString());
         } catch (Exception e) {
             if (log.isDebugEnabled()) {
                 log.debug(sm.getString("catalina.configFail", file), e);
@@ -674,7 +688,13 @@ public class Catalina {
         try {
             getServer().start();
         } catch (LifecycleException e) {
-            log.error("Catalina.start: ", e);
+            log.fatal(sm.getString("catalina.serverStartFail"), e);
+            try {
+                getServer().destroy();
+            } catch (LifecycleException e1) {
+                log.debug("destroy() failed for failed Server ", e1);
+            }
+            return;
         }
 
         long t2 = System.nanoTime();
