@@ -30,7 +30,10 @@ import org.apache.tomcat.util.digester.RuleSet;
  * Wrapper class around the Digester that hide Digester's initialization details
  *
  * @author Jean-Francois Arcand
+ * 
+ * @deprecated Use {@link org.apache.tomcat.util.descriptor.DigesterFactory}
  */
+@Deprecated
 public class DigesterFactory {
     /**
      * The log.
@@ -40,7 +43,10 @@ public class DigesterFactory {
     /**
      * Create a <code>Digester</code> parser with no <code>Rule</code>
      * associated and XML validation turned off.
+     *
+     * @deprecated  Unused - will be removed in 8.0.x
      */
+    @Deprecated
     public static Digester newDigester(){
         return newDigester(false, false, null);
     }
@@ -49,7 +55,10 @@ public class DigesterFactory {
     /**
      * Create a <code>Digester</code> parser with XML validation turned off.
      * @param rule an instance of <code>RuleSet</code> used for parsing the xml.
+     *
+     * @deprecated  Unused - will be removed in 8.0.x
      */
+    @Deprecated
     public static Digester newDigester(RuleSet rule){
         return newDigester(false,false,rule);
     }
