@@ -21,15 +21,12 @@ import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.JarURLConnection;
-import java.net.URL;
-import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
 import java.util.StringTokenizer;
-import java.util.jar.JarEntry;
 
 import javax.servlet.ServletContext;
 import javax.servlet.descriptor.TaglibDescriptor;
@@ -39,12 +36,16 @@ import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.core.StandardContext;
+import org.apache.catalina.deploy.ApplicationListener;
 import org.apache.tomcat.JarScanner;
 import org.apache.tomcat.JarScannerCallback;
 import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.descriptor.DigesterFactory;
+import org.apache.tomcat.util.descriptor.XmlErrorHandler;
 import org.apache.tomcat.util.digester.Digester;
 import org.apache.tomcat.util.res.StringManager;
-import org.apache.tomcat.util.scan.NonClosingJarInputStream;
+import org.apache.tomcat.util.scan.Jar;
+import org.apache.tomcat.util.scan.JarFactory;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 
@@ -78,45 +79,76 @@ public final class TldConfig  implements LifecycleListener {
     /**
      * The <code>Digester</code>s available to process tld files.
      */
-    private static Digester[] tldDigesters = new Digester[4];
+    private static Digester[] tldDigesters = new Digester[2];
 
     /**
      * Create (if necessary) and return a Digester configured to process the
      * tld.
      */
-    private static Digester createTldDigester(boolean namespaceAware,
-            boolean validation) {
+    private static Digester createTldDigester(boolean validation,
+            boolean blockExternal) {
         
         Digester digester = null;
-        if (!namespaceAware && !validation) {
+        if (!validation) {
             if (tldDigesters[0] == null) {
                 tldDigesters[0] = DigesterFactory.newDigester(validation,
-                        namespaceAware, new TldRuleSet());
+                        true, new TldRuleSet(), blockExternal);
+                tldDigesters[0].getParser();
             }
             digester = tldDigesters[0];
-        } else if (!namespaceAware && validation) {
+        } else {
             if (tldDigesters[1] == null) {
                 tldDigesters[1] = DigesterFactory.newDigester(validation,
-                        namespaceAware, new TldRuleSet());
+                        true, new TldRuleSet(), blockExternal);
+                tldDigesters[1].getParser();
             }
             digester = tldDigesters[1];
-        } else if (namespaceAware && !validation) {
-            if (tldDigesters[2] == null) {
-                tldDigesters[2] = DigesterFactory.newDigester(validation,
-                        namespaceAware, new TldRuleSet());
-            }
-            digester = tldDigesters[2];
-        } else {
-            if (tldDigesters[3] == null) {
-                tldDigesters[3] = DigesterFactory.newDigester(validation,
-                        namespaceAware, new TldRuleSet());
-            }
-            digester = tldDigesters[3];
         }
         return digester;
     }
 
 
+    static {
+        // Set the default list of JARs to skip for TLDs
+        StringBuilder jarList = new StringBuilder(System.getProperty(
+                Constants.DEFAULT_JARS_TO_SKIP, ""));
+
+        String tldJars = System.getProperty(Constants.TLD_JARS_TO_SKIP, "");
+        if (tldJars.length() > 0) {
+            if (jarList.length() > 0) {
+                jarList.append(',');
+            }
+            jarList.append(tldJars);
+        }
+
+        if (jarList.length() > 0) {
+            setNoTldJars(jarList.toString());
+        }
+    }
+
+    /**
+     * Sets the list of JARs that are known not to contain any TLDs.
+     *
+     * @param jarNames List of comma-separated names of JAR files that are
+     * known not to contain any TLDs.
+     */
+    public static synchronized void setNoTldJars(String jarNames) {
+        if (jarNames == null) {
+            noTldJars = null;
+        } else {
+            if (noTldJars == null) {
+                noTldJars = new HashSet<String>();
+            } else {
+                noTldJars.clear();
+            }
+            StringTokenizer tokenizer = new StringTokenizer(jarNames, ",");
+            while (tokenizer.hasMoreElements()) {
+                noTldJars.add(tokenizer.nextToken());
+            }
+        }
+    }
+
+
     // ----------------------------------------------------- Instance Variables
 
     /**
@@ -132,19 +164,6 @@ public final class TldConfig  implements LifecycleListener {
     private Digester tldDigester = null;
 
 
-    /**
-     * Attribute value used to turn on/off TLD validation
-     */
-    private boolean tldValidation = false;
-
-
-    /**
-     * Attribute value used to turn on/off TLD  namespace awareness.
-     */
-    private boolean tldNamespaceAware = false;
-
-    private boolean rescan=true;
-
     /**
      * Set of URIs discovered for the associated context. Used to enforce the
      * correct processing priority. Only the TLD associated with the first
@@ -180,77 +199,17 @@ public final class TldConfig  implements LifecycleListener {
     }
 
     /**
-     * Sets the list of JARs that are known not to contain any TLDs.
-     *
-     * @param jarNames List of comma-separated names of JAR files that are 
-     * known not to contain any TLDs.
+     * @deprecated  Unused - will be removed in 8.0.x
      */
-    public static void setNoTldJars(String jarNames) {
-        if (jarNames == null) {
-            noTldJars = null;
-        } else {
-            if (noTldJars == null) {
-                noTldJars = new HashSet<String>();
-            } else {
-                noTldJars.clear();
-            }
-            StringTokenizer tokenizer = new StringTokenizer(jarNames, ",");
-            while (tokenizer.hasMoreElements()) {
-                noTldJars.add(tokenizer.nextToken());
-            }
-        }
-    }
-
-    /**
-     * Set the validation feature of the XML parser used when
-     * parsing xml instances.
-     * @param tldValidation true to enable xml instance validation
-     */
-    public void setTldValidation(boolean tldValidation){
-        this.tldValidation = tldValidation;
-    }
-
-    /**
-     * Get the server.xml &lt;host&gt; attribute's xmlValidation.
-     * @return true if validation is enabled.
-     *
-     */
-    public boolean getTldValidation(){
-        return this.tldValidation;
-    }
-
-    /**
-     * Get the server.xml &lt;host&gt; attribute's xmlNamespaceAware.
-     * @return true if namespace awareness is enabled.
-     *
-     */
-    public boolean getTldNamespaceAware(){
-        return this.tldNamespaceAware;
-    }
-
-
-    /**
-     * Set the namespace aware feature of the XML parser used when
-     * parsing xml instances.
-     * @param tldNamespaceAware true to enable namespace awareness
-     */
-    public void setTldNamespaceAware(boolean tldNamespaceAware){
-        this.tldNamespaceAware = tldNamespaceAware;
-    }    
-
-
-    public boolean isRescan() {
-        return rescan;
-    }
-
-    public void setRescan(boolean rescan) {
-        this.rescan = rescan;
-    }
-
+    @Deprecated
     public Context getContext() {
         return context;
     }
 
+    /**
+     * @deprecated  Unused - will be removed in 8.0.x
+     */
+    @Deprecated
     public void setContext(Context context) {
         this.context = context;
     }
@@ -316,7 +275,8 @@ public final class TldConfig  implements LifecycleListener {
                     Integer.valueOf(list.length)));
 
         for( int i=0; list!=null && i<list.length; i++ ) {
-            context.addApplicationListener(list[i]);
+            context.addApplicationListener(
+                    new ApplicationListener(list[i], true));
         }
 
         long t2=System.currentTimeMillis();
@@ -382,10 +342,15 @@ public final class TldConfig  implements LifecycleListener {
                 try {
                     stream = context.getServletContext().getResourceAsStream(
                             resourcePath);
-                    XmlErrorHandler handler = tldScanStream(stream);
-                    handler.logFindings(log, resourcePath);
-                    taglibUris.add(descriptor.getTaglibURI());
-                    webxmlTaglibUris.add(descriptor.getTaglibURI());
+                    if (stream != null) {
+                        XmlErrorHandler handler = tldScanStream(stream);
+                        handler.logFindings(log, resourcePath);
+                        taglibUris.add(descriptor.getTaglibURI());
+                        webxmlTaglibUris.add(descriptor.getTaglibURI());
+                    } else {
+                        log.warn(sm.getString("tldConfig.webxmlFailPathDoesNotExist", resourcePath,
+                                descriptor.getTaglibURI()));
+                    }
                 } catch (IOException ioe) {
                     log.warn(sm.getString("tldConfig.webxmlFail", resourcePath,
                             descriptor.getTaglibURI()), ioe);
@@ -510,38 +475,40 @@ public final class TldConfig  implements LifecycleListener {
      */
     private void tldScanJar(JarURLConnection jarConn) {
 
-        // JarURLConnection#getJarFile() creates temporary copies of the JAR if
-        // the underlying resource is not a file URL. That can be slow so the
-        // InputStream for the resource is used
-        URL resourceURL = jarConn.getJarFileURL();
-        NonClosingJarInputStream jarInputStream = null;
-        String name = null;
-
+        Jar jar = null;
+        InputStream is;
+        
         try {
-            URLConnection resourceConn = resourceURL.openConnection();
-            resourceConn.setUseCaches(false);
-            jarInputStream =
-                new NonClosingJarInputStream(resourceConn.getInputStream());
-
-            JarEntry entry = jarInputStream.getNextJarEntry();
-            while (entry != null) {
-                name = entry.getName();
-                if (name.startsWith("META-INF/") && name.endsWith(".tld")) {
-                    XmlErrorHandler handler = tldScanStream(jarInputStream);
-                    handler.logFindings(log, jarConn.getURL() + name);
+            jar = JarFactory.newInstance(jarConn.getURL());
+            
+            jar.nextEntry();
+            String entryName = jar.getEntryName();
+            while (entryName != null) {
+                if (entryName.startsWith("META-INF/") &&
+                        entryName.endsWith(".tld")) {
+                    is = null;
+                    try {
+                        is = jar.getEntryInputStream();
+                        XmlErrorHandler handler = tldScanStream(is);
+                        handler.logFindings(log, jarConn.getURL() + entryName);
+                    } finally {
+                        if (is != null) {
+                            try {
+                                is.close();
+                            } catch (IOException ioe) {
+                                // Ignore
+                            }
+                        }
+                    }
                 }
-                entry = jarInputStream.getNextJarEntry();
+                jar.nextEntry();
+                entryName = jar.getEntryName();
             }
         } catch (IOException ioe) {
-            log.warn(sm.getString("tldConfig.jarFail", jarConn.getURL() + name),
-                    ioe);
+            log.warn(sm.getString("tldConfig.jarFail", jarConn.getURL()), ioe);
         } finally {
-            if (jarInputStream != null) {
-                try {
-                    jarInputStream.reallyClose();
-                } catch (Throwable t) {
-                    ExceptionUtils.handleThrowable(t);
-                }
+            if (jar != null) {
+                jar.close();
             }
         }
     }
@@ -606,9 +573,8 @@ public final class TldConfig  implements LifecycleListener {
     
     private void init() {
         if (tldDigester == null){
-            setTldValidation(context.getTldValidation());
-            setTldNamespaceAware(context.getTldNamespaceAware());
-            tldDigester = createTldDigester(tldNamespaceAware, tldValidation);
+            tldDigester = createTldDigester(context.getTldValidation(),
+                    context.getXmlBlockExternal());
         }
     }
 
