@@ -16,10 +16,9 @@
  */
 package org.apache.catalina.tribes;
 
-import java.io.Serializable;
 import java.io.Externalizable;
-import java.io.ObjectInput;
 import java.io.IOException;
+import java.io.ObjectInput;
 import java.io.ObjectOutput;
 
 /**
@@ -37,10 +36,8 @@ import java.io.ObjectOutput;
  * so that you can decide with the <code>ChannelListener.accept()</code> whether this message was intended
  * for you.
  * @author Filip Hanik
- * @version $Id: ByteMessage.java 939515 2010-04-29 23:59:49Z kkolinko $
  */
-
-public class ByteMessage implements Serializable, Externalizable {
+public class ByteMessage implements Externalizable {
     /**
      * Storage for the message to be sent
      */
@@ -83,10 +80,11 @@ public class ByteMessage implements Serializable, Externalizable {
      * @param in ObjectInput
      * @throws IOException
      */
+    @Override
     public void readExternal(ObjectInput in ) throws IOException {
         int length = in.readInt();
         message = new byte[length];
-        in.read(message,0,length);
+        in.readFully(message);
     }
 
     /**
@@ -94,6 +92,7 @@ public class ByteMessage implements Serializable, Externalizable {
      * @param out ObjectOutput
      * @throws IOException
      */
+    @Override
     public void writeExternal(ObjectOutput out) throws IOException {
         out.writeInt(message!=null?message.length:0);
         if ( message!=null ) out.write(message,0,message.length);
