@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -17,15 +17,22 @@
 
 package org.apache.catalina.tribes.membership;
 
+import java.io.IOException;
+import java.net.DatagramPacket;
 import java.util.Properties;
 
+import org.apache.catalina.tribes.Channel;
+import org.apache.catalina.tribes.ChannelException;
+import org.apache.catalina.tribes.ChannelMessage;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.MembershipListener;
 import org.apache.catalina.tribes.MembershipService;
+import org.apache.catalina.tribes.MessageListener;
+import org.apache.catalina.tribes.io.ChannelData;
+import org.apache.catalina.tribes.io.XByteBuffer;
 import org.apache.catalina.tribes.util.Arrays;
 import org.apache.catalina.tribes.util.StringManager;
 import org.apache.catalina.tribes.util.UUIDGenerator;
-import java.io.IOException;
 
 /**
  * A <b>membership</b> implementation using simple multicast.
@@ -34,19 +41,16 @@ import java.io.IOException;
  * If a node fails to send out a heartbeat, the node will be dismissed.
  *
  * @author Filip Hanik
- * @version $Revision: 892864 $, $Date: 2009-12-21 17:26:07 +0100 (Mo, 21. Dez 2009) $
  */
+public class McastService implements MembershipService,MembershipListener,MessageListener {
 
-
-public class McastService implements MembershipService,MembershipListener {
-
-    private static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog( McastService.class );
 
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     /**
      * The descriptive information about this implementation.
@@ -65,15 +69,19 @@ public class McastService implements MembershipService,MembershipListener {
      * A membership listener delegate (should be the cluster :)
      */
     protected MembershipListener listener;
+    /**
+     * A message listener delegate for broadcasts
+     */
+    protected MessageListener msglistener;
     /**
      * The local member
      */
     protected MemberImpl localMember ;
     private int mcastSoTimeout;
     private int mcastTTL;
-    
+
     protected byte[] payload;
-    
+
     protected byte[] domain;
 
     /**
@@ -96,7 +104,7 @@ public class McastService implements MembershipService,MembershipListener {
     public String getInfo() {
         return (info);
     }
-    
+
     /**
      *
      * @param properties
@@ -110,6 +118,7 @@ public class McastService implements MembershipService,MembershipListener {
      * 8. tcpListenHost - the bind address of this member<BR>
      * @exception java.lang.IllegalArgumentException if a property is missing.
      */
+    @Override
     public void setProperties(Properties properties) {
         hasProperty(properties,"mcastPort");
         hasProperty(properties,"mcastAddress");
@@ -123,6 +132,7 @@ public class McastService implements MembershipService,MembershipListener {
     /**
      * Return the properties, see setProperties
      */
+    @Override
     public Properties getProperties() {
         return properties;
     }
@@ -133,21 +143,25 @@ public class McastService implements MembershipService,MembershipListener {
     public String getLocalMemberName() {
         return localMember.toString() ;
     }
- 
+
     /**
      * Return the local member
      */
+    @Override
     public Member getLocalMember(boolean alive) {
         if ( alive && localMember != null && impl != null) localMember.setMemberAliveTime(System.currentTimeMillis()-impl.getServiceStartTime());
         return localMember;
     }
-    
+
     /**
      * Sets the local member properties for broadcasting
      */
-    public void setLocalMemberProperties(String listenHost, int listenPort) {
+    @Override
+    public void setLocalMemberProperties(String listenHost, int listenPort, int securePort, int udpPort) {
         properties.setProperty("tcpListenHost",listenHost);
         properties.setProperty("tcpListenPort",String.valueOf(listenPort));
+        properties.setProperty("udpListenPort",String.valueOf(udpPort));
+        properties.setProperty("tcpSecurePort",String.valueOf(securePort));
         try {
             if (localMember != null) {
                 localMember.setHostname(listenHost);
@@ -158,32 +172,36 @@ public class McastService implements MembershipService,MembershipListener {
                 localMember.setPayload(getPayload());
                 localMember.setDomain(getDomain());
             }
+            localMember.setSecurePort(securePort);
+            localMember.setUdpPort(udpPort);
             localMember.getData(true, true);
         }catch ( IOException x ) {
             throw new IllegalArgumentException(x);
         }
     }
-    
+
     public void setAddress(String addr) {
         properties.setProperty("mcastAddress", addr);
     }
-    
+
     /**
      * @deprecated use setAddress
      * @param addr String
      */
+    @Deprecated
     public void setMcastAddr(String addr) {
         setAddress(addr);
     }
-    
+
     public String getAddress() {
         return properties.getProperty("mcastAddress");
     }
-    
+
     /**
      * @deprecated use getAddress
      * @return String
      */
+    @Deprecated
     public String getMcastAddr() {
         return getAddress();
     }
@@ -191,7 +209,7 @@ public class McastService implements MembershipService,MembershipListener {
     public void setMcastBindAddress(String bindaddr) {
         setBind(bindaddr);
     }
-    
+
     public void setBind(String bindaddr) {
         properties.setProperty("mcastBindAddress", bindaddr);
     }
@@ -199,6 +217,7 @@ public class McastService implements MembershipService,MembershipListener {
      * @deprecated use getBind
      * @return String
      */
+    @Deprecated
     public String getMcastBindAddress() {
         return getBind();
     }
@@ -211,6 +230,7 @@ public class McastService implements MembershipService,MembershipListener {
      * @deprecated use setPort
      * @param port int
      */
+    @Deprecated
     public void setMcastPort(int port) {
         setPort(port);
     }
@@ -218,24 +238,60 @@ public class McastService implements MembershipService,MembershipListener {
     public void setPort(int port) {
         properties.setProperty("mcastPort", String.valueOf(port));
     }
-    
+
     public void setRecoveryCounter(int recoveryCounter) {
         properties.setProperty("recoveryCounter", String.valueOf(recoveryCounter));
     }
+    
+    public int getRecoveryCounter(){
+        String p = properties.getProperty("recoveryCounter");
+        if(p != null){
+            return new Integer(p).intValue();
+        }
+        return -1;
+    }
 
     public void setRecoveryEnabled(boolean recoveryEnabled) {
         properties.setProperty("recoveryEnabled", String.valueOf(recoveryEnabled));
     }
+    
+    public boolean getRecoveryEnabled() {
+        String p = properties.getProperty("recoveryEnabled");
+        if(p != null){
+            return Boolean.valueOf(p).booleanValue();
+        }
+        return false;
+    }
 
     public void setRecoverySleepTime(long recoverySleepTime) {
         properties.setProperty("recoverySleepTime", String.valueOf(recoverySleepTime));
     }
+    
+    public long getRecoverySleepTime(){
+        String p = properties.getProperty("recoverySleepTime");
+        if(p != null){
+            return new Long(p).longValue();
+        }
+        return -1;
+    }
 
+    public void setLocalLoopbackDisabled(boolean localLoopbackDisabled) {
+        properties.setProperty("localLoopbackDisabled",String.valueOf(localLoopbackDisabled));
+    }
+    
+    public boolean getLocalLoopbackDisabled(boolean localLoopbackDisabled) {
+        String p = properties.getProperty("localLoopbackDisabled");
+        if(p != null){
+            return Boolean.valueOf(p).booleanValue();
+        }
+        return false;
+    }
 
     /**
      * @deprecated use getPort()
      * @return int
      */
+    @Deprecated
     public int getMcastPort() {
         return getPort();
     }
@@ -243,15 +299,16 @@ public class McastService implements MembershipService,MembershipListener {
         String p = properties.getProperty("mcastPort");
         return new Integer(p).intValue();
     }
-    
+
     /**
      * @deprecated use setFrequency
      * @param time long
      */
+    @Deprecated
     public void setMcastFrequency(long time) {
         setFrequency(time);
     }
-    
+
     public void setFrequency(long time) {
         properties.setProperty("mcastFrequency", String.valueOf(time));
     }
@@ -260,6 +317,7 @@ public class McastService implements MembershipService,MembershipListener {
      * @deprecated use getFrequency
      * @return long
      */
+    @Deprecated
     public long getMcastFrequency() {
         return getFrequency();
     }
@@ -275,11 +333,12 @@ public class McastService implements MembershipService,MembershipListener {
     public void setDropTime(long time) {
         properties.setProperty("memberDropTime", String.valueOf(time));
     }
-    
+
     /**
      * @deprecated use getDropTime
      * @return long
      */
+    @Deprecated
     public long getMcastDropTime() {
         return getDropTime();
     }
@@ -302,11 +361,13 @@ public class McastService implements MembershipService,MembershipListener {
      * Start broadcasting and listening to membership pings
      * @throws java.lang.Exception if a IO error occurs
      */
+    @Override
     public void start() throws java.lang.Exception {
         start(MembershipService.MBR_RX);
         start(MembershipService.MBR_TX);
     }
-    
+
+    @Override
     public void start(int level) throws java.lang.Exception {
         hasProperty(properties,"mcastPort");
         hasProperty(properties,"mcastAddress");
@@ -314,6 +375,9 @@ public class McastService implements MembershipService,MembershipListener {
         hasProperty(properties,"mcastFrequency");
         hasProperty(properties,"tcpListenPort");
         hasProperty(properties,"tcpListenHost");
+        hasProperty(properties,"tcpSecurePort");
+        hasProperty(properties,"udpListenPort");
+
 
         if ( impl != null ) {
             impl.start(level);
@@ -321,7 +385,9 @@ public class McastService implements MembershipService,MembershipListener {
         }
         String host = getProperties().getProperty("tcpListenHost");
         int port = Integer.parseInt(getProperties().getProperty("tcpListenPort"));
-        
+        int securePort = Integer.parseInt(getProperties().getProperty("tcpSecurePort"));
+        int udpPort = Integer.parseInt(getProperties().getProperty("udpListenPort"));
+
         if ( localMember == null ) {
             localMember = new MemberImpl(host, port, 100);
             localMember.setUniqueId(UUIDGenerator.randomUUID(true));
@@ -330,6 +396,8 @@ public class McastService implements MembershipService,MembershipListener {
             localMember.setPort(port);
             localMember.setMemberAliveTime(100);
         }
+        localMember.setSecurePort(securePort);
+        localMember.setUdpPort(udpPort);
         if ( this.payload != null ) localMember.setPayload(payload);
         if ( this.domain != null ) localMember.setDomain(domain);
         localMember.setServiceStartTime(System.currentTimeMillis());
@@ -354,32 +422,35 @@ public class McastService implements MembershipService,MembershipListener {
             }
         }
 
-        impl = new McastServiceImpl((MemberImpl)localMember,Long.parseLong(properties.getProperty("mcastFrequency")),
+        impl = new McastServiceImpl(localMember,Long.parseLong(properties.getProperty("mcastFrequency")),
                                     Long.parseLong(properties.getProperty("memberDropTime")),
                                     Integer.parseInt(properties.getProperty("mcastPort")),
                                     bind,
                                     java.net.InetAddress.getByName(properties.getProperty("mcastAddress")),
                                     ttl,
                                     soTimeout,
-                                    this);
+                                    this,
+                                    this,
+                                    Boolean.valueOf(properties.getProperty("localLoopbackDisabled","false")).booleanValue());
         String value = properties.getProperty("recoveryEnabled","true");
         boolean recEnabled = Boolean.valueOf(value).booleanValue() ;
-        impl.setRecoveryEnabled(recEnabled);        
+        impl.setRecoveryEnabled(recEnabled);
         int recCnt = Integer.parseInt(properties.getProperty("recoveryCounter","10"));
         impl.setRecoveryCounter(recCnt);
         long recSlpTime = Long.parseLong(properties.getProperty("recoverySleepTime","5000"));
         impl.setRecoverySleepTime(recSlpTime);
-        
-        
+
+
         impl.start(level);
-		
+
 
     }
 
- 
+
     /**
      * Stop broadcasting and listening to membership pings
      */
+    @Override
     public void stop(int svc) {
         try  {
             if ( impl != null && impl.stop(svc) ) impl = null;
@@ -392,6 +463,7 @@ public class McastService implements MembershipService,MembershipListener {
     /**
      * Return all the members by name
      */
+    @Override
     public String[] getMembersByName() {
         Member[] currentMembers = getMembers();
         String [] membernames ;
@@ -404,10 +476,11 @@ public class McastService implements MembershipService,MembershipListener {
             membernames = new String[0] ;
         return membernames ;
     }
- 
+
     /**
      * Return the member by name
      */
+    @Override
     public Member findMemberByName(String name) {
         Member[] currentMembers = getMembers();
         for (int i = 0; i < currentMembers.length; i++) {
@@ -420,11 +493,13 @@ public class McastService implements MembershipService,MembershipListener {
     /**
      * has members?
      */
+    @Override
     public boolean hasMembers() {
        if ( impl == null || impl.membership == null ) return false;
        return impl.membership.hasMembers();
     }
-    
+
+    @Override
     public Member getMember(Member mbr) {
         if ( impl == null || impl.membership == null ) return null;
         return impl.membership.getMember(mbr);
@@ -434,6 +509,7 @@ public class McastService implements MembershipService,MembershipListener {
      * Return all the members
      */
     protected static final Member[]EMPTY_MEMBERS = new Member[0];
+    @Override
     public Member[] getMembers() {
         if ( impl == null || impl.membership == null ) return EMPTY_MEMBERS;
         return impl.membership.getMembers();
@@ -443,16 +519,27 @@ public class McastService implements MembershipService,MembershipListener {
      * so calling this method twice will result in only the second listener being active.
      * @param listener The listener
      */
+    @Override
     public void setMembershipListener(MembershipListener listener) {
         this.listener = listener;
     }
+    
+    public void setMessageListener(MessageListener listener) {
+        this.msglistener = listener;
+    }
+    
+    public void removeMessageListener() {
+        this.msglistener = null;
+    }
     /**
      * Remove the membership listener
      */
+    @Override
     public void removeMembershipListener(){
         listener = null;
     }
 
+    @Override
     public void memberAdded(Member member) {
         if ( listener!=null ) listener.memberAdded(member);
     }
@@ -461,44 +548,74 @@ public class McastService implements MembershipService,MembershipListener {
      * Callback from the impl when a new member has been received
      * @param member The member
      */
+    @Override
     public void memberDisappeared(Member member)
     {
         if ( listener!=null ) listener.memberDisappeared(member);
     }
+    
+    @Override
+    public void messageReceived(ChannelMessage msg) {
+        if (msglistener!=null && msglistener.accept(msg)) msglistener.messageReceived(msg); 
+    }
+    
+    @Override
+    public boolean accept(ChannelMessage msg) {
+        return true;
+    }
+    @Override
+    public void broadcast(ChannelMessage message) throws ChannelException {
+        if (impl==null || (impl.startLevel & Channel.MBR_TX_SEQ)!=Channel.MBR_TX_SEQ )
+            throw new ChannelException("Multicast send is not started or enabled.");
+        
+        byte[] data = XByteBuffer.createDataPackage((ChannelData)message);
+        if (data.length>McastServiceImpl.MAX_PACKET_SIZE) {
+            throw new ChannelException("Packet length["+data.length+"] exceeds max packet size of "+McastServiceImpl.MAX_PACKET_SIZE+" bytes.");
+        }
+        DatagramPacket packet = new DatagramPacket(data,0,data.length);
+        try {
+            impl.send(false, packet);
+        } catch (Exception x) {
+            throw new ChannelException(x);
+        }
+    }
 
     /**
      * @deprecated use getSoTimeout
      * @return int
      */
+    @Deprecated
     public int getMcastSoTimeout() {
         return getSoTimeout();
     }
-    
+
     public int getSoTimeout() {
         return mcastSoTimeout;
     }
-    
+
     /**
      * @deprecated use setSoTimeout
      * @param mcastSoTimeout int
      */
+    @Deprecated
     public void setMcastSoTimeout(int mcastSoTimeout) {
         setSoTimeout(mcastSoTimeout);
     }
-    
+
     public void setSoTimeout(int mcastSoTimeout) {
         this.mcastSoTimeout = mcastSoTimeout;
         properties.setProperty("mcastSoTimeout", String.valueOf(mcastSoTimeout));
     }
-    
+
     /**
      * @deprecated use getTtl
      * @return int
      */
+    @Deprecated
     public int getMcastTTL() {
         return getTtl();
     }
-    
+
     public int getTtl() {
         return mcastTTL;
     }
@@ -506,15 +623,16 @@ public class McastService implements MembershipService,MembershipListener {
     public byte[] getPayload() {
         return payload;
     }
-    
+
     public byte[] getDomain() {
         return domain;
     }
-    
+
     /**
      * @deprecated use setTtl
      * @param mcastTTL int
      */
+    @Deprecated
     public void setMcastTTL(int mcastTTL) {
         setTtl(mcastTTL);
     }
@@ -524,6 +642,7 @@ public class McastService implements MembershipService,MembershipListener {
         properties.setProperty("mcastTTL", String.valueOf(mcastTTL));
     }
 
+    @Override
     public void setPayload(byte[] payload) {
         this.payload = payload;
         if ( localMember != null ) {
@@ -536,7 +655,8 @@ public class McastService implements MembershipService,MembershipListener {
             }
         }
     }
-    
+
+    @Override
     public void setDomain(byte[] domain) {
         this.domain = domain;
         if ( localMember != null ) {
@@ -562,7 +682,7 @@ public class McastService implements MembershipService,MembershipListener {
      * @throws Exception If an error occurs
      */
     public static void main(String args[]) throws Exception {
-		if(log.isInfoEnabled())
+        if(log.isInfoEnabled())
             log.info("Usage McastService hostname tcpport");
         McastService service = new McastService();
         java.util.Properties p = new java.util.Properties();
@@ -574,6 +694,8 @@ public class McastService implements MembershipService,MembershipListener {
         p.setProperty("mcastFrequency","500");
         p.setProperty("tcpListenPort","4000");
         p.setProperty("tcpListenHost","127.0.0.1");
+        p.setProperty("tcpSecurePort","4100");
+        p.setProperty("udpListenPort","4200");
         service.setProperties(p);
         service.start();
         Thread.sleep(60*1000*60);
