@@ -19,17 +19,23 @@ package org.apache.catalina.tribes.membership;
 
 
 import java.io.IOException;
+import java.net.BindException;
 import java.net.DatagramPacket;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.MulticastSocket;
 import java.net.SocketTimeoutException;
 import java.util.Arrays;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.TimeUnit;
 
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.MembershipListener;
-import java.net.BindException;
+import org.apache.catalina.tribes.MessageListener;
+import org.apache.catalina.tribes.io.ChannelData;
+import org.apache.catalina.tribes.io.XByteBuffer;
+import org.apache.catalina.tribes.util.ExecutorFactory;
 
 /**
  * A <b>membership</b> implementation using simple multicast.
@@ -40,19 +46,18 @@ import java.net.BindException;
  * Need to fix this, could use java.nio and only need one thread to send and receive, or
  * just use a timeout on the receive
  * @author Filip Hanik
- * @version $Id: McastServiceImpl.java 939515 2010-04-29 23:59:49Z kkolinko $
  */
 public class McastServiceImpl
 {
-    private static org.apache.juli.logging.Log log =
+    private static final org.apache.juli.logging.Log log =
         org.apache.juli.logging.LogFactory.getLog( McastService.class );
     
     protected static int MAX_PACKET_SIZE = 65535;
     /**
      * Internal flag used for the listen thread that listens to the multicasting socket.
      */
-    protected boolean doRunSender = false;
-    protected boolean doRunReceiver = false;
+    protected volatile boolean doRunSender = false;
+    protected volatile boolean doRunReceiver = false;
     protected int startLevel = 0;
     /**
      * Socket that we intend to listen to
@@ -91,9 +96,13 @@ public class McastServiceImpl
      */
     protected Membership membership;
     /**
-     * The actual listener, for callback when shits goes down
+     * The actual listener, for callback when stuff goes down
      */
     protected MembershipListener service;
+    /**
+     * The actual listener for broadcast callbacks
+     */
+    protected MessageListener msgservice;
     /**
      * Thread to listen for pings
      */
@@ -103,11 +112,6 @@ public class McastServiceImpl
      */
     protected SenderThread sender;
 
-    /**
-     * When was the service started
-     */
-    protected long serviceStartTime = System.currentTimeMillis();
-    
     /**
      * Time to live for the multicast packets that are being sent out
      */
@@ -135,6 +139,17 @@ public class McastServiceImpl
      * Add the ability to turn on/off recovery
      */
     protected boolean recoveryEnabled = true;
+    
+    /**
+     * Dont interrupt the sender/receiver thread, but pass off to an executor
+     */
+    protected ExecutorService executor = ExecutorFactory.newThreadPool(0, 2, 2, TimeUnit.SECONDS);
+    
+    /**
+     * disable/enable local loopback message
+     */
+    protected boolean localLoopbackDisabled = false;
+    
     /**
      * Create a new mcast service impl
      * @param member - the local member
@@ -144,6 +159,7 @@ public class McastServiceImpl
      * @param bind - the bind address (not sure this is used yet)
      * @param mcastAddress - the mcast address
      * @param service - the callback service
+     * @param localLoopbackDisabled - disable loopbackMode
      * @throws IOException
      */
     public McastServiceImpl(
@@ -155,7 +171,9 @@ public class McastServiceImpl
         InetAddress mcastAddress,
         int ttl,
         int soTimeout,
-        MembershipListener service)
+        MembershipListener service,
+        MessageListener msgservice,
+        boolean localLoopbackDisabled)
     throws IOException {
         this.member = member;
         this.address = mcastAddress;
@@ -165,7 +183,9 @@ public class McastServiceImpl
         this.mcastBindAddress = bind;
         this.timeToExpiration = expireTime;
         this.service = service;
+        this.msgservice = msgservice;
         this.sendFrequency = sendFrequency;
+        this.localLoopbackDisabled = localLoopbackDisabled;
         init();
     }
 
@@ -189,7 +209,7 @@ public class McastServiceImpl
                 socket = new MulticastSocket(new InetSocketAddress(address,port));
             } catch (BindException e) {
                 /*
-                 * On some plattforms (e.g. Linux) it is not possible to bind
+                 * On some platforms (e.g. Linux) it is not possible to bind
                  * to the multicast address. In this case only bind to the
                  * port.
                  */
@@ -199,9 +219,9 @@ public class McastServiceImpl
         } else {
             socket = new MulticastSocket(port);
         }
-        socket.setLoopbackMode(false);
+        socket.setLoopbackMode(localLoopbackDisabled); //hint if we want disable loop back(local machine) messages
         if (mcastBindAddress != null) {
-			if(log.isInfoEnabled())
+            if(log.isInfoEnabled())
                 log.info("Setting multihome multicast interface to:" +mcastBindAddress);
             socket.setInterface(mcastBindAddress);
         } //end if
@@ -212,13 +232,12 @@ public class McastServiceImpl
         socket.setSoTimeout(mcastSoTimeout);
 
         if ( mcastTTL >= 0 ) {
-			if(log.isInfoEnabled())
+            if(log.isInfoEnabled())
                 log.info("Setting cluster mcast TTL to " + mcastTTL);
             socket.setTimeToLive(mcastTTL);
         }
     }
-    
-    
+
 
     /**
      * Start the service
@@ -230,7 +249,12 @@ public class McastServiceImpl
         boolean valid = false;
         if ( (level & Channel.MBR_RX_SEQ)==Channel.MBR_RX_SEQ ) {
             if ( receiver != null ) throw new IllegalStateException("McastService.receive already running.");
-            if ( sender == null ) socket.joinGroup(address);
+            try {
+                if ( sender == null ) socket.joinGroup(address);
+            }catch (IOException iox) {
+                log.error("Unable to join multicast group, make sure your system has multicasting enabled.");
+                throw iox;
+            }
             doRunReceiver = true;
             receiver = new ReceiverThread();
             receiver.setDaemon(true);
@@ -243,7 +267,6 @@ public class McastServiceImpl
             //make sure at least one packet gets out there
             send(false);
             doRunSender = true;
-            serviceStartTime = System.currentTimeMillis();
             sender = new SenderThread(sendFrequency);
             sender.setDaemon(true);
             sender.start();
@@ -300,7 +323,7 @@ public class McastServiceImpl
             //leave mcast group
             try {socket.leaveGroup(address);}catch ( Exception ignore){}
             try {socket.close();}catch ( Exception ignore){}
-            serviceStartTime = Long.MAX_VALUE;
+            member.setServiceStartTime(-1);
         }
         return (startLevel == 0);
     }
@@ -310,57 +333,137 @@ public class McastServiceImpl
      * @throws IOException
      */
     public void receive() throws IOException {
+        boolean checkexpired = true;
         try {
+            
             socket.receive(receivePacket);
             if(receivePacket.getLength() > MAX_PACKET_SIZE) {
                 log.error("Multicast packet received was too long, dropping package:"+receivePacket.getLength());
             } else {
                 byte[] data = new byte[receivePacket.getLength()];
                 System.arraycopy(receivePacket.getData(), receivePacket.getOffset(), data, 0, data.length);
-                final MemberImpl m = MemberImpl.getMember(data);
-                if (log.isTraceEnabled()) log.trace("Mcast receive ping from member " + m);
-                Thread t = null;
-                if (Arrays.equals(m.getCommand(), Member.SHUTDOWN_PAYLOAD)) {
-                    if (log.isDebugEnabled()) log.debug("Member has shutdown:" + m);
-                    membership.removeMember(m);
-                    t = new Thread() {
-                        public void run() {
-                            service.memberDisappeared(m);
-                        }
-                    };
-                } else if (membership.memberAlive(m)) {
-                    if (log.isDebugEnabled()) log.debug("Mcast add member " + m);
-                    t = new Thread() {
-                        public void run() {
-                            service.memberAdded(m);
-                        }
-                    };
-                } //end if
-                if ( t != null ) t.start();
+                if (XByteBuffer.firstIndexOf(data,0,MemberImpl.TRIBES_MBR_BEGIN)==0) {
+                    memberDataReceived(data);
+                } else {
+                    memberBroadcastsReceived(data);
+                }
+                
             }
         } catch (SocketTimeoutException x ) { 
             //do nothing, this is normal, we don't want to block forever
             //since the receive thread is the same thread
             //that does membership expiration
         }
-        checkExpired();
+        if (checkexpired) checkExpired();
+    }
+
+    private void memberDataReceived(byte[] data) {
+        final MemberImpl m = MemberImpl.getMember(data);
+        if (log.isTraceEnabled()) log.trace("Mcast receive ping from member " + m);
+        Runnable t = null;
+        if (Arrays.equals(m.getCommand(), Member.SHUTDOWN_PAYLOAD)) {
+            if (log.isDebugEnabled()) log.debug("Member has shutdown:" + m);
+            membership.removeMember(m);
+            t = new Runnable() {
+                @Override
+                public void run() {
+                    String name = Thread.currentThread().getName();
+                    try {
+                        Thread.currentThread().setName("Membership-MemberDisappeared.");
+                        service.memberDisappeared(m);
+                    }finally {
+                        Thread.currentThread().setName(name);
+                    }
+                }
+            };
+        } else if (membership.memberAlive(m)) {
+            if (log.isDebugEnabled()) log.debug("Mcast add member " + m);
+            t = new Runnable() {
+                @Override
+                public void run() {
+                    String name = Thread.currentThread().getName();
+                    try {
+                        Thread.currentThread().setName("Membership-MemberAdded.");
+                        service.memberAdded(m);
+                    }finally {
+                        Thread.currentThread().setName(name);
+                    }
+                }
+            };
+        } //end if
+        if ( t != null ) {
+            executor.execute(t);
+        }
     }
     
-    protected Object expiredMutex = new Object();
+    private void memberBroadcastsReceived(final byte[] b) {
+        if (log.isTraceEnabled()) log.trace("Mcast received broadcasts.");
+        XByteBuffer buffer = new XByteBuffer(b,true);
+        if (buffer.countPackages(true)>0) {
+            int count = buffer.countPackages();
+            final ChannelData[] data = new ChannelData[count];
+            for (int i=0; i<count; i++) {
+                try {
+                    data[i] = buffer.extractPackage(true);
+                }catch (IllegalStateException ise) {
+                    log.debug("Unable to decode message.",ise);
+                }catch (IOException x) {
+                    log.debug("Unable to decode message.",x);
+                }
+            }
+            Runnable t = new Runnable() {
+                @Override
+                public void run() {
+                    String name = Thread.currentThread().getName();
+                    try {
+                        Thread.currentThread().setName("Membership-MemberAdded.");
+                        for (int i=0; i<data.length; i++ ) {
+                            try {
+                                if (data[i]!=null && !member.equals(data[i].getAddress())) {
+                                    msgservice.messageReceived(data[i]);
+                                }
+                            } catch (Throwable t) {
+                                if (t instanceof ThreadDeath) {
+                                    throw (ThreadDeath) t;
+                                }
+                                if (t instanceof VirtualMachineError) {
+                                    throw (VirtualMachineError) t;
+                                }
+                                log.error("Unable to receive broadcast message.",t);
+                            }
+                        }
+                    }finally {
+                        Thread.currentThread().setName(name);
+                    }
+                }
+            };
+            executor.execute(t);
+        }
+    }
+
+    protected final Object expiredMutex = new Object();
     protected void checkExpired() {
         synchronized (expiredMutex) {
             MemberImpl[] expired = membership.expire(timeToExpiration);
             for (int i = 0; i < expired.length; i++) {
                 final MemberImpl member = expired[i];
                 if (log.isDebugEnabled())
-                    log.debug("Mcast exipre  member " + expired[i]);
+                    log.debug("Mcast expire  member " + expired[i]);
                 try {
-                    Thread t = new Thread() {
+                    Runnable t = new Runnable() {
+                        @Override
                         public void run() {
-                            service.memberDisappeared(member);
+                            String name = Thread.currentThread().getName();
+                            try {
+                                Thread.currentThread().setName("Membership-MemberExpired.");
+                                service.memberDisappeared(member);
+                            }finally {
+                                Thread.currentThread().setName(name);
+                            }
+                            
                         }
                     };
-                    t.start();
+                    executor.execute(t);
                 } catch (Exception x) {
                     log.error("Unable to process member disappeared message.", x);
                 }
@@ -370,24 +473,39 @@ public class McastServiceImpl
 
     /**
      * Send a ping
-     * @throws Exception
+     * @throws IOException
      */ 
     public void send(boolean checkexpired) throws IOException{
+        send(checkexpired,null);
+    }
+    
+    private final Object sendLock = new Object();
+
+    public void send(boolean checkexpired, DatagramPacket packet) throws IOException{
+        checkexpired = (checkexpired && (packet==null));
         //ignore if we haven't started the sender
         //if ( (startLevel&Channel.MBR_TX_SEQ) != Channel.MBR_TX_SEQ ) return;
-        member.inc();
-        if(log.isTraceEnabled())
-            log.trace("Mcast send ping from member " + member);
-        byte[] data = member.getData();
-        DatagramPacket p = new DatagramPacket(data,data.length);
-        p.setAddress(address);
-        p.setPort(port);
-        socket.send(p);
+        if (packet==null) {
+            member.inc();
+            if(log.isTraceEnabled()) {
+                log.trace("Mcast send ping from member " + member);
+            }
+            byte[] data = member.getData();
+            packet = new DatagramPacket(data,data.length);
+        } else if (log.isTraceEnabled()) {
+            log.trace("Sending message broadcast "+packet.getLength()+ " bytes from "+ member);
+        }
+        packet.setAddress(address);
+        packet.setPort(port);
+        //TODO this operation is not thread safe
+        synchronized (sendLock) {
+            socket.send(packet);
+        }
         if ( checkexpired ) checkExpired();
     }
 
     public long getServiceStartTime() {
-       return this.serviceStartTime;
+        return (member!=null) ? member.getServiceStartTime() : -1l;
     }
 
     public int getRecoveryCounter() {
@@ -408,6 +526,7 @@ public class McastServiceImpl
             super();
             setName("Tribes-MembershipReceiver");
         }
+        @Override
         public void run() {
             while ( doRunReceiver ) {
                 try {
@@ -419,12 +538,17 @@ public class McastServiceImpl
                     if ( log.isDebugEnabled() )
                         log.debug("Invalid member mcast package.",ax);
                 } catch ( Exception x ) {
-                    if (errorCounter==0) log.warn("Error receiving mcast package. Sleeping 500ms",x);
-                    else log.debug("Error receiving mcast package. Sleeping 500ms",x);
-                    try { Thread.sleep(500); } catch ( Exception ignore ){}
-                    if ( (++errorCounter)>=recoveryCounter ) {
-                        errorCounter=0;
-                        new RecoveryThread(McastServiceImpl.this);
+                    if (x instanceof InterruptedException) interrupted();
+                    else {
+                        if (errorCounter==0 && doRunReceiver) log.warn("Error receiving mcast package. Sleeping 500ms",x);
+                        else if (log.isDebugEnabled()) log.debug("Error receiving mcast package"+(doRunReceiver?". Sleeping 500ms":"."),x);
+                        if (doRunReceiver) {
+                            try { Thread.sleep(500); } catch ( Exception ignore ){}
+                            if ( (++errorCounter)>=recoveryCounter ) {
+                                errorCounter=0;
+                                RecoveryThread.recover(McastServiceImpl.this);
+                            }
+                        }
                     }
                 }
             }
@@ -439,6 +563,7 @@ public class McastServiceImpl
             setName("Tribes-MembershipSender");
 
         }
+        @Override
         public void run() {
             while ( doRunSender ) {
                 try {
@@ -449,7 +574,7 @@ public class McastServiceImpl
                     else log.debug("Unable to send mcast message.",x);
                     if ( (++errorCounter)>=recoveryCounter ) {
                         errorCounter=0;
-                        new RecoveryThread(McastServiceImpl.this);
+                        RecoveryThread.recover(McastServiceImpl.this);
                     }
                 }
                 try { Thread.sleep(time); } catch ( Exception ignore ) {}
@@ -458,23 +583,28 @@ public class McastServiceImpl
     }//class SenderThread
 
     protected static class RecoveryThread extends Thread {
-        static boolean running = false;
-        McastServiceImpl parent = null;
-        public RecoveryThread(McastServiceImpl parent) {
-            this.parent = parent;
-            if (!init(this)) parent = null;
-        }
-        
-        public static synchronized boolean init(RecoveryThread t) {
-            if ( running ) return false;
-            if ( !t.parent.isRecoveryEnabled()) return false;
+        static volatile boolean running = false;
+
+        public static synchronized void recover(McastServiceImpl parent) {
+            if (running) return;
+            if (!parent.isRecoveryEnabled())
+                return;
+            
             running = true;
+            
+            Thread t = new RecoveryThread(parent);
+            
             t.setName("Tribes-MembershipRecovery");
             t.setDaemon(true);
             t.start();
-            return true;
         }
 
+
+        McastServiceImpl parent = null;
+        public RecoveryThread(McastServiceImpl parent) {
+            this.parent = parent;
+        }
+        
         public boolean stopService() {
             try {
                 parent.stop(Channel.MBR_RX_SEQ | Channel.MBR_TX_SEQ);
@@ -494,6 +624,7 @@ public class McastServiceImpl
                 return false;
             }
         }
+        @Override
         public void run() {
             boolean success = false;
             int attempt = 0;
