@@ -28,7 +28,6 @@ import org.apache.catalina.tribes.RemoteProcessException;
 import org.apache.catalina.tribes.io.XByteBuffer;
 import org.apache.catalina.tribes.transport.AbstractSender;
 import org.apache.catalina.tribes.transport.Constants;
-import org.apache.catalina.tribes.transport.DataSender;
 import org.apache.catalina.tribes.transport.SenderState;
 import org.apache.catalina.tribes.util.StringManager;
 
@@ -38,17 +37,16 @@ import org.apache.catalina.tribes.util.StringManager;
  * 
  * @author Peter Rossbach
  * @author Filip Hanik
- * @version $Id: BioSender.java 939515 2010-04-29 23:59:49Z kkolinko $
  * @since 5.5.16
  */
-public class BioSender extends AbstractSender implements DataSender {
+public class BioSender extends AbstractSender {
 
-    private static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(BioSender.class);
+    private static final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(BioSender.class);
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -71,6 +69,7 @@ public class BioSender extends AbstractSender implements DataSender {
     // ------------------------------------------------------------- Constructor
     
     public BioSender()  {
+        // NO-OP
     }
 
 
@@ -89,8 +88,9 @@ public class BioSender extends AbstractSender implements DataSender {
 
     /**
      * Connect other cluster member receiver 
-     * @see org.apache.catalina.tribes.transport.IDataSender#connect()
+     * @see org.apache.catalina.tribes.transport.DataSender#connect()
      */
+    @Override
     public  void connect() throws IOException {
         openSocket();
    }
@@ -99,8 +99,9 @@ public class BioSender extends AbstractSender implements DataSender {
     /**
      * disconnect and close socket
      * 
-     * @see IDataSender#disconnect()
+     * @see org.apache.catalina.tribes.transport.DataSender#disconnect()
      */
+    @Override
     public  void disconnect() {
         boolean connect = isConnected();
         closeSocket();
@@ -112,10 +113,7 @@ public class BioSender extends AbstractSender implements DataSender {
     }
 
     /**
-     * Send message
-     * 
-     * @see org.apache.catalina.tribes.transport.IDataSender#sendMessage(,
-     *      ChannelMessage)
+     * Send message.
      */
     public  void sendMessage(byte[] data, boolean waitForAck) throws IOException {
         IOException exception = null;
@@ -149,8 +147,9 @@ public class BioSender extends AbstractSender implements DataSender {
     /**
      * Name of this SockerSender
      */
+    @Override
     public String toString() {
-        StringBuffer buf = new StringBuffer("DataSender[(");
+        StringBuilder buf = new StringBuilder("DataSender[(");
         buf.append(super.toString()).append(")");
         buf.append(getAddress()).append(":").append(getPort()).append("]");
         return buf.toString();
@@ -194,10 +193,9 @@ public class BioSender extends AbstractSender implements DataSender {
      }
 
     /**
-     * close socket
+     * Close socket.
      * 
-     * @see DataSender#disconnect()
-     * @see DataSender#closeSocket()
+     * @see #disconnect()
      */
     protected void closeSocket() {
         if(isConnected()) {
@@ -205,6 +203,7 @@ public class BioSender extends AbstractSender implements DataSender {
                 try {
                     socket.close();
                 } catch (IOException x) {
+                    // Ignore
                 } finally {
                     socket = null;
                     soOut = null;
@@ -223,13 +222,13 @@ public class BioSender extends AbstractSender implements DataSender {
      * Wait for ack is needed and make auto retry when write message is failed.
      * After sending error close and reopen socket again.
      * 
-     * After successfull sending update stats
+     * After successful sending update stats
      * 
-     * WARNING: Subclasses must be very carefull that only one thread call this pushMessage at once!!!
+     * WARNING: Subclasses must be very careful that only one thread call this pushMessage at once!!!
      * 
      * @see #closeSocket()
      * @see #openSocket()
-     * @see #writeData(ChannelMessage)
+     * @see #sendMessage(byte[], boolean)
      * 
      * @param data
      *            data to send
@@ -248,9 +247,8 @@ public class BioSender extends AbstractSender implements DataSender {
     }
     
     /**
-     * Wait for Acknowledgement from other server
-     * FIXME Please, not wait only for three charcters, better control that the wait ack message is correct.
-     * @param timeout
+     * Wait for Acknowledgement from other server.
+     * FIXME Please, not wait only for three characters, better control that the wait ack message is correct.
      * @throws java.io.IOException
      * @throws java.net.SocketTimeoutException
      */
