@@ -14,11 +14,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.users;
 
-
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -35,23 +32,19 @@ import org.apache.catalina.User;
 import org.apache.catalina.UserDatabase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.digester.AbstractObjectCreationFactory;
 import org.apache.tomcat.util.digester.Digester;
 import org.apache.tomcat.util.res.StringManager;
 import org.xml.sax.Attributes;
 
-
 /**
  * <p>Concrete implementation of {@link UserDatabase} that loads all
  * defined users, groups, and roles into an in-memory data structure,
  * and uses a specified XML file for its persistent storage.</p>
  *
  * @author Craig R. McClanahan
- * @version $Id: MemoryUserDatabase.java 1140070 2011-06-27 09:27:06Z markt $
  * @since 4.1
  */
-
 public class MemoryUserDatabase implements UserDatabase {
 
 
@@ -64,9 +57,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * Create a new instance with default values.
      */
     public MemoryUserDatabase() {
-
-        super();
-
+        this(null);
     }
 
 
@@ -76,10 +67,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * @param id Unique global identifier of this user database
      */
     public MemoryUserDatabase(String id) {
-
-        super();
         this.id = id;
-
     }
 
 
@@ -90,13 +78,13 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link Group}s defined in this database, keyed by
      * group name.
      */
-    protected HashMap<String,Group> groups = new HashMap<String,Group>();
+    protected final HashMap<String,Group> groups = new HashMap<String,Group>();
 
 
     /**
      * The unique global identifier of this user database.
      */
-    protected String id = null;
+    protected final String id;
 
 
     /**
@@ -129,7 +117,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link Role}s defined in this database, keyed by
      * role name.
      */
-    protected HashMap<String,Role> roles = new HashMap<String,Role>();
+    protected final HashMap<String,Role> roles = new HashMap<String,Role>();
 
 
     /**
@@ -143,7 +131,7 @@ public class MemoryUserDatabase implements UserDatabase {
      * The set of {@link User}s defined in this database, keyed by
      * user name.
      */
-    protected HashMap<String,User> users = new HashMap<String,User>();
+    protected final HashMap<String,User> users = new HashMap<String,User>();
 
 
     // ------------------------------------------------------------- Properties
@@ -413,9 +401,10 @@ public class MemoryUserDatabase implements UserDatabase {
                                     pathname);
                 }
                 if (!file.exists()) {
+                    log.error(sm.getString("memoryUserDatabase.fileNotFound",
+                            file.getAbsolutePath()));
                     return;
                 }
-                FileInputStream fis = new FileInputStream(file);
 
                 // Construct a digester to read the XML input file
                 Digester digester = new Digester();
@@ -437,16 +426,18 @@ public class MemoryUserDatabase implements UserDatabase {
                      new MemoryUserCreationFactory(this), true);
 
                 // Parse the XML input file to load this database
+                FileInputStream fis = null;
                 try {
+                    fis =  new FileInputStream(file);
                     digester.parse(fis);
-                    fis.close();
-                } catch (Exception e) {
-                    try {
-                        fis.close();
-                    } catch (Throwable t) {
-                        ExceptionUtils.handleThrowable(t);
+                } finally {
+                    if (fis != null) {
+                        try {
+                            fis.close();
+                        } catch (IOException ioe) {
+                            // Ignore
+                        }
                     }
-                    throw e;
                 }
 
             }
