@@ -23,6 +23,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.Locale;
 import java.util.NoSuchElementException;
 import java.util.StringTokenizer;
 import java.util.jar.JarInputStream;
@@ -33,36 +34,38 @@ import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
 import javax.naming.directory.DirContext;
 
-import org.apache.catalina.core.StandardContext;
+import org.apache.catalina.Context;
 import org.apache.naming.resources.Resource;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
- * Ensures that all extension dependies are resolved for a WEB application
+ * Ensures that all extension dependencies are resolved for a WEB application
  * are met. This class builds a master list of extensions available to an
- * applicaiton and then validates those extensions.
+ * application and then validates those extensions.
  *
- * See http://java.sun.com/j2se/1.4/docs/guide/extensions/spec.html for
- * a detailed explanation of the extension mechanism in Java.
+ * See http://docs.oracle.com/javase/1.4.2/docs/guide/extensions/spec.html
+ * for a detailed explanation of the extension mechanism in Java.
  *
  * @author Greg Murray
  * @author Justyna Horwat
- * @version $Id: ExtensionValidator.java 939353 2010-04-29 15:50:43Z kkolinko $
- *
  */
 public final class ExtensionValidator {
 
-    private static org.apache.juli.logging.Log log=
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog(ExtensionValidator.class);
 
     /**
      * The string resources for this package.
      */
-    private static StringManager sm =
+    private static final StringManager sm =
         StringManager.getManager("org.apache.catalina.util");
     
-    private static ArrayList containerAvailableExtensions = null;
-    private static ArrayList containerManifestResources = new ArrayList();
+    private static volatile ArrayList<Extension> containerAvailableExtensions =
+        null;
+    private static ArrayList<ManifestResource> containerManifestResources =
+        new ArrayList<ManifestResource>();
 
 
     // ----------------------------------------------------- Static Initializer
@@ -87,7 +90,7 @@ public final class ExtensionValidator {
         // build a list of jar files in the classpath
         while (strTok.hasMoreTokens()) {
             String classpathItem = strTok.nextToken();
-            if (classpathItem.toLowerCase().endsWith(".jar")) {
+            if (classpathItem.toLowerCase(Locale.ENGLISH).endsWith(".jar")) {
                 File item = new File(classpathItem);
                 if (item.isFile()) {
                     try {
@@ -102,8 +105,6 @@ public final class ExtensionValidator {
 
         // add specified folders to the list
         addFolderList("java.ext.dirs");
-        addFolderList("catalina.ext.dirs");
-
     }
 
 
@@ -111,7 +112,7 @@ public final class ExtensionValidator {
 
 
     /**
-     * Runtime validation of a Web Applicaiton.
+     * Runtime validation of a Web Application.
      *
      * This method uses JNDI to look up the resources located under a 
      * <code>DirContext</code>. It locates Web Application MANIFEST.MF 
@@ -129,20 +130,22 @@ public final class ExtensionValidator {
      */
     public static synchronized boolean validateApplication(
                                            DirContext dirContext, 
-                                           StandardContext context)
+                                           Context context)
                     throws IOException {
 
-        String appName = context.getPath();
-        ArrayList appManifestResources = new ArrayList();
+        String appName = context.getName();
+        ArrayList<ManifestResource> appManifestResources =
+            new ArrayList<ManifestResource>();
         // If the application context is null it does not exist and 
         // therefore is not valid
         if (dirContext == null) return false;
-        // Find the Manifest for the Web Applicaiton
+        // Find the Manifest for the Web Application
         InputStream inputStream = null;
         try {
-            NamingEnumeration wne = dirContext.listBindings("/META-INF/");
-            Binding binding = (Binding) wne.nextElement();
-            if (binding.getName().toUpperCase().equals("MANIFEST.MF")) {
+            NamingEnumeration<Binding> wne =
+                dirContext.listBindings("/META-INF/");
+            Binding binding = wne.nextElement();
+            if (binding.getName().toUpperCase(Locale.ENGLISH).equals("MANIFEST.MF")) {
                 Resource resource = (Resource)dirContext.lookup
                                     ("/META-INF/" + binding.getName());
                 inputStream = resource.streamContent();
@@ -163,41 +166,51 @@ public final class ExtensionValidator {
                 try {
                     inputStream.close();
                 } catch (Throwable t) {
-                    // Ignore
+                    ExceptionUtils.handleThrowable(t);
                 }
             }
         }
 
         // Locate the Manifests for all bundled JARs
-        NamingEnumeration ne = null;
+        NamingEnumeration<Binding> ne = null;
+        // Primarily used for error reporting
+        String jarName = null;
         try {
-            if (dirContext != null) {
-                ne = dirContext.listBindings("WEB-INF/lib/");
-            }
+            ne = dirContext.listBindings("WEB-INF/lib/");
             while ((ne != null) && ne.hasMoreElements()) {
-                Binding binding = (Binding)ne.nextElement();
-                if (!binding.getName().toLowerCase().endsWith(".jar")) {
+                Binding binding = ne.nextElement();
+                jarName = binding.getName();
+                if (!jarName.toLowerCase(Locale.ENGLISH).endsWith(".jar")) {
                     continue;
                 }
                 Object obj =
-                    dirContext.lookup("/WEB-INF/lib/" + binding.getName());
+                    dirContext.lookup("/WEB-INF/lib/" + jarName);
                 if (!(obj instanceof Resource)) {
                     // Probably a directory named xxx.jar - ignore it
                     continue;
                 }
                 Resource resource = (Resource) obj;
-                Manifest jmanifest = getManifest(resource.streamContent());
+                inputStream = resource.streamContent();
+                Manifest jmanifest = getManifest(inputStream);
                 if (jmanifest != null) {
-                    ManifestResource mre = new ManifestResource(
-                                                binding.getName(),
-                                                jmanifest, 
-                                                ManifestResource.APPLICATION);
+                    ManifestResource mre = new ManifestResource(jarName,
+                            jmanifest, ManifestResource.APPLICATION);
                     appManifestResources.add(mre);
                 }
             }
         } catch (NamingException nex) {
             // Jump out of the check for this application because it 
             // has no resources
+        } catch (IOException ioe) {
+            throw new IOException("Jar: " + jarName, ioe);
+        } finally {
+            if (inputStream != null) {
+                try {
+                    inputStream.close();
+                } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
+                }
+            }
         }
 
         return validateManifestResources(appName, appManifestResources);
@@ -234,7 +247,7 @@ public final class ExtensionValidator {
      * represented by any given <code>ManifestResource</code> objects 
      * is not met.
      *
-     * This method should also provide static validation of a Web Applicaiton 
+     * This method should also provide static validation of a Web Application 
      * if provided with the necessary parameters.
      *
      * @param appName The name of the Application that will appear in the 
@@ -244,16 +257,16 @@ public final class ExtensionValidator {
      *
      * @return true if manifest resource file requirements are met
      */
-    private static boolean validateManifestResources(String appName, 
-                                                     ArrayList resources) {
+    private static boolean validateManifestResources(String appName,
+            ArrayList<ManifestResource> resources) {
         boolean passes = true;
         int failureCount = 0;        
-        ArrayList availableExtensions = null;
+        ArrayList<Extension> availableExtensions = null;
 
-        Iterator it = resources.iterator();
+        Iterator<ManifestResource> it = resources.iterator();
         while (it.hasNext()) {
-            ManifestResource mre = (ManifestResource)it.next();
-            ArrayList requiredList = mre.getRequiredExtensions();
+            ManifestResource mre = it.next();
+            ArrayList<Extension> requiredList = mre.getRequiredExtensions();
             if (requiredList == null) {
                 continue;
             }
@@ -271,15 +284,15 @@ public final class ExtensionValidator {
             }
 
             // iterate through the list of required extensions
-            Iterator rit = requiredList.iterator();
+            Iterator<Extension> rit = requiredList.iterator();
             while (rit.hasNext()) {
                 boolean found = false;
-                Extension requiredExt = (Extension)rit.next();
-                // check the applicaion itself for the extension
+                Extension requiredExt = rit.next();
+                // check the application itself for the extension
                 if (availableExtensions != null) {
-                    Iterator ait = availableExtensions.iterator();
+                    Iterator<Extension> ait = availableExtensions.iterator();
                     while (ait.hasNext()) {
-                        Extension targetExt = (Extension) ait.next();
+                        Extension targetExt = ait.next();
                         if (targetExt.isCompatibleWith(requiredExt)) {
                             requiredExt.setFulfilled(true);
                             found = true;
@@ -289,9 +302,10 @@ public final class ExtensionValidator {
                 }
                 // check the container level list for the extension
                 if (!found && containerAvailableExtensions != null) {
-                    Iterator cit = containerAvailableExtensions.iterator();
+                    Iterator<Extension> cit =
+                        containerAvailableExtensions.iterator();
                     while (cit.hasNext()) {
-                        Extension targetExt = (Extension) cit.next();
+                        Extension targetExt = cit.next();
                         if (targetExt.isCompatibleWith(requiredExt)) {
                             requiredExt.setFulfilled(true);
                             found = true;
@@ -337,20 +351,21 @@ public final class ExtensionValidator {
     *
     * @return HashMap Map of available extensions
     */
-    private static ArrayList buildAvailableExtensionsList(ArrayList resources) {
+    private static ArrayList<Extension> buildAvailableExtensionsList(
+            ArrayList<ManifestResource> resources) {
 
-        ArrayList availableList = null;
+        ArrayList<Extension> availableList = null;
 
-        Iterator it = resources.iterator();
+        Iterator<ManifestResource> it = resources.iterator();
         while (it.hasNext()) {
-            ManifestResource mre = (ManifestResource)it.next();
-            ArrayList list = mre.getAvailableExtensions();
+            ManifestResource mre = it.next();
+            ArrayList<Extension> list = mre.getAvailableExtensions();
             if (list != null) {
-                Iterator values = list.iterator();
+                Iterator<Extension> values = list.iterator();
                 while (values.hasNext()) {
-                    Extension ext = (Extension) values.next();
+                    Extension ext = values.next();
                     if (availableList == null) {
-                        availableList = new ArrayList();
+                        availableList = new ArrayList<Extension>();
                         availableList.add(ext);
                     } else {
                         availableList.add(ext);
@@ -384,7 +399,7 @@ public final class ExtensionValidator {
                 try {
                     jin.close();
                 } catch (Throwable t) {
-                    // Ignore
+                    ExceptionUtils.handleThrowable(t);
                 }
             }
         }
@@ -410,7 +425,7 @@ public final class ExtensionValidator {
                 }
                 File[] files = targetDir.listFiles();
                 for (int i = 0; i < files.length; i++) {
-                    if (files[i].getName().toLowerCase().endsWith(".jar") &&
+                    if (files[i].getName().toLowerCase(Locale.ENGLISH).endsWith(".jar") &&
                             files[i].isFile()) {
                         try {
                             addSystemResource(files[i]);
