@@ -27,6 +27,7 @@ import java.nio.charset.UnmappableCharacterException;
 import org.apache.coyote.http11.upgrade.UpgradeInbound;
 import org.apache.coyote.http11.upgrade.UpgradeOutbound;
 import org.apache.coyote.http11.upgrade.UpgradeProcessor;
+import org.apache.tomcat.util.buf.Utf8Decoder;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 
 /**
@@ -34,7 +35,11 @@ import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
  * on streams. Applications should extend this class to provide application
  * specific functionality. Applications that wish to operate on a message basis
  * rather than a stream basis should use {@link MessageInbound}.
+ * 
+ * @deprecated  Replaced by the JSR356 WebSocket 1.0 implementation and will be
+ *              removed in Tomcat 8.0.x.  
  */
+@Deprecated
 public abstract class StreamInbound implements UpgradeInbound {
 
     private final ClassLoader applicationClassLoader;
@@ -87,7 +92,7 @@ public abstract class StreamInbound implements UpgradeInbound {
 
     @Override
     public final void setUpgradeOutbound(UpgradeOutbound upgradeOutbound) {
-        outbound = new WsOutbound(upgradeOutbound, outboundByteBufferSize,
+        outbound = new WsOutbound(upgradeOutbound, this, outboundByteBufferSize,
                 outboundCharBufferSize);
     }
 
@@ -206,7 +211,13 @@ public abstract class StreamInbound implements UpgradeInbound {
         }
     }
 
-    private void doOnClose(int status) {
+    /**
+     * Package private so the outbound connection can signal that the connection
+     * has been closed - usually due to an error.
+     *  
+     * @param status    The WebSocket status code to report to the application
+     */
+    void doOnClose(int status) {
         // Need to call onClose using the web application's class loader
         Thread t = Thread.currentThread();
         ClassLoader cl = t.getContextClassLoader();
