@@ -24,12 +24,17 @@ import java.nio.charset.CoderResult;
 
 import org.apache.catalina.util.Conversions;
 import org.apache.coyote.http11.upgrade.UpgradeProcessor;
+import org.apache.tomcat.util.buf.Utf8Decoder;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Represents a complete WebSocket frame with the exception of the payload for
  * non-control frames.
+ * 
+ * @deprecated  Replaced by the JSR356 WebSocket 1.0 implementation and will be
+ *              removed in Tomcat 8.0.x.  
  */
+@Deprecated
 public class WsFrame {
 
     private static final StringManager sm =
