@@ -31,14 +31,26 @@ import org.apache.tomcat.util.res.StringManager;
  * that write to the client (or update a buffer that is later written to the
  * client) are synchronized to prevent multiple threads trying to write to the
  * client at the same time.
+ * 
+ * @deprecated  Replaced by the JSR356 WebSocket 1.0 implementation and will be
+ *              removed in Tomcat 8.0.x.  
  */
+@Deprecated
 public class WsOutbound {
 
     private static final StringManager sm =
             StringManager.getManager(Constants.Package);
     public static final int DEFAULT_BUFFER_SIZE = 8192;
 
+    /**
+     * This state lock is used rather than synchronized methods to allow error
+     * handling to be managed outside of the synchronization else deadlocks may
+     * occur such as https://issues.apache.org/bugzilla/show_bug.cgi?id=55524
+     */
+    private final Object stateLock = new Object();
+    
     private UpgradeOutbound upgradeOutbound;
+    private StreamInbound streamInbound;
     private ByteBuffer bb;
     private CharBuffer cb;
     private boolean closed = false;
@@ -46,14 +58,17 @@ public class WsOutbound {
     private boolean firstFrame = true;
 
 
-    public WsOutbound(UpgradeOutbound upgradeOutbound) {
-        this(upgradeOutbound, DEFAULT_BUFFER_SIZE, DEFAULT_BUFFER_SIZE);
+    public WsOutbound(UpgradeOutbound upgradeOutbound,
+            StreamInbound streamInbound) {
+        this(upgradeOutbound, streamInbound, DEFAULT_BUFFER_SIZE,
+                DEFAULT_BUFFER_SIZE);
     }
 
 
-    public WsOutbound(UpgradeOutbound upgradeOutbound, int byteBufferSize,
-            int charBufferSize) {
+    public WsOutbound(UpgradeOutbound upgradeOutbound, StreamInbound streamInbound,
+            int byteBufferSize, int charBufferSize) {
         this.upgradeOutbound = upgradeOutbound;
+        this.streamInbound = streamInbound;
         this.bb = ByteBuffer.allocate(byteBufferSize);
         this.cb = CharBuffer.allocate(charBufferSize);
     }
@@ -71,22 +86,34 @@ public class WsOutbound {
      * @throws IOException  If a flush is required and an error occurs writing
      *                      the WebSocket frame to the client
      */
-    public synchronized void writeBinaryData(int b) throws IOException {
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+    public void writeBinaryData(int b) throws IOException {
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+        
+                if (bb.position() == bb.capacity()) {
+                    doFlush(false);
+                }
+                if (text == null) {
+                    text = Boolean.FALSE;
+                } else if (text == Boolean.TRUE) {
+                    // Flush the character data
+                    flush();
+                    text = Boolean.FALSE;
+                }
+                bb.put((byte) (b & 0xFF));
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-
-        if (bb.position() == bb.capacity()) {
-            doFlush(false);
-        }
-        if (text == null) {
-            text = Boolean.FALSE;
-        } else if (text == Boolean.TRUE) {
-            // Flush the character data
-            flush();
-            text = Boolean.FALSE;
-        }
-        bb.put((byte) (b & 0xFF));
     }
 
 
@@ -101,23 +128,35 @@ public class WsOutbound {
      * @throws IOException  If a flush is required and an error occurs writing
      *                      the WebSocket frame to the client
      */
-    public synchronized void writeTextData(char c) throws IOException {
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+    public void writeTextData(char c) throws IOException {
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+        
+                if (cb.position() == cb.capacity()) {
+                    doFlush(false);
+                }
+        
+                if (text == null) {
+                    text = Boolean.TRUE;
+                } else if (text == Boolean.FALSE) {
+                    // Flush the binary data
+                    flush();
+                    text = Boolean.TRUE;
+                }
+                cb.append(c);
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-
-        if (cb.position() == cb.capacity()) {
-            doFlush(false);
-        }
-
-        if (text == null) {
-            text = Boolean.TRUE;
-        } else if (text == Boolean.FALSE) {
-            // Flush the binary data
-            flush();
-            text = Boolean.TRUE;
-        }
-        cb.append(c);
     }
 
 
@@ -130,19 +169,30 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void writeBinaryMessage(ByteBuffer msgBb)
-            throws IOException {
+    public void writeBinaryMessage(ByteBuffer msgBb) throws IOException {
 
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+        
+                if (text != null) {
+                    // Empty the buffer
+                    flush();
+                }
+                text = Boolean.FALSE;
+                doWriteBytes(msgBb, true);
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-
-        if (text != null) {
-            // Empty the buffer
-            flush();
-        }
-        text = Boolean.FALSE;
-        doWriteBytes(msgBb, true);
     }
 
 
@@ -155,19 +205,30 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void writeTextMessage(CharBuffer msgCb)
-            throws IOException {
+    public void writeTextMessage(CharBuffer msgCb) throws IOException {
 
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+        
+                if (text != null) {
+                    // Empty the buffer
+                    flush();
+                }
+                text = Boolean.TRUE;
+                doWriteText(msgCb, true);
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-
-        if (text != null) {
-            // Empty the buffer
-            flush();
-        }
-        text = Boolean.TRUE;
-        doWriteText(msgCb, true);
     }
 
 
@@ -176,11 +237,23 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void flush() throws IOException {
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+    public void flush() throws IOException {
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+                doFlush(true);
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-        doFlush(true);
     }
 
 
@@ -263,33 +336,50 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void close(int status, ByteBuffer data)
-            throws IOException {
+    public void close(int status, ByteBuffer data) throws IOException {
 
-        if (closed) {
-            return;
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    return;
+                }
+        
+                // Send any partial data we have
+                try {
+                    doFlush(false);
+                } finally {
+                    closed = true;
+                }
+        
+                upgradeOutbound.write(0x88);
+                if (status == 0) {
+                    upgradeOutbound.write(0);
+                } else if (data == null || data.position() == data.limit()) {
+                    upgradeOutbound.write(2);
+                    upgradeOutbound.write(status >>> 8);
+                    upgradeOutbound.write(status);
+                } else {
+                    upgradeOutbound.write(2 + data.limit() - data.position());
+                    upgradeOutbound.write(status >>> 8);
+                    upgradeOutbound.write(status);
+                    upgradeOutbound.write(data.array(), data.position(),
+                            data.limit() - data.position());
+                }
+                upgradeOutbound.flush();
+        
+                bb = null;
+                cb = null;
+                upgradeOutbound = null;
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-        closed = true;
-
-        upgradeOutbound.write(0x88);
-        if (status == 0) {
-            upgradeOutbound.write(0);
-        } else if (data == null || data.position() == data.limit()) {
-            upgradeOutbound.write(2);
-            upgradeOutbound.write(status >>> 8);
-            upgradeOutbound.write(status);
-        } else {
-            upgradeOutbound.write(2 + data.limit() - data.position());
-            upgradeOutbound.write(status >>> 8);
-            upgradeOutbound.write(status);
-            upgradeOutbound.write(data.array(), data.position(),
-                    data.limit() - data.position());
-        }
-        upgradeOutbound.flush();
-
-        bb = null;
-        cb = null;
-        upgradeOutbound = null;
     }
 
 
@@ -300,7 +390,7 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void pong(ByteBuffer data) throws IOException {
+    public void pong(ByteBuffer data) throws IOException {
         sendControlMessage(data, Constants.OPCODE_PONG);
     }
 
@@ -311,7 +401,7 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    public synchronized void ping(ByteBuffer data) throws IOException {
+    public void ping(ByteBuffer data) throws IOException {
         sendControlMessage(data, Constants.OPCODE_PING);
     }
 
@@ -323,24 +413,36 @@ public class WsOutbound {
      *
      * @throws IOException  If an error occurs writing to the client
      */
-    private synchronized void sendControlMessage(ByteBuffer data, byte opcode) throws IOException {
+    private void sendControlMessage(ByteBuffer data, byte opcode) throws IOException {
 
-        if (closed) {
-            throw new IOException(sm.getString("outbound.closed"));
+        try {
+            synchronized (stateLock) {
+                if (closed) {
+                    throw new IOException(sm.getString("outbound.closed"));
+                }
+        
+                doFlush(false);
+        
+                upgradeOutbound.write(0x80 | opcode);
+                if (data == null) {
+                    upgradeOutbound.write(0);
+                } else {
+                    upgradeOutbound.write(data.limit() - data.position());
+                    upgradeOutbound.write(data.array(), data.position(),
+                            data.limit() - data.position());
+                }
+        
+                upgradeOutbound.flush();
+            }
+        } catch (IOException ioe) {
+            // Any IOException is terminal. Make sure the Inbound side knows
+            // that something went wrong.
+            // The exception handling needs to be outside of the sync to avoid
+            // possible deadlocks (e.g. BZ55524) when triggering the inbound
+            // close as that will execute user code
+            streamInbound.doOnClose(Constants.STATUS_CLOSED_UNEXPECTEDLY);
+            throw ioe;
         }
-
-        doFlush(true);
-
-        upgradeOutbound.write(0x80 | opcode);
-        if (data == null) {
-            upgradeOutbound.write(0);
-        } else {
-            upgradeOutbound.write(data.limit() - data.position());
-            upgradeOutbound.write(data.array(), data.position(),
-                    data.limit() - data.position());
-        }
-
-        upgradeOutbound.flush();
     }
 
 
@@ -355,6 +457,10 @@ public class WsOutbound {
     private void doWriteBytes(ByteBuffer buffer, boolean finalFragment)
             throws IOException {
 
+        if (closed) {
+            throw new IOException(sm.getString("outbound.closed"));
+        }
+
         // Work out the first byte
         int first = 0x00;
         if (finalFragment) {
@@ -387,11 +493,11 @@ public class WsOutbound {
             upgradeOutbound.write(buffer.limit() >>> 16);
             upgradeOutbound.write(buffer.limit() >>> 8);
             upgradeOutbound.write(buffer.limit() & 0xFF);
-
         }
 
         // Write the content
-        upgradeOutbound.write(buffer.array(), 0, buffer.limit());
+        upgradeOutbound.write(buffer.array(), buffer.arrayOffset(),
+                buffer.limit());
         upgradeOutbound.flush();
 
         // Reset
