@@ -29,8 +29,8 @@ import javax.management.MBeanServer;
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 
-import org.apache.coyote.http11.upgrade.UpgradeInbound;
-import org.apache.coyote.http11.upgrade.UpgradeProcessor;
+import org.apache.coyote.http11.upgrade.servlet31.HttpUpgradeHandler;
+import org.apache.coyote.http11.upgrade.servlet31.WebConnection;
 import org.apache.juli.logging.Log;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.modeler.Registry;
@@ -250,6 +250,11 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         endpoint.setMaxHeaderCount(maxHeaderCount);
     }
 
+    public long getConnectionCount() {
+        return endpoint.getConnectionCount();
+    }
+
+
     // ---------------------------------------------------------- Public methods
 
     public synchronized int getNameIndex() {
@@ -549,17 +554,28 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         }
         
 
-        public SocketState process(SocketWrapper<S> socket,
+        @SuppressWarnings("deprecation") // Old HTTP upgrade method has been deprecated
+        public SocketState process(SocketWrapper<S> wrapper,
                 SocketStatus status) {
-            Processor<S> processor = connections.remove(socket.getSocket());
-
-            if (status == SocketStatus.DISCONNECT && processor == null) {
-                //nothing more to be done endpoint requested a close
-                //and there are no object associated with this connection
+            if (wrapper == null) {
+                // Nothing to do. Socket has been closed.
                 return SocketState.CLOSED;
             }
 
-            socket.setAsync(false);
+            S socket = wrapper.getSocket();
+            if (socket == null) {
+                // Nothing to do. Socket has been closed.
+                return SocketState.CLOSED;
+            }
+
+            Processor<S> processor = connections.get(socket);
+            if (status == SocketStatus.DISCONNECT && processor == null) {
+                // Nothing to do. Endpoint requested a close and there is no
+                // longer a processor associated with this socket.
+                return SocketState.CLOSED;
+            }
+
+            wrapper.setAsync(false);
 
             try {
                 if (processor == null) {
@@ -569,7 +585,7 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                     processor = createProcessor();
                 }
 
-                initSsl(socket, processor);
+                initSsl(wrapper, processor);
 
                 SocketState state = SocketState.CLOSED;
                 do {
@@ -583,10 +599,12 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                         state = processor.asyncDispatch(status);
                     } else if (processor.isComet()) {
                         state = processor.event(status);
-                    } else if (processor.isUpgrade()) {
+                    } else if (processor.getUpgradeInbound() != null) {
                         state = processor.upgradeDispatch();
+                    } else if (processor.isUpgrade()) {
+                        state = processor.upgradeDispatch(status);
                     } else {
-                        state = processor.process(socket);
+                        state = processor.process(wrapper);
                     }
     
                     if (state != SocketState.CLOSED && processor.isAsync()) {
@@ -594,38 +612,82 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                     }
 
                     if (state == SocketState.UPGRADING) {
-                        // Get the UpgradeInbound handler
-                        UpgradeInbound inbound = processor.getUpgradeInbound();
+                        // Get the HTTP upgrade handler
+                        HttpUpgradeHandler httpUpgradeHandler =
+                                processor.getHttpUpgradeHandler();
                         // Release the Http11 processor to be re-used
-                        release(socket, processor, false, false);
+                        release(wrapper, processor, false, false);
+                        // Create the upgrade processor
+                        processor = createUpgradeProcessor(
+                                wrapper, httpUpgradeHandler);
+                        // Mark the connection as upgraded
+                        wrapper.setUpgraded(true);
+                        // Associate with the processor with the connection
+                        connections.put(socket, processor);
+                        // Initialise the upgrade handler (which may trigger
+                        // some IO using the new protocol which is why the lines
+                        // above are necessary)
+                        // This cast should be safe. If it fails the error
+                        // handling for the surrounding try/catch will deal with
+                        // it.
+                        httpUpgradeHandler.init((WebConnection) processor);
+                    } else if (state == SocketState.UPGRADING_TOMCAT) {
+                        // Get the UpgradeInbound handler
+                        org.apache.coyote.http11.upgrade.UpgradeInbound inbound =
+                                processor.getUpgradeInbound();
+                        // Release the Http11 processor to be re-used
+                        release(wrapper, processor, false, false);
                         // Create the light-weight upgrade processor
-                        processor = createUpgradeProcessor(socket, inbound);
+                        processor = createUpgradeProcessor(wrapper, inbound);
                         inbound.onUpgradeComplete();
                     }
+                    if (getLog().isDebugEnabled()) {
+                        getLog().debug("Socket: [" + wrapper +
+                                "], Status in: [" + status +
+                                "], State out: [" + state + "]");
+                    }
                 } while (state == SocketState.ASYNC_END ||
-                        state == SocketState.UPGRADING);
+                        state == SocketState.UPGRADING ||
+                        state == SocketState.UPGRADING_TOMCAT);
 
                 if (state == SocketState.LONG) {
                     // In the middle of processing a request/response. Keep the
                     // socket associated with the processor. Exact requirements
                     // depend on type of long poll
-                    longPoll(socket, processor);
+                    connections.put(socket, processor);
+                    longPoll(wrapper, processor);
                 } else if (state == SocketState.OPEN) {
                     // In keep-alive but between requests. OK to recycle
                     // processor. Continue to poll for the next request.
-                    release(socket, processor, false, true);
+                    connections.remove(socket);
+                    release(wrapper, processor, false, true);
                 } else if (state == SocketState.SENDFILE) {
                     // Sendfile in progress. If it fails, the socket will be
                     // closed. If it works, the socket will be re-added to the
                     // poller
-                    release(socket, processor, false, false);
+                    connections.remove(socket);
+                    release(wrapper, processor, false, false);
                 } else if (state == SocketState.UPGRADED) {
                     // Need to keep the connection associated with the processor
-                    longPoll(socket, processor);
+                    connections.put(socket, processor);
+                    // Don't add sockets back to the poller if this was a
+                    // non-blocking write otherwise the poller may trigger
+                    // multiple read events which may lead to thread starvation
+                    // in the connector. The write() method will add this socket
+                    // to the poller if necessary.
+                    if (status != SocketStatus.OPEN_WRITE) {
+                        longPoll(wrapper, processor);
+                    }
                 } else {
-                    // Connection closed. OK to recycle the processor.
-                    if (!(processor instanceof UpgradeProcessor)) {
-                        release(socket, processor, true, false);
+                    // Connection closed. OK to recycle the processor. Upgrade
+                    // processors are not recycled.
+                    connections.remove(socket);
+                    if (processor.isUpgrade()) {
+                        processor.getHttpUpgradeHandler().destroy();
+                    } else if (processor instanceof org.apache.coyote.http11.upgrade.UpgradeProcessor) {
+                        // NO-OP
+                    } else {
+                        release(wrapper, processor, true, false);
                     }
                 }
                 return state;
@@ -649,9 +711,13 @@ public abstract class AbstractProtocol implements ProtocolHandler,
                 getLog().error(
                         sm.getString("abstractConnectionHandler.error"), e);
             }
+            // Make sure socket/processor is removed from the list of current
+            // connections
+            connections.remove(socket);
             // Don't try to add upgrade processors back into the pool
-            if (!(processor instanceof UpgradeProcessor)) {
-                release(socket, processor, true, false);
+            if (!(processor instanceof org.apache.coyote.http11.upgrade.UpgradeProcessor)
+                    && !processor.isUpgrade()) {
+                release(wrapper, processor, true, false);
             }
             return SocketState.CLOSED;
         }
@@ -664,10 +730,17 @@ public abstract class AbstractProtocol implements ProtocolHandler,
         protected abstract void release(SocketWrapper<S> socket,
                 Processor<S> processor, boolean socketClosing,
                 boolean addToPoller);
+        /**
+         * @deprecated  Will be removed in Tomcat 8.0.x.
+         */
+        @Deprecated
         protected abstract Processor<S> createUpgradeProcessor(
                 SocketWrapper<S> socket,
-                UpgradeInbound inbound) throws IOException;
-
+                org.apache.coyote.http11.upgrade.UpgradeInbound inbound) throws IOException;
+        protected abstract Processor<S> createUpgradeProcessor(
+                SocketWrapper<S> socket,
+                HttpUpgradeHandler httpUpgradeProcessor) throws IOException;
+        
         protected void register(AbstractProcessor<S> processor) {
             if (getProtocol().getDomain() != null) {
                 synchronized (this) {
