@@ -49,42 +49,48 @@ import org.apache.tomcat.util.res.StringManager;
  * DISPATCHING   - The dispatch is being processed.
  * ERROR         - Something went wrong.
  *
- * |----------------->--------------|                                         
- * |                               \|/                                        
- * |   |----------<---------------ERROR                                    
- * |   |    complete()           /|\ |postProcess()
- * |   |                   error()|  |
- * |   |                          |  |  |--|timeout()
- * |   |           postProcess()  | \|/ | \|/            auto
- * |   |         |--------------->DISPATCHED<------------------COMPLETING<----|
- * |   |         |               /|\  |                          | /|\        | 
- * |   |         |    |--->-------|   |                          |--|         | 
- * |   |         ^    |               |startAsync()            timeout()      |
- * |   |         |    |               |                                       | 
- * |  \|/        |    |  complete()  \|/        postProcess()                 | 
- * | MUST_COMPLETE-<- | ----<------STARTING-->----------------|               ^     
- * |      /|\         |               |                       |               |         
- * |       |          |               |                       |               |         
- * |       |          ^               |dispatch()             |               |         
- * |       |          |               |                       |               |         
- * |       |          |              \|/                     \|/   complete() |         
- * |       |          |         MUST_DISPATCH              STARTED---->-------|   
- * |       |          |           |                         |   |                        
- * |       |          |           |postProcess()            |   |                        
- * ^       ^          |           |              dispatch() |   |auto                        
- * |       |          |           |    |--------------------|   |                        
- * |       |          | auto     \|/  \|/                      \|/                         
- * |       |          |---<----DISPATCHING<-----------------TIMING_OUT                              
- * |       |                                  dispatch()      |   |             
- * |       |                                                  |   |            
- * |       |-------<-------------------------------------<----|   |            
- * |                              complete()                      |            
- * |                                                              |            
- * |----<------------------------<-----------------------------<--|            
- *                                 error()
- * </pre>                               
+ * |----------------->--------------|
+ * |                               \|/
+ * |   |----------<---------------ERROR
+ * |   |      complete()         /|\ | \                                                          
+ * |   |                          |  |  \---------------|                                         
+ * |   |                          |  |                  |dispatch()                               
+ * |   |                          |  |postProcess()    \|/                                        
+ * |   |                   error()|  |                  |                                         
+ * |   |                          |  |  |--|timeout()   |                                         
+ * |   |           postProcess()  | \|/ | \|/           |         auto                            
+ * |   |         |--------------->DISPATCHED<---------- | --------------COMPLETING<-----|         
+ * |   |         |               /|\  |                 |                 | /|\         |         
+ * |   |         |    |--->-------|   |                 |                 |--|          |         
+ * |   |         ^    |               |startAsync()     |               timeout()       |         
+ * |   |         |    |               |                 |                               |         
+ * |  \|/        |    |  complete()  \|/  postProcess() |                               |         
+ * | MUST_COMPLETE-<- | ----<------STARTING-->--------- | ------------|                 ^         
+ * |         /|\      |               |                 |             |      complete() |         
+ * |          |       |               |                 |             |     /-----------|         
+ * |          |       ^               |dispatch()       |             |    /                      
+ * |          |       |               |                 |             |   /                       
+ * |          |       |              \|/                /            \|/ /                        
+ * |          |       |         MUST_DISPATCH          /           STARTED
+ * |          |       |           |                   /            /|  
+ * |          |       |           |postProcess()     /            / |   
+ * ^          ^       |           |                 /  dispatch()/  |    
+ * |          |       |           |                /            /   |    
+ * |          |       |           |   |---------- / -----------/    |auto
+ * |          |       |           |   |          /                  |   
+ * |          |       |           |   |   |-----/                   |   
+ * |          |       | auto     \|/ \|/ \|/                       \|/  
+ * |          |       |---<------DISPATCHING<-----------------TIMING_OUT
+ * |          |                               dispatch()        |   |
+ * |          |                                                 |   |
+ * |          |-------<----------------------------------<------|   |
+ * |                              complete()                        |  
+ * |                                                                |  
+ * |<--------<-------------------<-------------------------------<--|  
+ *                                 error()                             
+ * </pre>
  */
-public class AsyncStateMachine {
+public class AsyncStateMachine<S> {
 
     /**
      * The string manager for this package.
@@ -99,7 +105,7 @@ public class AsyncStateMachine {
         MUST_COMPLETE(true, false, false),
         COMPLETING(true, false, false),
         TIMING_OUT(true, false, false),
-        MUST_DISPATCH(true, false, true),
+        MUST_DISPATCH(true, true, true),
         DISPATCHING(true, false, true),
         ERROR(true,false,false);
     
@@ -131,10 +137,10 @@ public class AsyncStateMachine {
     private volatile AsyncState state = AsyncState.DISPATCHED;
     // Need this to fire listener on complete
     private AsyncContextCallback asyncCtxt = null;
-    private Processor processor;
+    private Processor<S> processor;
     
     
-    public AsyncStateMachine(Processor processor) {
+    public AsyncStateMachine(Processor<S> processor) {
         this.processor = processor;
     }
 
@@ -155,6 +161,9 @@ public class AsyncStateMachine {
         return state == AsyncState.TIMING_OUT;
     }
 
+    public boolean isAsyncError() {
+        return state == AsyncState.ERROR;
+    }
 
     public synchronized void asyncStart(AsyncContextCallback asyncCtxt) {
         if (state == AsyncState.DISPATCHED) {
@@ -191,13 +200,10 @@ public class AsyncStateMachine {
         } else if (state == AsyncState.DISPATCHING) {
             state = AsyncState.DISPATCHED;
             return SocketState.ASYNC_END;
-        } else if (state == AsyncState.ERROR) {
-            asyncCtxt.fireOnComplete();
-            state = AsyncState.DISPATCHED;
-            return SocketState.ASYNC_END;
-        //} else if (state == AsyncState.DISPATCHED) {
-        //    // No state change
-        //    return SocketState.OPEN;
+        } else if (state == AsyncState.STARTED) {
+            // This can occur if an async listener does a dispatch to an async
+            // servlet during onTimeout
+            return SocketState.LONG;
         } else {
             throw new IllegalStateException(
                     sm.getString("asyncStateMachine.invalidAsyncState",
@@ -249,7 +255,8 @@ public class AsyncStateMachine {
         if (state == AsyncState.STARTING) {
             state = AsyncState.MUST_DISPATCH;
         } else if (state == AsyncState.STARTED ||
-                state == AsyncState.TIMING_OUT) {
+                state == AsyncState.TIMING_OUT ||
+                state == AsyncState.ERROR) {
             state = AsyncState.DISPATCHING;
             doDispatch = true;
         } else {
@@ -325,7 +332,7 @@ public class AsyncStateMachine {
     }
     
     
-    public void recycle() {
+    public synchronized void recycle() {
         asyncCtxt = null;
         state = AsyncState.DISPATCHED;
     }
