@@ -25,23 +25,30 @@ import org.apache.tomcat.util.buf.ByteChunk;
 /**
  * Output buffer.
  *
- * This class is used internally by the protocol implementation. All writes from higher level code should happen
- * via Resonse.doWrite().
+ * This class is used internally by the protocol implementation. All writes from
+ * higher level code should happen via Resonse.doWrite().
  * 
  * @author Remy Maucherat
  */
 public interface OutputBuffer {
 
 
-    /** Write the response. The caller ( tomcat ) owns the chunks.
+    /**
+     * Write the response. The caller ( tomcat ) owns the chunks.
      *
      * @param chunk data to write
-     * @param response used to allow buffers that can be shared by multiple responses.
-     * @return
+     * @param response used to allow buffers that can be shared by multiple
+     *          responses.
      * @throws IOException
      */
     public int doWrite(ByteChunk chunk, Response response)
         throws IOException;
 
-
+    /**
+     * Bytes written to the underlying socket. This includes the effects of
+     * chunking, compression, etc.
+     * 
+     * @return  Bytes written for the current request
+     */
+    public long getBytesWritten();
 }
