@@ -135,7 +135,8 @@ public class AjpAprProtocol extends AbstractAjpProtocol {
             if (addToPoller) {
                 ((AprEndpoint)proto.endpoint).getPoller().add(
                         socket.getSocket().longValue(),
-                        proto.endpoint.getKeepAliveTimeout());
+                        proto.endpoint.getKeepAliveTimeout(),
+                        true, false);
             }
         }
 
