@@ -66,13 +66,6 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
 
     // ----------------------------------------------------- Instance Variables
 
-
-    /**
-     * Socket associated with the current connection.
-     */
-    protected NioChannel socket;
-
-    
     /**
      * Selector pool for the associated endpoint.
      */
@@ -95,9 +88,10 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
         rp.setStage(org.apache.coyote.Constants.STAGE_PARSE);
 
         // Setting up the socket
-        this.socket = socket.getSocket();
+        this.socketWrapper = socket;
         
         long soTimeout = endpoint.getSoTimeout();
+        boolean cping = false;
 
         // Error flag
         error = false;
@@ -118,6 +112,11 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
                 // not regular request processing
                 int type = requestHeaderMessage.getByte();
                 if (type == Constants.JK_AJP13_CPING_REQUEST) {
+                    if (endpoint.isPaused()) {
+                        recycle(true);
+                        break;
+                    }
+                    cping = true;
                     try {
                         output(pongMessageArray, 0, pongMessageArray.length);
                     } catch (IOException e) {
@@ -163,12 +162,13 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
                 }
             }
 
-            if (endpoint.isPaused()) {
+            if (!error && !cping && endpoint.isPaused()) {
                 // 503 - Service unavailable
                 response.setStatus(503);
                 adapter.log(request, response, 0);
                 error = true;
             }
+            cping = false;
 
             // Process the request in the adapter
             if (!error) {
@@ -186,7 +186,7 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
                     error = true;
                 }
             }
-            
+
             if (isAsync() && !error) {
                 break;
             }
@@ -216,7 +216,7 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
 
             recycle(false);
         }
-        
+
         rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
 
         if (!error && !endpoint.isPaused()) {
@@ -228,10 +228,9 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
         } else {
             return SocketState.CLOSED;
         }
-        
     }
 
-    
+
     // ----------------------------------------------------- ActionHook Methods
 
 
@@ -246,39 +245,62 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
 
         if (actionCode == ActionCode.ASYNC_COMPLETE) {
             if (asyncStateMachine.asyncComplete()) {
-                ((NioEndpoint)endpoint).processSocket(this.socket,
-                        SocketStatus.OPEN, false);
+                ((NioEndpoint)endpoint).processSocket(this.socketWrapper.getSocket(),
+                        SocketStatus.OPEN_READ, false);
             }
+
         } else if (actionCode == ActionCode.ASYNC_SETTIMEOUT) {
             if (param == null) return;
             long timeout = ((Long)param).longValue();
-            final KeyAttachment ka = (KeyAttachment)socket.getAttachment(false);
-            if (keepAliveTimeout > 0) {
-                ka.setTimeout(timeout);
-            }
+            final KeyAttachment ka =
+                    (KeyAttachment)socketWrapper.getSocket().getAttachment(false);
+            ka.setTimeout(timeout);
+
         } else if (actionCode == ActionCode.ASYNC_DISPATCH) {
             if (asyncStateMachine.asyncDispatch()) {
-                ((NioEndpoint)endpoint).processSocket(this.socket,
-                        SocketStatus.OPEN, true);
+                ((NioEndpoint)endpoint).processSocket(this.socketWrapper.getSocket(),
+                        SocketStatus.OPEN_READ, true);
             }
         }
     }
 
 
-    // ------------------------------------------------------ Protected Methods
+    @Override
+    protected void resetTimeouts() {
+        // The NIO connector uses the timeout configured on the wrapper in the
+        // poller. Therefore, it needs to be reset once asycn processing has
+        // finished.
+        final KeyAttachment attach =
+                (KeyAttachment)socketWrapper.getSocket().getAttachment(false);
+        if (!error && attach != null &&
+                asyncStateMachine.isAsyncDispatching()) {
+            long soTimeout = endpoint.getSoTimeout();
+
+            //reset the timeout
+            if (keepAliveTimeout > 0) {
+                attach.setTimeout(keepAliveTimeout);
+            } else {
+                attach.setTimeout(soTimeout);
+            }
+        }
+
+    }
+
 
     @Override
     protected void output(byte[] src, int offset, int length)
             throws IOException {
-        ByteBuffer writeBuffer = socket.getBufHandler() .getWriteBuffer();
+        ByteBuffer writeBuffer =
+                socketWrapper.getSocket().getBufHandler().getWriteBuffer();
 
         writeBuffer.put(src, offset, length);
         
         writeBuffer.flip();
         
-        NioEndpoint.KeyAttachment att = (NioEndpoint.KeyAttachment)socket.getAttachment(false);
+        KeyAttachment att =
+                (KeyAttachment) socketWrapper.getSocket().getAttachment(false);
         if ( att == null ) throw new IOException("Key must be cancelled");
-        long writeTimeout = att.getTimeout();
+        long writeTimeout = att.getWriteTimeout();
         Selector selector = null;
         try {
             selector = pool.get();
@@ -286,8 +308,8 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
             //ignore
         }
         try {
-            pool.write(writeBuffer, socket, selector, writeTimeout, true,
-                    null);
+            pool.write(writeBuffer, socketWrapper.getSocket(), selector,
+                    writeTimeout, true);
         }finally { 
             if ( selector != null ) pool.put(selector);
         }
@@ -312,7 +334,7 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
             } else if (res == 0 && !block) {
                 break;
             } else {
-                throw new IOException(sm.getString("ajpprotocol.failedread"));
+                throw new IOException(sm.getString("ajpprocessor.failedread"));
             }
             block = true;
         }
@@ -322,8 +344,10 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
     private int readSocket(byte[] buf, int pos, int n, boolean block)
             throws IOException {
         int nRead = 0;
-        socket.getBufHandler().getReadBuffer().clear();
-        socket.getBufHandler().getReadBuffer().limit(n);
+        ByteBuffer readBuffer =
+                socketWrapper.getSocket().getBufHandler().getReadBuffer();
+        readBuffer.clear();
+        readBuffer.limit(n);
         if ( block ) {
             Selector selector = null;
             try {
@@ -332,21 +356,23 @@ public class AjpNioProcessor extends AbstractAjpProcessor<NioChannel> {
                 // Ignore
             }
             try {
-                NioEndpoint.KeyAttachment att = (NioEndpoint.KeyAttachment)socket.getAttachment(false);
+                NioEndpoint.KeyAttachment att =
+                        (NioEndpoint.KeyAttachment) socketWrapper.getSocket().getAttachment(false);
                 if ( att == null ) throw new IOException("Key must be cancelled.");
-                nRead = pool.read(socket.getBufHandler().getReadBuffer(),socket,selector,att.getTimeout());
+                nRead = pool.read(readBuffer, socketWrapper.getSocket(),
+                        selector, att.getTimeout());
             } catch ( EOFException eof ) {
                 nRead = -1;
             } finally { 
                 if ( selector != null ) pool.put(selector);
             }
         } else {
-            nRead = socket.read(socket.getBufHandler().getReadBuffer());
+            nRead = socketWrapper.getSocket().read(readBuffer);
         }
         if (nRead > 0) {
-            socket.getBufHandler().getReadBuffer().flip();
-            socket.getBufHandler().getReadBuffer().limit(nRead);
-            socket.getBufHandler().getReadBuffer().get(buf, pos, nRead);
+            readBuffer.flip();
+            readBuffer.limit(nRead);
+            readBuffer.get(buf, pos, nRead);
             return nRead;
         } else if (nRead == -1) {
             //return false;
