@@ -36,7 +36,7 @@ import org.apache.coyote.http11.filters.IdentityOutputFilter;
 import org.apache.coyote.http11.filters.SavedRequestInputFilter;
 import org.apache.coyote.http11.filters.VoidInputFilter;
 import org.apache.coyote.http11.filters.VoidOutputFilter;
-import org.apache.coyote.http11.upgrade.UpgradeInbound;
+import org.apache.coyote.http11.upgrade.servlet31.HttpUpgradeHandler;
 import org.apache.juli.logging.Log;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.Ascii;
@@ -45,6 +45,7 @@ import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
+import org.apache.tomcat.util.log.UserDataHelper;
 import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.SocketStatus;
@@ -54,6 +55,7 @@ import org.apache.tomcat.util.res.StringManager;
 public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
 
     protected abstract Log getLog();
+    private final UserDataHelper userDataHelper;
 
     /**
      * The string manager for this package.
@@ -256,13 +258,24 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
 
     /**
      * Listener to which data available events are passed once the associated
-     * connection has completed the HTTP upgrade process.
+     * connection has completed the proprietary Tomcat HTTP upgrade process.
+     * 
+     * @deprecated  Will be removed in Tomcat 8.0.x.
      */
-    protected UpgradeInbound upgradeInbound = null;
+    @Deprecated
+    protected org.apache.coyote.http11.upgrade.UpgradeInbound upgradeInbound = null;
 
 
+    /**
+     * Instance of the new protocol to use after the HTTP connection has been
+     * upgraded using the Servlet 3.1 based upgrade process.
+     */
+    protected HttpUpgradeHandler httpUpgradeHandler = null;
+    
+    
     public AbstractHttp11Processor(AbstractEndpoint endpoint) {
         super(endpoint);
+        userDataHelper = new UserDataHelper(getLog());
     }
 
 
@@ -658,12 +671,6 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
     }
 
 
-    /**
-     * Allows the super class to set the socket wrapper being used.
-     */
-    protected abstract void setSocketWrapper(SocketWrapper<S> socketWrapper);
-
-
     /**
      * Exposes input buffer to super class to allow better code re-use.
      * @return  The input buffer used by the processor.
@@ -681,13 +688,14 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
     /**
      * Initialize standard input and output filters.
      */
-    protected void initializeFilters(int maxTrailerSize) {
+    protected void initializeFilters(int maxTrailerSize, int maxExtensionSize) {
         // Create and add the identity filters.
         getInputBuffer().addFilter(new IdentityInputFilter());
         getOutputBuffer().addFilter(new IdentityOutputFilter());
 
         // Create and add the chunked filters.
-        getInputBuffer().addFilter(new ChunkedInputFilter(maxTrailerSize));
+        getInputBuffer().addFilter(
+                new ChunkedInputFilter(maxTrailerSize, maxExtensionSize));
         getOutputBuffer().addFilter(new ChunkedOutputFilter());
 
         // Create and add the void filters.
@@ -740,6 +748,7 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
      * @param param Action parameter
      */
     @Override
+    @SuppressWarnings("deprecation") // Inbound/Outbound based upgrade mechanism
     public final void action(ActionCode actionCode, Object param) {
 
         if (actionCode == ActionCode.CLOSE) {
@@ -794,6 +803,9 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
                 response.setErrorException(e);
             }
 
+        } else if (actionCode == ActionCode.IS_ERROR) {
+            ((AtomicBoolean) param).set(error);
+
         } else if (actionCode == ActionCode.DISABLE_SWALLOW_INPUT) {
             // Do not swallow request input but
             // make sure we are closing the connection
@@ -822,6 +834,8 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             internalBuffer.addActiveFilter(savedBody);
         } else if (actionCode == ActionCode.ASYNC_START) {
             asyncStateMachine.asyncStart((AsyncContextCallback) param);
+            // Async time out is based on SocketWrapper access time
+            getSocketWrapper().access();
         } else if (actionCode == ActionCode.ASYNC_DISPATCHED) {
             asyncStateMachine.asyncDispatched();
         } else if (actionCode == ActionCode.ASYNC_TIMEOUT) {
@@ -841,8 +855,12 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             ((AtomicBoolean) param).set(asyncStateMachine.isAsyncTimingOut());
         } else if (actionCode == ActionCode.ASYNC_IS_ERROR) {
             ((AtomicBoolean) param).set(asyncStateMachine.isAsyncError());
+        } else if (actionCode == ActionCode.UPGRADE_TOMCAT) {
+            upgradeInbound = (org.apache.coyote.http11.upgrade.UpgradeInbound) param;
+            // Stop further HTTP output
+            getOutputBuffer().finished = true;
         } else if (actionCode == ActionCode.UPGRADE) {
-            upgradeInbound = (UpgradeInbound) param;
+            httpUpgradeHandler = (HttpUpgradeHandler) param;
             // Stop further HTTP output
             getOutputBuffer().finished = true;
         } else {
@@ -920,7 +938,8 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
         }
 
         while (!error && keepAlive && !comet && !isAsync() &&
-                upgradeInbound == null && !endpoint.isPaused()) {
+                upgradeInbound == null &&
+                httpUpgradeHandler == null && !endpoint.isPaused()) {
 
             // Parsing the request header
             try {
@@ -937,7 +956,12 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
                     response.setStatus(503);
                     error = true;
                 } else {
-                    request.setStartTime(System.currentTimeMillis());
+                    // Make sure that connectors that are non-blocking during
+                    // header processing (NIO) only set the start time the first
+                    // time a request is processed.
+                    if (request.getStartTime() < 0) {
+                        request.setStartTime(System.currentTimeMillis());
+                    }
                     keptAlive = true;
                     // Set this every time in case limit has been changed via JMX
                     request.getMimeHeaders().setLimit(endpoint.getMaxHeaderCount());
@@ -962,9 +986,21 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
                 break;
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
-                if (getLog().isDebugEnabled()) {
-                    getLog().debug(
-                            sm.getString("http11processor.header.parse"), t);
+                UserDataHelper.Mode logMode = userDataHelper.getNextMode();
+                if (logMode != null) {
+                    String message = sm.getString(
+                            "http11processor.header.parse");
+                    switch (logMode) {
+                        case INFO_THEN_DEBUG:
+                            message += sm.getString(
+                                    "http11processor.fallToDebug");
+                            //$FALL-THROUGH$
+                        case INFO:
+                            getLog().info(message);
+                            break;
+                        case DEBUG:
+                            getLog().debug(message);
+                    }
                 }
                 // 400 - Bad Request
                 response.setStatus(400);
@@ -1045,6 +1081,18 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
                     // thread if the servlet has rejected it.
                     getInputBuffer().setSwallowInput(false);
                 }
+                if (response.getStatus() < 200 || response.getStatus() > 299) {
+                    if (expectation) {
+                        // Client sent Expect: 100-continue but received a
+                        // non-2xx response. Disable keep-alive (if enabled) to
+                        // ensure the connection is closed. Some clients may
+                        // still send the body, some may send the next request.
+                        // No way to differentiate, so close the connection to
+                        // force the client to send the next request.
+                        getInputBuffer().setSwallowInput(false);
+                        keepAlive = false;
+                    }
+                }
                 endRequest();
             }
 
@@ -1063,7 +1111,11 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             }
 
             if (!disableUploadTimeout) {
-                setSocketTimeout(endpoint.getSoTimeout());
+                if(endpoint.getSoTimeout() > 0) {
+                    setSocketTimeout(endpoint.getSoTimeout());
+                } else {
+                    setSocketTimeout(0);
+                }
             }
 
             rp.setStage(org.apache.coyote.Constants.STAGE_KEEPALIVE);
@@ -1081,6 +1133,8 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             return SocketState.LONG;
         } else if (isUpgrade()) {
             return SocketState.UPGRADING;
+        } else if (getUpgradeInbound() != null) {
+            return SocketState.UPGRADING_TOMCAT;
         } else {
             if (sendfileInProgress) {
                 return SocketState.SENDFILE;
@@ -1254,10 +1308,20 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
 
         // Parse content-length header
         long contentLength = request.getContentLengthLong();
-        if (contentLength >= 0 && !contentDelimitation) {
-            getInputBuffer().addActiveFilter
-                (inputFilters[Constants.IDENTITY_FILTER]);
-            contentDelimitation = true;
+        if (contentLength >= 0) {
+            if (contentDelimitation) {
+                // contentDelimitation being true at this point indicates that
+                // chunked encoding is being used but chunked encoding should
+                // not be used with a content length. RFC 2616, section 4.4,
+                // bullet 3 states Content-Length must be ignored in this case -
+                // so remove it.
+                headers.removeHeader("content-length");
+                request.setContentLength(-1);
+            } else {
+                getInputBuffer().addActiveFilter
+                        (inputFilters[Constants.IDENTITY_FILTER]);
+                contentDelimitation = true;
+            }
         }
 
         MessageBytes valueMB = headers.getValue("host");
@@ -1375,7 +1439,9 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
         MimeHeaders headers = response.getMimeHeaders();
         if (!entityBody) {
             response.setContentLength(-1);
-        } else {
+        }
+        // A SC_NO_CONTENT response may include entity headers
+        if (entityBody || statusCode == 204) {
             String contentType = response.getContentType();
             if (contentType != null) {
                 headers.setValue("Content-Type").setString(contentType);
@@ -1429,8 +1495,12 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             }
         }
 
-        // Add date header
-        headers.setValue("Date").setString(FastHttpDateFormat.getCurrentDate());
+        // Add date header unless application has already set one (e.g. in a
+        // Caching Filter)
+        if (headers.getValue("Date") == null) {
+            headers.setValue("Date").setString(
+                    FastHttpDateFormat.getCurrentDate());
+        }
 
         // FIXME: Add transfer encoding header
 
@@ -1599,13 +1669,6 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
     }
 
 
-    @Override
-    public boolean isUpgrade() {
-        return upgradeInbound != null;
-    }
-
-
-
     @Override
     public SocketState upgradeDispatch() throws IOException {
         // Should never reach this code but in case we do...
@@ -1615,12 +1678,36 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
     }
 
 
+    /**
+     * @deprecated  Will be removed in Tomcat 8.0.x.
+     */
+    @Deprecated
     @Override
-    public UpgradeInbound getUpgradeInbound() {
+    public org.apache.coyote.http11.upgrade.UpgradeInbound getUpgradeInbound() {
         return upgradeInbound;
     }
 
 
+    @Override
+    public boolean isUpgrade() {
+        return httpUpgradeHandler != null;
+    }
+    
+    
+    @Override
+    public SocketState upgradeDispatch(SocketStatus status) throws IOException {
+        // Should never reach this code but in case we do...
+        throw new IOException(
+                sm.getString("ajpprocessor.httpupgrade.notsupported"));
+    }
+
+
+    @Override
+    public HttpUpgradeHandler getHttpUpgradeHandler() {
+        return httpUpgradeHandler;
+    }
+    
+    
     /**
      * Provides a mechanism for those connector implementations (currently only
      * NIO) that need to reset timeouts from Async timeouts to standard HTTP
@@ -1687,6 +1774,7 @@ public abstract class AbstractHttp11Processor<S> extends AbstractProcessor<S> {
             asyncStateMachine.recycle();
         }
         upgradeInbound = null;
+        httpUpgradeHandler = null;
         remoteAddr = null;
         remoteHost = null;
         localAddr = null;
