@@ -145,7 +145,8 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
 
 
     /**
-     * Maximum allowed size of the HTTP request line plus headers.
+     * Maximum allowed size of the HTTP request line plus headers plus any
+     * leading blank lines.
      */
     private final int headerBufferSize;
 
@@ -154,18 +155,6 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
      */
     private int socketReadBufferSize;
 
-    /**
-     * Additional size we allocate to the buffer to be more effective when
-     * skipping empty lines that may precede the request.
-     */
-    private static final int skipBlankLinesSize = 1024;
-
-    /**
-     * How many bytes in the buffer are occupied by skipped blank lines that
-     * precede the request.
-     */
-    private int skipBlankLinesBytes;
-
 
     // --------------------------------------------------------- Public Methods
 
@@ -175,11 +164,6 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
      */
     @Override
     public void recycle() {
-        // Recycle filters
-        for (int i = 0; i <= lastActiveFilter; i++) {
-            activeFilters[i].recycle();
-        }
-        // This must be after filters since it resets the lastFilterIndex
         super.recycle();
         socket = null;
         headerParsePos = HeaderParsePosition.HEADER_START;
@@ -239,22 +223,15 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
                     if (useAvailableDataOnly) {
                         return false;
                     }
-                    // Ignore bytes that were read
-                    pos = lastValid = 0;
                     // Do a simple read with a short timeout
-                    if ( readSocket(true, false)==0 ) return false;
+                    if (!fill(true, false)) {
+                        return false;
+                    }
                 }
                 chr = buf[pos++];
             } while ((chr == Constants.CR) || (chr == Constants.LF));
             pos--;
-            if (pos >= skipBlankLinesSize) {
-                // Move data, to have enough space for further reading
-                // of headers and body
-                System.arraycopy(buf, pos, buf, 0, lastValid - pos);
-                lastValid -= pos;
-                pos = 0;
-            }
-            skipBlankLinesBytes = pos;
+
             parsingRequestLineStart = pos;
             parsingRequestLinePhase = 2;
             if (log.isDebugEnabled()) {
@@ -362,11 +339,11 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
             }
             parsingRequestLineStart = pos;
             parsingRequestLinePhase = 6;
+
+            // Mark the current buffer position
+            end = 0;
         }
         if (parsingRequestLinePhase == 6) { 
-            // Mark the current buffer position
-            
-            end = 0;
             //
             // Reading the protocol
             // Protocol is always US-ASCII
@@ -437,9 +414,14 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
                 // Ignore
             }
             try {
-                NioEndpoint.KeyAttachment att = (NioEndpoint.KeyAttachment)socket.getAttachment(false);
-                if ( att == null ) throw new IOException("Key must be cancelled.");
-                nRead = pool.read(socket.getBufHandler().getReadBuffer(),socket,selector,att.getTimeout());
+                NioEndpoint.KeyAttachment att =
+                        (NioEndpoint.KeyAttachment) socket.getAttachment(false);
+                if (att == null) {
+                    throw new IOException("Key must be cancelled.");
+                }
+                nRead = pool.read(socket.getBufHandler().getReadBuffer(),
+                        socket, selector,
+                        socket.getIOChannel().socket().getSoTimeout());
             } catch ( EOFException eof ) {
                 nRead = -1;
             } finally { 
@@ -478,10 +460,6 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
         
         do {
             status = parseHeader();
-        } while ( status == HeaderParseStatus.HAVE_MORE_HEADERS );
-        if (status == HeaderParseStatus.DONE) {
-            parsingHeader = false;
-            end = pos;
             // Checking that
             // (1) Headers plus request line size does not exceed its limit
             // (2) There are enough bytes to avoid expanding the buffer when
@@ -490,11 +468,15 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
             // limitation to enforce the meaning of headerBufferSize
             // From the way how buf is allocated and how blank lines are being
             // read, it should be enough to check (1) only.
-            if (end - skipBlankLinesBytes > headerBufferSize
-                    || buf.length - end < socketReadBufferSize) {
+            if (pos > headerBufferSize
+                    || buf.length - pos < socketReadBufferSize) {
                 throw new IllegalArgumentException(
                         sm.getString("iib.requestheadertoolarge.error"));
             }
+        } while ( status == HeaderParseStatus.HAVE_MORE_HEADERS );
+        if (status == HeaderParseStatus.DONE) {
+            parsingHeader = false;
+            end = pos;
             return true;
         } else {
             return false;
@@ -773,8 +755,7 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
         socketReadBufferSize =
             socket.getBufHandler().getReadBuffer().capacity();
 
-        int bufLength = skipBlankLinesSize + headerBufferSize
-                + socketReadBufferSize;
+        int bufLength = headerBufferSize + socketReadBufferSize;
         if (buf == null || buf.length < bufLength) {
             buf = new byte[bufLength];
         }
@@ -800,7 +781,7 @@ public class InternalNioInputBuffer extends AbstractInputBuffer<NioChannel> {
 
         if (parsingHeader) {
 
-            if (lastValid == buf.length) {
+            if (lastValid > headerBufferSize) {
                 throw new IllegalArgumentException
                     (sm.getString("iib.requestheadertoolarge.error"));
             }
