@@ -33,7 +33,6 @@ import org.apache.el.lang.ELSupport;
  * Utilities for Managing Serialization and Reflection
  * 
  * @author Jacob Hookom [jacob@hookom.net]
- * @version $Id: ReflectionUtil.java 1304933 2012-03-24 21:33:47Z markt $
  */
 public class ReflectionUtil {
 
@@ -250,7 +249,6 @@ public class ReflectionUtil {
         return match;
     }
 
-    @SuppressWarnings("null")
     private static Method resolveAmbiguousMethod(Set<Method> candidates,
             Class<?>[] paramTypes) {
         // Identify which parameter isn't an exact match
@@ -267,6 +265,11 @@ public class ReflectionUtil {
             }
         }
         
+        if (nonMatchClass == null) {
+            // Null will always be ambiguous
+            return null;
+        }
+
         for (Method c : candidates) {
            if (c.getParameterTypes()[nonMatchIndex] ==
                    paramTypes[nonMatchIndex]) {
@@ -294,6 +297,12 @@ public class ReflectionUtil {
 
     // src will always be an object
     private static boolean isAssignableFrom(Class<?> src, Class<?> target) {
+        // Short-cut. null is always assignable to an object and in EL null
+        // can always be coerced to a valid value for a primitive
+        if (src == null) {
+            return true;
+        }
+
         Class<?> targetClass;
         if (target.isPrimitive()) {
             if (target == Boolean.TYPE) {
@@ -334,7 +343,11 @@ public class ReflectionUtil {
         if (types != null) {
             StringBuilder sb = new StringBuilder();
             for (int i = 0; i < types.length; i++) {
-                sb.append(types[i].getName()).append(", ");
+                if (types[i] == null) {
+                    sb.append("null, ");
+                } else {
+                    sb.append(types[i].getName()).append(", ");
+                }
             }
             if (sb.length() > 2) {
                 sb.setLength(sb.length() - 2);
