@@ -18,16 +18,18 @@
 package org.apache.jasper;
 
 import java.io.File;
-import java.util.*;
+import java.util.Enumeration;
+import java.util.Map;
+import java.util.Properties;
 
 import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
+import javax.servlet.jsp.tagext.TagLibraryInfo;
 
-import org.apache.jasper.compiler.TldLocationsCache;
 import org.apache.jasper.compiler.JspConfig;
-import org.apache.jasper.compiler.TagPluginManager;
 import org.apache.jasper.compiler.Localizer;
-import org.apache.jasper.xmlparser.ParserUtils;
+import org.apache.jasper.compiler.TagPluginManager;
+import org.apache.jasper.compiler.TldLocationsCache;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
@@ -41,7 +43,7 @@ import org.apache.juli.logging.LogFactory;
 public final class EmbeddedServletOptions implements Options {
     
     // Logger
-    private Log log = LogFactory.getLog(EmbeddedServletOptions.class);
+    private final Log log = LogFactory.getLog(EmbeddedServletOptions.class);
     
     private Properties settings = new Properties();
     
@@ -88,7 +90,7 @@ public final class EmbeddedServletOptions implements Options {
     private int checkInterval = 0;
     
     /**
-     * Is the generation of SMAP info for JSR45 debuggin suppressed?
+     * Is the generation of SMAP info for JSR45 debugging suppressed?
      */
     private boolean isSmapSuppressed = false;
     
@@ -130,12 +132,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Compiler target VM.
      */
-    private String compilerTargetVM = "1.5";
+    private String compilerTargetVM = "1.6";
     
     /**
      * The compiler source VM.
      */
-    private String compilerSourceVM = "1.5";
+    private String compilerSourceVM = "1.6";
     
     /**
      * The compiler class name.
@@ -185,6 +187,18 @@ public final class EmbeddedServletOptions implements Options {
     private boolean displaySourceFragment = true;
 
     
+    /**
+     * The maximum number of loaded jsps per web-application. If there are more
+     * jsps loaded, they will be unloaded.
+     */
+    private int maxLoadedJsps = -1;
+
+    /**
+     * The idle time in seconds after which a JSP is unloaded.
+     * If unset or less or equal than 0, no jsps are unloaded.
+     */
+    private int jspIdleTimeout = -1;
+
     public String getProperty(String name ) {
         return settings.getProperty( name );
     }
@@ -198,6 +212,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Are we keeping generated code around?
      */
+    @Override
     public boolean getKeepGenerated() {
         return keepGenerated;
     }
@@ -205,10 +220,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Should white spaces between directives or actions be trimmed?
      */
+    @Override
     public boolean getTrimSpaces() {
         return trimSpaces;
     }
     
+    @Override
     public boolean isPoolingEnabled() {
         return isPoolingEnabled;
     }
@@ -216,29 +233,23 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Are we supporting HTML mapped servlets?
      */
+    @Override
     public boolean getMappedFile() {
         return mappedFile;
     }
     
-    /**
-     * Should errors be sent to client or thrown into stderr?
-     * @deprecated
-     */
-    @Deprecated
-    public boolean getSendErrorToClient() {
-        return true;
-    }
-    
     /**
      * Should class files be compiled with debug information?
      */
+    @Override
     public boolean getClassDebugInfo() {
         return classDebugInfo;
     }
     
     /**
-     * Background JSP compile thread check intervall
+     * Background JSP compile thread check interval
      */
+    @Override
     public int getCheckInterval() {
         return checkInterval;
     }
@@ -246,6 +257,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Modification test interval.
      */
+    @Override
     public int getModificationTestInterval() {
         return modificationTestInterval;
     }
@@ -253,6 +265,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Re-compile on failure.
      */
+    @Override
     public boolean getRecompileOnFail() {
         return recompileOnFail;
     }
@@ -260,13 +273,15 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Is Jasper being used in development mode?
      */
+    @Override
     public boolean getDevelopment() {
         return development;
     }
     
     /**
-     * Is the generation of SMAP info for JSR45 debuggin suppressed?
+     * Is the generation of SMAP info for JSR45 debugging suppressed?
      */
+    @Override
     public boolean isSmapSuppressed() {
         return isSmapSuppressed;
     }
@@ -274,6 +289,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Should SMAP info for JSR45 debugging be dumped to a file?
      */
+    @Override
     public boolean isSmapDumped() {
         return isSmapDumped;
     }
@@ -281,6 +297,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Are Text strings to be generated as char arrays?
      */
+    @Override
     public boolean genStringAsCharArray() {
         return this.genStringAsCharArray;
     }
@@ -288,6 +305,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Class ID for use in the plugin tag when the browser is IE. 
      */
+    @Override
     public String getIeClassId() {
         return ieClassId;
     }
@@ -295,6 +313,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * What is my scratch dir?
      */
+    @Override
     public File getScratchDir() {
         return scratchDir;
     }
@@ -303,6 +322,7 @@ public final class EmbeddedServletOptions implements Options {
      * What classpath should I use while compiling the servlets
      * generated from JSP files?
      */
+    @Override
     public String getClassPath() {
         return classpath;
     }
@@ -310,6 +330,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Is generation of X-Powered-By response header enabled/disabled?
      */
+    @Override
     public boolean isXpoweredBy() {
         return xpoweredBy;
     }
@@ -317,6 +338,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Compiler to use.
      */
+    @Override
     public String getCompiler() {
         return compiler;
     }
@@ -324,6 +346,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * @see Options#getCompilerTargetVM
      */
+    @Override
     public String getCompilerTargetVM() {
         return compilerTargetVM;
     }
@@ -331,6 +354,7 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * @see Options#getCompilerSourceVM
      */
+    @Override
     public String getCompilerSourceVM() {
         return compilerSourceVM;
     }
@@ -338,10 +362,12 @@ public final class EmbeddedServletOptions implements Options {
     /**
      * Java compiler class to use.
      */
+    @Override
     public String getCompilerClassName() {
         return compilerClassName;
     }
 
+    @Override
     public boolean getErrorOnUseBeanInvalidClassAttribute() {
         return errorOnUseBeanInvalidClassAttribute;
     }
@@ -350,6 +376,7 @@ public final class EmbeddedServletOptions implements Options {
         errorOnUseBeanInvalidClassAttribute = b;
     }
     
+    @Override
     public TldLocationsCache getTldLocationsCache() {
         return tldLocationsCache;
     }
@@ -358,27 +385,33 @@ public final class EmbeddedServletOptions implements Options {
         tldLocationsCache = tldC;
     }
     
+    @Override
     public String getJavaEncoding() {
         return javaEncoding;
     }
     
+    @Override
     public boolean getFork() {
         return fork;
     }
     
+    @Override
     public JspConfig getJspConfig() {
         return jspConfig;
     }
     
+    @Override
     public TagPluginManager getTagPluginManager() {
         return tagPluginManager;
     }
     
+    @Override
     public boolean isCaching() {
         return false;
     }
     
-    public Map getCache() {
+    @Override
+    public Map<String, TagLibraryInfo> getCache() {
         return null;
     }
 
@@ -386,10 +419,29 @@ public final class EmbeddedServletOptions implements Options {
      * Should we include a source fragment in exception messages, which could be displayed
      * to the developer ?
      */
+    @Override
     public boolean getDisplaySourceFragment() {
         return displaySourceFragment;
     }
 
+    /**
+     * Should jsps be unloaded if to many are loaded?
+     * If set to a value greater than 0 eviction of jsps is started. Default: -1
+     */
+    @Override
+    public int getMaxLoadedJsps() {
+        return maxLoadedJsps;
+    }
+
+    /**
+     * Should any jsps be unloaded when being idle for this time in seconds?
+     * If set to a value greater than 0 eviction of jsps is started. Default: -1
+     */
+    @Override
+    public int getJspIdleTimeout() {
+        return jspIdleTimeout;
+    }
+
     /**
      * Create an EmbeddedServletOptions object using data available from
      * ServletConfig and ServletContext. 
@@ -397,17 +449,13 @@ public final class EmbeddedServletOptions implements Options {
     public EmbeddedServletOptions(ServletConfig config,
             ServletContext context) {
         
-        Enumeration enumeration=config.getInitParameterNames();
+        Enumeration<String> enumeration=config.getInitParameterNames();
         while( enumeration.hasMoreElements() ) {
-            String k=(String)enumeration.nextElement();
+            String k=enumeration.nextElement();
             String v=config.getInitParameter( k );
             setProperty( k, v);
         }
-        
-        // quick hack
-        String validating=config.getInitParameter( "validating");
-        if( "false".equals( validating )) ParserUtils.validating=false;
-        
+                
         String keepgen = config.getInitParameter("keepgenerated");
         if (keepgen != null) {
             if (keepgen.equalsIgnoreCase("true")) {
@@ -445,7 +493,7 @@ public final class EmbeddedServletOptions implements Options {
             } else {
                 if (log.isWarnEnabled()) {
                     log.warn(Localizer.getMessage("jsp.warning.enablePooling"));
-                }		       	   
+                }
             }
         }
         
@@ -548,7 +596,7 @@ public final class EmbeddedServletOptions implements Options {
             }
         }
         
-        String genCharArray = config.getInitParameter("genStrAsCharArray");
+        String genCharArray = config.getInitParameter("genStringAsCharArray");
         if (genCharArray != null) {
             if (genCharArray.equalsIgnoreCase("true")) {
                 genStringAsCharArray = true;
@@ -591,7 +639,7 @@ public final class EmbeddedServletOptions implements Options {
             scratchDir = new File(dir);
         } else {
             // First try the Servlet 2.2 javax.servlet.context.tempdir property
-            scratchDir = (File) context.getAttribute(Constants.TMP_DIR);
+            scratchDir = (File) context.getAttribute(ServletContext.TEMPDIR);
             if (scratchDir == null) {
                 // Not running in a Servlet 2.2 container.
                 // Try to get the JDK 1.2 java.io.tmpdir property
@@ -671,9 +719,31 @@ public final class EmbeddedServletOptions implements Options {
             }
         }
         
+        String maxLoadedJsps = config.getInitParameter("maxLoadedJsps");
+        if (maxLoadedJsps != null) {
+            try {
+                this.maxLoadedJsps = Integer.parseInt(maxLoadedJsps);
+            } catch(NumberFormatException ex) {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.maxLoadedJsps", ""+this.maxLoadedJsps));
+                }
+            }
+        }
+        
+        String jspIdleTimeout = config.getInitParameter("jspIdleTimeout");
+        if (jspIdleTimeout != null) {
+            try {
+                this.jspIdleTimeout = Integer.parseInt(jspIdleTimeout);
+            } catch(NumberFormatException ex) {
+                if (log.isWarnEnabled()) {
+                    log.warn(Localizer.getMessage("jsp.warning.jspIdleTimeout", ""+this.jspIdleTimeout));
+                }
+            }
+        }
+
         // Setup the global Tag Libraries location cache for this
         // web-application.
-        tldLocationsCache = new TldLocationsCache(context);
+        tldLocationsCache = TldLocationsCache.getInstance(context);
         
         // Setup the jsp config info for this web app.
         jspConfig = new JspConfig(context);
