@@ -18,7 +18,6 @@ package org.apache.jasper.compiler;
 
 import java.io.CharArrayWriter;
 import java.io.FileNotFoundException;
-import java.net.URL;
 import java.util.Iterator;
 import java.util.List;
 
@@ -29,6 +28,7 @@ import javax.servlet.jsp.tagext.TagLibraryInfo;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
+import org.apache.jasper.util.UniqueAttributesImpl;
 import org.xml.sax.Attributes;
 import org.xml.sax.helpers.AttributesImpl;
 
@@ -50,8 +50,6 @@ class Parser implements TagConstants {
 
     private JspReader reader;
 
-    private String currentFile;
-
     private Mark start;
 
     private ErrorDispatcher err;
@@ -62,33 +60,42 @@ class Parser implements TagConstants {
 
     private boolean directivesOnly;
 
-    private URL jarFileUrl;
+    private JarResource jarResource;
 
     private PageInfo pageInfo;
 
     // Virtual body content types, to make parsing a little easier.
     // These are not accessible from outside the parser.
-    private static final String JAVAX_BODY_CONTENT_PARAM = "JAVAX_BODY_CONTENT_PARAM";
+    private static final String JAVAX_BODY_CONTENT_PARAM =
+        "JAVAX_BODY_CONTENT_PARAM";
 
-    private static final String JAVAX_BODY_CONTENT_PLUGIN = "JAVAX_BODY_CONTENT_PLUGIN";
+    private static final String JAVAX_BODY_CONTENT_PLUGIN =
+        "JAVAX_BODY_CONTENT_PLUGIN";
 
-    private static final String JAVAX_BODY_CONTENT_TEMPLATE_TEXT = "JAVAX_BODY_CONTENT_TEMPLATE_TEXT";
+    private static final String JAVAX_BODY_CONTENT_TEMPLATE_TEXT =
+        "JAVAX_BODY_CONTENT_TEMPLATE_TEXT";
 
+    /* System property that controls if the strict white space rules are
+     * applied.
+     */ 
+    private static final boolean STRICT_WHITESPACE = Boolean.valueOf(
+            System.getProperty(
+                    "org.apache.jasper.compiler.Parser.STRICT_WHITESPACE",
+                    "true")).booleanValue();
     /**
      * The constructor
      */
     private Parser(ParserController pc, JspReader reader, boolean isTagFile,
-            boolean directivesOnly, URL jarFileUrl) {
+            boolean directivesOnly, JarResource jarResource) {
         this.parserController = pc;
         this.ctxt = pc.getJspCompilationContext();
         this.pageInfo = pc.getCompiler().getPageInfo();
         this.err = pc.getCompiler().getErrorDispatcher();
         this.reader = reader;
-        this.currentFile = reader.mark().getFile();
         this.scriptlessCount = 0;
         this.isTagFile = isTagFile;
         this.directivesOnly = directivesOnly;
-        this.jarFileUrl = jarFileUrl;
+        this.jarResource = jarResource;
         start = reader.mark();
     }
 
@@ -106,11 +113,12 @@ class Parser implements TagConstants {
      */
     public static Node.Nodes parse(ParserController pc, JspReader reader,
             Node parent, boolean isTagFile, boolean directivesOnly,
-            URL jarFileUrl, String pageEnc, String jspConfigPageEnc,
-            boolean isDefaultPageEncoding, boolean isBomPresent) throws JasperException {
+            JarResource jarResource, String pageEnc, String jspConfigPageEnc,
+            boolean isDefaultPageEncoding, boolean isBomPresent)
+            throws JasperException {
 
         Parser parser = new Parser(pc, reader, isTagFile, directivesOnly,
-                jarFileUrl);
+                jarResource);
 
         Node.Root root = new Node.Root(reader.mark(), parent, false);
         root.setPageEncoding(pageEnc);
@@ -142,11 +150,26 @@ class Parser implements TagConstants {
      * Attributes ::= (S Attribute)* S?
      */
     Attributes parseAttributes() throws JasperException {
-        AttributesImpl attrs = new AttributesImpl();
+        return parseAttributes(false);
+    }
+    Attributes parseAttributes(boolean pageDirective) throws JasperException {
+        UniqueAttributesImpl attrs = new UniqueAttributesImpl(pageDirective);
 
         reader.skipSpaces();
-        while (parseAttribute(attrs))
-            reader.skipSpaces();
+        int ws = 1;
+
+        try {
+            while (parseAttribute(attrs)) {
+                if (ws == 0 && STRICT_WHITESPACE) {
+                    err.jspError(reader.mark(),
+                            "jsp.error.attribute.nowhitespace");
+                }
+                ws = reader.skipSpaces();
+            }
+        } catch (IllegalArgumentException iae) {
+            // Duplicate attribute
+            err.jspError(reader.mark(), "jsp.error.attribute.duplicate");
+        }
 
         return attrs;
     }
@@ -157,7 +180,7 @@ class Parser implements TagConstants {
     public static Attributes parseAttributes(ParserController pc,
             JspReader reader) throws JasperException {
         Parser tmpParser = new Parser(pc, reader, false, false, null);
-        return tmpParser.parseAttributes();
+        return tmpParser.parseAttributes(true);
     }
 
     /**
@@ -167,7 +190,8 @@ class Parser implements TagConstants {
      * around Eq. It is added to be backward compatible with Tomcat, and with
      * other xml parsers.
      */
-    private boolean parseAttribute(AttributesImpl attrs) throws JasperException {
+    private boolean parseAttribute(AttributesImpl attrs)
+            throws JasperException {
 
         // Get the qualified name
         String qName = parseName();
@@ -213,7 +237,7 @@ class Parser implements TagConstants {
     private String parseName() throws JasperException {
         char ch = (char) reader.peekChar();
         if (Character.isLetter(ch) || ch == '_' || ch == ':') {
-            StringBuffer buf = new StringBuffer();
+            StringBuilder buf = new StringBuilder();
             buf.append(ch);
             reader.nextChar();
             ch = (char) reader.peekChar();
@@ -293,7 +317,7 @@ class Parser implements TagConstants {
         }
 
         try {
-            parserController.parse(file, parent, jarFileUrl);
+            parserController.parse(file, parent, jarResource);
         } catch (FileNotFoundException ex) {
             err.jspError(start, "jsp.error.file.not.found", file);
         } catch (Exception ex) {
@@ -306,7 +330,7 @@ class Parser implements TagConstants {
      * Attribute)*
      */
     private void parsePageDirective(Node parent) throws JasperException {
-        Attributes attrs = parseAttributes();
+        Attributes attrs = parseAttributes(true);
         Node.PageDirective n = new Node.PageDirective(attrs, start, parent);
 
         /*
@@ -337,11 +361,11 @@ class Parser implements TagConstants {
      * Add a list of files. This is used for implementing include-prelude and
      * include-coda of jsp-config element in web.xml
      */
-    private void addInclude(Node parent, List files) throws JasperException {
+    private void addInclude(Node parent, List<String> files) throws JasperException {
         if (files != null) {
-            Iterator iter = files.iterator();
+            Iterator<String> iter = files.iterator();
             while (iter.hasNext()) {
-                String file = (String) iter.next();
+                String file = iter.next();
                 AttributesImpl attrs = new AttributesImpl();
                 attrs.addAttribute("", "file", "file", "CDATA", file);
 
@@ -382,9 +406,9 @@ class Parser implements TagConstants {
                                 .getCache().get(uri);
                     }
                     if (impl == null) {
-                        String[] location = ctxt.getTldLocation(uri);
-                        impl = new TagLibraryInfoImpl(ctxt, parserController, pageInfo,
-                                prefix, uri, location, err,
+                        TldLocation location = ctxt.getTldLocation(uri);
+                        impl = new TagLibraryInfoImpl(ctxt, parserController,
+                                pageInfo, prefix, uri, location, err,
                                 reader.mark());
                         if (ctxt.getOptions().isCaching()) {
                             ctxt.getOptions().getCache().put(uri, impl);
@@ -393,8 +417,8 @@ class Parser implements TagConstants {
                         // Current compilation context needs location of cached
                         // tag files
                         for (TagFileInfo info : impl.getTagFiles()) {
-                            ctxt.setTagFileJarUrl(info.getPath(),
-                                    ctxt.getTagFileJarUrl());
+                            ctxt.setTagFileJarResource(info.getPath(),
+                                    ctxt.getTagFileJarResource());
                         }
                     }
                     pageInfo.addTaglib(uri, impl);
@@ -407,7 +431,8 @@ class Parser implements TagConstants {
                     if (pageInfo.getTaglib(urnTagdir) == null) {
                         pageInfo.addTaglib(urnTagdir,
                                 new ImplicitTagLibraryInfo(ctxt,
-                                        parserController, pageInfo, prefix, tagdir, err));
+                                        parserController, pageInfo, prefix,
+                                        tagdir, err));
                     }
                     pageInfo.addPrefixMapping(prefix, urnTagdir);
                 }
@@ -547,7 +572,7 @@ class Parser implements TagConstants {
      * Attribute)*
      */
     private void parseTagDirective(Node parent) throws JasperException {
-        Attributes attrs = parseAttributes();
+        Attributes attrs = parseAttributes(true);
         Node.TagDirective n = new Node.TagDirective(attrs, start, parent);
 
         /*
@@ -568,18 +593,16 @@ class Parser implements TagConstants {
      */
     private void parseAttributeDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
-        Node.AttributeDirective n = new Node.AttributeDirective(attrs, start,
-                parent);
+        new Node.AttributeDirective(attrs, start, parent);
     }
 
     /*
-     * Parses a variable directive with the following syntax: PageDirective ::= (
-     * S Attribute)*
+     * Parses a variable directive with the following syntax:
+     * PageDirective ::= ( S Attribute)*
      */
     private void parseVariableDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
-        Node.VariableDirective n = new Node.VariableDirective(attrs, start,
-                parent);
+        new Node.VariableDirective(attrs, start, parent);
     }
 
     /*
@@ -712,7 +735,8 @@ class Parser implements TagConstants {
      * ELExpressionBody (following "${" to first unquoted "}") // XXX add formal
      * production and confirm implementation against it, // once it's decided
      */
-    private void parseELExpression(Node parent, char type) throws JasperException {
+    private void parseELExpression(Node parent, char type)
+            throws JasperException {
         start = reader.mark();
         Mark last = null;
         boolean singleQuoted = false, doubleQuoted = false;
@@ -1174,10 +1198,14 @@ class Parser implements TagConstants {
         // Check if this is a user-defined tag.
         String uri = pageInfo.getURI(prefix);
         if (uri == null) {
-            reader.reset(start);
-            // Remember the prefix for later error checking
-            pageInfo.putNonCustomTagPrefix(prefix, reader.mark());
-            return false;
+            if (pageInfo.isErrorOnUndeclaredNamespace()) {
+                err.jspError(start, "jsp.error.undeclared_namespace", prefix);
+            } else {
+                reader.reset(start);
+                // Remember the prefix for later error checking
+                pageInfo.putNonCustomTagPrefix(prefix, reader.mark());
+                return false;
+            }
         }
 
         TagLibraryInfo tagLibInfo = pageInfo.getTaglib(uri);
@@ -1186,7 +1214,7 @@ class Parser implements TagConstants {
         if (tagInfo == null && tagFileInfo == null) {
             err.jspError(start, "jsp.error.bad_tag", shortTagName, prefix);
         }
-        Class tagHandlerClass = null;
+        Class<?> tagHandlerClass = null;
         if (tagInfo != null) {
             // Must be a classic tag, load it here.
             // tag files will be loaded later, in TagFileProcessor
@@ -1250,7 +1278,7 @@ class Parser implements TagConstants {
 
     /*
      * Parse for a template text string until '<' or "${" or "#{" is encountered,
-     * recognizing escape sequences "\%", "\$", and "\#".
+     * recognizing escape sequences "<\%", "\$", and "\#".
      */
     private void parseTemplateText(Node parent) throws JasperException {
 
@@ -1267,6 +1295,7 @@ class Parser implements TagConstants {
         }
 
         while (reader.hasMoreInput()) {
+            int prev = ch;
             ch = reader.nextChar();
             if (ch == '<') {
                 reader.pushChar();
@@ -1291,8 +1320,9 @@ class Parser implements TagConstants {
                 }
                 char next = (char) reader.peekChar();
                 // Looking for \% or \$ or \#
-                if (next == '%' || ((next == '$' || next == '#') &&
-                        !pageInfo.isELIgnored())) {
+                if ((prev == '<' && next == '%') ||
+                        ((next == '$' || next == '#') &&
+                                !pageInfo.isELIgnored())) {
                     ch = reader.nextChar();
                 }
             }
