@@ -19,13 +19,10 @@ package org.apache.jasper.compiler;
 
 import java.io.FileNotFoundException;
 import java.io.IOException;
-import java.net.MalformedURLException;
-import java.net.URL;
-import java.net.URLClassLoader;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Vector;
 import java.util.HashMap;
+import java.util.Iterator;
+import java.util.Map.Entry;
+import java.util.Vector;
 
 import javax.el.MethodExpression;
 import javax.el.ValueExpression;
@@ -39,8 +36,8 @@ import javax.servlet.jsp.tagext.VariableInfo;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
-import org.apache.jasper.servlet.JspServletWrapper;
 import org.apache.jasper.runtime.JspSourceDependent;
+import org.apache.jasper.servlet.JspServletWrapper;
 
 /**
  * 1. Processes and extracts the directive info in a tag file. 2. Compiles and
@@ -51,7 +48,7 @@ import org.apache.jasper.runtime.JspSourceDependent;
 
 class TagFileProcessor {
 
-    private Vector tempVector;
+    private Vector<Compiler> tempVector;
 
     /**
      * A visitor the tag file
@@ -118,9 +115,9 @@ class TagFileProcessor {
 
         private String example = null;
 
-        private Vector attributeVector;
+        private Vector<TagAttributeInfo> attributeVector;
 
-        private Vector variableVector;
+        private Vector<TagVariableInfo> variableVector;
 
         private static final String ATTR_NAME = "the name attribute of the attribute directive";
 
@@ -132,9 +129,11 @@ class TagFileProcessor {
 
         private static final String TAG_DYNAMIC = "the dynamic-attributes attribute of the tag directive";
 
-        private HashMap nameTable = new HashMap();
+        private HashMap<String,NameEntry> nameTable =
+            new HashMap<String,NameEntry>();
 
-        private HashMap nameFromTable = new HashMap();
+        private HashMap<String,NameEntry> nameFromTable =
+            new HashMap<String,NameEntry>();
 
         public TagFileDirectiveVisitor(Compiler compiler,
                 TagLibraryInfo tagLibInfo, String name, String path) {
@@ -142,10 +141,11 @@ class TagFileProcessor {
             this.tagLibInfo = tagLibInfo;
             this.name = name;
             this.path = path;
-            attributeVector = new Vector();
-            variableVector = new Vector();
+            attributeVector = new Vector<TagAttributeInfo>();
+            variableVector = new Vector<TagVariableInfo>();
         }
 
+        @Override
         public void visit(Node.TagDirective n) throws JasperException {
 
             JspUtil.checkAttributes("Tag directive", n, tagDirectiveAttrs, err);
@@ -188,6 +188,7 @@ class TagFileProcessor {
             return result;
         }
 
+        @Override
         public void visit(Node.AttributeDirective n) throws JasperException {
 
             JspUtil.checkAttributes("Attribute directive", n,
@@ -287,6 +288,7 @@ class TagFileProcessor {
             checkUniqueName(attrName, ATTR_NAME, n, tagAttributeInfo);
         }
 
+        @Override
         public void visit(Node.VariableDirective n) throws JasperException {
 
             JspUtil.checkAttributes("Variable directive", n,
@@ -349,28 +351,6 @@ class TagFileProcessor {
                     nameFromAttribute, className, declare, scope));
         }
 
-        /*
-         * Returns the vector of attributes corresponding to attribute
-         * directives.
-         */
-        public Vector getAttributesVector() {
-            return attributeVector;
-        }
-
-        /*
-         * Returns the vector of variables corresponding to variable directives.
-         */
-        public Vector getVariablesVector() {
-            return variableVector;
-        }
-
-        /*
-         * Returns the value of the dynamic-attributes tag directive attribute.
-         */
-        public String getDynamicAttributesMapName() {
-            return dynamicAttrsMapName;
-        }
-
         public TagInfo getTagInfo() throws JasperException {
 
             if (name == null) {
@@ -446,10 +426,11 @@ class TagFileProcessor {
         private void checkUniqueName(String name, String type, Node n,
                 TagAttributeInfo attr) throws JasperException {
 
-            HashMap table = (type == VAR_NAME_FROM) ? nameFromTable : nameTable;
-            NameEntry nameEntry = (NameEntry) table.get(name);
+            HashMap<String, NameEntry> table = (type == VAR_NAME_FROM) ? nameFromTable : nameTable;
+            NameEntry nameEntry = table.get(name);
             if (nameEntry != null) {
-                if (type != TAG_DYNAMIC || nameEntry.getType() != TAG_DYNAMIC) {
+                if (!TAG_DYNAMIC.equals(type) ||
+                        !TAG_DYNAMIC.equals(nameEntry.getType())) {
                     int line = nameEntry.getNode().getStart().getLineNumber();
                     err.jspError(n, "jsp.error.tagfile.nameNotUnique", type,
                             nameEntry.getType(), Integer.toString(line));
@@ -460,16 +441,15 @@ class TagFileProcessor {
         }
 
         /**
-         * Perform miscellean checks after the nodes are visited.
+         * Perform miscellaneous checks after the nodes are visited.
          */
         void postCheck() throws JasperException {
             // Check that var.name-from-attributes has valid values.
-            Iterator iter = nameFromTable.keySet().iterator();
+            Iterator<String> iter = nameFromTable.keySet().iterator();
             while (iter.hasNext()) {
-                String nameFrom = (String) iter.next();
-                NameEntry nameEntry = (NameEntry) nameTable.get(nameFrom);
-                NameEntry nameFromEntry = (NameEntry) nameFromTable
-                        .get(nameFrom);
+                String nameFrom = iter.next();
+                NameEntry nameEntry = nameTable.get(nameFrom);
+                NameEntry nameFromEntry = nameFromTable.get(nameFrom);
                 Node nameFromNode = nameFromEntry.getNode();
                 if (nameEntry == null) {
                     err.jspError(nameFromNode,
@@ -500,50 +480,24 @@ class TagFileProcessor {
      *            the current ParserController used in this compilation
      * @param name
      *            the tag name as specified in the TLD
-     * @param tagfile
+     * @param path
      *            the path for the tagfile
-     * @param tagLibInfo
-     *            the TagLibraryInfo object associated with this TagInfo
-     * @return a TagInfo object assembled from the directives in the tag file.
-     * @deprecated Use {@link TagFileProcessor#parseTagFileDirectives(
-     *                  ParserController, String, String, URL, TagLibraryInfo)}
-     *             See https://issues.apache.org/bugzilla/show_bug.cgi?id=46471
-     */
-    public static TagInfo parseTagFileDirectives(ParserController pc,
-            String name, String path, TagLibraryInfo tagLibInfo)
-            throws JasperException {
-        return parseTagFileDirectives(pc, name, path,
-                pc.getJspCompilationContext().getTagFileJarUrl(path),
-                tagLibInfo);
-    }
-    
-    /**
-     * Parses the tag file, and collects information on the directives included
-     * in it. The method is used to obtain the info on the tag file, when the
-     * handler that it represents is referenced. The tag file is not compiled
-     * here.
-     * 
-     * @param pc
-     *            the current ParserController used in this compilation
-     * @param name
-     *            the tag name as specified in the TLD
-     * @param tagfile
-     *            the path for the tagfile
-     * @param tagFileJarUrl
-     *            the url for the Jar containign the tag file 
+     * @param jarResource
+     *            the Jar resource containing the tag file 
      * @param tagLibInfo
      *            the TagLibraryInfo object associated with this TagInfo
      * @return a TagInfo object assembled from the directives in the tag file.
      */
     public static TagInfo parseTagFileDirectives(ParserController pc,
-            String name, String path, URL tagFileJarUrl, TagLibraryInfo tagLibInfo)
+            String name, String path, JarResource jarResource, TagLibraryInfo tagLibInfo)
             throws JasperException {
 
+
         ErrorDispatcher err = pc.getCompiler().getErrorDispatcher();
 
         Node.Nodes page = null;
         try {
-            page = pc.parseTagFileDirectives(path, tagFileJarUrl);
+            page = pc.parseTagFileDirectives(path, jarResource);
         } catch (FileNotFoundException e) {
             err.jspError("jsp.error.file.not.found", path);
         } catch (IOException e) {
@@ -561,41 +515,36 @@ class TagFileProcessor {
     /**
      * Compiles and loads a tagfile.
      */
-    private Class loadTagFile(Compiler compiler, String tagFilePath,
+    private Class<?> loadTagFile(Compiler compiler, String tagFilePath,
             TagInfo tagInfo, PageInfo parentPageInfo) throws JasperException {
 
-        URL tagFileJarUrl = null;
+        JarResource tagJarResouce = null;
         if (tagFilePath.startsWith("/META-INF/")) {
-            try { 
-                tagFileJarUrl = new URL("jar:" +
-                        compiler.getCompilationContext().getTldLocation(
-                        tagInfo.getTagLibrary().getURI())[0] + "!/");
-            } catch (MalformedURLException e) {
-                // Ignore - tagFileJarUrl will be null
-            }
+            tagJarResouce = 
+                compiler.getCompilationContext().getTldLocation(
+                        tagInfo.getTagLibrary().getURI()).getJarResource();
         }
-        String tagFileJarPath;
-        if (tagFileJarUrl == null) {
-            tagFileJarPath = "";
+        String wrapperUri;
+        if (tagJarResouce == null) {
+            wrapperUri = tagFilePath;
         } else {
-            tagFileJarPath = tagFileJarUrl.toString();
+            wrapperUri = tagJarResouce.getEntry(tagFilePath).toString();
         }
 
         JspCompilationContext ctxt = compiler.getCompilationContext();
         JspRuntimeContext rctxt = ctxt.getRuntimeContext();
-        String wrapperUri = tagFileJarPath + tagFilePath;
-        JspServletWrapper wrapper = rctxt.getWrapper(wrapperUri);
 
         synchronized (rctxt) {
+            JspServletWrapper wrapper = rctxt.getWrapper(wrapperUri);
             if (wrapper == null) {
                 wrapper = new JspServletWrapper(ctxt.getServletContext(), ctxt
                         .getOptions(), tagFilePath, tagInfo, ctxt
-                        .getRuntimeContext(), tagFileJarUrl);
+                        .getRuntimeContext(), tagJarResouce);
                 rctxt.addWrapper(wrapperUri, wrapper);
 
                 // Use same classloader and classpath for compiling tag files
                 wrapper.getJspEngineContext().setClassLoader(
-                        (URLClassLoader) ctxt.getClassLoader());
+                        ctxt.getClassLoader());
                 wrapper.getJspEngineContext().setClassPath(ctxt.getClassPath());
             } else {
                 // Make sure that JspCompilationContext gets the latest TagInfo
@@ -605,19 +554,23 @@ class TagFileProcessor {
                 wrapper.getJspEngineContext().setTagInfo(tagInfo);
             }
 
-            Class tagClazz;
+            Class<?> tagClazz;
             int tripCount = wrapper.incTripCount();
             try {
                 if (tripCount > 0) {
                     // When tripCount is greater than zero, a circular
-                    // dependency exists. The circularily dependant tag
+                    // dependency exists. The circularly dependent tag
                     // file is compiled in prototype mode, to avoid infinite
                     // recursion.
 
                     JspServletWrapper tempWrapper = new JspServletWrapper(ctxt
                             .getServletContext(), ctxt.getOptions(),
                             tagFilePath, tagInfo, ctxt.getRuntimeContext(),
-                            ctxt.getTagFileJarUrl(tagFilePath));
+                            ctxt.getTagFileJarResource(tagFilePath));
+                    // Use same classloader and classpath for compiling tag files
+                    tempWrapper.getJspEngineContext().setClassLoader(
+                            ctxt.getClassLoader());
+                    tempWrapper.getJspEngineContext().setClassPath(ctxt.getClassPath());
                     tagClazz = tempWrapper.loadTagFilePrototype();
                     tempVector.add(tempWrapper.getJspEngineContext()
                             .getCompiler());
@@ -628,16 +581,18 @@ class TagFileProcessor {
                 wrapper.decTripCount();
             }
 
-            // Add the dependants for this tag file to its parent's
-            // dependant list. The only reliable dependency information
+            // Add the dependents for this tag file to its parent's
+            // Dependent list. The only reliable dependency information
             // can only be obtained from the tag instance.
             try {
                 Object tagIns = tagClazz.newInstance();
                 if (tagIns instanceof JspSourceDependent) {
-                    Iterator iter = ((List) ((JspSourceDependent) tagIns)
-                            .getDependants()).iterator();
+                    Iterator<Entry<String,Long>> iter = ((JspSourceDependent)
+                            tagIns).getDependants().entrySet().iterator();
                     while (iter.hasNext()) {
-                        parentPageInfo.addDependant((String) iter.next());
+                        Entry<String,Long> entry = iter.next();
+                        parentPageInfo.addDependant(entry.getKey(),
+                                entry.getValue());
                     }
                 }
             } catch (Exception e) {
@@ -664,25 +619,40 @@ class TagFileProcessor {
             this.pageInfo = compiler.getPageInfo();
         }
 
+        @Override
         public void visit(Node.CustomTag n) throws JasperException {
             TagFileInfo tagFileInfo = n.getTagFileInfo();
             if (tagFileInfo != null) {
                 String tagFilePath = tagFileInfo.getPath();
                 if (tagFilePath.startsWith("/META-INF/")) {
                     // For tags in JARs, add the TLD and the tag as a dependency
-                    String[] location =
+                    TldLocation location =
                         compiler.getCompilationContext().getTldLocation(
                             tagFileInfo.getTagInfo().getTagLibrary().getURI());
-                    // Add TLD
-                    pageInfo.addDependant("jar:" + location[0] + "!/" +
-                            location[1]);
-                    // Add Tag
-                    pageInfo.addDependant("jar:" + location[0] + "!" +
-                            tagFilePath);
+                    JarResource jarResource = location.getJarResource();
+                    if (jarResource != null) {
+                        try {
+                            // Add TLD
+                            pageInfo.addDependant(jarResource.getEntry(location.getName()).toString(),
+                                    Long.valueOf(jarResource.getJarFile().getEntry(location.getName()).getTime()));
+                            // Add Tag
+                            pageInfo.addDependant(jarResource.getEntry(tagFilePath.substring(1)).toString(),
+                                    Long.valueOf(jarResource.getJarFile().getEntry(tagFilePath.substring(1)).getTime()));
+                        } catch (IOException ioe) {
+                            throw new JasperException(ioe);
+                        }
+                    }
+                    else {
+                        pageInfo.addDependant(tagFilePath,
+                                compiler.getCompilationContext().getLastModified(
+                                        tagFilePath));
+                    }
                 } else {
-                    pageInfo.addDependant(tagFilePath);
+                    pageInfo.addDependant(tagFilePath,
+                            compiler.getCompilationContext().getLastModified(
+                                    tagFilePath));
                 }
-                Class c = loadTagFile(compiler, tagFilePath, n.getTagInfo(),
+                Class<?> c = loadTagFile(compiler, tagFilePath, n.getTagInfo(),
                         pageInfo);
                 n.setTagHandlerClass(c);
             }
@@ -693,13 +663,13 @@ class TagFileProcessor {
     /**
      * Implements a phase of the translation that compiles (if necessary) the
      * tag files used in a JSP files. The directives in the tag files are
-     * assumed to have been proccessed and encapsulated as TagFileInfo in the
+     * assumed to have been processed and encapsulated as TagFileInfo in the
      * CustomTag nodes.
      */
     public void loadTagFiles(Compiler compiler, Node.Nodes page)
             throws JasperException {
 
-        tempVector = new Vector();
+        tempVector = new Vector<Compiler>();
         page.visit(new TagFileLoaderVisitor(compiler));
     }
 
@@ -711,9 +681,9 @@ class TagFileProcessor {
      *            If non-null, remove only the class file with with this name.
      */
     public void removeProtoTypeFiles(String classFileName) {
-        Iterator iter = tempVector.iterator();
+        Iterator<Compiler> iter = tempVector.iterator();
         while (iter.hasNext()) {
-            Compiler c = (Compiler) iter.next();
+            Compiler c = iter.next();
             if (classFileName == null) {
                 c.removeGeneratedClassFiles();
             } else if (classFileName.equals(c.getCompilationContext()
