@@ -22,17 +22,19 @@ import javax.servlet.jsp.el.ELException;
 import javax.servlet.jsp.el.Expression;
 import javax.servlet.jsp.el.VariableResolver;
 
+@Deprecated
 public final class ExpressionImpl extends Expression {
 
-	private final ValueExpression ve;
-	
-	public ExpressionImpl(ValueExpression ve) {
-		this.ve = ve;
-	}
+    private final ValueExpression ve;
+    
+    public ExpressionImpl(ValueExpression ve) {
+        this.ve = ve;
+    }
 
-	public Object evaluate(VariableResolver vResolver) throws ELException {
-		ELContext ctx = new ELContextImpl(new ELResolverImpl(vResolver));
-		return ve.getValue(ctx);
-	}
+    @Override
+    public Object evaluate(VariableResolver vResolver) throws ELException {
+        ELContext ctx = new ELContextImpl(new ELResolverImpl(vResolver));
+        return ve.getValue(ctx);
+    }
 
 }
