@@ -49,6 +49,7 @@ import javax.servlet.jsp.el.ExpressionEvaluator;
 import javax.servlet.jsp.el.VariableResolver;
 import javax.servlet.jsp.tagext.BodyContent;
 
+import org.apache.jasper.Constants;
 import org.apache.jasper.compiler.Localizer;
 import org.apache.jasper.el.ELContextImpl;
 import org.apache.jasper.el.ExpressionEvaluatorImpl;
@@ -150,6 +151,9 @@ public class PageContextImpl extends PageContext {
 
         // initialize the initial out ...
         depth = -1;
+        if (bufferSize == JspWriter.DEFAULT_BUFFER) {
+            bufferSize = Constants.DEFAULT_BUFFER_SIZE;
+        }
         if (this.baseOut == null) {
             this.baseOut = new JspWriterImpl(response, bufferSize, autoFlush);
         } else {
@@ -723,6 +727,7 @@ public class PageContextImpl extends PageContext {
         // JSP.4.5 If the buffer was flushed, throw IllegalStateException
         try {
             out.clear();
+            baseOut.clear();
         } catch (IOException ex) {
             IllegalStateException ise = new IllegalStateException(Localizer
                     .getMessage("jsp.error.attempt_to_clear_flushed_buffer"));
@@ -912,29 +917,6 @@ public class PageContextImpl extends PageContext {
         }
     }
 
-    private static String XmlEscape(String s) {
-        if (s == null)
-            return null;
-        StringBuilder sb = new StringBuilder();
-        for (int i = 0; i < s.length(); i++) {
-            char c = s.charAt(i);
-            if (c == '<') {
-                sb.append("&lt;");
-            } else if (c == '>') {
-                sb.append("&gt;");
-            } else if (c == '\'') {
-                sb.append("&#039;"); // &apos;
-            } else if (c == '&') {
-                sb.append("&amp;");
-            } else if (c == '"') {
-                sb.append("&#034;"); // &quot;
-            } else {
-                sb.append(c);
-            }
-        }
-        return sb.toString();
-    }
-
     /**
      * Proprietary method to evaluate EL expressions. XXX - This method should
      * go away once the EL interpreter moves out of JSTL and into its own
@@ -984,9 +966,6 @@ public class PageContextImpl extends PageContext {
             ValueExpression ve = exprFactory.createValueExpression(ctx, expression, expectedType);
             retValue = ve.getValue(ctx);
         }
-        if (escape && retValue != null) {
-            retValue = XmlEscape(retValue.toString());
-        }
 
         return retValue;
     }
