@@ -22,11 +22,12 @@ import java.security.PrivilegedAction;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Properties;
+
 import javax.mail.Authenticator;
 import javax.mail.PasswordAuthentication;
 import javax.mail.Session;
-import javax.naming.Name;
 import javax.naming.Context;
+import javax.naming.Name;
 import javax.naming.RefAddr;
 import javax.naming.Reference;
 import javax.naming.spi.ObjectFactory;
@@ -43,8 +44,8 @@ import javax.naming.spi.ObjectFactory;
  *     connections.  Defaults to <code>localhost</code> if not specified.</li>
  * </ul>
  *
- * <p>This factory can be configured in a <code>&lt;DefaultContext&gt;</code>
- * or <code>&lt;Context&gt;</code> element in your <code>conf/server.xml</code>
+ * <p>This factory can be configured in a 
+ * <code>&lt;Context&gt;</code> element in your <code>conf/server.xml</code>
  * configuration file.  An example of factory configuration is:</p>
  * <pre>
  * &lt;Resource name="mail/smtp" auth="CONTAINER"
@@ -62,9 +63,7 @@ import javax.naming.spi.ObjectFactory;
  * </pre>
  *
  * @author Craig R. McClanahan
- * @version $Id: MailSessionFactory.java 939511 2010-04-29 23:48:20Z kkolinko $
  */
-
 public class MailSessionFactory implements ObjectFactory {
 
 
@@ -88,12 +87,12 @@ public class MailSessionFactory implements ObjectFactory {
      *
      * @exception Exception if an error occurs during object creation
      */
+    @Override
     public Object getObjectInstance(Object refObj, Name name, Context context,
-				    Hashtable env) throws Exception 
-    {
+            Hashtable<?,?> env) throws Exception {
 
         // Return null if we cannot create an object of the requested type
-	final Reference ref = (Reference) refObj;
+        final Reference ref = (Reference) refObj;
         if (!ref.getClassName().equals(factoryType))
             return (null);
 
@@ -102,8 +101,9 @@ public class MailSessionFactory implements ObjectFactory {
         // exceptions.
         //
         // Bugzilla 31288, 33077: add support for authentication.
-        return AccessController.doPrivileged( new PrivilegedAction() {
-		public Object run() {
+        return AccessController.doPrivileged(new PrivilegedAction<Session>() {
+                @Override
+                public Session run() {
 
                     // Create the JavaMail properties we will use
                     Properties props = new Properties();
@@ -112,9 +112,9 @@ public class MailSessionFactory implements ObjectFactory {
 
                     String password = null;
 
-                    Enumeration attrs = ref.getAll();
+                    Enumeration<RefAddr> attrs = ref.getAll();
                     while (attrs.hasMoreElements()) {
-                        RefAddr attr = (RefAddr) attrs.nextElement();
+                        RefAddr attr = attrs.nextElement();
                         if ("factory".equals(attr.getType())) {
                             continue;
                         }
@@ -124,7 +124,7 @@ public class MailSessionFactory implements ObjectFactory {
                             continue;
                         }
 
-                        props.put(attr.getType(), (String) attr.getContent());
+                        props.put(attr.getType(), attr.getContent());
                     }
 
                     Authenticator auth = null;
@@ -137,6 +137,7 @@ public class MailSessionFactory implements ObjectFactory {
                         if(user != null) {
                             final PasswordAuthentication pa = new PasswordAuthentication(user, password);
                             auth = new Authenticator() {
+                                    @Override
                                     protected PasswordAuthentication getPasswordAuthentication() {
                                         return pa;
                                     }
@@ -148,10 +149,7 @@ public class MailSessionFactory implements ObjectFactory {
                     Session session = Session.getInstance(props, auth);
                     return (session);
 
-		}
-	    } );
-
+                }
+        } );
     }
-
-
 }
