@@ -20,20 +20,25 @@ package org.apache.tomcat.jni;
 /** Poll
  *
  * @author Mladen Turk
- * @version $Id: Poll.java 939309 2010-04-29 13:54:26Z kkolinko $
  */
 
 public class Poll {
 
     /**
-     * Poll options
+     * Poll return values
      */
-    public static final int APR_POLLIN   = 0x001; /** Can read without blocking */
-    public static final int APR_POLLPRI  = 0x002; /** Priority data available */
-    public static final int APR_POLLOUT  = 0x004; /** Can write without blocking */
-    public static final int APR_POLLERR  = 0x010; /** Pending error */
-    public static final int APR_POLLHUP  = 0x020; /** Hangup occurred */
-    public static final int APR_POLLNVAL = 0x040; /** Descriptor invalid */
+    /** Can read without blocking */
+    public static final int APR_POLLIN   = 0x001;
+    /** Priority data available */
+    public static final int APR_POLLPRI  = 0x002;
+    /** Can write without blocking */
+    public static final int APR_POLLOUT  = 0x004;
+    /** Pending error */
+    public static final int APR_POLLERR  = 0x010;
+    /** Hangup occurred */
+    public static final int APR_POLLHUP  = 0x020;
+    /** Descriptor invalid */
+    public static final int APR_POLLNVAL = 0x040;
 
     /**
      * Pollset Flags
@@ -73,17 +78,24 @@ public class Poll {
     public static native int destroy(long pollset);
 
     /**
-     * Add a socket or to a pollset
-     * If you set client_data in the descriptor, that value
-     * will be returned in the client_data field whenever this
-     * descriptor is signaled in apr_pollset_poll().
-     * @param pollset The pollset to which to add the descriptor
+     * Add a socket to a pollset with the default timeout.
+     * @param pollset The pollset to which to add the socket
      * @param sock The sockets to add
      * @param reqevents requested events
      */
     public static native int add(long pollset, long sock,
                                  int reqevents);
 
+    /**
+     * Add a socket to a pollset with a specific timeout.
+     * @param pollset The pollset to which to add the socket
+     * @param sock The sockets to add
+     * @param reqevents requested events
+     * @param timeout requested timeout in microseconds (-1 for infinite)
+     */
+    public static native int addWithTimeout(long pollset, long sock,
+                                            int reqevents, long timeout);
+
     /**
      * Remove a descriptor from a pollset
      * @param pollset The pollset from which to remove the descriptor
