@@ -24,22 +24,20 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.util.Map;
+import java.util.UUID;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.tomcat.util.http.fileupload.DeferredFileOutputStream;
 import org.apache.tomcat.util.http.fileupload.FileItem;
 import org.apache.tomcat.util.http.fileupload.FileItemHeaders;
-import org.apache.tomcat.util.http.fileupload.FileItemHeadersSupport;
 import org.apache.tomcat.util.http.fileupload.FileUploadException;
 import org.apache.tomcat.util.http.fileupload.IOUtils;
 import org.apache.tomcat.util.http.fileupload.ParameterParser;
 import org.apache.tomcat.util.http.fileupload.util.Streams;
 
-
 /**
  * <p> The default implementation of the
  * {@link org.apache.tomcat.util.http.fileupload.FileItem FileItem} interface.
@@ -54,42 +52,15 @@ import org.apache.tomcat.util.http.fileupload.util.Streams;
  * it into memory, which may come handy with large files.
  *
  * <p>Temporary files, which are created for file items, should be
- * deleted later on. The best way to do this is using a
- * {@link org.apache.tomcat.util.http.fileupload.FileCleaningTracker
- * FileCleaningTracker}, which you can set on the
- * {@link DiskFileItemFactory}. However, if you do use such a tracker,
- * then you must consider the following: Temporary files are automatically
- * deleted as soon as they are no longer needed. (More precisely, when the
- * corresponding instance of {@link java.io.File} is garbage collected.)
- * This is done by the so-called reaper thread, which is started
- * automatically when the class {@link org.apache.commons.io.FileCleaner}
- * is loaded.
- * It might make sense to terminate that thread, for example, if
- * your web application ends. See the section on "Resource cleanup"
- * in the users guide of commons-fileupload.</p>
- *
- * @author <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal Krzewski</a>
- * @author <a href="mailto:sean@informage.net">Sean Legassick</a>
- * @author <a href="mailto:jvanzyl@apache.org">Jason van Zyl</a>
- * @author <a href="mailto:jmcnally@apache.org">John McNally</a>
- * @author <a href="mailto:martinc@apache.org">Martin Cooper</a>
- * @author Sean C. Sullivan
+ * deleted later on.</p>
  *
  * @since FileUpload 1.1
- *
- * @version $Id: DiskFileItem.java 1154575 2011-08-06 20:19:29Z markt $
  */
 public class DiskFileItem
-    implements FileItem, FileItemHeadersSupport {
+    implements FileItem {
 
     // ----------------------------------------------------- Manifest constants
 
-    /**
-     * The UID to use when serializing this instance.
-     */
-    private static final long serialVersionUID = 2237570099615271025L;
-
-
     /**
      * Default content charset to be used when no explicit charset
      * parameter is provided by the sender. Media subtypes of the
@@ -98,47 +69,39 @@ public class DiskFileItem
      */
     public static final String DEFAULT_CHARSET = "ISO-8859-1";
 
-
     // ----------------------------------------------------------- Data members
 
-
     /**
      * UID used in unique file name generation.
      */
     private static final String UID =
-            new java.rmi.server.UID().toString()
-                .replace(':', '_').replace('-', '_');
+            UUID.randomUUID().toString().replace('-', '_');
 
     /**
      * Counter used in unique identifier generation.
      */
-    private static int counter = 0;
-
+    private static final AtomicInteger COUNTER = new AtomicInteger(0);
 
     /**
      * The name of the form field as provided by the browser.
      */
     private String fieldName;
 
-
     /**
      * The content type passed by the browser, or <code>null</code> if
      * not defined.
      */
-    private String contentType;
-
+    private final String contentType;
 
     /**
      * Whether or not this item is a simple form field.
      */
     private boolean isFormField;
 
-
     /**
      * The original filename in the user's filesystem.
      */
-    private String fileName;
-
+    private final String fileName;
 
     /**
      * The size of the item, in bytes. This is used to cache the size when a
@@ -150,21 +113,18 @@ public class DiskFileItem
     /**
      * The threshold above which uploads will be stored on disk.
      */
-    private int sizeThreshold;
-
+    private final int sizeThreshold;
 
     /**
      * The directory in which uploaded files will be stored, if stored on disk.
      */
-    private File repository;
-
+    private final File repository;
 
     /**
      * Cached contents of the file.
      */
     private byte[] cachedContent;
 
-
     /**
      * Output stream for this item.
      */
@@ -175,11 +135,6 @@ public class DiskFileItem
      */
     private transient File tempFile;
 
-    /**
-     * File to allow for serialization of the content of this item.
-     */
-    private File dfosFile;
-
     /**
      * The file items headers.
      */
@@ -187,7 +142,6 @@ public class DiskFileItem
 
     // ----------------------------------------------------------- Constructors
 
-
     /**
      * Constructs a new <code>DiskFileItem</code> instance.
      *
@@ -216,10 +170,8 @@ public class DiskFileItem
         this.repository = repository;
     }
 
-
     // ------------------------------- Methods from javax.activation.DataSource
 
-
     /**
      * Returns an {@link java.io.InputStream InputStream} that can be
      * used to retrieve the contents of the file.
@@ -242,7 +194,6 @@ public class DiskFileItem
         return new ByteArrayInputStream(cachedContent);
     }
 
-
     /**
      * Returns the content type passed by the agent or <code>null</code> if
      * not defined.
@@ -255,7 +206,6 @@ public class DiskFileItem
         return contentType;
     }
 
-
     /**
      * Returns the content charset passed by the agent or <code>null</code> if
      * not defined.
@@ -271,7 +221,6 @@ public class DiskFileItem
         return params.get("charset");
     }
 
-
     /**
      * Returns the original filename in the client's filesystem.
      *
@@ -279,17 +228,16 @@ public class DiskFileItem
      * @throws org.apache.tomcat.util.http.fileupload.InvalidFileNameException
      *   The file name contains a NUL character, which might be an indicator of
      *   a security attack. If you intend to use the file name anyways, catch
-     *   the exception and use InvalidFileNameException#getName().
+     *   the exception and use {@link
+     *   org.apache.tomcat.util.http.fileupload.InvalidFileNameException#getName()}.
      */
     @Override
     public String getName() {
         return Streams.checkFileName(fileName);
     }
 
-
     // ------------------------------------------------------- FileItem methods
 
-
     /**
      * Provides a hint as to whether or not the file contents will be read
      * from memory.
@@ -305,7 +253,6 @@ public class DiskFileItem
         return dfos.isInMemory();
     }
 
-
     /**
      * Returns the size of the file.
      *
@@ -324,7 +271,6 @@ public class DiskFileItem
         }
     }
 
-
     /**
      * Returns the contents of the file as an array of bytes.  If the
      * contents of the file were not yet cached in memory, they will be
@@ -342,10 +288,10 @@ public class DiskFileItem
         }
 
         byte[] fileData = new byte[(int) getSize()];
-        FileInputStream fis = null;
+        InputStream fis = null;
 
         try {
-            fis = new FileInputStream(dfos.getFile());
+            fis = new BufferedInputStream(new FileInputStream(dfos.getFile()));
             fis.read(fileData);
         } catch (IOException e) {
             fileData = null;
@@ -362,7 +308,6 @@ public class DiskFileItem
         return fileData;
     }
 
-
     /**
      * Returns the contents of the file as a String, using the specified
      * encoding.  This method uses {@link #get()} to retrieve the
@@ -381,15 +326,14 @@ public class DiskFileItem
         return new String(get(), charset);
     }
 
-
     /**
      * Returns the contents of the file as a String, using the default
      * character encoding.  This method uses {@link #get()} to retrieve the
      * contents of the file.
      *
-     * @return The contents of the file, as a string.
+     * <b>TODO</b> Consider making this method throw UnsupportedEncodingException.
      *
-     * TODO Consider making this method throw UnsupportedEncodingException.
+     * @return The contents of the file, as a string.
      */
     @Override
     public String getString() {
@@ -405,7 +349,6 @@ public class DiskFileItem
         }
     }
 
-
     /**
      * A convenience method to write an uploaded item to disk. The client code
      * is not concerned with whether or not the item is stored in memory, or on
@@ -485,7 +428,6 @@ public class DiskFileItem
         }
     }
 
-
     /**
      * Deletes the underlying storage for a file item, including deleting any
      * associated temporary disk file. Although this storage will be deleted
@@ -502,7 +444,6 @@ public class DiskFileItem
         }
     }
 
-
     /**
      * Returns the name of the field in the multipart form corresponding to
      * this file item.
@@ -517,7 +458,6 @@ public class DiskFileItem
         return fieldName;
     }
 
-
     /**
      * Sets the field name used to reference this file item.
      *
@@ -531,7 +471,6 @@ public class DiskFileItem
         this.fieldName = fieldName;
     }
 
-
     /**
      * Determines whether or not a <code>FileItem</code> instance represents
      * a simple form field.
@@ -547,7 +486,6 @@ public class DiskFileItem
         return isFormField;
     }
 
-
     /**
      * Specifies whether or not a <code>FileItem</code> instance represents
      * a simple form field.
@@ -563,7 +501,6 @@ public class DiskFileItem
         isFormField = state;
     }
 
-
     /**
      * Returns an {@link java.io.OutputStream OutputStream} that can
      * be used for storing the contents of the file.
@@ -583,10 +520,8 @@ public class DiskFileItem
         return dfos;
     }
 
-
     // --------------------------------------------------------- Public methods
 
-
     /**
      * Returns the {@link java.io.File} object for the <code>FileItem</code>'s
      * data's temporary location on the disk. Note that for
@@ -601,13 +536,14 @@ public class DiskFileItem
      *         memory.
      */
     public File getStoreLocation() {
-        return dfos == null ? null : dfos.getFile();
+        if (dfos == null) {
+            return null;
+        }
+        return dfos.getFile();
     }
 
-
     // ------------------------------------------------------ Protected methods
 
-
     /**
      * Removes the file contents from the temporary storage.
      */
@@ -620,7 +556,6 @@ public class DiskFileItem
         }
     }
 
-
     /**
      * Creates and returns a {@link java.io.File File} representing a uniquely
      * named temporary file in the configured repository path. The lifetime of
@@ -637,17 +572,15 @@ public class DiskFileItem
             }
 
             String tempFileName =
-                "upload_" + UID + "_" + getUniqueId() + ".tmp";
+                    String.format("upload_%s_%s.tmp", UID, getUniqueId());
 
             tempFile = new File(tempDir, tempFileName);
         }
         return tempFile;
     }
 
-
     // -------------------------------------------------------- Private methods
 
-
     /**
      * Returns an identifier that is unique within the class loader used to
      * load this class, but does not have random-like apearance.
@@ -656,10 +589,7 @@ public class DiskFileItem
      */
     private static String getUniqueId() {
         final int limit = 100000000;
-        int current;
-        synchronized (DiskFileItem.class) {
-            current = counter++;
-        }
+        int current = COUNTER.getAndIncrement();
         String id = Integer.toString(current);
 
         // If you manage to get more than 100 million of ids, you'll
@@ -670,7 +600,6 @@ public class DiskFileItem
         return id;
     }
 
-
     /**
      * Returns a string representation of this object.
      *
@@ -678,66 +607,9 @@ public class DiskFileItem
      */
     @Override
     public String toString() {
-        return "name=" + this.getName()
-            + ", StoreLocation="
-            + String.valueOf(this.getStoreLocation())
-            + ", size="
-            + this.getSize()
-            + "bytes, "
-            + "isFormField=" + isFormField()
-            + ", FieldName="
-            + this.getFieldName();
-    }
-
-
-    // -------------------------------------------------- Serialization methods
-
-
-    /**
-     * Writes the state of this object during serialization.
-     *
-     * @param out The stream to which the state should be written.
-     *
-     * @throws IOException if an error occurs.
-     */
-    private void writeObject(ObjectOutputStream out) throws IOException {
-        // Read the data
-        if (dfos.isInMemory()) {
-            cachedContent = get();
-        } else {
-            cachedContent = null;
-            dfosFile = dfos.getFile();
-        }
-
-        // write out values
-        out.defaultWriteObject();
-    }
-
-    /**
-     * Reads the state of this object during deserialization.
-     *
-     * @param in The stream from which the state should be read.
-     *
-     * @throws IOException if an error occurs.
-     * @throws ClassNotFoundException if class cannot be found.
-     */
-    private void readObject(ObjectInputStream in)
-            throws IOException, ClassNotFoundException {
-        // read values
-        in.defaultReadObject();
-
-        OutputStream output = getOutputStream();
-        if (cachedContent != null) {
-            output.write(cachedContent);
-        } else {
-            FileInputStream input = new FileInputStream(dfosFile);
-            IOUtils.copy(input, output);
-            dfosFile.delete();
-            dfosFile = null;
-        }
-        output.close();
-
-        cachedContent = null;
+        return String.format("name=%s, StoreLocation=%s, size=%s bytes, isFormField=%s, FieldName=%s",
+                      getName(), getStoreLocation(), Long.valueOf(getSize()),
+                      Boolean.valueOf(isFormField()), getFieldName());
     }
 
     /**
@@ -757,4 +629,5 @@ public class DiskFileItem
     public void setHeaders(FileItemHeaders pHeaders) {
         headers = pHeaders;
     }
+
 }
