@@ -42,9 +42,7 @@ import javax.management.ServiceNotFoundException;
  * descriptor.</p>
  *
  * @author Craig R. McClanahan
- * @version $Id: ManagedBean.java 1176157 2011-09-27 01:21:08Z kkolinko $
  */
-
 public class ManagedBean implements java.io.Serializable {
 
     private static final long serialVersionUID = 1L;
@@ -291,7 +289,7 @@ public class ManagedBean implements java.io.Serializable {
      * @param operation The new operation descriptor
      */
     public void addOperation(OperationInfo operation) {
-        operations.put(operation.getName(), operation);
+        operations.put(createOperationKey(operation), operation);
     }
 
 
@@ -305,8 +303,6 @@ public class ManagedBean implements java.io.Serializable {
      *
      * @exception InstanceNotFoundException if the managed resource
      *  object cannot be found
-     * @exception javax.management.modelmbean.InvalidTargetObjectTypeException
-     *  if our MBean cannot handle object references (should never happen)
      * @exception MBeanException if a problem occurs instantiating the
      *  <code>ModelMBean</code> instance
      * @exception RuntimeOperationsException if a JMX runtime error occurs
@@ -333,8 +329,6 @@ public class ManagedBean implements java.io.Serializable {
      *
      * @exception InstanceNotFoundException if the managed resource
      *  object cannot be found
-     * @exception javax.management.modelmbean.InvalidTargetObjectTypeException
-     *  if our MBean cannot handle object references (should never happen)
      * @exception MBeanException if a problem occurs instantiating the
      *  <code>ModelMBean</code> instance
      * @exception RuntimeOperationsException if a JMX runtime error occurs
@@ -581,7 +575,8 @@ public class ManagedBean implements java.io.Serializable {
 
         // Acquire the ModelMBeanOperationInfo information for
         // the requested operation
-        OperationInfo opInfo = operations.get(aname);
+        OperationInfo opInfo =
+                operations.get(createOperationKey(aname, signature));
         if (opInfo == null)
             throw new MBeanException(new ServiceNotFoundException(
                     "Cannot find operation " + aname),
@@ -622,4 +617,31 @@ public class ManagedBean implements java.io.Serializable {
     }
 
 
+    private String createOperationKey(OperationInfo operation) {
+        StringBuilder key = new StringBuilder(operation.getName());
+        key.append('(');
+        for (ParameterInfo parameterInfo: operation.getSignature()) {
+            key.append(parameterInfo.getType());
+            // Note: A trailing ',' does not matter in this case
+            key.append(',');
+        }
+        key.append(')');
+
+        return key.toString();
+    }
+
+
+    private String createOperationKey(String methodName,
+            String[] parameterTypes) {
+        StringBuilder key = new StringBuilder(methodName);
+        key.append('(');
+        for (String parameter: parameterTypes) {
+            key.append(parameter);
+            // Note: A trailing ',' does not matter in this case
+            key.append(',');
+        }
+        key.append(')');
+
+        return key.toString();
+    }
 }
