@@ -55,7 +55,8 @@ public abstract class AbstractEndpoint {
         public enum SocketState {
             // TODO Add a new state to the AsyncStateMachine and remove
             //      ASYNC_END (if possible)
-            OPEN, CLOSED, LONG, ASYNC_END, SENDFILE, UPGRADING, UPGRADED
+            OPEN, CLOSED, LONG, ASYNC_END, SENDFILE, UPGRADING_TOMCAT,
+            UPGRADING, UPGRADED
         }
 
 
@@ -116,6 +117,7 @@ public abstract class AbstractEndpoint {
      */
     protected volatile boolean internalExecutor = false;
 
+
     /**
      * counter for nr of connections handled by an endpoint
      */
@@ -137,10 +139,27 @@ public abstract class AbstractEndpoint {
 
     // ----------------------------------------------------------------- Properties
 
+    /**
+     * Time to wait for the internal executor (if used) to terminate when the
+     * endpoint is stopped in milliseconds. Defaults to 5000 (5 seconds).
+     */
+    private long executorTerminationTimeoutMillis = 5000;
+
+    public long getExecutorTerminationTimeoutMillis() {
+        return executorTerminationTimeoutMillis;
+    }
+
+    public void setExecutorTerminationTimeoutMillis(
+            long executorTerminationTimeoutMillis) {
+        this.executorTerminationTimeoutMillis = executorTerminationTimeoutMillis;
+    }
+
+
     /**
      * Acceptor thread count.
      */
     protected int acceptorThreadCount = 0;
+
     public void setAcceptorThreadCount(int acceptorThreadCount) {
         this.acceptorThreadCount = acceptorThreadCount;
     }
@@ -503,6 +522,17 @@ public abstract class AbstractEndpoint {
                 //this is our internal one, so we need to shut it down
                 ThreadPoolExecutor tpe = (ThreadPoolExecutor) executor;
                 tpe.shutdownNow();
+                long timeout = getExecutorTerminationTimeoutMillis();
+                if (timeout > 0) {
+                    try {
+                        tpe.awaitTermination(timeout, TimeUnit.MILLISECONDS);
+                    } catch (InterruptedException e) {
+                        // Ignore
+                    }
+                    if (tpe.isTerminating()) {
+                        getLog().warn(sm.getString("endpoint.warn.executorShutdown", getName()));
+                    }
+                }
                 TaskQueue queue = (TaskQueue) tpe.getQueue();
                 queue.setParent(null);
             }
@@ -626,7 +656,9 @@ public abstract class AbstractEndpoint {
 
         for (int i = 0; i < count; i++) {
             acceptors[i] = createAcceptor();
-            Thread t = new Thread(acceptors[i], getName() + "-Acceptor-" + i);
+            String threadName = getName() + "-Acceptor-" + i;
+            acceptors[i].setThreadName(threadName);
+            Thread t = new Thread(acceptors[i], threadName);
             t.setPriority(getAcceptorThreadPriority());
             t.setDaemon(getDaemon());
             t.start();
@@ -677,6 +709,11 @@ public abstract class AbstractEndpoint {
 
 
     public String adjustRelativePath(String path, String relativeTo) {
+        // Empty or null path can't point to anything useful. The assumption is
+        // that the value is deliberately empty / null so leave it that way.
+        if (path == null || path.length() == 0) {
+            return path;
+        }
         String newPath = path;
         File f = new File(newPath);
         if ( !f.isAbsolute()) {
@@ -775,9 +812,8 @@ public abstract class AbstractEndpoint {
     private String keystoreFile = System.getProperty("user.home")+"/.keystore";
     public String getKeystoreFile() { return keystoreFile;}
     public void setKeystoreFile(String s ) {
-        String file = adjustRelativePath(s,
+        keystoreFile = adjustRelativePath(s,
                 System.getProperty(Constants.CATALINA_BASE_PROP));
-        this.keystoreFile = file;
     }
 
     private String keystorePass = null;
@@ -796,20 +832,10 @@ public abstract class AbstractEndpoint {
     public String getSslProtocol() { return sslProtocol;}
     public void setSslProtocol(String s) { sslProtocol = s;}
 
-    // Note: Some implementations use the comma separated string, some use
-    // the array
     private String ciphers = null;
-    private String[] ciphersarr = new String[0];
-    public String[] getCiphersArray() { return this.ciphersarr;}
     public String getCiphers() { return ciphers;}
     public void setCiphers(String s) {
         ciphers = s;
-        if ( s == null ) ciphersarr = new String[0];
-        else {
-            StringTokenizer t = new StringTokenizer(s,",");
-            ciphersarr = new String[t.countTokens()];
-            for (int i=0; i<ciphersarr.length; i++ ) ciphersarr[i] = t.nextToken();
-        }
     }
 
     private String keyAlias = null;
@@ -823,13 +849,8 @@ public abstract class AbstractEndpoint {
     private String truststoreFile = System.getProperty("javax.net.ssl.trustStore");
     public String getTruststoreFile() {return truststoreFile;}
     public void setTruststoreFile(String s) {
-        if (s == null) {
-            this.truststoreFile = null;
-        } else {
-            String file = adjustRelativePath(s,
-                    System.getProperty(Constants.CATALINA_BASE_PROP));
-            this.truststoreFile = file;
-        }
+        truststoreFile = adjustRelativePath(s,
+                System.getProperty(Constants.CATALINA_BASE_PROP));
     }
 
     private String truststorePass =
