@@ -71,6 +71,9 @@ public class JIoEndpoint extends AbstractEndpoint {
         // Set maxConnections to zero so we can tell if the user has specified
         // their own value on the connector when we reach bind()
         setMaxConnections(0);
+        // Reduce the executor timeout for BIO as threads in keep-alive will not
+        // terminate when the executor interrupts them.
+        setExecutorTerminationTimeoutMillis(0);
     }
 
     // ------------------------------------------------------------- Properties
@@ -215,6 +218,7 @@ public class JIoEndpoint extends AbstractEndpoint {
                         // socket
                         socket = serverSocketFactory.acceptSocket(serverSocket);
                     } catch (IOException ioe) {
+                        countDownConnection();
                         // Introduce delay if necessary
                         errorDelay = handleExceptionWithDelay(errorDelay);
                         // re-throw
@@ -227,10 +231,12 @@ public class JIoEndpoint extends AbstractEndpoint {
                     if (running && !paused && setSocketOptions(socket)) {
                         // Hand this socket off to an appropriate processor
                         if (!processSocket(socket)) {
+                            countDownConnection();
                             // Close socket right away
                             closeSocket(socket);
                         }
                     } else {
+                        countDownConnection();
                         // Close socket right away
                         closeSocket(socket);
                     }
@@ -304,7 +310,7 @@ public class JIoEndpoint extends AbstractEndpoint {
 
                     if ((state != SocketState.CLOSED)) {
                         if (status == null) {
-                            state = handler.process(socket, SocketStatus.OPEN);
+                            state = handler.process(socket, SocketStatus.OPEN_READ);
                         } else {
                             state = handler.process(socket,status);
                         }
@@ -321,7 +327,8 @@ public class JIoEndpoint extends AbstractEndpoint {
                             // Ignore
                         }
                     } else if (state == SocketState.OPEN ||
-                            state == SocketState.UPGRADING  ||
+                            state == SocketState.UPGRADING ||
+                            state == SocketState.UPGRADING_TOMCAT  ||
                             state == SocketState.UPGRADED){
                         socket.setKeptAlive(true);
                         socket.access();
@@ -333,7 +340,17 @@ public class JIoEndpoint extends AbstractEndpoint {
                 } finally {
                     if (launch) {
                         try {
-                            getExecutor().execute(new SocketProcessor(socket, SocketStatus.OPEN));
+                            getExecutor().execute(new SocketProcessor(socket, SocketStatus.OPEN_READ));
+                        } catch (RejectedExecutionException x) {
+                            log.warn("Socket reprocessing request was rejected for:"+socket,x);
+                            try {
+                                //unable to handle connection at this time
+                                handler.process(socket, SocketStatus.DISCONNECT);
+                            } finally {
+                                countDownConnection();
+                            }
+
+
                         } catch (NullPointerException npe) {
                             if (running) {
                                 log.error(sm.getString("endpoint.launch.fail"),
@@ -362,7 +379,7 @@ public class JIoEndpoint extends AbstractEndpoint {
         // Initialize maxConnections
         if (getMaxConnections() == 0) {
             // User hasn't set a value - use the default
-            setMaxConnections(getMaxThreads());
+            setMaxConnections(getMaxThreadsExecutor(true));
         }
 
         if (serverSocketFactory == null) {
@@ -505,6 +522,7 @@ public class JIoEndpoint extends AbstractEndpoint {
         try {
             SocketWrapper<Socket> wrapper = new SocketWrapper<Socket>(socket);
             wrapper.setKeepAliveLeft(getMaxKeepAliveRequests());
+            wrapper.setSecure(isSSLEnabled());
             // During shutdown, executor may be null - avoid NPE
             if (!running) {
                 return false;
@@ -559,6 +577,7 @@ public class JIoEndpoint extends AbstractEndpoint {
                             return false;
                         }
                         getExecutor().execute(proc);
+                        //TODO gotta catch RejectedExecutionException and properly handle it
                     } finally {
                         if (Constants.IS_SECURITY_ENABLED) {
                             PrivilegedAction<Void> pa = new PrivilegedSetTccl(loader);
