@@ -16,6 +16,7 @@
  */
 package org.apache.tomcat.jdbc.pool;
 
+import java.lang.ref.WeakReference;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.Proxy;
@@ -391,8 +392,6 @@ public class ConnectionPool {
             } catch (InterruptedException ex) {
                 if (getPoolProperties().getPropagateInterruptState()) {
                     Thread.currentThread().interrupt();
-                } else {
-                    Thread.interrupted();
                 }
             }
             if (pool.size()==0 && force && pool!=busy) pool = busy;
@@ -401,7 +400,9 @@ public class ConnectionPool {
         PoolProperties.InterceptorDefinition[] proxies = getPoolProperties().getJdbcInterceptorsAsArray();
         for (int i=0; i<proxies.length; i++) {
             try {
-                proxies[i].getInterceptorClass().newInstance().poolClosed(this);
+                JdbcInterceptor interceptor = proxies[i].getInterceptorClass().newInstance();
+                interceptor.setProperties(proxies[i].getProperties());
+                interceptor.poolClosed(this);
             }catch (Exception x) {
                 log.debug("Unable to inform interceptor of pool closure.",x);
             }
@@ -486,9 +487,12 @@ public class ConnectionPool {
             } //for
 
         } catch (SQLException x) {
-            if (jmxPool!=null) jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
-            close(true);
-            throw x;
+            log.error("Unable to create initial connections of pool.", x);
+            if (!poolProperties.isIgnoreExceptionOnPreLoad()) {
+                if (jmxPool!=null) jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                close(true);
+                throw x;
+            }
         } finally {
             //return the members as idle to the pool
             for (int i = 0; i < initialPool.length; i++) {
@@ -647,8 +651,6 @@ public class ConnectionPool {
             } catch (InterruptedException ex) {
                 if (getPoolProperties().getPropagateInterruptState()) {
                     Thread.currentThread().interrupt();
-                } else {
-                    Thread.interrupted();
                 }
                 SQLException sx = new SQLException("Pool wait interrupted.");
                 sx.initCause(ex);
@@ -710,7 +712,7 @@ public class ConnectionPool {
             } else {
                 //validation failed, make sure we disconnect
                 //and clean up
-                error =true;
+                throw new SQLException("Validation Query Failed, enable logValidationErrors for more details.");
             } //end if
         } catch (Exception e) {
             error = true;
@@ -730,7 +732,6 @@ public class ConnectionPool {
             }
             con.unlock();
         }//catch
-        return null;
     }
 
     /**
@@ -1168,7 +1169,7 @@ public class ConnectionPool {
         Future<PooledConnection> pcFuture = null;
         AtomicBoolean configured = new AtomicBoolean(false);
         CountDownLatch latch = new CountDownLatch(1);
-        Connection result = null;
+        volatile Connection result = null;
         SQLException cause = null;
         AtomicBoolean cancelled = new AtomicBoolean(false);
         volatile PooledConnection pc = null;
@@ -1315,14 +1316,13 @@ public class ConnectionPool {
         return poolCleanTimer;
     }
 
-    protected class PoolCleaner extends TimerTask {
-        protected ConnectionPool pool;
+    protected static class PoolCleaner extends TimerTask {
+        protected WeakReference<ConnectionPool> pool;
         protected long sleepTime;
-        protected volatile boolean run = true;
         protected volatile long lastRun = 0;
 
         PoolCleaner(ConnectionPool pool, long sleepTime) {
-            this.pool = pool;
+            this.pool = new WeakReference<ConnectionPool>(pool);
             this.sleepTime = sleepTime;
             if (sleepTime <= 0) {
                 log.warn("Database connection pool evicter thread interval is set to 0, defaulting to 30 seconds");
@@ -1334,11 +1334,11 @@ public class ConnectionPool {
 
         @Override
         public void run() {
-            if (pool.isClosed()) {
-                if (pool.getSize() <= 0) {
-                    run = false;
-                }
-            } else if ((System.currentTimeMillis() - lastRun) > sleepTime) {
+            ConnectionPool pool = this.pool.get();
+            if (pool == null) {
+                stopRunning();
+            } else if (!pool.isClosed() &&
+                    (System.currentTimeMillis() - lastRun) > sleepTime) {
                 lastRun = System.currentTimeMillis();
                 try {
                     if (pool.getPoolProperties().isRemoveAbandoned())
@@ -1350,9 +1350,9 @@ public class ConnectionPool {
                         pool.testAllIdle();
                 } catch (Exception x) {
                     log.error("", x);
-                } // catch
-            } // end if
-        } // run
+                }
+            }
+        }
 
         public void start() {
             registerCleaner(this);
