@@ -310,6 +310,11 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
         return getPoolProperties().getValidationQuery();
     }
 
+    @Override
+    public int getValidationQueryTimeout() {
+        return getPoolProperties().getValidationQueryTimeout();
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -663,6 +668,11 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
         getPoolProperties().setValidationQuery(validationQuery);
     }
 
+    @Override
+    public void setValidationQueryTimeout(int validationQueryTimeout) {
+        getPoolProperties().setValidationQueryTimeout(validationQueryTimeout);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -828,6 +838,22 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
         getPoolProperties().setPropagateInterruptState(propagateInterruptState);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean isIgnoreExceptionOnPreLoad() {
+        return getPoolProperties().isIgnoreExceptionOnPreLoad();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setIgnoreExceptionOnPreLoad(boolean ignoreExceptionOnPreLoad) {
+        getPoolProperties().setIgnoreExceptionOnPreLoad(ignoreExceptionOnPreLoad);
+    }
+
     /**
      * {@inheritDoc}
      */
