@@ -23,20 +23,14 @@ import java.util.concurrent.CountDownLatch;
 
 import javax.sql.DataSource;
 
+import org.junit.Test;
 
-/**
- * @author Filip Hanik
- * @version 1.0
- */
 public class CheckOutThreadTest extends DefaultTestCase {
-    public CheckOutThreadTest(String name) {
-        super(name);
-    }
 
     CountDownLatch latch = null;
 
+    @Test
     public void testDBCPThreads10Connections10() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.threadcount = 10;
         this.transferProperties();
@@ -55,8 +49,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections10() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setFairQueue(false);
         this.threadcount = 10;
@@ -76,8 +70,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections10Fair() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setFairQueue(true);
         this.threadcount = 10;
@@ -97,8 +91,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+//    @Test
 //    public void testC3P0Threads10Connections10() throws Exception {
-//        init();
 //        this.datasource.getPoolProperties().setMaxActive(10);
 //        this.threadcount = 10;
 //        this.transferPropertiesToC3P0();
@@ -117,8 +111,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
 //        tearDown();
 //    }
 
+    @Test
     public void testDBCPThreads20Connections10() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.threadcount = 20;
         this.transferProperties();
@@ -137,8 +131,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads20Connections10() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setFairQueue(false);
         this.threadcount = 20;
@@ -158,8 +152,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads20Connections10Fair() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setFairQueue(true);
         this.threadcount = 20;
@@ -179,8 +173,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+//    @Test
 //    public void testC3P0Threads20Connections10() throws Exception {
-//        init();
 //        this.datasource.getPoolProperties().setMaxActive(10);
 //        this.threadcount = 20;
 //        this.transferPropertiesToC3P0();
@@ -199,8 +193,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
 //        tearDown();
 //    }
 
+    @Test
     public void testDBCPThreads10Connections10Validate() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.threadcount = 10;
@@ -220,8 +214,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections10Validate() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.datasource.getPoolProperties().setFairQueue(false);
@@ -242,8 +236,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections10ValidateFair() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.datasource.getPoolProperties().setFairQueue(true);
@@ -264,8 +258,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+//    @Test
 //    public void testC3P0Threads10Connections10Validate() throws Exception {
-//        init();
 //        this.datasource.getPoolProperties().setMaxActive(10);
 //        this.datasource.getPoolProperties().setTestOnBorrow(true);
 //        this.threadcount = 10;
@@ -285,8 +279,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
 //        tearDown();
 //    }
 
+    @Test
     public void testDBCPThreads20Connections10Validate() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.threadcount = 20;
@@ -306,8 +300,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections20Validate() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.datasource.getPoolProperties().setFairQueue(false);
@@ -328,8 +322,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+    @Test
     public void testPoolThreads10Connections20ValidateFair() throws Exception {
-        init();
         this.datasource.getPoolProperties().setMaxActive(10);
         this.datasource.getPoolProperties().setTestOnBorrow(true);
         this.datasource.getPoolProperties().setFairQueue(true);
@@ -350,8 +344,8 @@ public class CheckOutThreadTest extends DefaultTestCase {
         tearDown();
     }
 
+//    @Test
 //    public void testC3P0Threads10Connections20Validate() throws Exception {
-//        init();
 //        this.datasource.getPoolProperties().setMaxActive(10);
 //        this.datasource.getPoolProperties().setTestOnBorrow(true);
 //        this.threadcount = 20;
