@@ -17,12 +17,17 @@
 
 package org.apache.catalina.connector;
 
+import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.net.HttpURLConnection;
 import java.net.URL;
+import java.util.ArrayList;
 import java.util.Enumeration;
+import java.util.List;
 import java.util.TreeMap;
 
 import javax.servlet.ServletException;
@@ -33,12 +38,16 @@ import javax.servlet.http.HttpServletResponse;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import org.junit.Test;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.authenticator.BasicAuthenticator;
+import org.apache.catalina.deploy.FilterDef;
+import org.apache.catalina.deploy.FilterMap;
 import org.apache.catalina.deploy.LoginConfig;
+import org.apache.catalina.filters.FailedRequestFilter;
 import org.apache.catalina.startup.SimpleHttpClient;
 import org.apache.catalina.startup.TestTomcat.MapRealm;
 import org.apache.catalina.startup.Tomcat;
@@ -58,8 +67,7 @@ public class TestRequest extends TomcatBaseTest {
      */
     @Test
     public void testBug37794() {
-        Bug37794Client client = new Bug37794Client();
-        client.setPort(getPort());
+        Bug37794Client client = new Bug37794Client(true);
 
         // Edge cases around zero
         client.doRequest(-1, false); // Unlimited
@@ -71,14 +79,14 @@ public class TestRequest extends TomcatBaseTest {
         assertTrue(client.isResponseBodyOK());
         client.reset();
         client.doRequest(1, false); // 1 byte - too small should fail
-        assertTrue(client.isResponse500());
+        assertTrue(client.isResponse400());
 
         client.reset();
 
         // Edge cases around actual content length
         client.reset();
         client.doRequest(6, false); // Too small should fail
-        assertTrue(client.isResponse500());
+        assertTrue(client.isResponse400());
         client.reset();
         client.doRequest(7, false); // Just enough should pass
         assertTrue(client.isResponse200());
@@ -101,6 +109,22 @@ public class TestRequest extends TomcatBaseTest {
         assertTrue(client.isResponseBodyOK());
     }
 
+    /**
+     * Additional test for failed requests handling when no FailedRequestFilter
+     * is defined.
+     */
+    @Test
+    public void testBug37794withoutFilter() {
+        Bug37794Client client = new Bug37794Client(false);
+
+        // Edge cases around actual content length
+        client.reset();
+        client.doRequest(6, false); // Too small should fail
+        // Response code will be OK, but parameters list will be empty
+        assertTrue(client.isResponse200());
+        assertEquals("", client.getResponseBody());
+    }
+
     private static class Bug37794Servlet extends HttpServlet {
 
         private static final long serialVersionUID = 1L;
@@ -130,8 +154,14 @@ public class TestRequest extends TomcatBaseTest {
      */
     private class Bug37794Client extends SimpleHttpClient {
 
+        private final boolean createFilter;
+
         private boolean init;
 
+        public Bug37794Client(boolean createFilter) {
+            this.createFilter = createFilter;
+        }
+
         private synchronized void init() throws Exception {
             if (init) return;
 
@@ -139,8 +169,23 @@ public class TestRequest extends TomcatBaseTest {
             Context root = tomcat.addContext("", TEMP_DIR);
             Tomcat.addServlet(root, "Bug37794", new Bug37794Servlet());
             root.addServletMapping("/test", "Bug37794");
+
+            if (createFilter) {
+                FilterDef failedRequestFilter = new FilterDef();
+                failedRequestFilter.setFilterName("failedRequestFilter");
+                failedRequestFilter.setFilterClass(
+                        FailedRequestFilter.class.getName());
+                FilterMap failedRequestFilterMap = new FilterMap();
+                failedRequestFilterMap.setFilterName("failedRequestFilter");
+                failedRequestFilterMap.addURLPattern("/*");
+                root.addFilterDef(failedRequestFilter);
+                root.addFilterMap(failedRequestFilterMap);
+            }
+
             tomcat.start();
 
+            setPort(tomcat.getConnector().getLocalPort());
+
             init = true;
         }
 
@@ -323,7 +368,7 @@ public class TestRequest extends TomcatBaseTest {
         root.addServletMapping("/", "Bug37794");
         tomcat.start();
 
-        HttpURLConnection conn = getConnection();
+        HttpURLConnection conn = getConnection("http://localhost:" + getPort() + "/");
         InputStream is = conn.getInputStream();
         assertNotNull(is);
     }
@@ -337,7 +382,7 @@ public class TestRequest extends TomcatBaseTest {
         root.addServletMapping("/", "Bug37794");
         tomcat.start();
 
-        HttpURLConnection conn = getConnection();
+        HttpURLConnection conn = getConnection("http://localhost:" + getPort() + "/");
         conn.setChunkedStreamingMode(8 * 1024);
         InputStream is = conn.getInputStream();
         assertNotNull(is);
@@ -352,7 +397,6 @@ public class TestRequest extends TomcatBaseTest {
     @Test
     public void testBug48692() {
         Bug48692Client client = new Bug48692Client();
-        client.setPort(getPort());
 
         // Make sure GET works properly
         client.doRequest("GET", "foo=bar", null, null, false);
@@ -421,6 +465,34 @@ public class TestRequest extends TomcatBaseTest {
         */
     }
 
+    @Test
+    public void testBug54984() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+        Context root = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        root.setAllowCasualMultipartParsing(true);
+        Tomcat.addServlet(root, "Bug54984", new Bug54984Servlet());
+        root.addServletMapping("/", "Bug54984");
+        tomcat.start();
+
+        HttpURLConnection conn = getConnection("http://localhost:" + getPort()
+                + "/parseParametersBeforeParseParts");
+
+        prepareRequestBug54984(conn);
+
+        checkResponseBug54984(conn);
+
+        conn.disconnect();
+
+        conn = getConnection("http://localhost:" + getPort() + "/");
+
+        prepareRequestBug54984(conn);
+
+        checkResponseBug54984(conn);
+
+        conn.disconnect();
+    }
+
     /**
      *
      */
@@ -479,6 +551,8 @@ public class TestRequest extends TomcatBaseTest {
             root.addServletMapping("/echo", "EchoParameters");
             tomcat.start();
 
+            setPort(tomcat.getConnector().getLocalPort());
+
             init = true;
         }
 
@@ -536,8 +610,7 @@ public class TestRequest extends TomcatBaseTest {
         }
     }
 
-    private HttpURLConnection getConnection() throws IOException {
-        final String query = "http://localhost:" + getPort() + "/";
+    private HttpURLConnection getConnection(String query) throws IOException {
         URL postURL;
         postURL = new URL(query);
         HttpURLConnection conn = (HttpURLConnection) postURL.openConnection();
@@ -550,4 +623,78 @@ public class TestRequest extends TomcatBaseTest {
 
         return conn;
     }
+
+    private static class Bug54984Servlet extends HttpServlet {
+
+        private static final long serialVersionUID = 1L;
+
+        @Override
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
+            req.setCharacterEncoding("UTF-8");
+
+            if (req.getRequestURI().endsWith("parseParametersBeforeParseParts")) {
+                req.getParameterNames();
+            }
+
+            req.getPart("part");
+
+            resp.setContentType("text/plain");
+            resp.setCharacterEncoding("UTF-8");
+
+            resp.getWriter().println("Part " + req.getParameter("part"));
+        }
+    }
+
+    private void prepareRequestBug54984(HttpURLConnection conn)
+            throws Exception {
+        String boundary = "-----" + System.currentTimeMillis();
+        conn.setRequestProperty("Content-Type",
+                "multipart/form-data; boundary=" + boundary);
+
+        PrintWriter writer = null;
+        try {
+            writer = new PrintWriter(new OutputStreamWriter(
+                    conn.getOutputStream(), "UTF-8"), true);
+            writer.append("--" + boundary).append("\r\n");
+            writer.append("Content-Disposition: form-data; name=\"part\"\r\n");
+            writer.append("Content-Type: text/plain; charset=UTF-8\r\n");
+            writer.append("\r\n");
+            writer.append("").append("\r\n");
+            writer.flush();
+
+            writer.append("\r\n");
+            writer.flush();
+
+            writer.append("--" + boundary + "--").append("\r\n");
+        } finally {
+            if (writer != null) {
+                writer.close();
+            }
+        }
+    }
+
+    private void checkResponseBug54984(HttpURLConnection conn)
+            throws Exception {
+        List<String> response = new ArrayList<String>();
+        int status = conn.getResponseCode();
+        if (status == HttpURLConnection.HTTP_OK) {
+            BufferedReader reader = null;
+            try {
+                reader = new BufferedReader(new InputStreamReader(
+                        conn.getInputStream(), "UTF-8"));
+                String line = null;
+                while ((line = reader.readLine()) != null) {
+                    response.add(line);
+                }
+                assertTrue(response.contains("Part "));
+            } finally {
+                if (reader != null) {
+                    reader.close();
+                }
+            }
+        } else {
+            fail("OK status was expected: " + status);
+        }
+    }
 }
