@@ -42,7 +42,9 @@ public abstract class SimpleHttpClient {
     public static final String TEMP_DIR =
             System.getProperty("java.io.tmpdir");
 
-    public static final String CRLF = "\r\n";
+    public static final String CR = "\r";
+    public static final String LF = "\n";
+    public static final String CRLF = CR + LF;
 
     public static final String INFO_100 = "HTTP/1.1 100";
     public static final String OK_200 = "HTTP/1.1 200";
@@ -60,7 +62,7 @@ public abstract class SimpleHttpClient {
 
     protected static final String SESSION_COOKIE_NAME = "JSESSIONID";
     protected static final String SESSION_PARAMETER_NAME =
-            SESSION_COOKIE_NAME.toLowerCase(Locale.US);
+            SESSION_COOKIE_NAME.toLowerCase(Locale.ENGLISH);
 
     private static final String COOKIE_HEADER_PREFIX = "Set-Cookie: ";
     private static final String SESSION_COOKIE_HEADER_PREFIX =
