@@ -143,7 +143,20 @@ public class TestContextConfig extends TomcatBaseTest {
         tomcat.start();
 
         assertPageContains("/test/testServlet",
-                "envEntry1: 1 envEntry2: 2 envEntry3: 33 envEntry4: 4");
+                "envEntry1: 1 envEntry2: 2 envEntry3: 33 envEntry4: 4 envEntry5: 55 envEntry6: 66");
+    }
+
+    @Test
+    public void testBug55210() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp-3.0-fragments");
+        tomcat.addWebapp(null, "/test", appDir.getAbsolutePath());
+
+        tomcat.start();
+
+        assertPageContains("/test/TesterServlet1", "OK");
+        assertPageContains("/test/TesterServlet2", "OK");
     }
 
     private static class CustomDefaultServletSCI
