@@ -30,6 +30,7 @@ import static org.junit.Assert.assertTrue;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.deploy.ApplicationListener;
 
 public class TestListener extends TomcatBaseTest {
 
@@ -52,7 +53,7 @@ public class TestListener extends TomcatBaseTest {
 
     /**
      * Check that a {@link ServletContextListener} cannot install a
-     * {@link ServletContextInitializer}.
+     * {@link javax.servlet.ServletContainerInitializer}.
      * @throws Exception
      */
     @Test
@@ -63,11 +64,12 @@ public class TestListener extends TomcatBaseTest {
                 System.getProperty("java.io.tmpdir"));
 
         // SCL2 pretends to be in web.xml, and tries to install a
-        // ServletContextInitializer.
-        context.addApplicationListener(SCL2.class.getName());
+        // ServletContainerInitializer.
+        context.addApplicationListener(new ApplicationListener(
+                SCL2.class.getName(), false));
         tomcat.start();
 
-        //check that the ServletContextInitializer wasn't initialized.
+        //check that the ServletContainerInitializer wasn't initialized.
         assertFalse(SCL3.initialized);
     }
 
