@@ -32,7 +32,7 @@ import org.apache.catalina.connector.Response;
 public class TesterAccessLogValve extends ValveBase implements AccessLog {
 
     // Timing tests need a small error margin to prevent failures
-    private static final long ERROR_MARGIN = 10;
+    private static final long ERROR_MARGIN = 100;
 
     private final List<Entry> entries = new ArrayList<Entry>();
 
@@ -78,9 +78,9 @@ public class TesterAccessLogValve extends ValveBase implements AccessLog {
         for (int j = 0; j < count; j++) {
             Entry entry = entries.get(j);
             assertEquals(status, entry.getStatus());
-            assertTrue(entry.toString(),
+            assertTrue(entry.toString() + " duration is not >= " + (minTime - ERROR_MARGIN),
                     entry.getTime() >= minTime - ERROR_MARGIN);
-            assertTrue(entry.toString(),
+            assertTrue(entry.toString() + " duration is not < " + (maxTime + ERROR_MARGIN),
                     entry.getTime() < maxTime + ERROR_MARGIN);
         }
     }
