@@ -16,14 +16,11 @@
  */
 package org.apache.catalina.websocket;
 
-import java.io.BufferedReader;
-import java.io.File;
+import java.io.BufferedInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
-import java.io.Reader;
 import java.io.Writer;
 import java.net.InetSocketAddress;
 import java.net.Socket;
@@ -36,6 +33,7 @@ import java.util.List;
 
 import javax.naming.InitialContext;
 import javax.naming.NamingException;
+import javax.servlet.http.HttpServletRequest;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
@@ -44,7 +42,9 @@ import static org.junit.Assert.assertTrue;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.deploy.ApplicationListener;
 import org.apache.catalina.deploy.ContextEnvironment;
+import org.apache.catalina.servlets.DefaultServlet;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.catalina.util.Base64;
@@ -52,7 +52,12 @@ import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.C2BConverter;
 import org.apache.tomcat.util.buf.CharChunk;
+import org.apache.tomcat.websocket.TesterEchoServer;
 
+/**
+ * @deprecated  Will be removed in Tomcat 8.0.x.
+ */
+@Deprecated
 public class TestWebSocket extends TomcatBaseTest {
 
     private static final String CRLF = "\r\n";
@@ -63,16 +68,21 @@ public class TestWebSocket extends TomcatBaseTest {
     @Test
     public void testSimple() throws Exception {
         Tomcat tomcat = getTomcatInstance();
-        File appDir = new File(getBuildDirectory(), "webapps/examples");
-        tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
+        // Must have a real docBase - just use temp
+        Context ctx = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        ctx.addApplicationListener(new ApplicationListener(
+                TesterEchoServer.Config.class.getName(), false));
+
+        Tomcat.addServlet(ctx, "default", new DefaultServlet());
+        ctx.addServletMapping("/", "default");
 
         tomcat.start();
 
-        WebSocketClient client= new WebSocketClient(getPort());
-
+        WebSocketClient client = new WebSocketClient(getPort());
 
         // Send the WebSocket handshake
-        client.writer.write("GET /examples/websocket/echoStream HTTP/1.1" + CRLF);
+        client.writer.write("GET " + TesterEchoServer.Config.PATH_BASIC + " HTTP/1.1" + CRLF);
         client.writer.write("Host: foo" + CRLF);
         client.writer.write("Upgrade: websocket" + CRLF);
         client.writer.write("Connection: keep-alive, upgrade" + CRLF);
@@ -104,14 +114,21 @@ public class TestWebSocket extends TomcatBaseTest {
     @Test
     public void testDetectWrongVersion() throws Exception {
         Tomcat tomcat = getTomcatInstance();
-        File appDir = new File(getBuildDirectory(), "webapps/examples");
-        tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
+        // Must have a real docBase - just use temp
+        Context ctx = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        ctx.addApplicationListener(new ApplicationListener(
+                TesterEchoServer.Config.class.getName(), false));
+
+        Tomcat.addServlet(ctx, "default", new DefaultServlet());
+        ctx.addServletMapping("/", "default");
 
         tomcat.start();
+
         WebSocketClient client= new WebSocketClient(getPort());
 
         // Send the WebSocket handshake
-        client.writer.write("GET /examples/websocket/echoStream HTTP/1.1" + CRLF);
+        client.writer.write("GET " + TesterEchoServer.Config.PATH_BASIC + " HTTP/1.1" + CRLF);
         client.writer.write("Host: foo" + CRLF);
         client.writer.write("Upgrade: websocket" + CRLF);
         client.writer.write("Connection: upgrade" + CRLF);
@@ -140,15 +157,21 @@ public class TestWebSocket extends TomcatBaseTest {
     @Test
     public void testNoConnection() throws Exception {
         Tomcat tomcat = getTomcatInstance();
-        File appDir = new File(getBuildDirectory(), "webapps/examples");
-        tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
+        // Must have a real docBase - just use temp
+        Context ctx = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        ctx.addApplicationListener(new ApplicationListener(
+                TesterEchoServer.Config.class.getName(), false));
+
+        Tomcat.addServlet(ctx, "default", new DefaultServlet());
+        ctx.addServletMapping("/", "default");
 
         tomcat.start();
+
         WebSocketClient client= new WebSocketClient(getPort());
 
-
         // Send the WebSocket handshake
-        client.writer.write("GET /examples/websocket/echoStream HTTP/1.1" + CRLF);
+        client.writer.write("GET " + TesterEchoServer.Config.PATH_BASIC + " HTTP/1.1" + CRLF);
         client.writer.write("Host: foo" + CRLF);
         client.writer.write("Upgrade: websocket" + CRLF);
         client.writer.write("Sec-WebSocket-Version: 13" + CRLF);
@@ -156,7 +179,7 @@ public class TestWebSocket extends TomcatBaseTest {
         client.writer.write(CRLF);
         client.writer.flush();
 
-        // Make sure we got an upgrade response
+        // Make sure we got an error response
         String responseLine = client.reader.readLine();
         assertTrue(responseLine.startsWith("HTTP/1.1 400"));
 
@@ -168,14 +191,21 @@ public class TestWebSocket extends TomcatBaseTest {
     @Test
     public void testNoUpgrade() throws Exception {
         Tomcat tomcat = getTomcatInstance();
-        File appDir = new File(getBuildDirectory(), "webapps/examples");
-        tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
+        // Must have a real docBase - just use temp
+        Context ctx = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        ctx.addApplicationListener(new ApplicationListener(
+                TesterEchoServer.Config.class.getName(), false));
+
+        Tomcat.addServlet(ctx, "default", new DefaultServlet());
+        ctx.addServletMapping("/", "default");
 
         tomcat.start();
+
         WebSocketClient client= new WebSocketClient(getPort());
 
         // Send the WebSocket handshake
-        client.writer.write("GET /examples/websocket/echoStream HTTP/1.1" + CRLF);
+        client.writer.write("GET " + TesterEchoServer.Config.PATH_BASIC + " HTTP/1.1" + CRLF);
         client.writer.write("Host: foo" + CRLF);
         client.writer.write("Connection: upgrade" + CRLF);
         client.writer.write("Sec-WebSocket-Version: 13" + CRLF);
@@ -183,9 +213,11 @@ public class TestWebSocket extends TomcatBaseTest {
         client.writer.write(CRLF);
         client.writer.flush();
 
-        // Make sure we got an upgrade response
+        // Make sure we got an error response
+        // No upgrade means it is not treated an as upgrade request so a 404 is
+        // generated when the request reaches the Default Servlet.s
         String responseLine = client.reader.readLine();
-        assertTrue(responseLine.startsWith("HTTP/1.1 400"));
+        assertTrue(responseLine.startsWith("HTTP/1.1 404"));
 
         // Finished with the socket
         client.close();
@@ -194,14 +226,21 @@ public class TestWebSocket extends TomcatBaseTest {
     @Test
     public void testKey() throws Exception {
         Tomcat tomcat = getTomcatInstance();
-        File appDir = new File(getBuildDirectory(), "webapps/examples");
-        tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
+        // Must have a real docBase - just use temp
+        Context ctx = tomcat.addContext("",
+                System.getProperty("java.io.tmpdir"));
+        ctx.addApplicationListener(new ApplicationListener(
+                TesterEchoServer.Config.class.getName(), false));
+
+        Tomcat.addServlet(ctx, "default", new DefaultServlet());
+        ctx.addServletMapping("/", "default");
 
         tomcat.start();
+
         WebSocketClient client= new WebSocketClient(getPort());
 
         // Send the WebSocket handshake
-        client.writer.write("GET /examples/websocket/echoStream HTTP/1.1" + CRLF);
+        client.writer.write("GET " + TesterEchoServer.Config.PATH_BASIC + " HTTP/1.1" + CRLF);
         client.writer.write("Host: foo" + CRLF);
         client.writer.write("Upgrade: websocket" + CRLF);
         client.writer.write("Connection: upgrade" + CRLF);
@@ -296,7 +335,8 @@ public class TestWebSocket extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
 
         @Override
-        protected StreamInbound createWebSocketInbound(String subProtocol) {
+        protected StreamInbound createWebSocketInbound(String subProtocol,
+                HttpServletRequest request) {
             return new Bug53339WsInbound();
         }
     }
@@ -340,12 +380,11 @@ public class TestWebSocket extends TomcatBaseTest {
 
     private static class WebSocketClient {
         private OutputStream os;
-        private InputStream is;
         private boolean isContinuation = false;
         final String encoding = "ISO-8859-1";
-        private Socket socket ;
-        private Writer writer ;
-        private BufferedReader reader;
+        private Socket socket;
+        private Writer writer;
+        private CustomReader reader;
 
         public WebSocketClient(int port) {
             SocketAddress addr = new InetSocketAddress("localhost", port);
@@ -355,9 +394,7 @@ public class TestWebSocket extends TomcatBaseTest {
                 socket.connect(addr, 10000);
                 os = socket.getOutputStream();
                 writer = new OutputStreamWriter(os, encoding);
-                is = socket.getInputStream();
-                Reader r = new InputStreamReader(is, encoding);
-                reader = new BufferedReader(r);
+                reader = new CustomReader(socket.getInputStream(), encoding);
             } catch (Exception e) {
                 throw new RuntimeException(e);
             }
@@ -370,9 +407,10 @@ public class TestWebSocket extends TomcatBaseTest {
         private void sendMessage(String message, boolean finalFragment)
                 throws IOException {
             ByteChunk bc = new ByteChunk(8192);
-            C2BConverter c2b = new C2BConverter(bc, "UTF-8");
-            c2b.convert(message);
-            c2b.flushBuffer();
+            CharChunk cc = new CharChunk(8192);
+            C2BConverter c2b = new C2BConverter("UTF-8");
+            cc.append(message);
+            c2b.convert(cc, bc);
 
             int len = bc.getLength();
             assertTrue(len < 126);
@@ -406,28 +444,100 @@ public class TestWebSocket extends TomcatBaseTest {
         }
 
         private String readMessage() throws IOException {
-            ByteChunk bc = new ByteChunk(125);
-            CharChunk cc = new CharChunk(125);
 
             // Skip first byte
-            is.read();
+            reader.read();
 
             // Get payload length
-            int len = is.read() & 0x7F;
+            int len = reader.read() & 0x7F;
             assertTrue(len < 126);
 
             // Read payload
-            int read = 0;
-            while (read < len) {
-                read = read + is.read(bc.getBytes(), read, len - read);
+            reader.setEncoding("UTF-8");
+            return reader.read(len);
+        }
+        /*
+         * This is not a real reader but just a thin wrapper around
+         * an input stream that allows to switch encoding during
+         * reading.
+         * An example is reading headers using ISO-8859-1 and a body
+         * using UTF-8.
+         * The class is not thread-safe and not well-performing.
+         */
+        private class CustomReader {
+            private InputStream is;
+            private String encoding;
+            private boolean markSupported;
+            private B2CConverter b2c;
+
+            public CustomReader(InputStream is, String encoding) throws IOException {
+                this.is = new BufferedInputStream(is);
+                this.encoding = encoding;
+                markSupported = is.markSupported();
+                b2c = new B2CConverter(encoding);
             }
 
-            bc.setEnd(len);
+            public String getEncoding() {
+                return encoding;
+            }
 
-            B2CConverter b2c = new B2CConverter("UTF-8");
-            b2c.convert(bc, cc, len);
+            public void setEncoding(String encoding) throws IOException {
+                this.encoding = encoding;
+                b2c = new B2CConverter(encoding);
+            }
 
-            return cc.toString();
+            public int read() throws IOException {
+                return is.read();
+            }
+
+            public String read(int len) throws IOException {
+                ByteChunk bc = new ByteChunk(125);
+                CharChunk cc = new CharChunk(125);
+                int read = 0;
+                while (read < len) {
+                    read = read + is.read(bc.getBytes(), read, len - read);
+                }
+
+                bc.setEnd(len);
+                b2c.convert(bc, cc, true);
+                return cc.toString();
+            }
+
+            public String readLine() throws IOException {
+                ByteChunk bc = new ByteChunk(125);
+                CharChunk cc = new CharChunk(125);
+                char c;
+                int i = is.read();
+                int read = 0;
+                while (i != -1) {
+                    if (i != '\r' && i != '\n') {
+                        bc.append((byte)i);
+                        read++;
+                    } else if (i == '\n') {
+                        break;
+                    } else if (i == '\r') {
+                        if (markSupported) {
+                            is.mark(2);
+                        }
+                        i = read();
+                        if (i == -1) {
+                            break;
+                        } else {
+                            if (i == '\n') {
+                                break;
+                            } else {
+                                if (markSupported) {
+                                    is.reset();
+                                }
+                            }
+                        }
+                    }
+                    i = is.read();
+                }
+                bc.setEnd(read);
+                b2c.convert(bc, cc, true);
+                return cc.toString();
+            }
         }
     }
 }
