@@ -60,7 +60,7 @@ public class TestCookiesAllowEquals extends TomcatBaseTest {
 
             tomcat.start();
             // Open connection
-            setPort(tomcat.getConnector().getPort());
+            setPort(tomcat.getConnector().getLocalPort());
             connect();
 
             String[] request = new String[1];
